unit T_SpPick;

interface
uses
  Elements,
  e_Geometry,
  G_Elems,
  T_Recurs;

type
  TPickUper = class(TRecursor)
  private
    FFound: boolean;
    FCrd: TCoordinate;
    FTolerance: double;
    FMulti: boolean;
    FMultiElement: TGeometry;

    procedure SetCoord(aVal: TCoordinate);
    function  GetCoord: TCoordinate;

    procedure SetX(aVal: double);
    procedure SetY(aVal: double);
    function  GetX: double;
    function  GetY: double;
    procedure SetMulti(const Value: boolean);
    procedure SetMultiElement(const Value: TGeometry);
  public
    constructor Create; override;
    destructor  Destroy; override;
    procedure   DoIt; override;

    procedure Process(Element: TElement); override;
    property Coord: TCoordinate read GetCoord write SetCoord;
  published
    property Found: boolean read FFound;
    property X: double read GetX write SetX;
    property Y: double read GetY write SetY;
    property Tolerance: double read FTolerance write FTolerance;
    property Multi: boolean read FMulti write SetMulti;
    property MultiElement: TGeometry read FMultiElement write SetMultiElement;
  end;


  TPickUpPoint = class(TPickUper)
  public
    function  QGoInside(Element: TElement): boolean; override;
    function  QProcessBefore(Element: TElement): boolean; override;
  end;



  TPickUpPolyLine = class(TPickUper)
  public
    function  QGoInside(Element: TElement): boolean; override;
    function  QProcessBefore(Element: TElement): boolean; override;
  end;


  TPickUpArea = class(TPickUper)
  public
    procedure Process(Element: TElement); override;
    function  QGoInside(Element: TElement): boolean; override;
    function  QProcessBefore(Element: TElement): boolean; override;
  end;

implementation

uses
   E_Class;



constructor TPickUper.Create;
begin
  inherited Create;
  FFound:= false;
end;


destructor  TPickUper.Destroy;
begin
  if FCrd <> nil then
     FCrd.Free;
  inherited Destroy
end;

procedure TPickUper.DoIt;
begin
  FFound:= false;
  MultiElement := nil;
  Multi := false;
  inherited DoIt
end;

procedure TPickUper.SetCoord(aVal: TCoordinate);
begin
  if FCrd <> nil then FCrd.Free;
  FCrd:= aVal
end;

function  TPickUper.GetCoord: TCoordinate;
begin
  result:= FCrd;
  if result <> nil then exit;

  FCrd:= T2DCoordinate.Create;
  result:= FCrd;
end;


procedure TPickUper.SetX(aVal: double);
begin
  if Coord = nil then exit;
  Coord.X:= aVal;
end;



procedure TPickUper.SetY(aVal: double);
begin
  if Coord = nil then exit;
  Coord.Y:= aVal;
end;

function  TPickUper.GetX: double;
begin
  result:= 0;
  if Coord = nil then exit;
  result:= Coord.X
end;

function  TPickUper.GetY: double;
begin
  result:= 0;
  if Coord = nil then exit;
  result:= Coord.Y
end;


procedure TPickUper.Process(Element: TElement);
begin
  if Found then exit;
  if not (Element is TGeometry) then exit;
  if Coord = nil then exit;
  FFound:= TGeometry(Element).IsHit(Coord, Tolerance);
  if Found then
     Output:= Element
end;



                     { TPickUpPoint }

function  TPickUpPoint.QGoInside(Element: TElement): boolean;
begin
  Result:= not Found and (Element is TRectEntity)
           and not (Element is TAreaString)
           and not (Element is TPolyLineString)
           and not (Element is TPolyLineElement)
           and not (Element is TAreaElement)
end;


function  TPickUpPoint.QProcessBefore(Element: TElement): boolean;
begin
  Result:= not Found and (Element is TPointElement)
end;


                   { TPickUpPolyLine }

function  TPickUpPolyLine.QGoInside(Element: TElement): boolean;
begin
  Result:= not Found and (Element is TRectEntity)
           and not (Element is TPolyLineElement)
           and not (Element is TAreaElement)
           and not (Element is TAreaString)
           and not (Element is TPointString)
end;


function  TPickUpPolyLine.QProcessBefore(Element: TElement): boolean;
begin
  Result:= not Found and (Element is TPolyLineElement)
end;





             { TPickUpArea }

procedure TPickUpArea.Process(Element: TElement);
var
   MA: TMultiAreaElement;
   A: TAreaElement;
   i: integer;
begin
  if Found then exit;
  if Coord = nil then exit;
  if Element is TMultiAreaElement then
  begin
     MA := TMultiAreaElement(Element);
     for i := 0 to MA.LastIndex do
     begin
        A := TAreaElement(MA.Elements[i]);
        FFound:= TGeometry(A).IsHit(Coord, Tolerance);
        if Found then
        begin
           Multi := true;
           MultiElement := MA;
           Output := A;
           break;
        end;
     end;
  end
  else
  begin
     FFound:= TGeometry(Element).IsHit(Coord, Tolerance);
     if Found then
        Output:= Element
  end;
end;


function  TPickUpArea.QGoInside(Element: TElement): boolean;
begin
  Result:= not Found and (Element is TRectEntity)
           and not (Element is TAreaElement)
           and not (Element is TPolyLineElement)
           and not (Element is TPolyLineString)
           and not (Element is TPointString)
end;


function  TPickUpArea.QProcessBefore(Element: TElement): boolean;
begin
  Result:= not Found and (Element is TAreaElement)
end;

procedure TPickUper.SetMulti(const Value: boolean);
begin
   FMulti := Value;
end;

procedure TPickUper.SetMultiElement(const Value: TGeometry);
begin
  FMultiElement := Value;
end;

initialization
   RegisterOrbisClass(TPickUper);
   RegisterOrbisClass(TPickUpPoint);
   RegisterOrbisClass(TPickUpPolyLine);
   RegisterOrbisClass(TPickUpArea);
end.

