{$D+}
unit T_IniFile;

interface
uses
  IniFiles, Classes,
  Elements, Tools,
  E_Type;

type

T_IniFile = class(TTool)
private
   FFileName: string;
   FSection: string;
   FIdent: string;
   FDefaultValue: string;
   FSLE: TStringListElement;
   FIni: TIniFile;
   procedure SetFileName(aValue: string);
   function GetValue: string;
   procedure SetValue(aValue: string);
   procedure SetIni(const Value: TIniFile);
    function GetSLE: TStringListElement;
    function GetSectionElement: TEntity;
    procedure SetSectionElement(const Value: TEntity);
public
   destructor Destroy; override;
   property  Ini: TIniFile read FIni write SetIni;
   property  SLE: TStringListElement read GetSLE;
published
  property FileName: string read FFileName write SetFileName;
  property Section: string read FSection write FSection;
  property Ident: string read FIdent write FIdent;
  property DefaultValue: string read FDefaultValue write FDefaultValue;
  property Value: string read GetValue write SetValue;
  property SectionElement: TEntity read GetSectionElement write SetSectionElement;
end;





implementation

uses
  SysUtils,
  E_Class,
  {!F_Start,}
  U_String;



procedure T_IniFile.SetFileName(aValue: string);
var
  aPath: string;
begin
  FFileName:= GetAbsolutePath(aValue);
  if Ini <> nil then
  begin
     Ini.Free;
     Ini := nil;
  end;
  Ini := TIniFile.Create(FFileName);
end;



function T_IniFile.GetValue: string;
begin
  result:= DefaultValue;
  if Ini = nil then exit;
  result:= Ini.ReadString(Section, Ident, DefaultValue);
end;

procedure T_IniFile.SetValue(aValue: string);
begin
  if Ini = nil then exit;
  Ini.WriteString(Section, Ident, aValue);
end;





destructor T_IniFile.Destroy;
begin
   if Ini <> nil then Ini.Free;
   Ini := nil;
   inherited;
end;

procedure T_IniFile.SetIni(const Value: TIniFile);
begin
   FIni := Value;
end;


function T_IniFile.GetSLE: TStringListElement;
begin
   if FSLE = nil then
   begin
      FSLE := TStringListElement.Create;
      FSLE.Name := 'SLE';
      AddElement(FSLE);
   end;
   result := FSLE;
end;


function T_IniFile.GetSectionElement: TEntity;
var
   SL: TStringList;
   i: integer;
begin
   SLE.FreeAll;
   result:= SLE;
   if Ini = nil then exit;
   SL:= TStringList.Create;
   Ini.ReadSection(Section, SL);
   if SL.Count > 0 then
   begin
      SLE.AddStringList(SL);
      { naplnn obsahu poloky }
      for I:= 0 to SLE.LastIndex do
      begin
        SLE.Strings[I]:= Ini.ReadString(Section, SLE.Names[I], '');
      end;
   end;
   SL.Free;
end;

procedure T_IniFile.SetSectionElement(const Value: TEntity);
var
   aSLE: TStringListElement;
   SE: TStringElement;
   i: integer;
begin
   SLE.FreeAll;
   if Value = nil then exit;
   if Value is TStringListElement then
   begin
      aSLE := TStringListElement(Value);
      for i := 0 to aSLE.LastIndex do
      begin
         SE := aSLE.StringElements[i];
         SLE.AddNameString(SE.Name,SE.Value);
      end;
   end
   else
   begin
      for i := 0 to Value.LastIndex do
      begin
         SLE.NextName := Value.Elements[i].Name;
      end;
   end;
   for i := 0 to SLE.LastIndex do
   begin
      SE := SLE.StringElements[i];
      Ini.WriteString(Section,SE.Name,SE.Value);
   end;
end;

INITIALIZATION
   RegisterOrbisClass(T_IniFile);
end.
