unit i_Gis_Georaster;

interface
uses
   Winapi.Windows,
   Classes,
   Vcl.Graphics,
   Elements,
   e_Page,
   e_Proje,
   e_Gis_GeoRaster,
   i_Basic;

type

TGis_GeoRasterInterpret = class(TRectInterpret)
  private
    FColor: TColor;
    FUseColor: boolean;
    FProj: TPageProjection;
    FPage: T_Page;
    procedure SetColor(const Value: TColor);
    procedure SetUseColor(const Value: boolean);
protected
   function  IsItsClass(AObject: TObject): boolean; override;
   function  QGoInside(Element: TElement): boolean; override;
   procedure Process(Element: TElement); override;
   property  Proj: TPageProjection read FProj;
   property  Page: T_Page read FPage;
public
   constructor Create; override;
   destructor  Destroy; override;
published
   property Color: TColor read FColor write SetColor;
   property UseColor: boolean read FUseColor write SetUseColor;
end;

implementation
uses
   Forms,
   SysUtils,
   Types,
   vcl.imaging.pngimage,
   G_Elems,
   e_Class;

{ TGis_GeoRasterInterpret }

constructor TGis_GeoRasterInterpret.Create;
begin
   inherited;
   FProj := TPageProjection.Create;
   AddElement(FProj);
   FPage := T_Page.Create;
   AddElement(FPage);
end;

destructor TGis_GeoRasterInterpret.Destroy;
begin
   inherited;
end;

function TGis_GeoRasterInterpret.IsItsClass(AObject: TObject): boolean;
begin
   result := aObject is TGis_GeoRaster;
end;

procedure TGis_GeoRasterInterpret.Process(Element: TElement);
var
   i,Xi,Yi,xi1,xi2,yi1,yi2: integer;
   X1,Y1,X2,Y2: double;
   Revi: integer;
   TileIndex: integer;
   S: string;
   bm1,bms: TBitmap;//TPngImage;//TGraphic;
   bmWidth,bmHeight: integer;
   aCanvas: TCanvas;
   DestRect,SrcRect,RR: TRect;
   dX: Single;
   Picture: TPicture;
   aGraphic: TGraphic;
   SrcModel: TDoubleRect;
   IsPNG: boolean;
   SrcPng: TPngImage;
   bmx,bmy: integer;
   aColor: TColor;
   GR: TGis_GeoRaster;
   TRNS: TChunkTRNS;

//   R: TDoubleRect;
  function DetectPixelFormat: TPixelFormat;
  begin
    with SrcPng.Header do
    begin
      {Always use 24bits for partial transparency}
      if SrcPng.TransparencyMode = ptmPartial then
        DetectPixelFormat := pf24bit
      else
        case BitDepth of
          {Only supported by COLOR_PALETTE}
          1: DetectPixelFormat := pf1bit;
          2, 4: DetectPixelFormat := pf4bit;
          {8 may be palette or r, g, b values}
          8, 16:
            case ColorType of
              COLOR_RGB, COLOR_GRAYSCALE: DetectPixelFormat := pf24bit;
              COLOR_PALETTE: DetectPixelFormat := pf8bit;
              else raise Exception.Create('');
            end {case ColorFormat of}
          else raise Exception.Create('');
        end {case BitDepth of}
    end {with Header}
  end;

begin
   if Device = nil then exit;
   if not(Element is TGis_GeoRaster) then exit;
   GR := TGis_GeoRaster(Element);
   if GR.Error > 0 then exit;
    X1 := GR.MinX;
    X2 := GR.MaxX;
    Y1 := GR.MinY;
    Y2 := GR.MaxY;
    IsPNG := true;
    if (X1 < MaxX) and (Y1 < MaxY) and
       (X2 > MinX) and (Y2 > MinY)then
    begin
       S := GR.FileName;
       if FileExists(S) then
       begin
         try
          Picture := TPicture.Create;
          Picture.LoadFromFile(S);
          aGraphic := Picture.Graphic;
          Application.ProcessMessages;
          bms := nil;
          SrcPng := TPngImage(aGraphic);
//          if IsPNG then
//          begin
//             SrcPng := TPngImage(aGraphic);
//             bms := TBitmap.Create;
//             bms.Width := SrcPng.Width;
//             bms.Height := SrcPng.Height;
//             bms.Canvas.Brush.Color:= aColor;
//             bms.Canvas.Brush.Style:= bsSolid;
//             RR := System.Classes.Rect(0,0,bms.Width,bms.Height);
//             bms.Canvas.FillRect(RR);
//             bms.Canvas.CopyMode := cmSrcPaint;
//             bms.Canvas.CopyRect(RR,SrcPng.Canvas,RR);
//             bms.Canvas.CopyMode := cmSrcCopy;
////                   bms.Transparent := true;
////                   bms.TransparentColor := SrcPng.TransparentColor;
//             aGraphic := bms;
////                   for bmx := 0 to SrcPng.Width - 1 do
////                   begin
////                       for bmy := 0 to SrcPng.Height - 1 do
////                       begin
////                          if SrcPng.Pixels[bmx,bmy] <> SrcPng.TransparentColor then
////                             SrcPng.Pixels[bmx,bmy] := $000000FF;
////                       end;
////                   end;
//          end;

          // CLipping tad nejak ... jaksi ... jakoze ... neni! ... ale bude asi
          // kontrola orezu
          SrcRect  := System.Classes.Rect(0,0,Picture.Graphic.Width,Picture.Graphic.Height);
          DestRect := System.Classes.Rect(Device.Projection.GetPageX(X1),Device.Projection.GetPageY(Y2),Device.Projection.GetPageX(X2),Device.Projection.GetPageY(Y1));
          bm1 := nil;
          if (X1 < MinX) or (X2 > MaxX) or (Y1 < MinY) or (Y2 > MaxY)   then
          begin
//                   Page.SetRect(0,0,Picture.Graphic.Width,Picture.Graphic.Height);
             Page.SetRect(0,0,Picture.Graphic.Width,Picture.Graphic.Height);
             Page.SetResolution(Page.Width / GR.PixelWidth,Page.Height / GR.PixelHeight);
             SrcModel.MinX := X1;
             SrcModel.MinY := Y1;
             SrcModel.MaxX := X2;
             SrcModel.MaxY := Y2;
             Proj.ChangeProjectionByRect(SrcModel, Page);
             if (X1  < MinX) then
             begin
                SrcRect.Left := Proj.GetPageX(MinX);
                DestRect.Left := Device.Projection.GetPageX(MinX);
             end;
             if (X2 > MaxX) then
             begin
                SrcRect.Right := Proj.GetPageX(MaxX);
                DestRect.Right := Device.Projection.GetPageX(MaxX);
             end;
             if (Y1 < MinY) then
             begin
                SrcRect.Bottom := Proj.GetPageY(MinY);
                DestRect.Bottom := Device.Projection.GetPageY(MinY);
             end;
             if (Y2 > MaxY) then
             begin
                SrcRect.Top := Proj.GetPageY(MaxY);
                DestRect.Top := Device.Projection.GetPageY(MaxY);
             end;
//             bm1 := TPngImage.CreateBlank(SrcPng.Header.ColorType,SrcPng.Header.BitDepth,SrcRect.Right - SrcRect.Left,SrcRect.Bottom - SrcRect.Top);
             bm1 := TBitmap.Create;
             bmWidth  := SrcRect.Right - SrcRect.Left;
             bmHeight := SrcRect.Bottom - SrcRect.Top;
             // kopie PngImage.AssignTo -------------------------
             bm1.SetSize(bmWidth, bmHeight);

             if (SrcPng.TransparencyMode = ptmPartial) then
             begin
               bm1.PixelFormat := pf32bit;
               bm1.AlphaFormat := afDefined;
               bm1.Canvas.Brush.Color := 0;
               bm1.Canvas.FillRect(Bounds(0,0,bmWidth, bmHeight));
             end
             else
             begin
               bm1.PixelFormat := DetectPixelFormat;
               bm1.AlphaFormat := afIgnored;
             end;

             if SrcPng.Palette <> 0 then
               bm1.Palette := CopyPalette(SrcPng.Palette);

             if (SrcPng.TransparencyMode = ptmBit) then
             begin
                bm1.Transparent := true;
                bm1.TransparentColor := SrcPng.TransparentColor;
             end; {if (TransparencyMode = ptmBit)}

//             if IsPNG then
//             begin
//                bm1.Transparent := true;
//                bm1.TransparentColor := SrcPng.TransparentColor;
//             end;
//             Bm1.Width := SrcRect.Right - SrcRect.Left;
//             Bm1.Height := SrcRect.Bottom - SrcRect.Top;
             aCanvas := bm1.Canvas;
             aCanvas.Draw(0 - SrcRect.Left,0 - SrcRect.Top,aGraphic);
             aGraphic := Bm1;
          end
          else
          if IsPNG then
          begin
//             bms.Transparent := true;
//             bms.TransparentColor := SrcPng.TransparentColor;
          end;


          Device.StretchDrawDevice(DestRect.Left,DestRect.Top,DestRect.Width,DestRect.Height,0,0,aGraphic.Width,aGraphic.Height,aGraphic);
//                Device.StretchDrawDevice(DestRect.Left,DestRect.Top,DestRect.Width,DestRect.Height,SrcRect.Left,SrcRect.Top,SrcRect.Width,SrcRect.Height,Picture.Graphic);
         finally
            //G.Dispose;
            Picture.Free;
            if Assigned(bm1)then
               FreeAndNil(bm1);
            if IsPNG and Assigned(bms) then
               FreeAndNil(bms);
         end;
          Application.ProcessMessages;
       end;
    end;
end;

function TGis_GeoRasterInterpret.QGoInside(Element: TElement): boolean;
begin
   result := false;
end;

procedure TGis_GeoRasterInterpret.SetColor(const Value: TColor);
begin
   FColor := Value;
end;

procedure TGis_GeoRasterInterpret.SetUseColor(const Value: boolean);
begin
   FUseColor := Value;
end;

initialization
   RegisterOrbisClass(TGis_GeoRasterInterpret);
end.
