unit t_GeometryEx;

interface
uses
   Tools;

type

Tgis_NoteTransfer = class(TTool)
  private
    FToDirectory: string;
    FFromDirectory: string;
    procedure SetFromDirectory(const Value: string);
    procedure SetToDirectory(const Value: string);
public
   procedure DoIt; override;
   function  NumberToPropertyName(aNumber: longInt): string; override;
published
   property FromDirectory: string read FFromDirectory write SetFromDirectory;
   property ToDirectory: string read FToDirectory write SetToDirectory;
end;

Tgis_NoteKillData = class(TTool)
public
   procedure DoIt; override;
end;

implementation
uses
   Forms,
   SysUtils,
   t_Main,
   Worlds,
   t_OilPro1,
   e_GeometryEx,
   e_Class;

{ Tgis_NoteTransfer }

procedure Tgis_NoteTransfer.DoIt;
var
   FromAuto,ToAuto: boolean;
   N: TGis_Note;
   AutoDirectory: string;
   i: integer;
   FileName: string;
   cpf: TCopyFile;
begin
   Error := 1;
   if Input = nil then exit;
   Error := 2;
   if not (Input is TGis_Note) then exit;
   N := TGis_Note(Input);
   FromAuto := (FromDirectory = '') or (UpperCase(FromDirectory)='AUTO');
   ToAuto := (ToDirectory = '') or (UpperCase(ToDirectory)='AUTO');
   Error := 3;
   if FromAuto and ToAuto then exit;
   AutoDirectory := TMainTool(World.MainTool).GetProjectDataDirectoryEx;
   if FromAuto  then
      FromDirectory := AutoDirectory
   else if ToAuto then
           TODirectory := AutoDirectory;
   FromDirectory := IncludeTrailingPathDelimiter(FromDirectory);
   ToDirectory   := IncludeTrailingPathDelimiter(ToDirectory);
   if not N.NoImages then
   begin
      ForceDirectories(ToDirectory+'img');
      for i := 0 to N.Images.LastIndex do
      begin
         cpf := TCopyFile.Create;
         try
         FileName := N.Images.Names[i];
         cpf.FileName  := FromDirectory + 'img' + PathDelim + FileName;
         cpf.FileName1 := ToDirectory   + 'img' + PathDelim + FileName;
         cpf.DoIt;
         finally
         cpf.Free;
         end;
      end;
   end;
   if not N.NoAudios then
   begin
      ForceDirectories(ToDirectory+'audio');
      for i := 0 to N.Audios.LastIndex do
      begin
         cpf := TCopyFile.Create;
         cpf.MuteFileEvents := MuteFileEvents;
         try
         FileName := N.Audios.Names[i];
         cpf.FileName  := FromDirectory + 'audio' + PathDelim + FileName;
         cpf.FileName1 := ToDirectory   + 'audio' + PathDelim + FileName;
         cpf.DoIt;
         finally
         cpf.Free;
         end;
      end;
   end;
   Error := 0;
end;

function Tgis_NoteTransfer.NumberToPropertyName(aNumber: Integer): string;
begin
   if aNumber = 0 then
      result := 'FromDirectory'
   else
      result := 'ToDirectory';
end;

procedure Tgis_NoteTransfer.SetFromDirectory(const Value: string);
begin
  FFromDirectory := Value;
end;

procedure Tgis_NoteTransfer.SetToDirectory(const Value: string);
begin
  FToDirectory := Value;
end;

{ Tgis_NoteKillData }

procedure Tgis_NoteKillData.DoIt;
var
   N: TGis_Note;
   AutoDirectory: string;
   i: integer;
   FileName: string;
begin
   Error := 1;
   if Input = nil then exit;
   Error := 2;
   if not (Input is TGis_Note) then exit;
   N := TGis_Note(Input);
   AutoDirectory := IncludeTrailingPathDelimiter(TMainTool(World.MainTool).GetProjectDataDirectoryEx);
   if not N.NoImages then
   begin
      for i := 0 to N.Images.LastIndex do
      begin
         try
         FileName := N.Images.Names[i];
         FileName := AutoDirectory + 'img' + PathDelim + FileName;
         if FileExists(FileName) then
         begin
            DeleteFile(FileName);
         end;
         {$IFDEF FILEEVENTS}
          PostTextMsg(tm_FileDelete,FileName,nil);
          Application.ProcessMessages;
         {$ENDIF}
         finally
         end;
      end;
   end;
   if not N.NoAudios then
   begin
      for i := 0 to N.Audios.LastIndex do
      begin
         try
         FileName := N.Audios.Names[i];
         FileName := AutoDirectory + 'audio' + PathDelim + FileName;
         if FileExists(FileName) then
         begin
            DeleteFile(FileName);
         end;
         {$IFDEF FILEEVENTS}
          PostTextMsg(tm_FileDelete,FileName,nil);
          Application.ProcessMessages;
         {$ENDIF}
         finally
         end;
      end;
   end;
end;

INITIALIZATION
   RegisterOrbisClass(Tgis_NoteTransfer);
   RegisterOrbisClass(Tgis_NoteKillData);
END.
