unit t_LHCPositionMonitor;

interface
uses
   e_LHCModel,
   e_KatastrModel,
   m_Intera,
   t_InterfaceTool,
   t_ForestModelSelector,
   t_ParcelModelSelector;

type

TOn_MonitorCoordsChange = procedure(X,Y: double) of object;
TOn_MonitorLHCChange = procedure(LHC,Oddeleni: integer; Dilec,Porost,Skupina: string) of object;
TOn_MonitorParcelyChange = procedure(Katastr,Parcis,Parpod: integer) of object;

TLHCPositionMonitor = class(TInterfaceTool)
  private
    FFMSelector: TForestModelSelector;
    FPMSelector: TParcelModelSelector;
    FOnCoordsChange: TOn_MonitorCoordsChange;
    FOnLHCChange: TOn_MonitorLHCChange;
    FOnParcelyChange: TOn_MonitorParcelyChange;
    FMonitorLHC: boolean;
    FMonitorCoords: boolean;
    FMonitorParcely: boolean;
    FActive: boolean;
    FLastY: double;
    FLastX: double;
    FLastParcast: Tgis_ParcelaCast;
    FLastSkupina: Tgis_PorostniSkupina;
    procedure SetOnCoordsChange(const Value: TOn_MonitorCoordsChange);
    procedure SetOnLHCChange(const Value: TOn_MonitorLHCChange);
    procedure SetOnParcelyChange(const Value: TOn_MonitorParcelyChange);
    procedure SetMonitorCoords(const Value: boolean);
    procedure SetMonitorLHC(const Value: boolean);
    procedure SetMonitorParcely(const Value: boolean);
    procedure SetActive(const Value: boolean);
    function GetFMSelector: TForestModelSelector;
    function GetPMSelector: TParcelModelSelector;
    procedure SetLastParcast(const Value: Tgis_ParcelaCast);
    procedure SetLastSkupina(const Value: Tgis_PorostniSkupina);
    procedure SetLastX(const Value: double);
    procedure SetLastY(const Value: double);
public
   procedure   Reactivate;
   procedure   On_tm_MouseMove(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
   property    FMSelector: TForestModelSelector read GetFMSelector;
   property    PMSelector: TParcelModelSelector read GetPMSelector;
   property    LastX: double read FLastX write SetLastX;
   property    LastY: double read FLastY write SetLastY;
   property    LastParcast: Tgis_ParcelaCast read FLastParcast write SetLastParcast;
   property    LastSkupina: Tgis_PorostniSkupina read FLastSkupina write SetLastSkupina;
published
   property Active: boolean read FActive write SetActive;
   property OnCoordsChange: TOn_MonitorCoordsChange read FOnCoordsChange write SetOnCoordsChange;
   property OnLHCChange: TOn_MonitorLHCChange read FOnLHCChange write SetOnLHCChange;
   property OnParcelyChange: TOn_MonitorParcelyChange read FOnParcelyChange write SetOnParcelyChange;
   property MonitorCoords: boolean read FMonitorCoords write SetMonitorCoords;
   property MonitorLHC: boolean read FMonitorLHC write SetMonitorLHC;
   property MonitorParcely: boolean read FMonitorParcely write SetMonitorParcely;
end;

implementation
uses
   e_Class;

{ TLHCPositionMonitor }

function TLHCPositionMonitor.GetFMSelector: TForestModelSelector;
begin
   if FFMSelector = nil then
   begin
      FFMSelector := TForestModelSelector.Create;
      //if not FindByZAK then
         FFMSelector.FrameClassName := 'Tgis_PorostniSkupina';
      //else
      //   FFMSelector.FrameClassName := 'Tgis_Porost';
      FFMSelector.FindInput;
      AddElement(FFMSelector);
   end;
   result := FFMSelector;
end;

function TLHCPositionMonitor.GetPMSelector: TParcelModelSelector;
begin
   if FPMSelector = nil then
   begin
      FPMSelector := TParcelModelSelector.Create;
      {
      if not FindByZAK then
         FFMSelector.FrameClassName := 'Tgis_PorostniSkupina'
      else
         FFMSelector.FrameClassName := 'Tgis_Porost';
      }
      FPMSelector.FindInput;
      AddElement(FPMSelector);
   end;
   result := FPMSelector;
end;


procedure TLHCPositionMonitor.On_tm_MouseMove(var aMessage: TOrbisMouseEvent; var Handled: boolean);
var
   aX,aY: double;
   PSK: Tgis_PorostniSkupina;
begin
   if not Active then exit;
   if ZeroActivity then exit;
   aX:= Device.Projection.GetModelX(aMessage.x);
   aY:= Device.Projection.GetModelY(aMessage.y);
   if MonitorCoords then
   begin
      if (LastX <> aX) and (LastY <> aY) and Assigned(OnCoordsChange) then
      begin
         OnCoordsChange(aX,aY);
         LastX := aX;
         LastY := aY;
      end;
   end;
   if MonitorParcely and not PMSelector.Busy then
   begin
      PMSelector.X := aX;
      PMSelector.Y := aY;
      PMSelector.DoIt;
      if PMSelector.ActualParCast <> LastParcast then
      begin
         if PMSelector.ActualParCast <> nil then
         begin
            if Assigned(OnParcelyChange) then
               OnParcelyChange(PMSelector.ActualParCast.KU_KOD,PMSelector.ActualParCast.PARCIS,PMSelector.ActualParCast.PARPOD);
         end
         else
         begin
            if Assigned(OnParcelyChange) then
               OnParcelyChange(0,0,0);
         end;
         LastParcast := PMSelector.ActualParCast;
      end;
   end;
   if MonitorLHC then
   begin
      if not FMSelector.Busy then
      begin
         FMSelector.X := aX;
         FMSelector.Y := aY;
         FMSelector.DoIt;
         if FMSelector.ActualFrame <> nil then
         begin
            PSK := Tgis_PorostniSkupina(FMSelector.ActualFrame);
            if Assigned(OnLHCChange) then
               OnLHCChange(PSK.LHC,PSK.Oddeleni,PSK.Dilec,PSK.Porost,PSK.Skupina);
         end
         else
         begin
            if Assigned(OnLHCChange) then
               OnLHCChange(0,0,'','','');
         end;
         LastSkupina := TGis_PorostniSkupina(FMSelector.ActualFrame);
      end;
   end;
end;

procedure TLHCPositionMonitor.Reactivate;
begin
   FMSelector.FindInput;
   FMSelector.Busy := false;
   PMSelector.FindInput;
   PMSelector.Busy := false;
   LastX := 0;
   LastY := 0;
   LastParcast := nil;
   LastSkupina := nil;
end;

procedure TLHCPositionMonitor.SetActive(const Value: boolean);
begin
   FActive := Value;
   if Value then Reactivate;
end;

procedure TLHCPositionMonitor.SetLastParcast(
  const Value: Tgis_ParcelaCast);
begin
  FLastParcast := Value;
end;

procedure TLHCPositionMonitor.SetLastSkupina(
  const Value: Tgis_PorostniSkupina);
begin
  FLastSkupina := Value;
end;

procedure TLHCPositionMonitor.SetLastX(const Value: double);
begin
  FLastX := Value;
end;

procedure TLHCPositionMonitor.SetLastY(const Value: double);
begin
  FLastY := Value;
end;

procedure TLHCPositionMonitor.SetMonitorCoords(const Value: boolean);
begin
  FMonitorCoords := Value;
end;

procedure TLHCPositionMonitor.SetMonitorLHC(const Value: boolean);
begin
  FMonitorLHC := Value;
end;

procedure TLHCPositionMonitor.SetMonitorParcely(const Value: boolean);
begin
  FMonitorParcely := Value;
end;

procedure TLHCPositionMonitor.SetOnCoordsChange(
  const Value: TOn_MonitorCoordsChange);
begin
  FOnCoordsChange := Value;
end;

procedure TLHCPositionMonitor.SetOnLHCChange(
  const Value: TOn_MonitorLHCChange);
begin
  FOnLHCChange := Value;
end;

procedure TLHCPositionMonitor.SetOnParcelyChange(
  const Value: TOn_MonitorParcelyChange);
begin
  FOnParcelyChange := Value;
end;

INITIALIZATION
   RegisterOrbisClass(TLHCPositionMonitor);
END.

