unit f_GraphicForm;

interface

uses
  //HintShadow,
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  f_ExtendedOrbisForm, ActnList, Menus, ComCtrls, MenuBar, ToolWin,
  ExtCtrls,
  // KERNEL
  Tools, StdCtrls, Buttons, System.Actions;

type
TGraphic_Form = class(TExtendedORBIS_Form)
    MenuBar1: TMenuBar;
    Panel1: TPanel;
    Image: TImage;
   procedure ImageDragDrop(Sender, Source: TObject; X, Y: Integer);
   procedure ImageDragOver(Sender, Source: TObject; X, Y: Integer; State: TDragState; var Accept: Boolean);
   procedure ImageEndDrag(Sender, Target: TObject; X, Y: Integer);
   procedure ImageMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
   procedure ImageMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
   procedure ImageMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
   procedure ImageStartDrag(Sender: TObject; var DragObject: TDragObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormCreate(Sender: TObject);
    procedure Panel1Resize(Sender: TObject);
    procedure Panel1ConstrainedResize(Sender: TObject; var MinWidth,
      MinHeight, MaxWidth, MaxHeight: Integer);
    procedure FormConstrainedResize(Sender: TObject; var MinWidth,
      MinHeight, MaxWidth, MaxHeight: Integer);
    procedure FormResize(Sender: TObject);
    procedure FormCanResize(Sender: TObject; var NewWidth,
      NewHeight: Integer; var Resize: Boolean);
    procedure FormMouseWheelDown(Sender: TObject; Shift: TShiftState;
      MousePos: TPoint; var Handled: Boolean);
    procedure FormMouseWheelUp(Sender: TObject; Shift: TShiftState;
      MousePos: TPoint; var Handled: Boolean);
  private
    FBusy: boolean;
    procedure SetBusy(const Value: boolean);
private
   IsActive: boolean;
   LockMouseUp: boolean;
   RealyDraged: boolean;
   DragShift: TShiftState;
    FImageWidth: integer;
    FImageHeight: integer;
    procedure SetImageHeight(const Value: integer);
    procedure SetImageWidth(const Value: integer);
    property Busy: boolean read FBusy write SetBusy;
public
   procedure EndSizeMove; override;
   procedure SetTool(const Value: TTool); override;
   property ImageHeight: integer read FImageHeight write SetImageHeight;
   Property ImageWidth: integer read FImageWidth write SetImageWidth;
end;


implementation
uses
   // KERNEL
   m_ORBIS,
   e_Type,
   Elements,
   Worlds,
   // DEVICE
   t_Device, f_OrbisForm;

{$R *.DFM}

procedure TGraphic_Form.ImageDragDrop(Sender, Source: TObject; X,
  Y: Integer);
begin
   inherited;
   if Tool = nil then exit;
   TDeviceTool(Tool).PostMouseEvent(tm_DragDrop, Tool, mbLeft, DragShift, X, Y, IsActive);
   //Tool.PostTextMsg(tm_AddToProtocol,'DragDrop',nil);
end;

procedure TGraphic_Form.SetTool(const Value: TTool);
begin
   inherited;
   if not (Tool is TDeviceTool) then Tool := nil;
end;

procedure TGraphic_Form.ImageDragOver(Sender, Source: TObject; X,
  Y: Integer; State: TDragState; var Accept: Boolean);
begin
   inherited;
   Accept := true;
   RealyDraged := true;
   if Tool = nil then exit;
   TDeviceTool(Tool).PostMouseEvent(tm_DragOver, Tool, mbLeft, DragShift, X, Y, IsActive);
   //Tool.PostTextMsg(tm_AddToProtocol,'...DragOver '+ IntToStr(X)+' '+IntToStr(Y),nil);
   //Application.ProcessMessages;
end;

procedure TGraphic_Form.ImageEndDrag(Sender, Target: TObject; X,
  Y: Integer);
var
   P: TPoint;
begin
   inherited;
   Image.DragCursor := crDrag;
   if Tool = nil then exit;
   TDeviceTool(Tool).PostMouseEvent(tm_DragEnd, Tool, mbLeft, DragShift, X, Y, IsActive);
   if not RealyDraged then
   begin
      P.X := X;
      P.Y := Y;
      P := Image.ScreenToClient(P);
      TDeviceTool(Tool).PostMouseEvent(tm_MouseUp, Tool, mbLeft, DragShift, P.X, P.Y, IsActive);
   end;
   //Tool.PostTextMsg(tm_AddToProtocol,'EndDrag',nil);
   RealyDraged := false;
   LockMouseUp := false;
end;

procedure TGraphic_Form.ImageMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
   inherited;
   if Tool = nil then exit;
   DragShift := Shift;
   TDeviceTool(Tool).PostMouseEvent(tm_MouseDown, Tool, Button, Shift, X, Y, IsActive);
   //Tool.PostTextMsg(tm_AddToProtocol,'***********************',nil);
   //Tool.PostTextMsg(tm_AddToProtocol,'MouseDown',nil);
end;

procedure TGraphic_Form.ImageMouseMove(Sender: TObject; Shift: TShiftState;
  X, Y: Integer);
begin
   inherited;
   if Tool = nil then exit;
   TDeviceTool(Tool).PostMouseEvent(tm_MouseMove, Tool, mbLeft, Shift, X, Y, IsActive);
   //Tool.PostTextMsg(tm_AddToProtocol,'MouseMove '+ IntToStr(X)+' '+IntToStr(Y),nil);
   if ssLeft in Shift then
   begin
      LockMouseUp := true;
      DragShift := Shift;
      Image.BeginDrag(false,1);
   end;

end;

procedure TGraphic_Form.ImageMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
   inherited;
   if Tool = nil then exit;
   if LockMouseUp then exit;
   TDeviceTool(Tool).PostMouseEvent(tm_MouseUp, Tool, Button, Shift, X, Y, IsActive);
   //Tool.PostTextMsg(tm_AddToProtocol,'MouseUp',nil);
   IsActive := true;
end;

procedure TGraphic_Form.ImageStartDrag(Sender: TObject;
  var DragObject: TDragObject);
begin
   inherited;
   if Tool = nil then exit;
   TDeviceTool(Tool).PostMouseEvent(tm_StartDrag, Tool, mbLeft, DragShift, 0, 0, IsActive);
   //Tool.PostTextMsg(tm_AddToProtocol,'StartDrag',nil);
end;

procedure TGraphic_Form.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
//   inherited;
   if Tool = nil then exit;
   TDeviceTool(Tool).PostKeyEvent(tm_KeyDown, Tool, Key, Shift);
   Key := 0;
   //Tool.PostTextMsg(tm_AddToProtocol,'Key: '+IntToStr(Key)+ ' '+CHar(Key),nil);
end;

procedure TGraphic_Form.FormCreate(Sender: TObject);
begin
  inherited;
  Panel1.ControlStyle := Panel1.ControlStyle + [csOpaque];
  //Panel1.Brush.Style := bsClear;
  //Maps.ItemIndex := Maps.Items.IndexOf(TStringElement(TEntity(World.SystemRegister.FindElementName('Projection')).FindElementName('MAP')).Value);
end;

procedure TGraphic_Form.Panel1Resize(Sender: TObject);
begin
   //inherited;
   try
      if not Visible then exit;
      if (ImageHeight = Image.Height) and (ImageWidth = Image.Width) then exit;
      //Tool.PostTextMsg(tm_AddToProtocol, 'resize', nil);
      //Tool.SetPropertyBooleanByName('BreakReView',true);
      Image.Picture.Bitmap.Height:= Image.Height;
      Image.Picture.Bitmap.Width:= Image.Width;
      //Image.ControlStyle := Image.ControlStyle - [csOpaque];
      //if not Image.Visible then ShowMessage('not Visible');
      //if not (csOpaque in Image.ControlStyle) then ShowMessage('not Opaque');
      if SizeMove  then exit;
      if Tool = nil then exit;
      if Tool.Idle then exit;
      //Tool.PostTextMsg(tm_AddToProtocol, 'Resize', nil);
      Tool.PostMsg(tm_ReSize, Tool, nil, false);
      Application.ProcessMessages;
      ImageHeight := Image.Height;
      ImageWidth := Image.Width;
      {
      Tool.Device.SetupPage(nil);
      Tool.PostMsg(tm_ReView, Tool, nil, DontKill);
      }
   except
      //Tool.PostTextMsg(tm_AddToProtocol, 'except', nil);
   end;
end;

procedure TGraphic_Form.Panel1ConstrainedResize(Sender: TObject;
  var MinWidth, MinHeight, MaxWidth, MaxHeight: Integer);
begin
   //inherited;
   exit;
   try
      if not Visible then exit;
      {
      if (ImageHeight = Image.Picture.Bitmap.Height) and
         (ImageWidth  = Image.Picture.Bitmap.Width) then
         exit;
      }
      Image.Picture.Bitmap.Height:= Image.Height;
      Image.Picture.Bitmap.Width:= Image.Width;
      ImageHeight := Image.Height;
      ImageWidth := Image.Width;
      if Tool = nil then exit;
      if Tool.Idle then exit;
      Tool.PostMsg(tm_ReSize, Tool, nil, false);
      {
      if Busy then exit;
      Busy := true;
      Application.ProcessMessages;
      Busy := false;
      }
   except
   end;
end;

procedure TGraphic_Form.FormConstrainedResize(Sender: TObject;
  var MinWidth, MinHeight, MaxWidth, MaxHeight: Integer);
begin
  inherited;
   exit;
   try
      if not Visible then exit;
      {
      if (Image.Picture.Bitmap.Height = Image.Height) and
         (Image.Picture.Bitmap.Width = Image.Width) then
         exit;
      }
      Image.Picture.Bitmap.Height:= Image.Height;
      Image.Picture.Bitmap.Width:= Image.Width;
      ImageHeight := Image.Height;
      ImageWidth := Image.Width;
      if Tool = nil then exit;
      if Tool.Idle then exit;
      Tool.PostMsg(tm_ReSize, Tool, nil, false);
   except
   end;
end;

procedure TGraphic_Form.SetImageHeight(const Value: integer);
begin
  FImageHeight := Value;
end;

procedure TGraphic_Form.SetImageWidth(const Value: integer);
begin
  FImageWidth := Value;
end;

procedure TGraphic_Form.SetBusy(const Value: boolean);
begin
  FBusy := Value;
end;

procedure TGraphic_Form.FormResize(Sender: TObject);
begin
   try
   if not Visible then exit;
   if Tool <> nil then
   begin
      Tool.PostTextMsg(tm_AddToProtocol,'Form resize',nil);
      Application.ProcessMessages;
   end;
   except
   if Tool <> nil then
      Tool.PostTextMsg(tm_AddToProtocol,'Form except',nil);
   end
end;

procedure TGraphic_Form.FormCanResize(Sender: TObject; var NewWidth,
  NewHeight: Integer; var Resize: Boolean);
begin
      Tool.PostTextMsg(tm_AddToProtocol,'Form can resize',nil);
      resize := true;
end;



procedure TGraphic_Form.EndSizeMove;
begin
   inherited;
   Panel1Resize(nil);
end;

procedure TGraphic_Form.FormMouseWheelDown(Sender: TObject; Shift: TShiftState; MousePos: TPoint; var Handled: Boolean);
var
   P: TPoint;
begin
    Handled := true;
    P := Image.ScreenToClient(MousePos);
    TDeviceTool(Tool).PostMouseEvent(tm_MouseWheelDown, Tool, mbMiddle, Shift, P.X, P.Y, IsActive);
end;

procedure TGraphic_Form.FormMouseWheelUp(Sender: TObject; Shift: TShiftState; MousePos: TPoint; var Handled: Boolean);
var
   P: TPoint;
begin
    Handled := true;
    P := Image.ScreenToClient(MousePos);
    TDeviceTool(Tool).PostMouseEvent(tm_MouseWheelUp, Tool, mbMiddle, Shift, P.X, P.Y, IsActive);
end;

end.
