unit t_ControlTool;

interface
uses
   Classes,
   Controls,
   Graphics,
   Tools;
type

TControlTool = class(TTechnology)
private
    FControl: TControl;
    FDestroyControl: boolean;
    function Get_Owner: TComponent;
    function GetParent: TControl;
    procedure SetControl(const Value: TControl);
    procedure Set_Owner(const Value: TComponent);
    function GetHeight: integer;
    function GetVisible: boolean;
    function GetWidth: integer;
    procedure SetHeight(const Value: integer);
    procedure SetLeft(const Value: integer);
    procedure SetTop(const Value: integer);
    procedure SetVisible(const Value: boolean);
    procedure SetWidth(const Value: integer);
    function GetAlign: TAlign;
    function GetEnabled: boolean;
    procedure SetEnabled(const Value: boolean);
    function GetFont_Color: TColor;
    function GetFont_Height: integer;
    function GetFont_Charset: TFontCharset;
    function GetFont_Name: string;
    function GetFont_Style: TFontStyles;
    procedure SetFont_Color(const Value: TColor);
    procedure SetFont_Height(const Value: integer);
    procedure SetFont_Charset(const Value: TFontCharset);
    procedure SetFont_Name(const Value: string);
    procedure SetFont_Style(const Value: TFontStyles);
    function GetFont_Size: integer;
    procedure SetFont_Size(const Value: integer);
    function GetAnchorBottom: boolean;
    function GetAnchorLeft: boolean;
    function GetAnchorRight: boolean;
    function GetAnchorTop: boolean;
    procedure SetAnchorBottom(const Value: boolean);
    procedure SetAnchorLeft(const Value: boolean);
    procedure SetAnchorRight(const Value: boolean);
    procedure SetAnchorTop(const Value: boolean);
    procedure SetDestroyControl(const Value: boolean);
protected
    function GetFont: TFont; virtual;
    function GetAutoSize: boolean; virtual;
    procedure SetAutoSize(const Value: boolean); virtual;
    function GetCaption: string; virtual;
    procedure SetCaption(const Value: string); virtual;
    function GetLeft: integer; virtual;
    function GetTop: integer;  virtual;
    procedure SetAlign(const Value: TAlign); virtual;
    procedure SetParent(const Value: TControl); virtual;
    function GetColor: integer; virtual;
    procedure SetColor(const Value: integer); virtual;
public
   constructor Create; override;
   destructor  Destroy; override;
   procedure CreateControl; virtual;
   function  AddTool(aTool: TTool): TTool; override;
   function  GetPropertyStringByName(AName: string): string; override;
   procedure SetPropertyStringByName(AName: string; aValue: string); override;

   property  Owner: TComponent read Get_Owner write Set_Owner;
   property  Parent: TControl  read GetParent write SetParent;
   property  Control: TControl read FControl write SetControl;
   property  Font: TFont read GetFont;
published
   property   Visible: boolean read GetVisible write SetVisible;
   property   Align: TAlign   read GetAlign write SetAlign;
   property   Left: integer   read GetLeft write SetLeft;
   property   Top: integer    read GetTop write SetTop;
   property   Width: integer  read GetWidth write SetWidth;
   property   Height: integer read GetHeight write SetHeight;
   property   Caption: string read GetCaption write SetCaption;
   property   AutoSize: boolean read GetAutoSize write SetAutoSize;
   property   Color: integer read GetColor write SetColor;
   property   Enabled: boolean read GetEnabled write SetEnabled;
   property   Font_Name: string read GetFont_Name write SetFont_Name;
   property   Font_Size: integer read GetFont_Size write SetFont_Size;
   property   Font_Height: integer read GetFont_Height write SetFont_Height;
   property   Font_Style: TFontStyles read GetFont_Style write SetFont_Style;
   property   Font_Color: TColor read GetFont_Color write SetFont_Color;
   property   Font_Charset: TFontCharset read GetFont_Charset write SetFont_Charset;
   property   AnchorLeft: boolean read GetAnchorLeft write SetAnchorLeft;
   property   AnchorRight: boolean read GetAnchorRight write SetAnchorRight;
   property   AnchorTop: boolean read GetAnchorTop write SetAnchorTop;
   property   AnchorBottom: boolean read GetAnchorBottom write SetAnchorBottom;
   property   DestroyControl: boolean read FDestroyControl write SetDestroyControl;
end;

implementation
uses
   u_Props,
   t_OrbisFrame;

{ TControlTool }

function TControlTool.AddTool(aTool: TTool): TTool;
var
   CT: TControlTool;
   FT: TOrbisFrame;
begin
   result := inherited AddTool(aTool);
   if result = nil then exit;
   if result is TControlTool then
   begin
   CT := TControlTool(result);
   CT.Owner := Control;
   CT.Parent := Control;
   end
   else
   if result is TOrbisFrame then
   begin
      FT := TOrbisFrame(result);
      FT.Owner  := Control;
      FT.Parent := Control;
   end;
end;

constructor TControlTool.Create;
begin
   inherited;
   CreateControl;
end;

procedure TControlTool.CreateControl;
begin

end;


function TControlTool.GetAlign: TAlign;
begin
   result := alNone;
   if Control = nil then exit;
   result := Control.Align;
end;



function TControlTool.GetAutoSize: boolean;
begin
  result := false;
end;

function TControlTool.GetCaption: string;
begin
   result := '';
end;


function TControlTool.GetColor: integer;
begin
   result := 0;
end;

function TControlTool.GetEnabled: boolean;
begin
   result := false;
   if Control = nil then exit;
   result := Control.Enabled;
end;

function TControlTool.GetFont: TFont;
begin
   result := nil;
end;

function TControlTool.GetFont_Color: TColor;
var
   F: TFont;
begin
   result := 0;
   F := Font;
   if F <> nil then
      result := F.Color;
end;

function TControlTool.GetFont_Height: integer;
var
   F: TFont;
begin
   result := 0;
   F := Font;
   if F <> nil then
      result := F.Height;
end;

function TControlTool.GetFont_Charset: TFontCharset;
var
   F: TFont;
begin
   result := 0;
   F := Font;
   if F <> nil then
      result := F.Charset;
end;

function TControlTool.GetFont_Name: string;
var
   F: TFont;
begin
   result := '';
   F := Font;
   if F <> nil then
      result := F.Name;
end;

function TControlTool.GetFont_Style: TFontStyles;
var
   F: TFont;
begin
   result := [];
   F := Font;
   if F <> nil then
      result := F.Style;
end;

function TControlTool.GetHeight: integer;
begin
   result := 0;
   if Control = nil then exit;
   result := Control.Height;
end;

function TControlTool.GetLeft: integer;
begin
   result := 0;
   if Control = nil then exit;
   result := Control.Left;
end;

function TControlTool.Get_Owner: TComponent;
begin
   result := nil;
   if Control = nil then exit;
   result := Control.Owner;
end;

function TControlTool.GetParent: TControl;
begin
   result := nil;
   if Control = nil then exit;
   result := Control.Parent;
end;


function TControlTool.GetTop: integer;
begin
   result := 0;
   if Control = nil then exit;
   result := Control.Top;
end;

function TControlTool.GetVisible: boolean;
begin
   result := false;
   if Control = nil then exit;
   result := Control.Visible;
end;

function TControlTool.GetWidth: integer;
begin
   result := 0;
   if Control = nil then exit;
   result := Control.Width;
end;

procedure TControlTool.SetAlign(const Value: TAlign);
begin
  if Control <> nil then Control.Align := Value;
end;


procedure TControlTool.SetAutoSize(const Value: boolean);
begin

end;

procedure TControlTool.SetCaption(const Value: string);
begin
end;


procedure TControlTool.SetColor(const Value: integer);
begin

end;

procedure TControlTool.SetControl(const Value: TControl);
begin
   FControl := Value;
end;

procedure TControlTool.SetEnabled(const Value: boolean);
begin
   if Control <> nil then
      Control.Enabled := Value;
end;

procedure TControlTool.SetFont_Color(const Value: TColor);
var
   F: TFont;
begin
   F := Font;
   if F <> nil then
      F.Color := Value;
end;

procedure TControlTool.SetFont_Height(const Value: integer);
var
   F: TFont;
begin
   F := Font;
   if F <> nil then
      F.Height := Value;
end;

procedure TControlTool.SetFont_Charset(const Value: TFontCharset);
var
   F: TFont;
begin
   F := Font;
   if F <> nil then
      F.Charset := Value;
end;

procedure TControlTool.SetFont_Name(const Value: string);
var
   F: TFont;
begin
   F := Font;
   if F <> nil then
      F.Name := Value;
end;

procedure TControlTool.SetFont_Style(const Value: TFontStyles);
var
   F: TFont;
begin
   F := Font;
   if F <> nil then
      F.Style := Value;
end;

procedure TControlTool.SetHeight(const Value: integer);
begin
   if Control <> nil then
      Control.Height := Value;
end;

procedure TControlTool.SetLeft(const Value: integer);
begin
   if Control <> nil then
      Control.Left := Value;
end;

procedure TControlTool.Set_Owner(const Value: TComponent);
begin
   if Control = nil then exit;
   if Owner <> nil then
   begin
      Owner.RemoveComponent(Control);
   end;
   Value.InsertComponent(Control);
end;

procedure TControlTool.SetParent(const Value: TControl);
begin
   if Control = nil then exit;
   if not (Value is TWinControl) then exit;
   Control.Parent := TWinControl(Value);
end;


procedure TControlTool.SetTop(const Value: integer);
begin
   if Control <> nil then
      Control.Top := Value;
end;

procedure TControlTool.SetVisible(const Value: boolean);
begin
   if Control <> nil then
      Control.Visible := Value;
end;

procedure TControlTool.SetWidth(const Value: integer);
begin
   if Control <> nil then
      Control.Width := Value;
end;

function TControlTool.GetFont_Size: integer;
var
   F: TFont;
begin
   result := 0;
   F := Font;
   if F <> nil then
      result := F.Size;
end;

procedure TControlTool.SetFont_Size(const Value: integer);
var
   F: TFont;
begin
   F := Font;
   if F <> nil then
      F.Size := Value;
end;

function TControlTool.GetAnchorBottom: boolean;
begin
   result := false;
   if Control <> nil then
      result:= akBottom in Control.Anchors;
end;

function TControlTool.GetAnchorLeft: boolean;
begin
   result := false;
   if Control <> nil then
      result:= akLeft in Control.Anchors;
end;

function TControlTool.GetAnchorRight: boolean;
begin
   result := false;
   if Control <> nil then
      result:= akRight in Control.Anchors;
end;

function TControlTool.GetAnchorTop: boolean;
begin
   result := false;
   if Control <> nil then
      result:= akTop in Control.Anchors;
end;

procedure TControlTool.SetAnchorBottom(const Value: boolean);
begin
   if Control <> nil then
   if Value then
      Control.Anchors := Control.Anchors + [akBottom]
   else
      Control.Anchors := Control.Anchors - [akBottom];
end;

procedure TControlTool.SetAnchorLeft(const Value: boolean);
begin
   if Control <> nil then
   if Value then
      Control.Anchors := Control.Anchors + [akLeft]
   else
      Control.Anchors := Control.Anchors - [akLeft];
end;

procedure TControlTool.SetAnchorRight(const Value: boolean);
begin
   if Control <> nil then
   if Value then
      Control.Anchors := Control.Anchors + [akRight]
   else
      Control.Anchors := Control.Anchors - [akRight];
end;

procedure TControlTool.SetAnchorTop(const Value: boolean);
begin
   if Control <> nil then
   if Value then
      Control.Anchors := Control.Anchors + [akTop]
   else
      Control.Anchors := Control.Anchors - [akTop];
end;

function TControlTool.GetPropertyStringByName(AName: string): string;
//var
//   l: integer;
begin
   if aName = ''  then exit;
   if aName[1]='@' then
      result := GetPropStringByName(Control, Copy(aName,2,Length(aName)-1))
   else
      result := inherited GetPropertyStringByName(aName);
end;

procedure TControlTool.SetPropertyStringByName(AName, aValue: string);
//var
//   l: integer;
begin
   if aName = ''  then exit;
   if aName[1]='@' then
      SetPropStringByName(Control, Copy(aName,2,Length(aName)-1),aValue)
   else
      inherited SetPropertyStringByName(aName,aValue);
end;

procedure TControlTool.SetDestroyControl(const Value: boolean);
begin
   FDestroyControl := Value;
end;

destructor TControlTool.Destroy;
begin
   if DestroyControl then
      Control.Free;
   inherited;
end;

INITIALIZATION
END.
