unit t_FormTool;

interface
uses
   COntrols,
   f_OrbisForm,
   // KERNEL
   Elements,
   Tools,
   // APP
   t_InterfaceTool;

type
TTaskAction = (taSelect,taRun,taRunSelect);

TFormTool = class(TInterfaceTool)
private
   FForm: TOrbisForm;
   FManualSetting: boolean;
    FOnCloseScript: string;
    FOnResizeScript: string;
    FOnConstrainedResizeScript: string;
    FOnShowScript: string;
    FOnCreateScript: string;
   procedure SetAutosize(const Value: boolean);
   procedure SetCaption(const Value: string);
   procedure SetHeight(const Value: integer);
   procedure SetLeft(const Value: integer);
   procedure SetTop(const Value: integer);
   procedure SetWidth(const Value: integer);
   function GetAutosize: boolean;
   function GetCaption: string;
   function GetHeight: integer;
   function GetLeft: integer;
   function GetTop: integer;
   function GetVisible: boolean;
   function GetWidth: integer;
   procedure SetManualSetting(const Value: boolean);
   function GetAcceptDropFiles: boolean;
   procedure SetAcceptDropFiles(const Value: boolean);
    function GetStayOnTop: boolean;
    procedure SetStayOnTop(const Value: boolean);
    procedure SetOnCloseScript(const Value: string);
    procedure SetOnResizeScript(const Value: string);
    procedure SetOnConstrainedResizeScript(const Value: string);
    procedure SetOnShowScript(const Value: string);
    function GetAlphaBlend: boolean;
    function GetColor: integer;
    function GetTRansparentColor: boolean;
    function GetTRansparentColorValue: integer;
    procedure SetAlphaBlend(const Value: boolean);
    procedure SetColor(const Value: integer);
    procedure SetTransparentColor(const Value: boolean);
    procedure SetTransparentColorValue(const Value: integer);
    function GetAlphaBlendValue: integer;
    procedure SetAlphaBlendValue(const Value: integer);
    procedure SetOnCreateScript(const Value: string);
    procedure SetModalResult(const Value: integer);
public
   constructor Create; override;
   destructor  Destroy; override;
   function    AddTool(aTool: TTool): TTool; override;
   procedure SetVisible(const Value: boolean); virtual;
   procedure CreateForm; virtual;
   function    GetCursor: TCursor; override;
   procedure   SetCursor(const Value: TCursor); override;
   procedure on_tm_DropFile(var aMessage: TOrbisTextMessage; var Handled: boolean); virtual;
   procedure on_tm_Show; virtual;
   procedure on_tm_Hide; virtual;
   procedure HandleMessage(var aMessage: TOrbisMessage; var Handled: boolean); override;
   procedure OnFormCanResize(Sender: TObject; var NewWidth,NewHeight: Integer; var Resize: Boolean); virtual;
   procedure OnFormConstrainedResize(Sender: TObject; var MinWidth,MinHeight, MaxWidth, MaxHeight: Integer); virtual;
   procedure OnFormResize(Sender: TObject); virtual;
   procedure OnFormCloseQuery(Sender: TObject; var CanClose: Boolean); virtual;
   procedure OnFormShow(Sender: TObject); virtual;
  property aForm: TOrbisForm read FForm write FForm;
published
   property ManualSetting: boolean read FManualSetting write SetManualSetting;
   property Caption: string read GetCaption write SetCaption;
   property Left: integer read GetLeft write SetLeft;
   property Top: integer read GetTop write SetTop;
   property Width: integer read GetWidth write SetWidth;
   property Height: integer read GetHeight write SetHeight;
   property Autosize: boolean read GetAutosize write SetAutosize;
   property Visible: boolean read GetVisible write SetVisible;
   property AcceptDropFiles: boolean read GetAcceptDropFiles write SetAcceptDropFiles;
   property StayOnTop: boolean read GetStayOnTop write SetStayOnTop;
   property OnCreateScript: string read FOnCreateScript write SetOnCreateScript;
   property OnCloseScript: string read FOnCloseScript write SetOnCloseScript;
   property OnShowScript: string read FOnShowScript write SetOnShowScript;
   property OnResizeScript: string read FOnResizeScript write SetOnResizeScript;
   property OnConstrainedResizeScript: string read FOnConstrainedResizeScript write SetOnConstrainedResizeScript;
   property Color: integer read GetColor write SetColor;
   property AlphaBlend: boolean read GetAlphaBlend write SetAlphaBlend;
   property AlphaBlendValue: integer read GetAlphaBlendValue write SetAlphaBlendValue;
   property TransparentColor: boolean read GetTRansparentColor write SetTransparentColor;
   property TransparentColorValue: integer read GetTRansparentColorValue write SetTransparentColorValue;
   property ModalResult: integer write SetModalResult;
end;

implementation
uses
   //KERNEL
   Forms,
   e_Class,
   m_ORBIS,
   m_Proj,
   t_ControlTool,
   t_OrbisFrame;

{ TFormTool }

function TFormTool.AddTool(aTool: TTool): TTool;
var
   CT: TControlTool;
   FT: TOrbisFrame;
begin
   result := inherited AddTool(aTool);
   if result = nil then exit;
   if result is TControlTool then
   begin
      CT := TControlTool(result);
      CT.Owner := aForm;
      CT.Parent := aForm;
   end
   else
   if result is TOrbisFrame then
   begin
      FT := TOrbisFrame(result);
      FT.Owner  := aForm;
      FT.Parent := aForm;
   end;
end;

constructor TFormTool.Create;
begin
   CreateForm;
end;

procedure TFormTool.CreateForm;
begin
   aForm := TOrbisForm.Create(Application);
   aForm.KeyPreview := true;
   aForm.OnCloseQuery := OnFormCloseQuery;
   aForm.OnCanResize := OnFormCanResize;
   aForm.OnConstrainedResize := OnFormConstrainedResize;
   aForm.OnResize := OnFormResize;
   aForm.OnShow := OnFormShow;
end;

destructor TFormTool.Destroy;
begin
   if aForm <> nil then
      aForm.Free;
   inherited;
end;

function TFormTool.GetAcceptDropFiles: boolean;
begin
   result := false;
   if FForm <> nil then result := FForm.AcceptDropFiles;
end;

function TFormTool.GetStayOnTop: boolean;
begin
   result := false;
   if FForm <> nil then
      result := FForm.PopupMode = pmExplicit;
end;

function TFormTool.GetAutosize: boolean;
begin
   result := false;
   if FForm <> nil then result := FForm.AutoSize;
end;

function TFormTool.GetCaption: string;
begin
   result := '';
   if FForm <> nil then result := FForm.Caption;
end;

function TFormTool.GetCursor: TCursor;
begin
   result := 0;
end;

function TFormTool.GetHeight: integer;
begin
   result := 0;
   if FForm <> nil then result := FForm.Height;
end;

function TFormTool.GetLeft: integer;
begin
   result := 0;
   if FForm <> nil then result := FForm.Left;
end;

function TFormTool.GetTop: integer;
begin
   result := 0;
   if FForm <> nil then result := FForm.Top;
end;

function TFormTool.GetVisible: boolean;
begin
   result := false;
   if FForm <> nil then result := FForm.Visible;
end;

function TFormTool.GetWidth: integer;
begin
   result := 0;
   if FForm <> nil then result := FForm.Width;
end;

procedure TFormTool.HandleMessage(var aMessage: TOrbisMessage; var Handled: boolean);
begin
   Handled := false;
   inherited HandleMessage(aMessage, Handled);
   if Handled then exit;
   case aMessage.Message of
      tm_DropFile:  on_tm_DropFile(TOrbisTextMessage(aMessage), Handled);
      tm_Show:      on_tm_Show;
      tm_Hide:      on_tm_Hide;
   end;
end;

procedure TFormTool.on_tm_DropFile(var aMessage: TOrbisTextMessage;
  var Handled: boolean);
begin
   //PostTextMsg(tm_AddToProtocol,Name +' Droped file: '+aMessage.Text,nil);
end;

procedure TFormTool.on_tm_Hide;
begin
   Visible := false;
end;

procedure TFormTool.on_tm_Show;
begin
   Visible := true;
end;

procedure TFormTool.SetAcceptDropFiles(const Value: boolean);
begin
   if FForm <> nil then FForm.AcceptDropFiles := Value;
end;

procedure TFormTool.SetStayOnTop(const Value: boolean);
begin
   if FForm <> nil then
   if Value then
      FForm.PopupMode := pmExplicit
//      FForm.FormStyle := fsStayOnTop
   else
//      FForm.FormStyle := fsNormal;
      FForm.PopupMode := pmNone;
end;

procedure TFormTool.SetAutosize(const Value: boolean);
begin
   if FForm <> nil then FForm.AutoSize := Value;
end;

procedure TFormTool.SetCaption(const Value: string);
begin
   if FForm <> nil then FForm.Caption := Value;
end;

procedure TFormTool.SetCursor(const Value: TCursor);
begin

end;

procedure TFormTool.SetHeight(const Value: integer);
begin
   if FForm <> nil then FForm.Height := Value;
end;

procedure TFormTool.SetLeft(const Value: integer);
begin
   if FForm <> nil then FForm.Left := Value;
end;

procedure TFormTool.SetManualSetting(const Value: boolean);
begin
  FManualSetting := Value;
end;

procedure TFormTool.SetTop(const Value: integer);
begin
   if FForm <> nil then FForm.Top := Value;
end;

procedure TFormTool.SetVisible(const Value: boolean);
begin
   if FForm <> nil then FForm.Visible := Value;
end;

procedure TFormTool.SetWidth(const Value: integer);
begin
   if FForm <> nil then FForm.Width := Value;
end;

procedure TFormTool.SetOnCloseScript(const Value: string);
begin
  FOnCloseScript := Value;
end;

procedure TFormTool.SetOnResizeScript(const Value: string);
begin
  FOnResizeScript := Value;
end;

procedure TFormTool.OnFormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
   CanClose := true;
   if OnCloseScript <> '' then
      PostTextMsg(tm_Run,OnCloseScript,nil);
end;

procedure TFormTool.OnFormResize(Sender: TObject);
begin
   if OnResizeScript <> '' then
   begin
      PostTextMsg(tm_Run,OnResizeScript,nil);
      Application.ProcessMessages;
   end;
end;

procedure TFormTool.OnFormCanResize(Sender: TObject; var NewWidth,
  NewHeight: Integer; var Resize: Boolean);
begin
   Resize := true;
end;

procedure TFormTool.OnFormConstrainedResize(Sender: TObject; var MinWidth,
  MinHeight, MaxWidth, MaxHeight: Integer);
begin
   if OnConstrainedResizeScript <> '' then
   begin
      PostTextMsg(tm_Run,OnConstrainedResizeScript,nil);
      Application.ProcessMessages;
   end;
end;

procedure TFormTool.SetOnConstrainedResizeScript(const Value: string);
begin
  FOnConstrainedResizeScript := Value;
end;

procedure TFormTool.OnFormShow(Sender: TObject);
begin
   if OnShowScript <> '' then
   begin
      PostTextMsg(tm_Run,OnShowScript,nil);
      Application.ProcessMessages;
   end;
end;

procedure TFormTool.SetOnShowScript(const Value: string);
begin
  FOnShowScript := Value;
end;

function TFormTool.GetAlphaBlend: boolean;
begin
   result := aForm.AlphaBlend;
end;

function TFormTool.GetColor: integer;
begin
   result := aForm.Color;
end;

function TFormTool.GetTRansparentColor: boolean;
begin
   result := aForm.TransparentColor;
end;

function TFormTool.GetTRansparentColorValue: integer;
begin
   result := aForm.TransparentColorValue;
end;

procedure TFormTool.SetAlphaBlend(const Value: boolean);
begin
   aForm.AlphaBlend := Value;
end;

procedure TFormTool.SetColor(const Value: integer);
begin
   aForm.Color := Value;
end;

procedure TFormTool.SetTransparentColor(const Value: boolean);
begin
   aForm.TransparentColor := Value;
end;

procedure TFormTool.SetTransparentColorValue(const Value: integer);
begin
   aForm.TransparentColorValue := Value;
end;

function TFormTool.GetAlphaBlendValue: integer;
begin
   result := aForm.AlphaBlendValue;
end;

procedure TFormTool.SetAlphaBlendValue(const Value: integer);
begin
   aForm.AlphaBlendValue := Value;
end;

procedure TFormTool.SetOnCreateScript(const Value: string);
begin
   if FOnCreateScript <> Value then
   begin
      FOnCreateScript := Value;
      if Value <> '' then
      begin
         PostTextMsg(tm_Run,Value,nil);
         Application.ProcessMessages;
      end;
   end
end;

procedure TFormTool.SetModalResult(const Value: integer);
begin
   if aForm <> nil then aForm.ModalResult := Value;
end;

INITIALIZATION
   RegisterOrbisClass(TFormTool);
END.
