unit T_PZ;

interface

uses
   Classes,
   f_TopImp,
   Elements, {e_lstBin,}
   u_TopoLT,
   e_TopMDL,
   e_WCS,
   f_Main,
   g_Elems,
   Tools,
   T_TopImp,
{$IFDEF RAS}
   e_Raster,
{$ENDIF}
{$IFDEF BINRAS}
   e_TopRas,
{$ENDIF}
   F_Gauge, F_Protcl;

const
   WCSID = cs_JTSK;
type
   TPZSelector = class(TTool)
    protected
      FDialog: TBlockSelectDLG;
      function  GetDialog: TBlockSelectDLG;
      procedure SetDialog(aDialog: TBlockSelectDLG);
    public
      procedure DoIt; override;
      property  Dialog: TBlockSelectDLG read GetDialog write setDialog;
   end;


   TPZReader = class(TPZReaderNoRastr)
    public
      constructor Create; override;
      procedure DoIt; override;
   end;


   TTopoLImport = class(TSeqTechnology)
    public
      constructor Create; override;
   end;


{$IFDEF BINRAS}
   function ReadTopolRaster(RasterName: string): TTopRas;
{$ENDIF}
{$IFDEF RAS}
   function ReadTopolRaster(RasterName: string): TRaster;
{$ENDIF}

implementation

uses
   WinProcs,
   Controls,
   Dialogs,
   Forms,
   SysUtils,
   e_Class,
   e_TopIS,
   e_TplPrp,
   M_Orbis,
   t_MainTl, {!T_Main,}
   U_File,
{$IFDEF BINRAS}
   U_Ras,
{$ENDIF}
   Worlds;





{ ******************* }

{ TPZSelector }

procedure TPZSelector.DoIt;
var
   i: longInt;
   Source: TTopoLImportSource;
   S: string;
begin
   inherited DoIt;
   Dialog.ed_Knihovny.text := PathOfLibraries;
   Dialog.Directory := TMainTool(World.MainTool).Options.DataDirectory;
   if (Dialog.ShowModal = mrOK) and
      (Dialog.Outline.ItemCount > 0) or
      (Dialog.ROutline.ItemCount > 0) then
   begin

      Dialog.Directory := ExecPath;

      Source := TTopoLImportSource.Create;
      OutPut := Source;
      with Dialog.Outline do
      begin
         for i := 1 to ItemCount  do
         begin
            Source.Blocks.AddItem(Items[i].Text);
         end;
      end;
{$IFDEF BINRAS}
      with Dialog.ROutline do
      begin
         for i := 1 to ItemCount  do
         begin
            S := Items[i].Text;
            if UpperCase(ExtractFileExt(Items[i].Text)) = '.RAK' then
            begin
               S := Copy(S, 1, Length(S) - 4);
               TOptions(TMainTool(World.MainTool).Options).DelRas.AddString(S + '.RAS');
              { TRaspTool.}FastDekomp(S);
               S := S + '.RAS';
            end;
            Source.Rasters.AddItem(S);
         end;
      end;
{$ENDIF}
      Source.Libraries.Name := Dialog.ed_Knihovny.Text;
   end
   else
      OutPut := nil;
   Dialog := nil;
end;

function  TPZSelector.GetDialog: TBlockSelectDLG;
begin
   if not assigned(FDialog)then
      FDialog:= TBlockSelectDLG.Create(Application);

   result := FDialog;
end;

procedure TPZSelector.SetDialog(aDialog: TBlockSelectDLG);
begin
   if assigned(FDialog) then FDialog.Free;
   FDialog := nil;
   FDialog := aDialog;
end;


{$IFDEF BINRAS}
function ReadTopolRaster(RasterName: string): TTopRas;
begin
   result:= nil;
   if not fileExists(RasterName) then exit;

   if UpperCase(ExtractFileExt(RasterName)) = '.RAK' then
   begin
      RasterName := Copy(RasterName, 1, Length(RasterName) - 4);
      TOptions(TMainTool(World.MainTool).Options).DelRas.AddString(RasterName + '.RAS');
      {TRaspTool.}FastDekomp(RasterName);
      RasterName := RasterName + '.RAS';
   end;

   result := TTopRas.Create(RasterName);
end;
{$ENDIF}

{$IFDEF RAS}
function ReadTopolRaster(RasterName: string): TRaster;
begin
   result:= nil;
   if not fileExists(RasterName) then exit;
   result := TRaster.Create;
   result.FileName := RasterName;
end;
{$ENDIF}

{ TPZReader }

constructor TPZReader.Create;
begin
   inherited Create;
   QReadAreas := true;
end;


procedure TPZReader.DoIt;
var
{$IFDEF BINRAS}
   R: TTopRas;
   RasterName: string;
{$ENDIF}
   BR: TTopoLBlokReader;
   Source: TTopoLImportSource;
   PZ : TTopoLPredmetZajmu;
   i: longInt;
begin
  try
     if Input = nil then
     begin
        MessageDlg('Nen vybrn dn blok', mtInformation, [mbOK], 0);
        OutPut := nil;
        exit;
     end;
     TEntity(World.ObjectModel).AddElement(Input);
     Screen.Cursor := crHourglass;
     Source := TTopoLImportSource(Input);
     PZ := TTopoLPredmetZajmu.Create;
     PZ.Name := 'TOPOL ... Pedmt zjmu:';
{$IFDEF BINRAS}
     { Rastry }
     for i := 0 to Source.Rasters.Count -1 do
     begin
        RasterName := Source.Rasters.Items[i].SourcePath;
        R:= ReadTopolRaster(Source.Rasters.Items[i].SourcePath);
        if R <> nil then
           PZ.AddAndUpdateRect(R);
     end;
{$ENDIF}
     { Bloky }
     for i := 0 to Source.Blocks.Count -1 do
     begin
       try
        BR := TTopoLBlokReader.CreateBlock(Source.Blocks.Items[i].SourcePath);
        BR.FiltrDruhu:= FiltrDruhu;
        BR.QReadAreas :=  QReadAreas;
        PZ.AddAndUpdateRect(Br.ReadBlock);
       finally
        BR.Free;
       end
     end;
     PathOfLibraries := Source.Libraries.Name;
     OutPut := PZ;
     Screen.Cursor := crDefault;
  except
    messageDlg('Chyba ten pedmtu zjmu', mtError, [mbOK], 0)
  end;
end;


{ TTopoLImport }

constructor TTopoLImport.Create;
begin
   inherited Create;
   AddTool(TPZSelector.Create);
   TPZReader(AddTool(TPZReader.Create)).QReadAreas:= true;
end;

{
   TFiltrDruhuObjektu = class(TElement)
     ProBod, ProLin, ProPlo, ProTxt: TFiltrDruhu;
}

BEGIN
   RegisterOrbisClass(TPZReader);
END.

