unit E_MemoryInfo;

interface
uses classes, elements, windows, sysutils;


const
     sMemoryLoad = 'Memory in used: %d%%';
     sTotalPhys = 'Total phy memory: %n MB';
     sAvailPhys = 'Avail phy memory: %n MB';
     sTotalPageFile = 'Total page file: %n MB';
     sAvailPageFile = 'Avail page file: %n MB';
     sTotalVirtual = 'Total virtual memory: %n MB';
     sAvailVirtual = 'Avail virtual memory: %n MB';
     sMB = 1000000;

type
  TMemoryStatus = record
     dwLength: DWORD;
     dwMemoryLoad: DWORD;
     dwTotalPhys: DWORD;
     dwAvailPhys: DWORD;
     dwTotalPageFile: DWORD;
     dwAvailPageFile: DWORD;
     dwTotalVirtual: DWORD;
     dwAvailVirtual: DWORD;
  end;



  TMemoryInfo = class(TOwnerEntity)
  private
     FMS: TMemoryStatus;
     function GetMs: TMemoryStatus;
     procedure SetMs(const Value: TMemoryStatus);
     function GetAvailPageFile: String;
     function GetAvailPhys: String;
     function GetAvailVirtual: String;
     procedure GetGlobalMemoryStatus;
     function GetMemoryLoad: String;
     function GetTotalPageFile: String;
     function GetTotalPhys: String;
     function GetTotalVirtual: String;
  public
      constructor Create; override;
      destructor  Destroy; override;

      procedure GetMemStatusInTSList(aSList: TStrings);
      procedure Refresh;
  published
      property MS: TMemoryStatus read GetMs write SetMs;

      property MemoryLoad: String read GetMemoryLoad;
      property TotalPhys: String read GetTotalPhys;
      property AvailPhys: String read GetAvailPhys;
      property TotalPageFile: String read GetTotalPageFile;
      property AvailPageFile: String read GetAvailPageFile;
      property TotalVirtual: String read GetTotalVirtual;
      property AvailVirtual: String read GetAvailVirtual;
  end;



implementation


constructor TMemoryInfo.Create;
begin
  inherited;

  Refresh;
//  GetGlobalMemoryStatus;

end;


destructor TMemoryInfo.Destroy;
begin
  inherited;
end;


procedure TMemoryInfo.Refresh;
begin
  GetGlobalMemoryStatus;
end;


function TMemoryInfo.GetAvailPageFile: String;
var
   LiAvailPageFile: LongInt;
begin
   LiAvailPageFile := FMS.dwAvailPageFile;
   Result := Format(sAvailPageFile,[LiAvailPageFile/sMB]);
end;



function TMemoryInfo.GetAvailPhys: String;
var
   LiAvailPhys: LongInt;
begin
   LiAvailPhys := FMS.dwAvailPhys;
   Result := Format(sAvailPhys,[LiAvailPhys/sMB]);
end;



function TMemoryInfo.GetAvailVirtual: String;
var
   LiAvailVirtual: LongInt;
begin
   LiAvailVirtual := FMS.dwAvailVirtual;
   Result := Format(sAvailVirtual,[LiAvailVirtual/sMB]);
end;



procedure TMemoryInfo.GetGlobalMemoryStatus;
begin
   FMS.dwLength := SizeOf(FMS);
   GlobalMemoryStatus(MemoryStatus(FMS));
end;



function TMemoryInfo.GetMemoryLoad: String;
begin
   Result := Format(sMemoryLoad,[FMS.dwMemoryLoad]);
end;



function TMemoryInfo.GetMs: TMemoryStatus;
begin
   FMS.dwLength := SizeOf(FMS);
   GlobalMemoryStatus(MemoryStatus(FMS));
   Result := FMS;
end;



function TMemoryInfo.GetTotalPageFile: String;
var
   LiTotalPageFile: LongInt;
begin
   LiTotalPageFile := FMS.dwTotalPageFile;
   Result := Format(sTotalPageFile,[LiTotalPageFile/sMB]);
end;



function TMemoryInfo.GetTotalPhys: String;
var
   LiTotalPhys: LongInt;
begin
   LiTotalPhys := FMS.dwTotalPhys;
   Result := Format(sTotalPhys,[LiTotalPhys/sMB]);
end;



function TMemoryInfo.GetTotalVirtual: String;
var
   LiTotalVirtual: LongInt;
begin
   LiTotalVirtual := FMS.dwTotalVirtual;
   Result := Format(sTotalVirtual,[LiTotalVirtual/sMB]);
end;



procedure TMemoryInfo.SetMs(const Value: TMemoryStatus);
begin
   FMS := Value;
end;



procedure TMemoryInfo.GetMemStatusInTSList(aSList: TStrings);
begin
   if aSList = nil then
      aSlist := TStringList.Create
   else
      aSlist.Clear;


   with aSList do
   begin
      add(MemoryLoad);
      add(TotalPhys);
      add(AvailPhys);
      add(TotalPageFile);
      add(AvailPageFile);
      add(TotalVirtual);
      add(AvailVirtual);
   end;
end;


END.
