unit e_ProjectionInfo;

interface
uses
   e_Proje,
   Elements;

type

TProjectionInfo = class(TOwnerElement)
private
   Fy: double;
   FScale: double;
   FX: double;
   FVirtualScale: double;
   procedure SetScale(const Value: double);
   procedure SetX(const Value: double);
   procedure Sety(const Value: double);
   procedure SetVirtualScale(const Value: double);
published
   property X: double read FX write SetX;
   property Y: double read Fy write Sety;
   property Scale: double read FScale write SetScale;
   property VirtualScale: double read FVirtualScale write SetVirtualScale;
end;

TProjectionInfoEntity = class(TOwnerEntity)
private
    FActualInfoIndex: integer;
    procedure SetActualInfoIndex(const Value: integer);
    function GetActualProjectionInfo: TProjectionInfo;
    function GetNoNext: boolean;
    function GetNoPrevious: boolean;
    procedure SetNewProjection(const Value: TProjection);
public
   constructor Create; override;
   procedure Next;
   procedure Previous;
   procedure ProjectionChange(aX,aY,aScale,aVirtualScale: double);
   property  ActualProjectionInfo: TProjectionInfo read GetActualProjectionInfo;
published
   property ActualInfoIndex: integer read FActualInfoIndex write SetActualInfoIndex;
   property NoNext: boolean read GetNoNext stored false;
   property NoPrevious: boolean read GetNoPrevious stored false;
   property NewProjection: TProjection write SetNewProjection stored false;
end;

TProjectionInfoModel = class(TOwnerEntity)
private
   FProjections: TProjectionInfoEntity;
   FNamedProjections: TOwnerEntity;
    function GetNamedProjections: TOwnerEntity;
    function GetProjections: TProjectionInfoEntity;
published
   property Projections: TProjectionInfoEntity read GetProjections stored false;
   property NamedProjections: TOwnerEntity read GetNamedProjections stored false;
end;


implementation
uses
   e_Class;

{ TProjectionInfo }

procedure TProjectionInfo.SetScale(const Value: double);
begin
   FScale := Value;
end;

procedure TProjectionInfo.SetVirtualScale(const Value: double);
begin
   FVirtualScale := Value;
end;

procedure TProjectionInfo.SetX(const Value: double);
begin
   FX := Value;
end;

procedure TProjectionInfo.Sety(const Value: double);
begin
   Fy := Value;
end;

{ TProjectionInfoEntity }

constructor TProjectionInfoEntity.Create;
begin
   inherited;
end;

function TProjectionInfoEntity.GetActualProjectionInfo: TProjectionInfo;
var
   P: TProjectionInfo;
begin
   result := nil;
   if NoElements then
   begin
      P := TProjectionInfo.Create;
      P.VirtualScale := 5000;
      AddElement(P);
   end;
   if (ActualInfoIndex < 0) or (ActualInfoIndex > LastIndex) then exit;
   result := TProjectionInfo(Elements[ActualInfoIndex]);
end;

function TProjectionInfoEntity.GetNoNext: boolean;
begin
   result := ActualInfoIndex <= 0;
end;

function TProjectionInfoEntity.GetNoPrevious: boolean;
begin
   result := ActualInfoIndex >= LastIndex;
end;

procedure TProjectionInfoEntity.Next;
begin
   if not NoNext then
      ActualInfoIndex := ActualInfoIndex - 1;
end;

procedure TProjectionInfoEntity.Previous;
begin
   if not NoPrevious then
      ActualInfoIndex := ActualInfoIndex + 1;
end;

procedure TProjectionInfoEntity.ProjectionChange(aX, aY, aScale, aVirtualScale: double);
var
   P: TProjectionInfo;
   i: integer;
begin
   P := ActualProjectionInfo;
   if (P.X = 0) or (P.Y = 0) then
   begin
      P.X := aX;
      P.Y := aY;
      P.Scale := aScale;
      P.VirtualScale := aVirtualScale;
   end
   else
   begin
      if not NoNext then
      for i := ActualInfoIndex - 1 downTo 0 do
      begin
         Elements[i].Free;
      end;
      P := TProjectionInfo.Create;
      P.X := aX;
      P.Y := aY;
      P.Scale := aScale;
      P.VirtualScale := aVirtualScale;
      InsertElement(0,P);
      ActualInfoIndex := 0;
   end;
end;

procedure TProjectionInfoEntity.SetActualInfoIndex(const Value: integer);
begin
  FActualInfoIndex := Value;
end;

procedure TProjectionInfoEntity.SetNewProjection(const Value: TProjection);
var
   P: TProjectionInfo;
   i: integer;
begin
   if not NoNext then
   for i := ActualInfoIndex - 1 downTo 0 do
   begin
      Elements[i].Free;
   end;
   P := TProjectionInfo.Create;
   P.X := Value.Center_X;
   P.Y := Value.Center_Y;
   P.Scale := Value.Scale;
   P.VirtualScale := Value.VirtualScale;
   InsertElement(0,P);
   ActualInfoIndex := 0;
end;

{ TProjectionModel }

function TProjectionInfoModel.GetNamedProjections: TOwnerEntity;
begin
   if FNamedProjections = nil then
      FNamedProjections := TOwnerEntity(FindOrCreateElementName(TOwnerEntity,'Named Projections'));
   result := FNamedProjections;
end;

function TProjectionInfoModel.GetProjections: TProjectionInfoEntity;
begin
   if FProjections = nil then
      FProjections := TProjectionInfoEntity(FindOrCreateElementName(TProjectionInfoEntity,'Projections'));
   result := FProjections;
end;

INITIALIZATION
   RegisterOrbisClass(TProjectionInfo);
   RegisterOrbisClass(TProjectionInfoEntity);
   RegisterOrbisClass(TProjectionInfoModel);
END.
