unit ORBISTreeView;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls,
// KERNEL
  Elements,
  e_SysReg,
  Tools;

type
  TORBISTreeView = class(TTreeView)
  private
     //W: TWatchForm;
     FCanClose: boolean;
     FShowOwners: boolean;
     //FDontShowList: boolean;
     FModel: TElement;
     LastClass: TElementClass;
     ClassModel: TRegisterClassModel;
     procedure SetModel(aValue: TElement);
     procedure CheckNode(aNode: TTreeNode);
     function  CheckClassModel(aValue: TElement): boolean;
  protected
     procedure Collapse(Node: TTreeNode); override;
     procedure Expand(Node: TTreeNode); override;
  public
     property CanClose: boolean read FCanClose write FCanClose;
     property ShowOwners: boolean read FShowOwners write FShowOwners;
     property Model: TElement read FModel write SetModel;
  published
  end;



procedure Register;

implementation
uses
// KERNEL
   e_Class,
   Worlds;

procedure Register;
begin
  RegisterComponents('ORBIS', [TORBISTreeView]);
end;

{ TORBISTreeView }

procedure TORBISTreeView.SetModel(aValue: TElement);
var
   N: TTreeNode;
begin
   N := Items.AddObject(nil, aValue.Name, aValue);
   CheckNode(N);
   Change(N);
end;

function TORBISTreeView.CheckClassModel(aValue: TElement): boolean;
var
   EC: TElementClass;
begin
   result := true;
   if aValue = nil then exit;
   EC := TElementClass(aValue.ClassType);
   if EC <> LastClass then
   begin
      result := false;
      LastClass := EC;
      ClassModel := TRegisterClassModel(World.SystemRegister.ClassModels.GetItemForClass(EC));
   end;
end;

procedure TORBISTreeView.CheckNode(aNode: TTreeNode);
var
   E: TEntity;
   El: TElement;
   Envelope: boolean;
begin
   if aNode.Data = nil then exit;
   El := aNode.Data;
   CheckClassModel(El);
   if ClassModel <> nil then
   begin
     aNode.ImageIndex := ClassModel.ImageIndex;
     aNode.SelectedIndex := ClassModel.ImageSelectedIndex;
   end;
   if not ShowOwners then
   begin
      if not(El is Tentity) then exit;
      E := TEntity(El);
      if E.NoElements then exit;
      El := E.Elements[0];
      Items.AddChildObject(aNode, El.Name, El);
   end
   else
   begin
      Envelope := El is TEnvelopeEntity;
      if Envelope then
      begin
         if TEnvelopeEntity(El).Owner = nil then exit;
         E := TEnvelopeEntity(El).Owner;
      end
      else
      begin
         if El.NoOwners then exit;
         E := El.Owners[0];
      end;
      Items.AddChildObject(aNode, E.Name, E);
   end;
end;


procedure TORBISTreeView.Collapse(Node: TTreeNode);
begin
   inherited;
   Node.DeleteChildren;
   CheckNode(Node);
end;

procedure TORBISTreeView.Expand(Node: TTreeNode);
var
   E: TEntity;
   El: TElement;
   i: integer;
   N: TTreeNode;
   Envelope: boolean;
begin
   inherited;
   El := TElement(Node.Data);
   if not ShowOwners then
   begin
      if not (TElement(El) is TEntity) then exit;
      E := TEntity(El);
      if E.NoElements then exit;
      Items.BeginUpdate;
      if Node.Count > 0 then CheckNode(Node.Item[0]);
      for i := 1 to E.LastIndex do
      begin
         El := E.Elements[i];
         N := Items.AddChildObject(Node, El.Name, El);
         CheckNode(N);
      end;
      Items.EndUpdate;
   end
   else
   begin
      Envelope := El is TEnvelopeEntity;
      if Envelope then
      begin
         if TEnvelopeEntity(El).Owner = nil then exit;
      end
      else if El.NoOwners then exit;
      Items.BeginUpdate;
      if Node.Count > 0 then CheckNode(Node.Item[0]);
      if not Envelope then
      begin
         for i := 1 to El.OwnersCount-1 do
         begin
            E := El.Owners[i];
            N := Items.AddChildObject(Node, E.Name, E);
            CheckNode(N);
         end;
      end;
      Items.EndUpdate;
   end;
end;


end.


