unit f_Explor;
{
[Unit_Description]
Description=Formul Explorer
Admin=Petr
TechTopic=Forms
Topic=Interface
Version=1.0
}

interface

uses
  WinApi.Windows, WinApi.Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Elements,
  Tools, ComCtrls, ToolWin, ExtCtrls, Buttons,
  e_SysReg, StdCtrls, Menus,
  f_Watch, ImgList, System.ImageList;

type
  TExplorerForm = class(TForm)
    StatusBar: TStatusBar;
    TreeView: TTreeView;
    ImageList: TImageList;
    ListView: TListView;
    ToolBar: TToolBar;
    Splitter: TSplitter;
    SpeedButton1: TSpeedButton;
    SpeedButton2: TSpeedButton;
    SpeedButton3: TSpeedButton;
    Memo: TMemo;
    PopupMenu: TPopupMenu;
    ExploreOwners: TMenuItem;
    ExploreFromHere: TMenuItem;
    BitBtn1: TBitBtn;
    Zobraz1: TMenuItem;
    b_W: TBitBtn;
    EditProperties: TMenuItem;
    ToolButton1: TToolButton;
    N1: TMenuItem;
    Load1: TMenuItem;
    Save1: TMenuItem;
    ToolButton2: TToolButton;
    ToolButton3: TToolButton;
    OIL1: TMenuItem;
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure TreeViewExpanding(Sender: TObject; Node: TTreeNode;
      var AllowExpansion: Boolean);
    procedure TreeViewExpanded(Sender: TObject; Node: TTreeNode);
    procedure TreeViewCollapsed(Sender: TObject; Node: TTreeNode);
    procedure SpeedButton1Click(Sender: TObject);
    procedure SpeedButton2Click(Sender: TObject);
    procedure SpeedButton3Click(Sender: TObject);
    procedure TreeViewChange(Sender: TObject; Node: TTreeNode);
    procedure FormCreate(Sender: TObject);
    procedure ExploreFromHereClick(Sender: TObject);
    procedure ExploreOwnersClick(Sender: TObject);
    procedure BitBtn1Click(Sender: TObject);
    procedure Zobraz1Click(Sender: TObject);
    procedure b_WClick(Sender: TObject);
    procedure EditPropertiesClick(Sender: TObject);
    procedure Save1Click(Sender: TObject);
    procedure Load1Click(Sender: TObject);
    procedure TreeViewDragOver(Sender, Source: TObject; X, Y: Integer;
      State: TDragState; var Accept: Boolean);
    procedure OIL1Click(Sender: TObject);
  private
     W: TWatchForm;
     FCanClose: boolean;
     FShowOwners: boolean;
     FDontShowList: boolean;
     FModel: TElement;
     FTool: TTool;
     LastClass: TElementClass;
     LastSubClass: TElementClass;
     ClassModel: TRegisterClassModel;
     SubClassModel: TRegisterClassModel;
     procedure SetModel(aValue: TElement);
     procedure CheckNode(aNode: TTreeNode);
     procedure SetDontShowList(aValue: boolean);
     function  CheckClassModel(aValue: TElement): boolean;
     function  CheckSubClassModel: boolean;
  public
     property CanClose: boolean read FCanClose write FCanClose;
     property ShowOwners: boolean read FShowOwners write FShowOwners;
     property DontShowList: boolean read FDontShowList write SetDontShowList;
     property Tool: TTool read FTool write FTool;
     property Model: TElement read FModel write SetModel;
  end;


implementation

uses
   e_Class,
   //e_Index,
   m_Orbis,
   u_String,
   Worlds;
{$R *.DFM}

procedure TExplorerForm.SetDontShowList(aValue: boolean);
var
   Wdth: integer;
begin
   if aValue = FDontShowList then exit;
   FDontShowList := aValue;
   if aValue then
   begin
      ListView.Align := alNone;
      Splitter.Align := alRight;
      ListView.Visible := false;
      TreeView.Align := alClient;
   end
   else
   begin
      Wdth := ListView.Width;
      //Splitter.Align := alNone;
      TreeView.Align := alLeft;
      TreeView.Width := TreeView.Width - Wdth;
      if TreeView.Width < 50 then TreeView.Width := 50;
      Splitter.Align := alLeft;
      ListView.Align := alClient;
      ListView.Visible := true;
      TreeViewChange(nil, TreeView.Selected);
   end;
end;

procedure TExplorerForm.SetModel(aValue: TElement);
var
   N: TTreeNode;
begin
   N := TReeView.Items.AddObject(nil, aValue.Name, aValue);
   CheckNode(N);
   TreeViewChange(nil, N);
end;

function TExplorerForm.CheckClassModel(aValue: TElement): boolean;
var
   EC: TElementClass;
begin
   result := true;
   if aValue = nil then exit;
   EC := TElementClass(aValue.ClassType);
   if EC <> LastClass then
   begin
      result := false;
      LastClass := EC;
      ClassModel := TRegisterClassModel(World.SystemRegister.ClassModels.GetItemForClass(EC));
   end;
end;

function TExplorerForm.CheckSubClassModel: boolean;
var
   EC: TElementClass;
begin
   result := true;
   if ClassModel = nil then
   begin
      LastSubClass := nil;
      SubClassModel := nil;
      result := false;
      exit;
   end;
   EC := GetOrbisClass(ClassModel.SubClass);
   if EC <> LastSubClass then
   if EC <> nil then
   begin
      result := false;
      LastSubClass := EC;
      SubClassModel := TRegisterClassModel(World.SystemRegister.ClassModels.GetItemForClass(EC));
   end
   else
   begin
      result := false;
      LastSubClass := nil;
      SubClassModel := nil;
   end;
end;

procedure TExplorerForm.CheckNode(aNode: TTreeNode);
var
   E: TEntity;
   El: TElement;
   Envelope: boolean;
begin
   if aNode.Data = nil then exit;
   El := aNode.Data;
   CheckClassModel(El);
   if ClassModel <> nil then
   begin
     aNode.ImageIndex := ClassModel.ImageIndex;
     aNode.SelectedIndex := ClassModel.ImageSelectedIndex;
   end;
   if not ShowOwners then
   begin
      if not(El is Tentity) then exit;
      E := TEntity(El);
      if E.NoElements then exit;
      El := E.Elements[0];
      TreeView.Items.AddChildObject(aNode, El.Name, El);
   end
   else
   begin
      Envelope := El is TEnvelopeEntity;
      if Envelope then
      begin
         if TEnvelopeEntity(El).Owner = nil then exit;
         E := TEnvelopeEntity(El).Owner;
      end
      else
      begin
         if El.NoOwners then exit;
         E := El.Owners[0];
      end;
      TreeView.Items.AddChildObject(aNode, E.Name, E);
   end;
end;


procedure TExplorerForm.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
begin
   Tool.PostMsg(tm_QuitTask, Tool, nil, DontKill);
   CanClose := true;
   if W <> nil then
   try
      W.Free;
   except
   end;
end;

procedure TExplorerForm.TreeViewExpanding(Sender: TObject;
  Node: TTreeNode; var AllowExpansion: Boolean);
begin
   AllowExpansion := true;
end;

procedure TExplorerForm.TreeViewExpanded(Sender: TObject;
  Node: TTreeNode);
var
   E: TEntity;
   El: TElement;
   i: integer;
   N: TTreeNode;
   Envelope: boolean;
begin
   El := TElement(Node.Data);
   if not ShowOwners then
   begin
      if not (TElement(El) is TEntity) then exit;
      E := TEntity(El);
      if E.NoElements then exit;
      TreeView.Items.BeginUpdate;
      if Node.Count > 0 then CheckNode(Node.Item[0]);
      for i := 1 to E.LastIndex do
      begin
         El := E.Elements[i];
         N := TreeView.Items.AddChildObject(Node, El.Name, El);
         CheckNode(N);
      end;
      TreeView.Items.EndUpdate;
   end
   else
   begin
      Envelope := El is TEnvelopeEntity;
      if Envelope then
      begin
         if TEnvelopeEntity(El).Owner = nil then exit;
      end
      else if El.NoOwners then exit;
      TreeView.Items.BeginUpdate;
      if Node.Count > 0 then CheckNode(Node.Item[0]);
      if not Envelope then
      begin
         for i := 1 to El.OwnersCount-1 do
         begin
            E := El.Owners[i];
            N := TreeView.Items.AddChildObject(Node, E.Name, E);
            CheckNode(N);
         end;
      end;
      TreeView.Items.EndUpdate;
   end;
end;

procedure TExplorerForm.TreeViewCollapsed(Sender: TObject;Node: TTreeNode);
begin
   Node.DeleteChildren;
   CheckNode(Node);
end;

procedure TExplorerForm.SpeedButton1Click(Sender: TObject);
begin
   if SpeedButton1.Down then ListView.ViewStyle := vsIcon;
end;

procedure TExplorerForm.SpeedButton2Click(Sender: TObject);
begin
   if SpeedButton2.Down then ListView.ViewStyle := vsList;
end;

procedure TExplorerForm.SpeedButton3Click(Sender: TObject);
begin
   if SpeedButton3.Down then ListView.ViewStyle := vsReport;
end;

procedure TExplorerForm.TreeViewChange(Sender: TObject; Node: TTreeNode);
var
   N: TTreeNode;
   L: TListItem;
   E: TEntity;
   El: TElement;
   i,j: integer;
   LC: TListColumn;
   PM: TRegisterPropertyModel;
   S: string;
   Envelope: boolean;
   VS: TViewStyle;
   RS: string;
procedure AddEToList;
var
   ONumber: integer;
begin
   L.Data := E;
   L.Caption := E.Name;
   CheckClassModel(E);
   if ClassModel <> nil then
   begin
      L.ImageIndex := ClassModel.ImageIndex;
      //L.StateIndex := ClassModel.ImageIndex;
   end;
   // podmnka ... nsledujc provdt jen v ppad detailu
   L.SubItems.Add(IntToStr(E.Count));
   if E is TEnvelopeEntity then
   begin
      ONumber := 0;
      if TEnvelopeEntity(E).Owner <> nil then ONumber := 1;
   end
   else ONumber := E.OwnersCount;
   L.SubItems.Add(IntToStr(ONumber));
end;

procedure BeginUpd;
begin
   ListView.Items.BeginUpdate;
   VS := ListView.ViewStyle;
   if VS = vsReport then ListView.ViewStyle := vsList;
end;

procedure EndUpd;
begin
   if ListView.ViewStyle <> VS then ListView.ViewStyle := VS;
   ListView.Items.EndUpdate;
end;

begin
   N := Node;//TreeView.Selected;
   El := TElement(N.Data);
   // !!!!!!!!!!!!!!!!
   if W <> nil then
   begin
      W.C.ActiveElement := El;
      W.Run;
   end;
   if GetOrbisClass(El.ClassName) <> nil then RS := ' R+'
                                         else RS := ' R-';
   StatusBar.Panels.Items[0].Text := El.Name + ' ('+El.ClassName + ')'+RS;
   if DontShowList then exit;
   if not ShowOwners then
   begin
      BeginUpd;
      ListView.Items.Clear;
      CheckClassModel(El);
      if not CheckSubClassModel then
      begin
         Memo.Lines.Clear;
         ListView.Columns.Clear;
         LC := ListView.Columns.Add;
         LC.Caption := 'Name';
         LC.Width := -2;
         if SubClassModel <> nil then
         begin
            for i := 0 to SubClassModel.PropertyModels.LastIndex do
            begin
               PM := TRegisterPropertyModel(SubClassModel.PropertyModels.Elements[i]);
               Memo.Lines.Add(PM.Name);
               LC := ListView.Columns.Add;
               LC.Caption := PM.Alias;
               LC.Width := -2;
            end;
         end;
      end;
      //ListView.Items.EndUpdate;
      if not (TElement(El) is TEntity) then
      begin
         EndUpd;
         exit;
      end;
      E := TEntity(El);
      if E.NoElements then
      begin
         EndUpd;
         exit;
      end;

      //LIstView.Items.BeginUpdate;
      for i := 0 to E.LastIndex do
      begin
         //Caption := IntToStr(i);
         L := ListView.Items.Add;
         El := E.Elements[i];
         L.Data := El;
         L.Caption := El.Name;
         CheckClassModel(El);
         if ClassModel <> nil then
         begin
            L.ImageIndex := ClassModel.ImageIndex;
            //L.StateIndex := ClassModel.ImageIndex;
         end;
         // podmnka ... nsledujc provdt jen v ppad detailu
         for j := 0 to Memo.Lines.Count - 1 do
         begin
            S := El.GetPropertyStringByName(Memo.Lines.Strings[j]);
            L.SubItems.Add(S);
         end;
      end;
      EndUpd;
   end
   else
   begin
      Envelope := (El is TEnvelopeEntity);
      if Envelope  then
      begin
         if TEnvelopeEntity(El).Owner = nil then exit;
      end
      else if El.NoOwners then exit;

      BeginUpd;
      ListView.Items.Clear;
      if not Envelope then
      begin
         for i := 0 to El.OwnersCount-1 do
         begin
            L := ListView.Items.Add;
            E := El.Owners[i];
            AddEToList;
         end;
      end
      else
      begin
            L := ListView.Items.Add;
            E := TEnvelopeEntity(El).Owner;
            AddEToList;
      end;
      EndUpd;
   end;
end;

procedure TExplorerForm.FormCreate(Sender: TObject);
var
   BM: TBitmap;
   i: integer;
   S: string;
begin
   //SS := World.ApplicExeName;
   //SS := ExtractFilePath(SS);
   //SS := CheckBackSlash(SS);
   for i := 0 to World.SystemRegister.Images.LastIndex do
   begin
      BM := TBitmap.Create;
      S := World.SystemRegister.Images.Strings[i];
      //if pos(':',S) = 0 then S := SS + S;
      S := GetAbsolutePath(S);
      //ShowMessage(S);
      try
         BM.LoadFromFile(S);
      except
      end;
      ImageList.AddMasked(BM,clGray);
   end;
end;

procedure TExplorerForm.ExploreFromHereClick(Sender: TObject);
begin
   Tool.PostMsg(tm_Explore, World.MainTool, TElement(TreeView.Selected.Data),DontKill);
end;

procedure TExplorerForm.ExploreOwnersClick(Sender: TObject);
begin
   Tool.PostMsg(tm_ExploreOwners, World.MainTool, TElement(TreeView.Selected.Data),DontKill);
end;

procedure TExplorerForm.BitBtn1Click(Sender: TObject);
begin
   if DontShowList then DontShowList := false
                   else DontShowList := true;
end;

procedure TExplorerForm.Zobraz1Click(Sender: TObject);
begin
   Tool.PostMsg(tm_AddView, World.MainTool, TElement(TreeView.Selected.Data),DontKill);
end;

procedure TExplorerForm.b_WClick(Sender: TObject);
begin
   if W = nil then
   begin
      Application.CreateForm(TWatchForm,W);
      W.C.ActiveElement := TElement(TreeView.Selected.Data);
   end
   else
   begin
      try
         W.Free;
         W := nil;
      except
      end;
   end;
end;

procedure TExplorerForm.EditPropertiesClick(Sender: TObject);
begin
   Tool.PostMsg(tm_EditElement, World.MainTool, TElement(TreeView.Selected.Data),DontKill);
end;

procedure TExplorerForm.Save1Click(Sender: TObject);
begin
   Tool.PostMsg(tm_SaveElement, World.MainTool, TElement(TreeView.Selected.Data),DontKill);
end;

procedure TExplorerForm.Load1Click(Sender: TObject);
begin
   Tool.PostMsg(tm_LoadElement, World.MainTool, TElement(TreeView.Selected.Data),DontKill);
end;
(*
procedure TExplorerForm.TreeViewDragOver(Sender, Source: TObject; X,
  Y: Integer; State: TDragState; var Accept: Boolean);
var
  aEl: TElement;
begin
  Accept:= false;
  if not (Source = Self.TreeView) then
     Exit;
//  Accept:= True;
  aEl:= TElement(TreeView.Selected.Data);
  Accept:= aEl is TElement;
end;

procedure TExplorerForm.TreeViewDragDrop(Sender, Source: TObject; X, Y: Integer);
var
  aNode: TTreeNode;
  aEl: TElement;
  aGoal: TElement;
begin
  aNode:= TreeView.DropTarget;
  if aNode = nil then
     Exit;
  if aNode.Data = nil then
     Exit;
  aEl:= TElement(TreeView.Selected.Data);
  if not (aEl is TElement) then
     Exit;
  aGoal:= TElement(aNode.Data);
  if not (aGoal is TElement) then
     Exit;

  if MessageDlg('Zdroj: ' + aEl.Name + #13 + #10 + 'Cil: ' + aGoal.Name,
                   mtConfirmation, [mbYes, mbNo], 0) = mrYes then
  begin
    if (aGoal is TElementIndex) and (aEl is TEntity) then
    begin
      TElementIndex(aGoal).Entity:= TEntity(aEl);
      Exit;
    end;
    if not (aGoal is TTool) then
       Exit;

    if aEl is TTool then
      TTool(aGoal).Input:= TTool(aEl).Output
    else
      TTool(aGoal).Input:= aEl;

    TTool(aGoal).DoIt;
  end;
end;
*)
procedure TExplorerForm.TreeViewDragOver(Sender, Source: TObject; X,
  Y: Integer; State: TDragState; var Accept: Boolean);
begin
  Accept:= True;
end;

procedure TExplorerForm.OIL1Click(Sender: TObject);
begin
   Tool.PostMsg(tm_OIL,nil,TElement(TreeView.Selected.Data),false);
end;

end.
