unit t_ElementKiller;

interface
uses
   Elements,
   t_CmdLn,
   t_GraphicEditors,
   Tools;

type

TAllElementKiller = class(TTool)
private
   FElementClass: TElementClass;
   FConfirmText: string;
   FGlobalPath: string;
   FOIL: TCommandLine;
   procedure SetConfirmText(const Value: string);
   procedure SetElementClassName(const Value: string);
   procedure SetGlobalPath(const Value: string);
   procedure SetElementClass(const Value: TElementClass);
    function GetElementClassName: string;
protected
   property  ElementClass: TElementClass read FElementClass write SetElementClass;
public
   procedure DoIt; override;
   function FindGlobalEntity: TEntity;
   property OIL: TCommandLine read FOIL;
published
   property  GlobalPath: string read FGlobalPath write SetGlobalPath;
   property  ElementClassName: string read GetElementClassName write SetElementClassName;
   property  ConfirmText: string read FConfirmText write SetConfirmText;
end;

TSelectElementKiller = class(TGraphicEditor)
end;

implementation
uses
   Dialogs,
   SysUtils,
   e_Class,
   g_Elems,
   m_Orbis,
   t_ConfirmDialog;

{ TAllElementKiller }

procedure TAllElementKiller.DoIt;
var
   E: TEntity;
   El: TElement;
   i,c: integer;
begin
   E := FindGlobalEntity;
   if E = nil then
   begin
      ShowMessage('Nemohu nalzt cestu: '+GlobalPath);
      exit;
   end;
   if not Confirm(ConfirmText) then exit;
   c := 0;
   for i := E.LastIndex downTo 0 do
   begin
      El := E.Elements[i];
      if El is ElementClass then
      begin
         E.FreeElement(El);
         //El.Free;
         inc(c);
      end;
   end;
   if c > 0 then
   begin
      if E is TRectEntity then
         TRectEntity(E).ResetAndUpdate;
      PostMsgToOwners(tm_BreakAndReview,nil,false);
      PostMsg(tm_ModelChanged,nil,nil,false);
   end;
   ShowMessage('Bylo odstranno celkem '+intToStr(c)+' element');
end;

function TAllElementKiller.FindGlobalEntity: TEntity;
var
   E: TElement;
begin
   result := nil;
   OIL.DoIt;
   {$IFDEF WIN64}
   E := TElement(StrToInt64Def(OIL.ResultLine, 0));
   {$ELSE}
   E := TElement(StrToIntDef(OIL.ResultLine, 0));
   {$ENDIF}
   if not (E is TEntity) then exit;
   result := TEntity(E);
end;

function TAllElementKiller.GetElementClassName: string;
begin
   result := '';
   if ElementClass = nil then exit;
   result := ElementClass.ClassName;
end;

procedure TAllElementKiller.SetConfirmText(const Value: string);
begin
   FConfirmText := Value;
end;

procedure TAllElementKiller.SetElementClass(const Value: TElementClass);
begin
   FElementClass := Value;
end;

procedure TAllElementKiller.SetElementClassName(const Value: string);
begin
   ElementClass := GetOrbisClass(Value);
   if ElementClass = nil then
      PostTextMsg(tm_AddToProtocol,'ElementKiller - unknown class: '+Value,nil);
end;

procedure TAllElementKiller.SetGlobalPath(const Value: string);
begin
   FGlobalPath := Value;
   if Value <> '' then
   begin
      if OIL = nil then
         ExchangeFieldElement(TElement(FOIL), TCommandLine.Create);
      OIL.Line := Value;
   end;
end;

INITIALIZATION
   RegisterOrbisClass(TAllElementKiller);
   RegisterOrbisClass(TSelectElementKiller);
END.
