unit t_SpatialAnalysis;

interface
uses
   e_Geometry,
   Tools;

type

TSpatialAnalysisTool = class(TTechnology)
  private
    FInside: boolean;
    FOver: boolean;
    FNotEnds: boolean;
    FTolerance: double;
    procedure SetInside(const Value: boolean);
    procedure SetOver(const Value: boolean);
    procedure SetNotEnds(const Value: boolean);
    procedure SetTolerance(const Value: double);
public
   function Relate(Geometry1, Geometry2: TGeometry): boolean;
   function Relate_Point_Point(Point1, Point2: TPointElement): boolean;
   function Relate_Point_Line(Point: TPointElement; Line: TPolyLineElement): boolean;
   function Relate_Point_Area(Point: TPointElement; Area: TAreaElement): boolean;
   function Relate_Line_Line(Line1, Line2: TPolyLineElement): boolean;
   function Relate_Line_Area(Line: TPolyLineElement; Area: TAreaElement): boolean;
   function Relate_Area_Area(Area1, Area2: TAreaElement): boolean;
   function Relate_2PolyElements(P1,P2: TPolyElement; P1Closed, P2Closed: boolean; Ends: boolean): boolean;
   property Inside: boolean read FInside write SetInside;
   property Over: boolean read FOver write SetOver;
   property NotEnds: boolean read FNotEnds write SetNotEnds; // ignorovat relace koncu linii
   property Tolerance: double read FTolerance write SetTolerance;
end;

implementation
uses
   Elements,
   e_Class,
   G_Elems,
   MathLib;

{ TSpatialAnalysisTool }

function TSpatialAnalysisTool.Relate(Geometry1,Geometry2: TGeometry): boolean;
begin
   result := false;
   Inside := false;
   Over := false;
   if (Geometry1 = nil) or (Geometry2 = nil) then exit;
   if Geometry1 is TPointElement then
   begin
      if Geometry2 is TPointElement then result := Relate_Point_Point(TPointElement(Geometry1),TPointElement(Geometry2))
      else
      if Geometry2 is TPolyLineElement then result := Relate_Point_Line(TPointElement(Geometry1),TPolyLineElement(Geometry2))
      else
      if Geometry2 is TAreaElement then result := Relate_Point_Area(TPointElement(Geometry1),TAreaElement(Geometry2))
   end
   else
   if Geometry1 is TpolyLineElement then
   begin
      if Geometry2 is TPointElement then result := Relate_Point_Line(TPointElement(Geometry2),TPolyLineElement(Geometry1))
      else
      if Geometry2 is TPolyLineElement then result := Relate_Line_Line(TPolyLineElement(Geometry1),TPolyLineElement(Geometry2))
      else
      if Geometry2 is TAreaElement then result := Relate_Line_Area(TPolyLineElement(Geometry1),TAreaElement(Geometry2))
   end
   else
   if Geometry1 is TAreaElement then
   begin
      if Geometry2 is TPointElement then result := Relate_Point_Area(TPointElement(Geometry2),TAreaElement(Geometry1))
      else
      if Geometry2 is TPolyLineElement then result := Relate_Line_Area(TPolyLineElement(Geometry2),TAreaElement(Geometry1))
      else
      if Geometry2 is TAreaElement then result := Relate_Area_Area(TAreaElement(Geometry1),TAreaElement(Geometry2))
   end;
end;

function TSpatialAnalysisTool.Relate_2PolyElements(P1, P2: TPolyElement;
  P1Closed, P2Closed: boolean; Ends: boolean): boolean;
var
   i, j: integer;
   XA1,YA1,XA2,YA2,XB1,YB1,XB2,YB2: double;
   A1,A2,B1,B2: boolean;  // info, zda bod lezi na druhe usecce

   function SameEnds: boolean;
   begin
      result := ((XA1 = XB1) and (YA1 = YB1)) or ((XA2 = XB1) and (YA2 = YB1)) or
                ((XA1 = XB2) and (YA1 = YB2)) or ((XA2 = XB2) and (YA2 = YB2));
   end;
begin
   result := false;
   if (P1=nil) or (P2=nil) then exit;
   if (P1.Count < 2) or (P2.Count < 2) then exit;
   for i := 0 to P1.LastIndex - 1 do
   begin
      for j := 0 to P2.LastIndex - 1 do
      begin
         XA1 := P1.Xi[i];
         YA1 := P1.Yi[i];
         XA2 := P1.Xi[i+1];
         YA2 := P1.Yi[i+1];
         XB1 := P2.Xi[j];
         YB1 := P2.Yi[j];
         XB2 := P2.Xi[j+1];
         YB2 := P2.Yi[j+1];
         if EqualPoints(XA1,YA1,XA2,YA2) or EqualPoints(XB1,YB1,XB2,YB2) then
            Continue; // jedna z usecek neni usecka, ale 2 stejne body
         A1 := BodLeziNaUsecce(XA1,YA1,XB1,YB1,XB2,YB2);
         A2 := BodLeziNaUsecce(XA2,YA2,XB1,YB1,XB2,YB2);
         B1 := BodLeziNaUsecce(XB1,YB1,XA1,YA1,XA2,YA2);
         B2 := BodLeziNaUsecce(XB2,YB2,XA1,YA1,XA2,YA2);
         if ((A1 and A2) or (B1 and B2)) or                    // jedna us. lezi na druhe
             ((A1 or A2) and (B1 or B2) and not SameEnds) then // prekryv us.
         begin
            if Ends then
            begin
               result := true;  // akceptujeme-li konce na us, trefa
               Inside := true;
               exit;
            end
            else
               Continue;        // jinak na to prdime
         end;
         if not(A1 or A2 or B1 or B2) then    // pokud neni dotyk usecek....
            result := ProtnutyUsecky(XA1,YA1,XA2,YA2,XB1,YB1,XB2,YB2);
         if result then exit;
      end;
   end;
   if P1Closed then
   begin
      for i := 0 to P2.LastIndex - 1 do
      begin
         XA1 := P1.Xi[P1.LastIndex];
         YA1 := P1.Yi[P1.LastIndex];
         XA2 := P1.Xi[0];
         YA2 := P1.Yi[0];
         XB1 := P2.Xi[i];
         YB1 := P2.Yi[i];
         XB2 := P2.Xi[i+1];
         YB2 := P2.Yi[i+1];
         if EqualPoints(XA1,YA1,XA2,YA2) or EqualPoints(XB1,YB1,XB2,YB2) then
            Continue; // jedna z usecek neni usecka, ale 2 stejne body
         A1 := BodLeziNaUsecce(XA1,YA1,XB1,YB1,XB2,YB2);
         A2 := BodLeziNaUsecce(XA2,YA2,XB1,YB1,XB2,YB2);
         B1 := BodLeziNaUsecce(XB1,YB1,XA1,YA1,XA2,YA2);
         B2 := BodLeziNaUsecce(XB2,YB2,XA1,YA1,XA2,YA2);
         if ((A1 and A2) or (B1 and B2)) or                    // jedna us. lezi na druhe
             ((A1 or A2) and (B1 or B2) and not SameEnds) then // prekryv us.
         begin
            if Ends then
            begin
               result := true;  // akceptujeme-li konce na us, trefa
               Inside := true;
               exit;
            end
            else
               Continue;        // jinak na to prdime
         end;
         if not(A1 or A2 or B1 or B2) then    // pokud neni dotyk usecek....
            result := ProtnutyUsecky(XA1,YA1,XA2,YA2,XB1,YB1,XB2,YB2);
         if result then exit;
      end;
   end;
   if P2Closed then
   begin
      for i := 0 to P1.LastIndex - 1 do
      begin
         XA1 := P2.Xi[P2.LastIndex];
         YA1 := P2.Yi[P2.LastIndex];
         XA2 := P2.Xi[0];
         YA2 := P2.Yi[0];
         XB1 := P1.Xi[i];
         YB1 := P1.Yi[i];
         XB2 := P1.Xi[i+1];
         YB2 := P1.Yi[i+1];
         if EqualPoints(XA1,YA1,XA2,YA2) or EqualPoints(XB1,YB1,XB2,YB2) then
            Continue; // jedna z usecek neni usecka, ale 2 stejne body
         A1 := BodLeziNaUsecce(XA1,YA1,XB1,YB1,XB2,YB2);
         A2 := BodLeziNaUsecce(XA2,YA2,XB1,YB1,XB2,YB2);
         B1 := BodLeziNaUsecce(XB1,YB1,XA1,YA1,XA2,YA2);
         B2 := BodLeziNaUsecce(XB2,YB2,XA1,YA1,XA2,YA2);
         if ((A1 and A2) or (B1 and B2)) or                    // jedna us. lezi na druhe
             ((A1 or A2) and (B1 or B2) and not SameEnds) then // prekryv us.
         begin
            if Ends then
            begin
               result := true;  // akceptujeme-li konce na us, trefa
               Inside := true;
               exit;
            end
            else
               Continue;        // jinak na to prdime
         end;
         if not(A1 or A2 or B1 or B2) then    // pokud neni dotyk usecek....
            result := ProtnutyUsecky(XA1,YA1,XA2,YA2,XB1,YB1,XB2,YB2);
         if result then exit;
      end;
   end;
end;

function TSpatialAnalysisTool.Relate_Area_Area(Area1, Area2: TAreaElement): boolean;
var
   anAre1,anAre2,Are1,Are2,A1,A2: TAreaElement;
   j: integer;

   function Relate_A1_Are2: boolean;
   var
      i: integer;
      KillA2: boolean;
      Crd,LastCrd: TCoordinate;
   begin
      result := false;
      KillA2 := false;
      // nejsou obdelniky mimo?
      if not A1.IsOverRegion(Are2.MinX, Are2.MinY,Are2.MaxX,Are2.MaxY) then exit;

      result := A1.IsHitByXY(Are2.X, Are2.Y, 0);   // Nelezi nahodou popis.bod Are2 v A1?
      if result then exit;                     // lezi ... hotovo.

      if Are2 is TPolyLineAreaElement then
      begin
         A2 := TPolyLineAreaElement(Are2).CreateAreaElement;
         KillA2 := True;
      end
      else
         A2 := Are2;

      result := A2.IsHitByXY(A1.X, A1.Y, 0);   // Nelezi nahodou popis.bod A1 v A2?
      if result then                   // ... asi ne
      begin
         if KillA2 then
            A2.Free;
         exit;
      end;
      // zkusime vrcholy hranice A1 v plose A2
      LastCrd := nil;
      for i := 0 to A1.LastIndex do
      begin
         crd := A1.GetOrientedCoordElement(i, true);
         if crd = nil then continue;
         result := A2.IsHit(crd,0);
         if result then
         begin
            if KillA2 then
               A2.Free;
            exit;
         end;
         if i > 0 then  // A co takhle stred usecky ?
         if A2.IsHitByXY((LastCrd.X+Crd.X)/2,(LastCrd.Y+Crd.Y)/2, 0) then
         begin
            if KillA2 then
               A2.Free;
            result := true;
            exit;
         end;
         LastCrd := Crd;
      end;
      // no nic ...zkusime vrcholy hranice A2 v plose A1
      LastCrd := nil;
      for i := 0 to A2.LastIndex do
      begin
         crd := A2.GetOrientedCoordElement(i, true);
         if crd = nil then continue;
         result := A1.IsHit(crd,0);
         if result then
         begin
            if KillA2 then
               A2.Free;
            exit;
         end;
         if i > 0 then  // A co takhle stred usecky ?
         if A1.IsHitByXY((LastCrd.X+Crd.X)/2,(LastCrd.Y+Crd.Y)/2, 0) then
         begin
            if KillA2 then
               A2.Free;
            result := true;
            exit;
         end;
         LastCrd := Crd;
      end;

      // nic nam nevyslo, tak mozna pruseciky hranic?
      result := Relate_2PolyElements(A1, A2, true, true, false);
      if KillA2 then
         A2.Free;
   end;

   function Relate_Are1_anAre2: boolean;
   var
      i: integer;
      KillA1: boolean;
   begin
      result := false;
      KillA1 := false;
      if Are1 is TPolyLineAreaElement then
      begin
         A1 := TPolyLineAreaElement(Are1).CreateAreaElement;
         KillA1 := True;
      end
      else
         A1 := Are1;
      if anAre2 is TMultiAreaElement  then
      begin
         for i := 0 to anAre2.LastIndex do
         begin
            Are2 := TMultiAreaElement(anAre2).AreaElement[i];
            result := Relate_A1_Are2;
            if result then break;
         end;
      end
      else
         result := Relate_A1_Are2;
      if KillA1 then
         A1.Free;
   end;
begin
   result := false;
   anAre1 := Area1;
   anAre2 := Area2;
   Are1 := Area1;
   Are2 := Area2;
   if anAre1 is TMultiAreaElement  then
   begin
      for j := 0 to anAre1.LastIndex do
      begin
         Are1 := TMultiAreaElement(anAre1).AreaElement[j];
         result := Relate_Are1_anAre2;
         if result then exit;
      end;
   end
   else
      result := Relate_Are1_anAre2;
end;

function TSpatialAnalysisTool.Relate_Line_Area(Line: TPolyLineElement;
  Area: TAreaElement): boolean;
var
   j: integer;
   Are,anAre: TAreaElement;
   Crd,LastCrd: TCoordinate;
   E: TElement;

   function Relate_Line_SimpleArea: boolean;
   var
      i: integer;
      KillArea: boolean;
   begin
      KillArea := false;
      // pokud je plocha liniov, udlm si z n oby Areu (ta se pak mus zruit)
      if Area is TpolyLineAreaElement then
      begin
         Are := TPolyLineAreaElement(Area).CreateAreaElement;
         KillArea := true;
      end
      else
         Are := Area;
      // kontrola, zda nkter z bod linie nele v ploe
      LastCrd := nil;
      for i := 0 to Line.LastIndex do
      begin
         E := Line.elements[i];
         if not (E is TCoordinate) then Continue;
         Crd := TCoordinate(E);
         if Are.IsHit(Crd, 0) then
         begin
            if KillArea then
               Are.Free;
            Inside := true;
            result := true;
            exit;
         end;
         if i > 0 then  // A co takhle stred usecky ?
         if Are.IsHitByXY((LastCrd.X+Crd.X)/2,(LastCrd.Y+Crd.Y)/2, 0) then
         begin
            if KillArea then
               Are.Free;
            Inside := true;
            result := true;
            exit;
         end;
         LastCrd := Crd;
      end;
      // dobr, body ani stedy v ploe nele, zkusme prseky seek
      result := Relate_2PolyElements(Line, Are, false, true, true);
      if KillArea then
         Are.Free;
   end;
begin
   result := false;
   Inside := false;
   Over := false;
   // nejdve kontrola opsanch obdlnk
   if not Line.IsOverRegion(Area.MinX,Area.MinY,Area.MaxX,Area.MaxY) then exit;
   // MultiArea se porovn po kouskch
   anAre := Area;
   if anAre is TMultiAreaElement then
   begin
      for j := 0 to anAre.LastIndex do
      begin
         Area := TMultiAreaElement(anAre).AreaElement[j];
         result := Relate_Line_SimpleArea;
         if result then exit;
      end;
   end
   else
      result := Relate_Line_SimpleArea;
end;

function TSpatialAnalysisTool.Relate_Line_Line(Line1, Line2: TPolyLineElement): boolean;
var
   i,j,li,lj: integer;
   //dx,dy,
   aDistance: double;
   X,Y,X1,Y1,X2,Y2,X3,Y3,pX,pY: double;
   End1,End2,IsIn: boolean;
begin
   result := false;
   X1 := 0;
   Y1 := 0;
   X3 := 0;
   Y3 := 0;
   // 1) Body
   li := Line1.LastIndex;
   for i := 0 to li do
   begin
      X := Line1.Xi[i];
      Y := Line1.Yi[i];
      End1 := (i = 0) or (i = li);
      lj := Line2.LastIndex;
      for j := 0 to lj do
      begin
         End2 := (j = 0) or (j = lj);
         if NotEnds and End1 and End2 then
            Continue;                // konce zasadne nekontrolujem, proste sou nam fuk
         X1 := Line2.Xi[i];
         Y1 := Line2.Yi[i];
         if (abs(X1-X) <= Tolerance) and (abs(Y1-Y) <= Tolerance) then
         begin
            aDistance := Distance(X,Y,X1,Y1);
            if (aDistance <= Tolerance) then
            begin
               Result := True;  // ha - body na sobe
               exit;
            end;
         end;
      end;
   end;
   // 2) body a usecky
   for i := 0 to li do
   begin
      X := Line1.Xi[i];
      Y := Line1.Yi[i];
      End1 := i = li;
      if not End1 then
      begin
         X1 := Line1.Xi[i+1];
         Y1 := Line1.Yi[i+1];
      end;
      lj := Line2.LastIndex;
      for j := 0 to lj do
      begin
         X2 := Line2.Xi[j];
         Y2 := Line2.Yi[j];
         End2 := j = lj;
         if not End2 then      // bod L2 v us L1
         begin
            X3 := Line1.Xi[j+1];
            Y3 := Line1.Yi[j+1];
            if RectanglesOverlay(X2,Y2,X3,Y3,X-1,Y-1,X+1,Y+1,Tolerance) then
            begin
               DejVztahyBod_Usecka( X, Y, X1, Y1, X2, Y2, IsIn, pX, pY, aDistance);
               if IsIn and (aDistance <= Tolerance) then
               begin
                  result := true;
                  exit;
               end;
            end;
         end;
         if not End1 then      // bod L1 v us L2
         begin
            if RectanglesOverlay(X,Y,X1,Y1,X2-1,Y2-1,X2+1,Y2+1,Tolerance) then
            begin
               DejVztahyBod_Usecka( X2, Y2, X, Y, X1, Y1, IsIn, pX, pY, aDistance);
               if IsIn and (aDistance <= Tolerance) then
               begin
                  result := true;
                  exit;
               end;
            end;
         end;
         if not (End1 or End2) then  // usecky vzajemne mezi sebou
         begin
            result := ProtnutyUsecky(X,Y,X1,Y1,X2,Y2,X3,Y3);
            if result then
               exit;
         end;
      end;
   end;
end;

function TSpatialAnalysisTool.Relate_Point_Area(Point: TPointElement;
  Area: TAreaElement): boolean;
begin
   result := false
end;

function TSpatialAnalysisTool.Relate_Point_Line(Point: TPointElement;
  Line: TPolyLineElement): boolean;
begin
   result := false
end;

function TSpatialAnalysisTool.Relate_Point_Point(Point1,
  Point2: TPointElement): boolean;
begin
   result := false
end;

procedure TSpatialAnalysisTool.SetInside(const Value: boolean);
begin
  FInside := Value;
end;

procedure TSpatialAnalysisTool.SetNotEnds(const Value: boolean);
begin
  FNotEnds := Value;
end;

procedure TSpatialAnalysisTool.SetOver(const Value: boolean);
begin
  FOver := Value;
end;

procedure TSpatialAnalysisTool.SetTolerance(const Value: double);
begin
  FTolerance := Value;
end;

INITIALIZATION
   RegisterOrbisClass(TSpatialAnalysisTool);
END.
