unit WGS84toGaussKrueger;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants,
  System.Classes, Vcl.Graphics,
  Vcl.Dialogs, Vcl.StdCtrls, System.Math;

procedure WGS84toGK(lat, lon: double; var northing, easting: double);
procedure GKtoWGS84(northing, easting: double; var lat, lon: double);

implementation

const
  flattening = 1 / 299.1528128;
//  flattening = 1 / 298.257223563;

const
  equatorialRadius = 6377397.155;
//  equatorialRadius = 6378137;

const
  centralMeridian = 16.33333333333333;

const
  scale = 1;

const
  falseNorthing = -5000000;

const
  falseEasting = 0;

const
  GradSize = 0.0174532925199433;



procedure GKtoWGS84(northing, easting: double; var lat, lon: double);
var
  e2, n, rectifyingRadius, xi,eta,delta1,delta2,delta3,delta4,AStar, BStar, CStar, DStar, phi, lambda, lambda0, deltaLambda,
    phiStar, xiPrim, etaPrim: double;
begin
  e2 := flattening * (2 - flattening); // e2: first eccentricity squared
  n := flattening / (2 - flattening); // n: 3rd flattening
  rectifyingRadius := equatorialRadius / (1 + n) *
    (1 + 0.25 * System.Math.Power(n, 2) + 0.015625 * System.Math.Power(n, 4));
  xi := (northing - falseNorthing) / (scale * rectifyingRadius);
  eta := (easting - falseEasting) / (scale * rectifyingRadius);

  delta1 := 1 / 2.0 * n - 2 / 3.0 * System.Math.Power(n, 2) + 37 / 96.0 *
    System.Math.Power(n, 3) - 1 / 360.0 * System.Math.Power(n, 4);
  delta2 := 1 / 48.0 * System.Math.Power(n, 2) + 1 / 15.0 *
    System.Math.Power(n, 3) - 437 / 1440.0 * System.Math.Power(n, 4);
  delta3 := 17 / 480.0 * System.Math.Power(n, 3) - 37 / 840.0 *
    System.Math.Power(n, 4);
  delta4 := 4397 / 161280.0 * System.Math.Power(n, 4);

  xiPrim := xi - delta1 * sin(2 * xi) * cosh(2 * eta) - delta2 * sin(4 * xi) *
    cosh(4 * eta) - delta3 * sin(6 * xi) * cosh(6 * eta) - delta4 * sin(8 * xi)
    * cosh(8 * eta);
  etaPrim := eta - delta1 * cos(2 * xi) * sinh(2 * eta) - delta2 * cos(4 * xi) *
    sinh(4 * eta) - delta3 * cos(6 * xi) * sinh(6 * eta) - delta4 * cos(8 * xi)
    * sinh(8 * eta);

  phiStar := arcsin(sin(xiPrim) / cosh(etaPrim)); // Conformal latitude
  deltaLambda := arctan(sinh(etaPrim) / cos(xiPrim));

  AStar := e2 + System.Math.Power(e2, 2) + System.Math.Power(e2, 3) +
    System.Math.Power(e2, 4);
  BStar := (7 * System.Math.Power(e2, 2) + 17 * System.Math.Power(e2, 3) + 30 *
    System.Math.Power(e2, 4)) / -6;
  CStar := (224 * System.Math.Power(e2, 3) + 889 * System.Math.Power(e2,
    4)) / 120;
  DStar := (4279 * System.Math.Power(e2, 4)) / -1260;

  phi := phiStar + sin(phiStar) * cos(phiStar) *
    (AStar + BStar * System.Math.Power(sin(phiStar), 2) + CStar *
    System.Math.Power(sin(phiStar), 4) + DStar *
    System.Math.Power(sin(phiStar), 6));

  // phi: latitude in radians, lambda: longitude in radians
  // Return latitude and longitude as degrees
  lat := phi * 180 / Pi;
  lon := centralMeridian + deltaLambda * 180 / Pi;


//  Edit1.Text := FloatToStr(lat);
//  Edit2.Text := FloatToStr(lon);

end;

procedure WGS84toGK(lat, lon: double; var northing, easting: double);
var
  e2, n, rectifyingRadius, A, B, C, D, phi, lambda, lambda0, deltaLambda,
    phiStar, xiPrim, etaPrim, beta1, beta2, beta3, beta4: double;
begin
  e2 := flattening * (2 - flattening); // e2: first eccentricity squared
  n := flattening / (2 - flattening); // n: 3rd flattening
  rectifyingRadius := equatorialRadius / (1 + n) *
    (1 + 0.25 * System.Math.Power(n, 2) + 0.015625 * System.Math.Power(n, 4));

  A := e2;
  B := (5 * System.Math.Power(e2, 2) - System.Math.Power(e2, 3)) / 6.0;
  C := (104 * System.Math.Power(e2, 3) - 45 * System.Math.Power(e2, 4)) / 120.0;
  D := (1237 * System.Math.Power(e2, 4)) / 1260.0;

  // Latitude and longitude are expected to be given in degrees
  // phi and lambda: latitude and longitude in radians
  phi := lat * Pi / 180;
  lambda := lon * Pi / 180;
  lambda0 := centralMeridian * Pi / 180;

  // deltaLambda: longitude relative to the central meridian
  deltaLambda := lambda - lambda0;

  // phiStar: conformal latitude
  phiStar := phi - sin(phi) * cos(phi) * (A +
     B * System.Math.Power(sin(phi), 2) +
     C * System.Math.Power(sin(phi), 4) +
     D * System.Math.Power(sin(phi), 6));

  xiPrim := arctan(tan(phiStar) / cos(deltaLambda));
  etaPrim := arctanh(cos(phiStar) * sin(deltaLambda));

  beta1 := 1 / 2.0 * n - 2 / 3.0 * System.Math.Power(n, 2) + 5 / 16.0 *
    System.Math.Power(n, 3) + 41 / 180.0 * System.Math.Power(n, 4);
  beta2 := 13 / 48.0 * System.Math.Power(n, 2) - 3 / 5.0 *
    System.Math.Power(n, 3) + 557 / 1440.0 * System.Math.Power(n, 4);
  beta3 := 61 / 240.0 * System.Math.Power(n, 3) - 103 / 140.0 *
    System.Math.Power(n, 4);
  beta4 := 49561 / 161280.0 * System.Math.Power(n, 4);

  northing := falseNorthing + scale * rectifyingRadius * (xiPrim +
     beta1 * sin(2 * xiPrim) * cosh(2 * etaPrim) +
     beta2 * sin(4 * xiPrim) * cosh(4 * etaPrim) +
     beta3 * sin(6 * xiPrim) * cosh(6 * etaPrim) +
     beta4 * sin(8 * xiPrim) * cosh(8 * etaPrim)) + 65.506;

  easting := falseEasting + scale * rectifyingRadius * (etaPrim +
     beta1 * cos(2 * xiPrim) * sinh(2 * etaPrim) +
     beta2 * cos(4 * xiPrim) * sinh(4 * etaPrim) +
     beta3 * cos(6 * xiPrim) * sinh(6 * etaPrim) +
     beta4 * cos(8 * xiPrim) * sinh(8 * etaPrim))+ 85.203;

//  Edit3.Text := FloatToStr(northing);
//  Edit4.Text := FloatToStr(easting);

end;

end.
