unit E_dynm;

interface
uses
// KERNEL
   Elements;

type

TDynamicModel = class(TOwnerEntity)
private
   FActualModel: TElement;
   FActualIndex: longInt;
   FActive: boolean;
   procedure SetActualModel(aValue: TElement);
   procedure SetActualIndex(aValue: longInt);
protected
   function Get(Index: longint): TElement; override;
public
   constructor Create; override;
   destructor  Destroy; override;
published
   property   ActualModel: TElement read FActualModel write SetActualModel;
   property   ActualIndex: longInt read FActualIndex write SetActualIndex;
   property   Active: boolean read FActive write FActive;
end;

implementation

uses
// KERNEL
   e_Class,
   Worlds,
// GISLAYER
   e_LayScr,
   T_LMCre;

{ TDynamicModel }

procedure TDynamicModel.SetActualModel(aValue: TElement);
var
   Reg: TElement;
begin
   if FActualModel <> nil then
   begin
      FActualModel.Free;
      FActualModel := nil;
      Reg := World.ObjectModel.FindElementName('TRegister');
      if Reg <> nil then Reg.Free;
   end;
   FActualModel := aValue;
end;

procedure TDynamicModel.SetActualIndex(aValue: longInt);
var
   MS: TModelScript;
   MBSC: TModelByScriptCreator;
begin
   if aValue <> FActualIndex then
   begin
      ActualModel := nil;
      MS := TModelScript(inherited Get(aValue));
      MBSC := TModelByScriptCreator.Create;
      MBSC.Input := MS;
      MBSC.DoIt;
      ActualModel := MBSC.Output;
      MBSC.Free;
   end;
end;

function TDynamicModel.Get(Index: longint): TElement;
begin
   if not Active then
   begin
      result := inherited Get(Index);
   end
   else
   begin
      ActualIndex := Index;
      result := ActualModel;
   end;
end;

constructor TDynamicModel.Create;
begin
   inherited Create;
   FActualIndex := -1;
end;

destructor  TDynamicModel.Destroy;
begin
   ActualModel := nil;
   inherited Destroy;
end;


BEGIN
   RegisterOrbisClass(TDynamicModel);
END.
