unit f_coltab;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Grids, StdCtrls;

type
  TColorTableForm = class(TForm)
    Grid: TStringGrid;
    btAdd: TButton;
    btDel: TButton;
    btInsert: TButton;
    procedure GridMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure GridDrawCell(Sender: TObject; ACol, ARow: Integer;
      Rect: TRect; State: TGridDrawState);
    procedure btAddClick(Sender: TObject);
    procedure GridDblClick(Sender: TObject);
    procedure btDelClick(Sender: TObject);
    procedure btInsertClick(Sender: TObject);
  private
    FCol, FRow: integer;
    FItem: string;
    function GetColor(aRow: integer): integer;
  public
    { Public declarations }
  published
    property Item: string read FItem write FItem;
  end;

var
  ColorTableForm: TColorTableForm;

implementation

{$R *.DFM}

procedure TColorTableForm.GridMouseMove(Sender: TObject; Shift: TShiftState;
  X, Y: Integer);
begin
  Grid.MouseToCell(X, Y, FCol, FRow);
end;


function TColorTableForm.GetColor(aRow: integer): integer;
var
  aStr: string;
  aVal, aCode: integer;
begin
  aStr:= Grid.Cells[1, aRow];
  val(aStr, aVal, aCode);
  if aCode <> 0 then
     Result:= -1
  else
     Result:= aVal
end;



procedure TColorTableForm.GridDrawCell(Sender: TObject; ACol, ARow: Integer;
  Rect: TRect; State: TGridDrawState);
var
  aVal: integer;
begin
  if aRow = 0 then Exit;
  if aCol = 0 then Exit;

  if aCol = 1 then
  begin
    aVal:= GetColor(aRow);
    if aVal = -1 then
       Exit;
    Grid.Canvas.Brush.Color:= aVal;
    Grid.Canvas.Rectangle(Rect.Left+2, Rect.Top+2, Rect.Right-2, Rect.Bottom-2);
  end;
end;


procedure TColorTableForm.btAddClick(Sender: TObject);
begin
  Grid.RowCount:=Grid.RowCount + 1;
end;


procedure TColorTableForm.GridDblClick(Sender: TObject);
var
  aCD: TColorDialog;
begin
  if FRow = 0 then Exit;
  if FCol = 0 then Exit;

  aCD:= TColorDialog.Create(Self);
  if GetColor(FRow) > -1 then
     aCD.Color:= GetColor(FRow);

  if aCD.Execute then
  begin
    Grid.Cells[FCol, FRow]:= IntToStr(aCD.Color);
  end;
  aCD.Free;
end;


procedure TColorTableForm.btDelClick(Sender: TObject);
begin
  Grid.Cols[0].Delete(Grid.Row);
  Grid.Cols[1].Delete(Grid.Row);
end;

procedure TColorTableForm.btInsertClick(Sender: TObject);
begin
  Grid.Cols[0].Insert(Grid.Row, '');
  Grid.Cols[1].Insert(Grid.Row, '');
end;

end.
