unit T_LocateIntoPSK;

interface
uses
// KERNEL
  Elements,
// GLIB
  g_elems,
// SPACE
  e_Space,
  T_HelSel;

type

TLocateIntoPSK = class(THeleBasic)
private
  FTemp: TEntity;
  FLayerName: string;
  FFeatureName: string;
    FPropertyClassName: string;
    FPropertyClass: TElementClass;
  function GetPSK(aObj: TSpaceEntity;  var JeToPSK: boolean): TRectEntity;
  procedure AddToPSK(JeToPSK: Boolean; aPSK: TRectEntity; aObj: TSpaceEntity);
    procedure SetPropertyClass(const Value: TElementClass);
    procedure SetPropertyClassName(const Value: string);
public
  procedure DoIt; override;
  procedure ReadLayer;
  procedure SetupProperties(aObj: TSpaceEntity);
  property  PropertyClass: TElementClass read FPropertyClass write SetPropertyClass;
published
  property LayerName: string read FLayerName write FLayerName;
  property FeatureName: string read FFeatureName write FFeatureName;
  property PropertyClassName: string read  FPropertyClassName write SetPropertyClassName;
end;

implementation
uses
  Dialogs, SysUtils,
// KERNEL
  e_Class,
  e_Spaprp,
  m_Orbis,
// OIL
  T_SelectByName,
  T_Helhie,
  u_Props;

procedure TLocateIntoPSK.AddToPSK(JeToPSK: Boolean; aPSK: TRectEntity; aObj: TSpaceEntity);
var
  aLayer: TRectEntity;
begin
  if JeToPSK then
     aLayer:= TRectEntity(aPSK.FindElementName(LayerName+'_'+FeatureName))
  else
     aLayer:= TRectEntity(aPSK.FindElementName(LayerName+'_'+FeatureName + ' NEZAAZENO'));

  if aLayer = nil then
  begin
    aLayer:= TRectEntity.Create;
    if JeToPSK then
       aLayer.Name:= LayerName+'_'+FeatureName
    else
       aLayer.Name:= LayerName+'_'+FeatureName + ' NEZAAZENO';

    aPSK.AddElement(aLayer);
  end;
  aLayer.AddAndUpdateRect(aObj);
  aPSK.UpdateRect(aObj);
end;



function TLocateIntoPSK.GetPSK(aObj: TSpaceEntity;  var JeToPSK: boolean): TRectEntity;
var
  HelSku: THeleSkupina;
  aLHC, aOdd, aDil, aPor, aPSK: string;

  procedure AddToProtocol(aStr: string);
  begin
     Protocol:= 'Zaazovn ' + FeatureName + ' .' + IntToStr(aObj.SpaceId) +
                aStr
  end;

begin
  Result:= nil;
  JeToPSK:= false;

  HelSku:= THeleSkupina.Create;
  HelSku.Input:= Hierarchy;
  HelSku.X:= aObj.X;
  HelSku.Y:= aObj.Y;
  HelSku.DoIt;
  if not HelSku.Found then
  begin
    HelSku.Free;
    Exit;
  end;

  aLHC := HelSku.LHC;
  aOdd := HelSku.Odd;
  aDIL := HelSku.Dil;
  aPOR := HelSku.Por;
  aPSK := HelSku.PSK;

  result:= HelSku.PSKObject;
  JeToPSK:= Result <> nil;
  HelSku.Free;
end;



procedure TLocateIntoPSK.ReadLayer;
var
  I: integer;
  aHier: TEntity;
  aObj: TSpaceEntity;
  aPSK: TRectEntity;
  JeToPSK: Boolean;
  SBN: TSelectByName;
  aSlu: TRectEntity;
begin
  FTemp:= FindLayer(LayerName, FeatureName);

  if not(FTemp is TSpaceEntityString) then exit;
  if FTemp.Count = 0 then exit;

  aHier:= Hierarchy;
  if aHier =nil then Exit;

  for I:= 0 to FTemp.Count-1 do
  begin
    aObj:= TSpaceEntity(FTemp.elements[I]);
    if not (TObject(aObj) is TSpaceEntity) then
       continue;
    SetupProperties(aObj);
    aPSK:= GetPSK(aObj, JeToPSK);
    if aPSK = nil then
    begin
      //ShowMessage('Pro ' + FeatureName + ' c.' + IntToStr(aObj.SpaceId) + ' nen nalezena PSK');
      PostTextmsg(tm_AddToProtocol,'Pro '+LayerName + '_' + FeatureName + ' c.' + IntToStr(aObj.SpaceId) + ' nen nalezena PSK',nil);
      continue;
    end
    else
      AddToPSK(JeToPSK, aPSK, aObj)
  end;

  aSlu:= CreateDataRectEntitu(LayerName+'_'+FeatureName);

  SBN:= TSelectByName.Create;
  SBN.CaseSensitive:= False;
  SBN.SelectedName:=LayerName+'_'+FeatureName;
  SBN.Input := aHier;
  SBN.Partialy:= false;
  SBN.Output:= aSlu;
  SBN.DoIt;
  SBN.Free;
end;


procedure TLocateIntoPSK.DoIt;
begin
   ReadLayer;
end;



procedure TLocateIntoPSK.SetPropertyClass(const Value: TElementClass);
begin
   FPropertyClass := Value;
end;

procedure TLocateIntoPSK.SetPropertyClassName(const Value: string);
begin
   if Value = '' then exit;
   FPropertyClassName := Value;
   PropertyClass := GetOrbisClass(Value);
   if PropertyClass = nil then
      PostTextMsg(tm_AddToProtocol, Name + ': Class '+ Value+' not found!',nil);
end;

procedure TLocateIntoPSK.SetupProperties(aObj: TSpaceEntity);
var
   aP: TSpaceEntityProps;
   aStart: integer;
   i: integer;
begin
   if PropertyClass = nil then exit;
   if not (aObj.Properties is PropertyClass) then
   begin
      aP:= TSpaceEntityProps(PropertyClass.Create);
      if aObj.Properties = nil then
      begin
         aObj.Properties := aP;
         exit;
      end;
      aStart:= GetPropIndex(aP,'LastIndex')+1;
      for I:= aStart to GetPropCount(aP)-1 do { misto startovani od nuly }
      begin
         SetPropString(aP, I, aObj.Properties.GetPropertyStringByName(GetPropName(aP, I)));
      end;
      aObj.Properties := aP;
   end;
end;

INITIALIZATION
  RegisterOrbisClass(TLocateIntoPSK);
END.

