unit e_LHCModel;

interface
uses
   e_GeometryElement,
   e_People,
   e_KatastrModel,
   Elements,
   g_Elems;

type
Tgis_ForestUnit = class;
Tgis_LHC = class;
Tgis_LS = class;
Tgis_Oddeleni = class;
Tgis_Dilec = class;
Tgis_Porost = class;
Tgis_PorostniSkupina = class;
Tgis_Etaz = class;

Tgis_Taxator = class(TPersonElement)
public
    procedure  AddElement(aElement: TElement); override;
    procedure  ResetFU(aFU: Tgis_ForestUnit);
end;

Tgis_Taxator_Collection = class(TPerson_Collection);

Tgis_ForestModelCollection = class(TOwnerEntity{TAreaCollection})
private
   function GetForestUnit(aIndex: integer): Tgis_ForestUnit;
protected
   procedure SetName(aName: string); override;
public
   property ForestUnit[aIndex: integer]: Tgis_ForestUnit read GetForestUnit;
end;

   Tgis_ForestModelCollectionClass = class of Tgis_ForestModelCollection;

Tgis_LSCollection = class(Tgis_ForestModelCollection)
protected
   function  GetName: string; override;
end;


Tgis_OddeleniCollection = class(Tgis_ForestModelCollection)
protected
   function  GetName: string; override;
end;

Tgis_DilecCollection = class(Tgis_ForestModelCollection)
protected
   function  GetName: string; override;
end;

Tgis_PorostCollection = class(Tgis_ForestModelCollection)
protected
   function  GetName: string; override;
end;

Tgis_PorostniSkupinaCollection = class(Tgis_ForestModelCollection)
protected
   function  GetName: string; override;
end;

Tgis_TezbaCollection = class(Tgis_ForestModelCollection)
protected
   function  GetName: string; override;
end;

Tgis_UserCollection = class(Tgis_ForestModelCollection)
protected
   function  GetName: string; override;
public
   function  FindUserID: integer;
end;


Tgis_ForestUnit = class(TPropertyGeometryElement)
  private
    function GetTaxatorPIN: integer;
    procedure Check_Taxator(const Value: Tgis_Taxator);
protected
   FZnacka: integer;
   FBarva: integer;
   procedure SetBarva(const Value: integer);
   procedure SetZnacka(const Value: integer);
   function Get_Dilec: Tgis_Dilec; virtual;
   function Get_LHC: Tgis_LHC; virtual;
   function Get_LS: Tgis_LS; virtual;
   function Get_Oddeleni: Tgis_Oddeleni; virtual;
   function Get_Porost: Tgis_Porost; virtual;
   function GetPlocha: double; virtual;
   function GetVymera: double; virtual;
   procedure Set_Porost(const Value: Tgis_Porost); virtual;
   procedure SetPlocha(const Value: double); virtual;
   procedure SetVymera(const Value: double); virtual;
   function  Get_Taxator: Tgis_Taxator; virtual;
   procedure Set_Taxator(const Value: Tgis_Taxator); virtual;
   function  GetTaxator: string;
   function GetTaxPro: string; virtual;
   procedure Reset_Taxator(aTaxator: Tgis_Taxator; FUClass: TClass); virtual;
    function GetMultiElement: boolean; override;
    function GetMultiElementClass: TElementClass; override;
public
   property _LHC: Tgis_LHC read Get_LHC;
   property _LS: Tgis_LS read Get_LS;
   property _Oddeleni: Tgis_Oddeleni read Get_Oddeleni;
   property _Dilec: Tgis_Dilec read Get_Dilec;
   property _Porost: Tgis_Porost read Get_Porost write Set_Porost;
   property _Taxator: Tgis_Taxator read Get_Taxator write Set_Taxator;
   property Taxator: string read GetTaxator;
   property TaxPro: string read GetTaxPro;
   property TaxatorPIN: integer read GetTaxatorPIN;
   property Plocha: double read GetPlocha write SetPlocha;
   property Vymera: double read GetVymera write SetVymera;
published
   property Barva: integer read FBarva write SetBarva;
   property Znacka: integer read FZnacka write SetZnacka;
end;

   Tgis_ForestUnitClass = class of Tgis_ForestUnit;

Tgis_ItemsForestUnit = class(Tgis_ForestUnit)
private
    FItems: Tgis_ForestModelCollection;
    procedure SetItems(const Value: Tgis_ForestModelCollection);
    function GetItems: Tgis_ForestModelCollection;
    function GetNoItems: boolean;
    procedure SetUpdateAll(const Value: boolean);
protected
   function GetItemsClass: Tgis_ForestModelCollectionClass; virtual;
   function GetItemClass: Tgis_ForestUnitClass; virtual;
   function GetPlocha: double; override;
   function GetVymera: double; override;
   procedure Set_Taxator(const Value: Tgis_Taxator); override;
   procedure Reset_Taxator(aTaxator: Tgis_Taxator; FUClass: TClass); override;
public
   function GetItemName(const Value: string): Tgis_ForestUnit;
   function CreateItemName(const Value: string): Tgis_ForestUnit;
   procedure CheckItems;
   property ItemClass: Tgis_ForestUnitClass read GetItemClass;
published
   property Items: Tgis_ForestModelCollection read GetItems write SetItems stored false;
   property NoItems: boolean read GetNoItems stored false;
   property UpdateAll: boolean write SetUpdateAll stored false;
end;

Tgis_CisloForestUnit = class(Tgis_ItemsForestUnit)
protected
   FCislo: integer;
   procedure SetCislo(const Value: integer);
   function  GetName: string; override;
   procedure SetName(aName: string); override;
public
   function GetItemCislo(const Value: integer): Tgis_CisloForestUnit;
   function CreateItemCislo(const Value: integer): Tgis_CisloForestUnit;
published
   property Cislo: integer read FCislo write SetCislo;
end;

Tgis_LHC = class(Tgis_CisloForestUnit)
private
    FNazev: string;
    FLHC_LCR: integer;
    FROK: integer;
    FLHC_TAX: string;
    procedure SetNazev(const Value: string);
    procedure SetLHC_LCR(const Value: integer);
    procedure SetROK(const Value: integer);
    procedure SetLHC_TAX(const Value: string);
protected
   function GetItemsClass: Tgis_ForestModelCollectionClass; override;
   function GetItemClass: Tgis_ForestUnitClass; override;
published
   property Plocha stored false;
   property Vymera stored false;
   property Nazev: string read FNazev write SetNazev;
   property LHC_KOD: integer read FCislo write SetCislo stored false;
   property LHC_LCR: integer read FLHC_LCR write SetLHC_LCR;
   property ROK: integer read FROK write SetROK;
   property LHC_TAX: string read FLHC_TAX write SetLHC_TAX;
end;

Tgis_LS = class(Tgis_CisloForestUnit)
private
    FNazev: string;
    FLS: integer;
    FLS_NAZ: string;
    FREV_TYP: integer;
    FUSEK: integer;
    FKR_NAZ: string;
    FPoznamka: string;
    FUSEK_NAZ: string;
    FKR: integer;
    procedure SetNazev(const Value: string);
    procedure SetLS(const Value: integer);
    procedure SetLS_NAZ(const Value: string);
    function GetLHC: integer;
    function GetLHC_LCR: integer;
    procedure SetREV_TYP(const Value: integer);
    procedure SetKR(const Value: integer);
    procedure SetKR_NAZ(const Value: string);
    procedure SetPoznamka(const Value: string);
    procedure SetUSEK(const Value: integer);
    procedure SetUSEK_NAZ(const Value: string);
protected
   function GetItemsClass: Tgis_ForestModelCollectionClass; override;
   function GetItemClass: Tgis_ForestUnitClass; override;
   function Get_LHC: Tgis_LHC; override;
published
   property _LHC stored false;
   property Plocha stored false;
   property Vymera stored false;
   property Nazev: string read FNazev write SetNazev;
   property LHC: integer read GetLHC stored false;
   property LHC_LCR: integer read GetLHC_LCR stored false;
   property LS_KOD: integer read FCislo write SetCislo stored false;
   property LS: integer read FLS write SetLS;
   property LS_NAZ: string read FLS_NAZ write SetLS_NAZ;
   property REVIR: integer read FCislo write SetCislo stored false;
   property REV_NAZ: string read FNazev write SetNazev stored false;
   property REV_TYP: integer read FREV_TYP write SetREV_TYP;
   property KR: integer read FKR write SetKR;
   property KR_NAZ: string read FKR_NAZ write SetKR_NAZ;
   property USEK: integer read FUSEK write SetUSEK;
   property USEK_NAZ: string read FUSEK_NAZ write SetUSEK_NAZ;
   property Poznamka: string read FPoznamka write SetPoznamka;
end;

Tgis_Oddeleni = class(Tgis_CisloForestUnit)
  private
    function GetLHC: integer;
    function GetLS_KOD: integer;
    function GetLHC_LCR: integer;
protected
   function GetItemsClass: Tgis_ForestModelCollectionClass; override;
   function GetItemClass: Tgis_ForestUnitClass; override;
   function Get_LHC: Tgis_LHC; override;
   function Get_LS: Tgis_LS; override;
   function GetTaxPro: string; override;
published
   property _LHC stored false;
   property _LS stored false;
   property Plocha stored false;
   property Vymera stored false;
   property LHC: integer read GetLHC stored false;
   property LHC_LCR: integer read GetLHC_LCR stored false;
   property LS_KOD: integer read GetLS_KOD stored false;
   property Oddeleni: integer read FCislo write SetCislo stored false;
   property ODD: integer read FCislo write SetCislo stored false;
   property _Taxator stored false;
   property Taxator stored false;
   property TaxatorPIN stored false;
   property TaxPro stored false;
end;

Tgis_Dilec = class(Tgis_ItemsForestUnit)
  private
    function GetLS_Kod: integer;
    function GetOddeleni: SmallInt;
    procedure SetLS_KOD(const Value: integer);
    procedure SetOddeleni(const Value: SmallInt);
    function GetLHC: integer;
    procedure SetLHC(const Value: integer);
    function GetDilec: char;
    procedure SetDilec(const Value: char);
protected
   function GetItemsClass: Tgis_ForestModelCollectionClass; override;
   function GetItemClass: Tgis_ForestUnitClass; override;
   function Get_LHC: Tgis_LHC; override;
   function Get_LS: Tgis_LS; override;
   function Get_Oddeleni: Tgis_Oddeleni; override;
   function Get_Taxator: Tgis_Taxator; override;
   function GetTaxPro: string; override;
published
   property _LHC stored false;
   property _LS stored false;
   property _Oddeleni stored false;
   property _Taxator stored false;
   property Plocha stored false;
   property Vymera stored false;
   property LS_Kod: integer read GetLS_Kod write SetLS_KOD stored false;
   property LHC: integer read GetLHC write SetLHC stored false;
   property Oddeleni: SmallInt read GetOddeleni write SetOddeleni stored false;
   property Dilec: char read GetDilec write SetDilec stored false;
   property DIL: char read GetDilec write SetDilec stored false;
   property Taxator stored false;
   property TaxatorPIN stored false;
   property TaxPro stored false;
end;

Tgis_Porost = class(Tgis_ItemsForestUnit)
  private
    FKVAL_P: byte;
    FMultiKatastr: boolean;
    FKATS: string;
    FZVL_STATUT: string;
    FKAT_Poznamka: string;
    function GetDilec: char;
    function GetKATPAR_KOD: byte;
    function GetKU_KOD: integer;
    function GetKVAL_P: byte;
    function GetLHC: integer;
    function GetLS_Kod: integer;
    function GetOddeleni: SmallInt;
    function GetPorost: char;
    function GetSK_PARC: integer;
    function GetSkupinaParcel: Tgis_SkupinaParcel;
    function GetVLASTNIK: string;
    procedure SetDilec(const Value: char);
    procedure SetKATPAR_KOD(const Value: byte);
    procedure SetKU_KOD(const Value: integer);
    procedure SetKVAL_P(const Value: byte);
    procedure SetLHC(const Value: integer);
    procedure SetLS_KOD(const Value: integer);
    procedure SetOddeleni(const Value: SmallInt);
    procedure SetPorost(const Value: char);
    procedure SetSK_PARC(const Value: integer);
    procedure SetSkupinaParcel(const Value: Tgis_SkupinaParcel);
    procedure SetVLASTNIK(const Value: string);
    procedure SetMultiKatastr(const Value: boolean);
    function GetLHC_LCR: integer;
    procedure SetKATS(const Value: string);
    function GetKAT: string;
    procedure SetKAT(const Value: string);
    procedure SetNextKAT(const Value: string);
    procedure SetKAT_Poznamka(const Value: string);
    procedure SetNextZVL_STATUT(const Value: string);
    procedure SetZVL_STATUT(const Value: string);
    function GetMajitel: Tgis_Majitel;
    function GetOLH: Tgis_OLH;
protected
   function GetItemsClass: Tgis_ForestModelCollectionClass; override;
   function GetItemClass: Tgis_ForestUnitClass; override;
   function Get_LHC: Tgis_LHC; override;
   function Get_LS: Tgis_LS; override;
   function Get_Oddeleni: Tgis_Oddeleni; override;
   function Get_Dilec: Tgis_Dilec; override;
   function GetPlocha: double; override;
   function GetVymera: double; override;
   function Get_Taxator: Tgis_Taxator; override;
   function GetTaxPro: string; override;
published
   property _LHC stored false;
   property _LS stored false;
   property _Oddeleni stored false;
   property _Dilec stored false;
   property _Taxator stored false;
   property Plocha stored false;
   property Vymera stored false; //!!!!!!!!!!
   property SkupinaParcel: Tgis_SkupinaParcel read GetSkupinaParcel write SetSkupinaParcel stored false;
   property Majitel: Tgis_Majitel read GetMajitel stored false;
   property OLH: Tgis_OLH read GetOLH stored false;
   property LHC: integer read GetLHC write SetLHC stored false;
   property LHC_LCR: integer read GetLHC_LCR stored false;
   property LS_Kod: integer read GetLS_Kod write SetLS_KOD stored false;
   property Oddeleni: SmallInt read GetOddeleni write SetOddeleni stored false;
   property Dilec: char read GetDilec write SetDilec stored false;
   property Porost: char read GetPorost write SetPorost stored false;
   property POR: char read GetPorost write SetPorost stored false;
   property SK_PARC: integer read GetSK_PARC write SetSK_PARC stored false;
   property KVAL_P: byte read GetKVAL_P write SetKVAL_P;
   property KU_KOD: integer read GetKU_KOD write SetKU_KOD stored false;
   property VLASTNIK: string read GetVLASTNIK write SetVLASTNIK stored false;
   property KATPAR_KOD: byte read GetKATPAR_KOD write SetKATPAR_KOD stored false;
   property KATS: string read FKATS write SetKATS;
   property KAT: string read GetKAT write SetKAT stored false;
   property NextKAT: string write SetNextKAT stored false;
   property ZVL_STATUT: string read FZVL_STATUT write SetZVL_STATUT;
   property NextZVL_STATUT: string write SetNextZVL_STATUT stored false;
   property KAT_Poznamka: string read FKAT_Poznamka write SetKAT_Poznamka;
   property MultiKatastr: boolean read FMultiKatastr write SetMultiKatastr;
   property Taxator stored false;
   property TaxatorPIN stored false;
   property TaxPro stored false;
end;

Tgis_PorostniSkupina = class(Tgis_ForestUnit)
private
   FVymera: double;
   FVEK_ST: byte;
   FKVAL_P: byte;
    FLVS: byte;
   //FVlastnik: integer;
    function GetPureSkupina: string;
    function GetLHC_LCR: integer;
    function GetSameKVAL_P: boolean;
    function GetMAJ_DRUH: integer;
    function GetMAJ_KOD: string;
    function GetKU_KOD_: integer;
    function GetKU_Nazev: string;
    function GetMajitel: Tgis_Majitel;
    function GetMAJ_Adresa: string;
    function GetMAJ_Name: string;
    function GetORP: integer;
    function GetORP_Nazev: string;
    function GetKAT: string;
    function GetKATS: string;
    procedure SetKAT(const Value: string);
    procedure SetKATS(const Value: string);
    procedure SetNextKAT(const Value: string);
    function GetKAT_Poznamka: string;
    function GetZVL_STATUT: string;
    procedure SetKAT_Poznamka(const Value: string);
    procedure SetNextZVL_STATUT(const Value: string);
    procedure SetZVL_STATUT(const Value: string);
    function GetKraj: string;
    function GetKraj_Nazev: string;
    function GetLV: integer;
    function GetLVc: integer;
    procedure SetLVS(const Value: byte);
    function GetOLH: Tgis_OLH;
    function GetOLH_Adresa: string;
    function GetOLH_ID: string;
    function GetOLH_Name: string;
    function GetOLH_PR_TIT: string;
protected
   function GetGroup: integer; override;
   function Get_LHC: Tgis_LHC; override;
   function Get_LS: Tgis_LS; override;
   function Get_Oddeleni: Tgis_Oddeleni; override;
   function Get_Dilec: Tgis_Dilec; override;
   function Get_Porost: Tgis_Porost; override;
   function GetPlocha: double; override;
   function GetVymera: double; override;
   procedure Set_Porost(const Value: Tgis_Porost); override;
   procedure SetVymera(const Value: double); override;
    function GetDilec: char; virtual;
    function GetKATPAR_KOD: byte; virtual;
    function GetKU_KOD: integer; virtual;
    function GetKVAL_P: byte; virtual;
    function GetLHC: integer; virtual;
    function GetLS_Kod: integer; virtual;
    function GetOddeleni: SmallInt; virtual;
    function GetPorost: char; virtual;
    function GetSK_PARC: integer; virtual;
    function GetSkupinaParcel: Tgis_SkupinaParcel;
    function GetSkupinaParcelDetail: Tgis_SkupinaParcel;
    function GetSkupina: string; virtual;
    function GetVLASTNIK: string; virtual;
    procedure SetDilec(const Value: char); virtual;
    procedure SetKATPAR_KOD(const Value: byte); virtual;
    procedure SetKU_KOD(const Value: integer); virtual;
    procedure SetKVAL_P(const Value: byte); virtual;
    procedure SetLHC(const Value: integer); virtual;
    procedure SetLS_KOD(const Value: integer); virtual;
    procedure SetOddeleni(const Value: SmallInt); virtual;
    procedure SetPorost(const Value: char); virtual;
    procedure SetSK_PARC(const Value: integer); virtual;
    procedure SetSkupinaParcel(const Value: Tgis_SkupinaParcel);
    procedure SetSkupina(const Value: string); virtual;
    procedure SetVEK_ST(const Value: byte); virtual;
    procedure SetVLASTNIK(const Value: string); virtual;
    function Get_Taxator: Tgis_Taxator; override;
    function GetTaxPro: string; override;
public
    function  GetPropertyStringByName(AName: string): string; override;
published
   property _LHC stored false;
   property _LS stored false;
   property _Oddeleni stored false;
   property _Dilec stored false;
   property _Porost stored false;
   property _Taxator stored false;
   property Plocha stored false;
   property Vymera stored true;
   property SkupinaParcel: Tgis_SkupinaParcel read GetSkupinaParcel write SetSkupinaParcel stored false;
   property Majitel: Tgis_Majitel read GetMajitel stored false;
   property OLH: Tgis_OLH read GetOLH stored false;
   property LHC: integer read GetLHC write SetLHC stored false;
   property LHC_LCR: integer read GetLHC_LCR stored false;
   property LS_Kod: integer read GetLS_Kod write SetLS_KOD stored false;
   property Oddeleni: SmallInt read GetOddeleni write SetOddeleni stored false;
   property Dilec: char read GetDilec write SetDilec stored false;
   property Porost: char read GetPorost write SetPorost stored false;
   property SK_PARC: integer read GetSK_PARC write SetSK_PARC stored false;
   property KVAL_P: byte read GetKVAL_P write SetKVAL_P stored true;
   property SameKVAL_P: boolean read GetSameKVAL_P stored false;
   property KU_KOD: integer read GetKU_KOD write SetKU_KOD stored false;
   property KU_KOD_: integer read GetKU_KOD_ stored false;
   property KU_Nazev: string read GetKU_Nazev stored false;
   property ORP: integer read GetORP stored false;
   property ORP_Nazev: string read GetORP_Nazev stored false;
   property Kraj: string read GetKraj stored false;
   property Kraj_Nazev: string read GetKraj_Nazev stored false;
   property KATPAR_KOD: byte read GetKATPAR_KOD write SetKATPAR_KOD stored false;
   property Skupina: string read GetSkupina write SetSkupina stored false;
   property PSK: string read GetSkupina write SetSkupina stored false;
   property BZL: string read GetSkupina write SetSkupina stored false;
   property OP: string read GetSkupina write SetSkupina stored false;
   property JP: string read GetSkupina write SetSkupina stored false;
   property PureSkupina: string read GetPureSkupina stored false;
   property VEK_ST: byte read FVEK_ST write SetVEK_ST stored true;
   property KOD_SRF: integer read FZnacka write SetZnacka stored false;
   property VLASTNIK: string read GetVLASTNIK write SetVLASTNIK;
   property MAJ_DRUH: integer read GetMAJ_DRUH stored false;
   property MAJ_KOD: string read GetMAJ_KOD stored false;
   property MAJ_Name: string read GetMAJ_Name stored false;
   property MAJ_Adresa: string read GetMAJ_Adresa stored false;
   property OLH_ID: string read GetOLH_ID stored false;
   property OLH_PR_TIT: string read GetOLH_PR_TIT stored false;
   property OLH_Name: string read GetOLH_Name stored false;
   property OLH_Adresa: string read GetOLH_Adresa stored false;
   property KATS: string read GetKATS write SetKATS;
   property KAT: string read GetKAT write SetKAT stored false;
   property NextKAT: string write SetNextKAT stored false;
   property ZVL_STATUT: string read GetZVL_STATUT write SetZVL_STATUT;
   property NextZVL_STATUT: string write SetNextZVL_STATUT stored false;
   property KAT_Poznamka: string read GetKAT_Poznamka write SetKAT_Poznamka;
   property LV: integer read GetLV stored false;
   property LVc: integer read GetLVc stored false;
   property Taxator stored false;
   property TaxatorPIN stored false;
   property TaxPro stored false;
   property LVS: byte read FLVS write SetLVS;
end;

Tgis_PorostniSkupinaAlone = class(Tgis_PorostniSkupina)
private
   FLHC:integer;
   FLS_Kod: integer;
   FOddeleni: SmallInt;
   FDilec: char;
   FPorost: char;
   FSK_PARC: integer;
   FKU_KOD: integer;
   FVLASTNIK: string;
   FKATPAR_KOD: byte;
protected
    function GetDilec: char; override;
    function GetKATPAR_KOD: byte; override;
    function GetKU_KOD: integer; override;
    function GetLHC: integer; override;
    function GetLS_Kod: integer; override;
    function GetOddeleni: SmallInt; override;
    function GetPorost: char; override;
    function GetSK_PARC: integer; override;
    function GetVLASTNIK: string; override;
    procedure SetDilec(const Value: char); override;
    procedure SetKATPAR_KOD(const Value: byte); override;
    procedure SetKU_KOD(const Value: integer); override;
    procedure SetLHC(const Value: integer); override;
    procedure SetLS_KOD(const Value: integer); override;
    procedure SetOddeleni(const Value: SmallInt); override;
    procedure SetPorost(const Value: char); override;
    procedure SetSK_PARC(const Value: integer); override;
    procedure SetVLASTNIK(const Value: string); override;
published
   property LHC stored true;
   property LS_Kod stored true;
   property Oddeleni stored true;
   property Dilec stored true;
   property Porost stored true;
   property SK_PARC stored true;
   property KU_KOD stored true;
   property VLASTNIK stored true;
   property KATPAR_KOD stored true;
end;

Tgis_Etaz = class(Tgis_ForestUnit);

Tgis_ForestModel = class(TRectEntity)
  private
    FActualDilec: Tgis_Dilec;
    FActualLHC: Tgis_LHC;
    FActualLS: Tgis_LS;
    FActualOddeleni: Tgis_Oddeleni;
    FActualPorost: Tgis_Porost;
    FActualSkupina: Tgis_PorostniSkupina;
    FAutoCreate: boolean;
    function GetDilec_Name: string;
    function GetLHC_Cislo: integer;
    function GetLS_Cislo: integer;
    function GetOddeleni_Cislo: integer;
    function GetPorost_Name: string;
    function GetSkupina_Name: string;
    procedure SetActualLHC(const Value: Tgis_LHC);
    procedure SetActualLS(const Value: Tgis_LS);
    procedure SetActualOddeleni(const Value: Tgis_Oddeleni);
    procedure SetActualDilec(const Value: Tgis_Dilec);
    procedure SetActualPorost(const Value: Tgis_Porost);
    procedure SetActualSkupina(const Value: Tgis_PorostniSkupina);
    procedure SetLHC_Cislo(const Value: integer);
    procedure SetLS_Cislo(const Value: integer);
    procedure SetOddeleni_Cislo(const Value: integer);
    procedure SetDilec_Name(const Value: string);
    procedure SetPorost_Name(const Value: string);
    procedure SetSkupina_Name(const Value: string);
    procedure SetUpdateAll(const Value: boolean);
    procedure SetSort(const Value: boolean);
    procedure SetAutoCreate(const Value: boolean);
    function GetLHC_ROK: integer;
public
   function GetLHCCislo(const Value: integer): Tgis_LHC;
   function GetLSCislo(const Value: integer): Tgis_LS;
   function GetOddeleniCislo(const Value: integer): Tgis_Oddeleni;
   function GetDilecName(const Value: string): Tgis_Dilec;
   function GetPorostName(const Value: string): Tgis_Porost;
   function GetSkupinaName(const Value: string): Tgis_PorostniSkupina;
   function CreateLHCCislo(const Value: integer): Tgis_LHC;
   function CreateLSCislo(const Value: integer): Tgis_LS;
   function CreateOddeleniCislo(const Value: integer): Tgis_Oddeleni;
   function CreateDilecName(const Value: string): Tgis_Dilec;
   function CreatePorostName(const Value: string): Tgis_Porost;
   function CreateSkupinaName(const Value: string): Tgis_PorostniSkupina;
   function CreatePorost(aLHC,aLS,aOddeleni: integer; aDilec,aPorost: string): Tgis_Porost;
   function CreateSkupina(aLHC,aLS,aOddeleni: integer; aDilec,aPorost,aSkupina: string): Tgis_PorostniSkupina;
   function GetOddCount: integer;
   function FindOdds: TEntity;
published
   property ActualLHC: Tgis_LHC read FActualLHC write SetActualLHC stored false;
   property ActualLS: Tgis_LS read FActualLS write SetActualLS stored false;
   property ActualOddeleni: Tgis_Oddeleni read FActualOddeleni write SetActualOddeleni stored false;
   property ActualDilec: Tgis_Dilec read FActualDilec write SetActualDilec stored false;
   property ActualPorost: Tgis_Porost read FActualPorost write SetActualPorost stored false;
   property ActualSkupina: Tgis_PorostniSkupina read FActualSkupina write SetActualSkupina stored false;
   property LHC_Cislo: integer read GetLHC_Cislo write SetLHC_Cislo stored false;
   property LHC_ROK: integer read GetLHC_ROK stored false;
   property LS_Cislo: integer read GetLS_Cislo write SetLS_Cislo stored false;
   property Oddeleni_Cislo: integer read GetOddeleni_Cislo write SetOddeleni_Cislo stored false;
   property Dilec_Name: string read GetDilec_Name write SetDilec_Name stored false;
   property Porost_Name: string read GetPorost_Name write SetPorost_Name stored false;
   property Skupina_Name: string read GetSkupina_Name write SetSkupina_Name stored false;
   property UpdateAll: boolean write SetUpdateAll stored false;
   property Sort: boolean write SetSort stored false;
   property AutoCreate: boolean read FAutoCreate write SetAutoCreate stored false;
end;



implementation
uses
   SysUtils,
   e_Geometry,
   e_Class,
   e_LHCModelSupport,
   t_Sort,
   u_String;

{ Tgis_ForestUnit }

procedure Tgis_ForestUnit.Check_Taxator(const Value: Tgis_Taxator);
begin

end;

function Tgis_ForestUnit.GetMultiElement: boolean;
begin
   result := true;
end;

function Tgis_ForestUnit.GetMultiElementClass: TElementClass;
begin
   result := TMulTiAreaElement;
end;

function Tgis_ForestUnit.GetPlocha: double;
begin
   result := 0;
end;

function Tgis_ForestUnit.GetTaxator: string;
var
   aT: Tgis_Taxator;
begin
   result := '-';
   aT := _Taxator;
   if aT <> nil then
      result := aT.Name;
end;

function Tgis_ForestUnit.GetTaxatorPIN: integer;
var
   aT: Tgis_Taxator;
begin
   result := 0;
   aT := _Taxator;
   if aT <> nil then
      result := aT.PIN;
end;

function Tgis_ForestUnit.GetTaxPro: string;
begin
   result := '?';
end;

function Tgis_ForestUnit.GetVymera: double;
begin
   result := 0;
end;

function Tgis_ForestUnit.Get_Dilec: Tgis_Dilec;
begin
   result := nil;
end;

function Tgis_ForestUnit.Get_LHC: Tgis_LHC;
begin
   result := nil;
end;

function Tgis_ForestUnit.Get_LS: Tgis_LS;
begin
   result := nil;
end;

function Tgis_ForestUnit.Get_Oddeleni: Tgis_Oddeleni;
begin
   result := nil;
end;

function Tgis_ForestUnit.Get_Porost: Tgis_Porost;
begin
   result := nil;
end;

function Tgis_ForestUnit.Get_Taxator: Tgis_Taxator;
begin
   result := Tgis_Taxator(FindOwnerClass(TGis_Taxator));
end;

procedure Tgis_ForestUnit.Reset_Taxator(aTaxator: Tgis_Taxator; FUClass: TClass);
var
   aT: Tgis_Taxator;
begin
   if ClassType = FUClass then
   begin
      aT := _Taxator;
      if aT = nil then
         _Taxator := aTaxator;
   end;
end;

procedure Tgis_ForestUnit.SetBarva(const Value: integer);
begin
  FBarva := Value;
end;

procedure Tgis_ForestUnit.SetPlocha(const Value: double);
begin
end;

procedure Tgis_ForestUnit.SetVymera(const Value: double);
begin
end;

procedure Tgis_ForestUnit.SetZnacka(const Value: integer);
begin
  FZnacka := Value;
end;

procedure Tgis_ForestUnit.Set_Porost(const Value: Tgis_Porost);
begin

end;

procedure Tgis_ForestUnit.Set_Taxator(const Value: Tgis_Taxator);
var
   T: Tgis_Taxator;
begin
   T := Tgis_Taxator(FindOwnerClass(TGis_Taxator));
   if Value = T then exit;   //ftejnej
   if T <> nil then
      T.RemoveElement(Self);
   if Value <> nil then
   begin
      Value.AddElement(Self);
      Check_Taxator(Value);
   end;
end;

{ Tgis_Oddeleni }

function Tgis_Oddeleni.GetItemClass: Tgis_ForestUnitClass;
begin
   result := Tgis_Dilec;
end;

function Tgis_Oddeleni.GetItemsClass: Tgis_ForestModelCollectionClass;
begin
   result := Tgis_DilecCollection;
end;

function Tgis_Oddeleni.GetLHC: integer;
var
   aLHC: Tgis_LHC;
begin
   result := 0;
   aLHC := _LHC;
   if aLHC <> nil then
      result := aLHC.Cislo;
end;

function Tgis_Oddeleni.GetLHC_LCR: integer;
var
   aLHC: Tgis_LHC;
begin
   result := 0;
   aLHC := _LHC;
   if aLHC <> nil then
      result := aLHC.LHC_LCR;
end;

function Tgis_Oddeleni.GetLS_KOD: integer;
var
   aLS: Tgis_LS;
begin
   result := 0;
   aLS := _LS;
   if aLS <> nil then
      result := aLS.Cislo;
end;

function Tgis_Oddeleni.GetTaxPro: string;
begin
   result := 'ODD';
end;

function Tgis_Oddeleni.Get_LHC: Tgis_LHC;
var
   aLS: Tgis_LS;
begin
   result := nil;
   aLS := _LS;
   if aLS = nil then exit;
   result := aLS._LHC;
end;


function Tgis_Oddeleni.Get_LS: Tgis_LS;
var
   c: Tgis_ForestModelCollection;
begin
   result := nil;
   c := Tgis_OddeleniCollection(FindOwnerClass(Tgis_OddeleniCollection));
   if c = nil then exit;
   result := Tgis_LS(c.FindOwnerClass(Tgis_LS));
end;

{ Tgis_Dilec }

function Tgis_Dilec.GetDilec: char;
begin
   result := ' ';
   if Name <> '' then
      result := Name[1];
end;

function Tgis_Dilec.GetItemClass: Tgis_ForestUnitClass;
begin
   result := Tgis_Porost;
end;

function Tgis_Dilec.GetItemsClass: Tgis_ForestModelCollectionClass;
begin
   result := Tgis_PorostCollection;
end;

function Tgis_Dilec.GetLHC: integer;
var
   l: Tgis_LHC;
begin
   result := 0;
   l := _LHC;
   if l = nil then exit;
   result := l.Cislo;
end;

function Tgis_Dilec.GetLS_Kod: integer;
var
   aLS: Tgis_LS;
begin
   result := 0;
   aLS := _LS;
   if aLS <> nil then
      result := aLS.Cislo;
end;

function Tgis_Dilec.GetOddeleni: SmallInt;
var
   o: Tgis_Oddeleni;
begin
   result := 0;
   o := _Oddeleni;
   if o = nil then exit;
   result := o.Cislo;
end;

function Tgis_Dilec.GetTaxPro: string;
var
   aOdd: Tgis_Oddeleni;
begin
   result := '?';
   aOdd := _Oddeleni;
   if (aOdd <> nil) and (aOdd._Taxator <> nil) then
      result := aOdd.TaxPro
   else
   if _Taxator <> nil then result := 'DIL';
end;

function Tgis_Dilec.Get_LHC: Tgis_LHC;
var
   Odd: Tgis_Oddeleni;
begin
   result := nil;
   Odd := _Oddeleni;
   if Odd = nil then exit;
   result := Odd._LHC;
end;

function Tgis_Dilec.Get_LS: Tgis_LS;
var
   Odd: Tgis_Oddeleni;
begin
   result := nil;
   Odd := _Oddeleni;
   if Odd = nil then exit;
   result := Odd._LS;
end;

function Tgis_Dilec.Get_Oddeleni: Tgis_Oddeleni;
var
   c: Tgis_ForestModelCollection;
begin
   result := nil;
   c := Tgis_DilecCollection(FindOwnerClass(Tgis_DilecCollection));
   if c = nil then exit;
   result := Tgis_Oddeleni(c.FindOwnerClass(Tgis_Oddeleni));
end;

function Tgis_Dilec.Get_Taxator: Tgis_Taxator;
var
   aOdd: Tgis_Oddeleni;
begin
   result := nil;
   aOdd := _Oddeleni;
   if (aOdd <> nil) and (aOdd._Taxator <> nil) then
      result := aOdd._Taxator
   else
      result := inherited Get_Taxator;
end;

procedure Tgis_Dilec.SetDilec(const Value: char);
begin

end;

procedure Tgis_Dilec.SetLHC(const Value: integer);
begin

end;

procedure Tgis_Dilec.SetLS_KOD(const Value: integer);
begin

end;

procedure Tgis_Dilec.SetOddeleni(const Value: SmallInt);
begin

end;

{ Tgis_Porost }

function Tgis_Porost.Get_LHC: Tgis_LHC;
var
   Odd: Tgis_Oddeleni;
begin
   result := nil;
   Odd := _Oddeleni;
   if Odd = nil then exit;
   result := Odd._LHC;
end;

function Tgis_Porost.Get_Oddeleni: Tgis_Oddeleni;
var
   Dil: Tgis_Dilec;
begin
   result := nil;
   Dil := _Dilec;
   if Dil = nil then exit;
   result := Dil._Oddeleni;
end;

function Tgis_Porost.Get_Dilec: Tgis_Dilec;
var
   c: Tgis_ForestModelCollection;
begin
   result := nil;
   c := Tgis_PorostCollection(FindOwnerClass(Tgis_PorostCollection));
   if c = nil then exit;
   result := Tgis_Dilec(c.FindOwnerClass(Tgis_Dilec));
end;

function Tgis_Porost.GetItemClass: Tgis_ForestUnitClass;
begin
   result := Tgis_PorostniSkupina;
end;

function Tgis_Porost.GetItemsClass: Tgis_ForestModelCollectionClass;
begin
   result := Tgis_PorostniSkupinaCollection;
end;

function Tgis_Porost.GetDilec: char;
var
   d: Tgis_Dilec;
begin
   result := ' ';
   d := _Dilec;
   if d = nil then exit;
   if d.Name <> '' then  result := d.Name[1];
end;

function Tgis_Porost.GetKATPAR_KOD: byte;
var
   s: Tgis_SkupinaParcel;
begin
   result := 0;
   s := SkupinaParcel;
   if s = nil then exit;
   result := s.KATPAR_KOD;
end;

function Tgis_Porost.GetKU_KOD: integer;
var
   s: Tgis_SkupinaParcel;
begin
   result := 0;
   s := SkupinaParcel;
   if s = nil then exit;
   result := s.KU_KOD;
end;

function Tgis_Porost.GetKVAL_P: byte;
begin
   result := FKVAL_P;
end;

function Tgis_Porost.GetLHC: integer;
var
   l: Tgis_LHC;
begin
   result := 0;
   l := _LHC;
   if l = nil then exit;
   result := l.Cislo;
end;

function Tgis_Porost.GetLS_Kod: integer;
var
   aLS: Tgis_LS;
begin
   result := 0;
   aLS := _LS;
   if aLS = nil then exit;
   result := aLS.Cislo;
end;

function Tgis_Porost.GetOddeleni: SmallInt;
var
   o: Tgis_Oddeleni;
begin
   result := 0;
   o := _Oddeleni;
   if o = nil then exit;
   result := o.Cislo;
end;

function Tgis_Porost.GetOLH: Tgis_OLH;
var
   SKP: Tgis_SkupinaParcel;
begin
   result := nil;
   SKP := SkupinaParcel;
   if SKP = nil then exit;
   result := SKP.OLH;
end;

function Tgis_Porost.GetPorost: char;
begin
   result := ' ';
   if Name <> '' then  result := Name[1];
end;

function Tgis_Porost.GetSK_PARC: integer;
var
   s: Tgis_SkupinaParcel;
begin
   result := 0;
   s := SkupinaParcel;
   if s = nil then exit;
   result := s.Cislo;
end;

function Tgis_Porost.GetSkupinaParcel: Tgis_SkupinaParcel;
var
   CE: TForest_ParcelConnectionEntity;
begin
   result := nil;
   CE := TForest_ParcelConnectionEntity(FindOwnerClass(TForest_ParcelConnectionEntity));
   if CE = nil then exit;
   result := Tgis_SkupinaParcel(CE.FindOwnerClass(Tgis_SkupinaParcel));
end;

function Tgis_Porost.GetVLASTNIK: string;
var
   M: Tgis_Majitel;
begin
   result := '0';
   M := Majitel;
   if M = nil then exit;
   result := M.Cislo;
end;

procedure Tgis_Porost.SetDilec(const Value: char);
begin

end;

procedure Tgis_Porost.SetKATPAR_KOD(const Value: byte);
begin

end;

procedure Tgis_Porost.SetKU_KOD(const Value: integer);
begin

end;

procedure Tgis_Porost.SetKVAL_P(const Value: byte);
begin
   FKVAL_P := Value;
end;

procedure Tgis_Porost.SetLHC(const Value: integer);
begin

end;

procedure Tgis_Porost.SetLS_KOD(const Value: integer);
begin

end;

procedure Tgis_Porost.SetOddeleni(const Value: SmallInt);
begin

end;

procedure Tgis_Porost.SetPorost(const Value: char);
begin
   Name := Value;
end;

procedure Tgis_Porost.SetSK_PARC(const Value: integer);
begin

end;

procedure Tgis_Porost.SetSkupinaParcel(const Value: Tgis_SkupinaParcel);
var
   SKP: Tgis_SkupinaParcel;
begin
   SKP := SkupinaParcel;
   if SKP <> nil then
   begin
      if SKP = Value then exit;
      SKP.RemovePorost(Self);
   end;
   if Value = nil then exit;
   Value.NextPorost := Self;
end;

procedure Tgis_Porost.SetVLASTNIK(const Value: string);
begin

end;

function Tgis_Porost.Get_LS: Tgis_LS;
var
   Dil: Tgis_Dilec;
begin
   result := nil;
   Dil := _Dilec;
   if Dil = nil then exit;
   result := Dil._LS;
end;

procedure Tgis_Porost.SetMultiKatastr(const Value: boolean);
begin
  FMultiKatastr := Value;
end;

function Tgis_Porost.GetVymera: double;
var
   G: TGeometry;
   A: TAreaElement;
   MA: TMultiAreaElement;
   i: integer;
begin
   result := 0;
   if NoGeometry then exit;
   G := Geometry;
   if not (G is TAreaElement) then exit;
   A := TAreaElement(G);
   if A is TMultiAreaElement then
   begin
      MA := TMultiAreaElement(A);
      for i := 0 to MA.LastIndex do
      begin
         A := MA.AreaElement[i];
         result := result + A.FixedArea;
      end;
   end
   else
   begin
      result := A.FixedArea;
   end;
end;

function Tgis_Porost.GetPlocha: double;
begin
   if NoGeometry then
   begin
      result := inherited GetPlocha;
   end
   else
   begin
      result := TAreaElement(Geometry).Area;
   end;
end;

function Tgis_Porost.GetLHC_LCR: integer;
var
   l: Tgis_LHC;
begin
   result := 0;
   l := _LHC;
   if l = nil then exit;
   result := l.LHC_LCR;
end;

procedure Tgis_Porost.SetKATS(const Value: string);
begin
  FKATS := Value;
end;

function Tgis_Porost.GetKAT: string;
var
   D: Char;
   P: integer;
begin
   result := ReadToDelimiter(KATS,[','],1,D,P);
end;

procedure Tgis_Porost.SetKAT(const Value: string);
var
   i: integer;
   //ch: char;
   l: integer;
begin
   i := pos(Value,KATS);
   if i = 1 then exit; // je prvni
   if i = 0 then // vubec tam neni
   begin
      if KATS <> '' then
         KATS := Value + ',' + KATS
      else
         KATS := Value;
   end
   else
   begin
      l := Length(Value);
      while (i+l <= length(KATS)) and not (KATS[i+l-1] = ',') do Inc(l);
      if i+l > length(KATS) then // posledni kat
      begin
         while (i > 1) and not (KATS[i] = ',') do // smazani carky pred
         begin
            Dec(i);
            inc(l);
         end;
      end;
      Delete(FKATS,i,l);
      KATS := Value + ',' + KATS
   end;
end;

procedure Tgis_Porost.SetNextKAT(const Value: string);
var
   i: integer;
begin
   i := pos(Value,KATS);
   if i > 0 then exit; // uz tam je
   if KATS <> '' then
      KATS := KATS + ',' + Value
   else
      KATS := Value;
end;

procedure Tgis_Porost.SetKAT_Poznamka(const Value: string);
begin
  FKAT_Poznamka := Value;
end;

procedure Tgis_Porost.SetNextZVL_STATUT(const Value: string);
var
   i: integer;
begin
   i := pos(Value,ZVL_STATUT);
   if i > 0 then exit; // uz tam je
   if ZVL_STATUT <> '' then
      ZVL_STATUT := ZVL_STATUT + ',' + Value
   else
      ZVL_STATUT := Value;
end;

procedure Tgis_Porost.SetZVL_STATUT(const Value: string);
begin
  FZVL_STATUT := Value;
end;

function Tgis_Porost.Get_Taxator: Tgis_Taxator;
var
   aDil: Tgis_Dilec;
begin
   aDil := _Dilec;
   if (aDil <> nil) and (aDil._Taxator <> nil) then
      result := aDil._Taxator
   else
      result := inherited Get_Taxator;
end;

function Tgis_Porost.GetTaxPro: string;
var
   aDil: Tgis_Dilec;
begin
   result := '?';
   aDil := _Dilec;
   if (aDil <> nil) and (aDil._Taxator <> nil) then
      result := aDil.TaxPro
   else
   if _Taxator <> nil then result := 'POR';
end;

function Tgis_Porost.GetMajitel: Tgis_Majitel;
var
   SKP: Tgis_SkupinaParcel;
begin
   result := nil;
   SKP := SkupinaParcel;
   if SKP = nil then exit;
   result := SKP.Majitel;
end;

{ Tgis_PorostniSkupina }

function Tgis_PorostniSkupina.Get_LHC: Tgis_LHC;
var
   Odd: Tgis_Oddeleni;
begin
   result := nil;
   Odd := _Oddeleni;
   if Odd = nil then exit;
   result := Odd._LHC;
end;

function Tgis_PorostniSkupina.Get_Oddeleni: Tgis_Oddeleni;
var
   Dil: Tgis_Dilec;
begin
   result := nil;
   Dil := _Dilec;
   if Dil = nil then exit;
   result := Dil._Oddeleni;
end;

function Tgis_PorostniSkupina.Get_Dilec: Tgis_Dilec;
var
   Por: Tgis_Porost;
begin
   result := nil;
   Por := _Porost;
   if Por = nil then exit;
   result := Por._Dilec;
end;

function Tgis_PorostniSkupina.Get_Porost: Tgis_Porost;
var
   c: Tgis_ForestModelCollection;
begin
   result := nil;
   c := Tgis_PorostniSkupinaCollection(FindOwnerClass(Tgis_PorostniSkupinaCollection));
   if c = nil then exit;
   result := Tgis_Porost(c.FindOwnerClass(Tgis_Porost));
end;


function Tgis_PorostniSkupina.GetDilec: char;
var
   d: Tgis_Dilec;
begin
   result := ' ';
   d := _Dilec;
   if d = nil then exit;
   if d.Name <> '' then  result := d.Name[1];
end;

function Tgis_PorostniSkupina.GetKATPAR_KOD: byte;
var
   SK: Tgis_SkupinaParcel;
begin
   result := 0;
   Sk := SkupinaParcel;
   if SK = nil then exit;
   result := SK.KATPAR_KOD;
end;

function Tgis_PorostniSkupina.GetKU_KOD: integer;
var
   SK: Tgis_SkupinaParcel;
begin
   result := 0;
   Sk := SkupinaParcel;
   if SK = nil then exit;
   result := SK.KU_KOD;
end;

function Tgis_PorostniSkupina.GetKVAL_P: byte;
begin
   result := 0;
   if NoGeometry then exit;
   result := TAreaElement(Geometry).FixedAreaIndex;
end;

function Tgis_PorostniSkupina.GetLHC: integer;
var
   l: Tgis_LHC;
begin
   result := 0;
   l := _LHC;
   if l = nil then exit;
   result := l.Cislo;
end;

function Tgis_PorostniSkupina.GetLS_Kod: integer;
var
   aLS: Tgis_LS;
begin
   result := 0;
   aLS := _LS;
   if aLS <> nil then
      result := aLS.Cislo;
end;

function Tgis_PorostniSkupina.GetOddeleni: SmallInt;
var
   o: Tgis_Oddeleni;
begin
   result := 0;
   o := _Oddeleni;
   if o = nil then exit;
   result := o.Cislo;
end;

function Tgis_PorostniSkupina.GetOLH: Tgis_OLH;
var
   SKP: Tgis_SkupinaParcel;
begin
   result := nil;
   SKP := SkupinaParcel;
   if SKP = nil then exit;
   result := SKP.OLH;
end;

function Tgis_PorostniSkupina.GetOLH_Adresa: string;
var
   O: TGis_OLH;
begin
   result := '';
   O := OLH;
   if O <> nil then
      result := O.Adresa;
end;

function Tgis_PorostniSkupina.GetOLH_ID: string;
var
   O: TGis_OLH;
begin
   result := '';
   O := OLH;
   if O <> nil then
      result := O.OLH_ID;
end;

function Tgis_PorostniSkupina.GetOLH_Name: string;
var
   O: TGis_OLH;
begin
   result := '';
   O := OLH;
   if O <> nil then
      result := O.FullName;
end;

function Tgis_PorostniSkupina.GetOLH_PR_TIT: string;
var
   O: TGis_OLH;
begin
   result := '';
   O := OLH;
   if O <> nil then
      result := O.OLH_PR_TIT;
end;

function Tgis_PorostniSkupina.GetPlocha: double;
begin
   result := 0;
   if NoGeometry then exit;
   result := TAreaElement(Geometry).Area / 10000;
end;

function Tgis_PorostniSkupina.GetPorost: char;
var
   p: Tgis_Porost;
begin
   result := ' ';
   p := _Porost;
   if p = nil then exit;
   if p.Name <> '' then  result := p.Name[1];
end;

function Tgis_PorostniSkupina.GetSK_PARC: integer;
var
   SK: Tgis_SkupinaParcel;
begin
   result := 0;
   Sk := SkupinaParcel;
   if SK = nil then exit;
   result := SK.Cislo;
end;

function Tgis_PorostniSkupina.GetSkupina: string;
begin
   result := Name;
end;

function Tgis_PorostniSkupina.GetVLASTNIK: string;
var
   M: Tgis_Majitel;
begin
   result := '0';
   M := Majitel;
   if M = nil then exit;
   result := M.Cislo;
end;

function Tgis_PorostniSkupina.GetVymera: double;
begin
   result := 0;
   if NoGeometry then exit;
   result := TAreaElement(Geometry).FixedArea;
end;

procedure Tgis_PorostniSkupina.SetDilec(const Value: char);
begin

end;

procedure Tgis_PorostniSkupina.SetKATPAR_KOD(const Value: byte);
begin

end;


procedure Tgis_PorostniSkupina.SetKU_KOD(const Value: integer);
begin

end;

procedure Tgis_PorostniSkupina.SetKVAL_P(const Value: byte);
begin
   FKVAL_P := Value;
end;

procedure Tgis_PorostniSkupina.SetLHC(const Value: integer);
begin

end;

procedure Tgis_PorostniSkupina.SetLS_KOD(const Value: integer);
begin

end;

procedure Tgis_PorostniSkupina.SetOddeleni(const Value: SmallInt);
begin

end;

procedure Tgis_PorostniSkupina.SetPorost(const Value: char);
begin

end;

procedure Tgis_PorostniSkupina.SetSK_PARC(const Value: integer);
begin

end;

procedure Tgis_PorostniSkupina.SetSkupina(const Value: string);
begin
   Name := Value;
end;

procedure Tgis_PorostniSkupina.SetVEK_ST(const Value: byte);
begin
  FVEK_ST := Value;
end;

procedure Tgis_PorostniSkupina.SetVLASTNIK(const Value: string);
begin
   //FVlastnik := Value;
end;

procedure Tgis_PorostniSkupina.SetVymera(const Value: double);
begin
   FVymera := Value;
end;

function Tgis_PorostniSkupina.Get_LS: Tgis_LS;
var
   Por: Tgis_Porost;
begin
   result := nil;
   Por := _Porost;
   if Por = nil then exit;
   result := Por._LS;
end;

function Tgis_PorostniSkupina.GetSkupinaParcel: Tgis_SkupinaParcel;
var
   Por: Tgis_Porost;
begin
   result := GetSkupinaParcelDetail;
   if result = nil then
   begin
      Por := _Porost;
      if Por = nil then exit;
      result := Por.SkupinaParcel;
   end;
end;

function Tgis_PorostniSkupina.GetSkupinaParcelDetail: Tgis_SkupinaParcel;
var
   CE: TForest_ParcelDetailConnectionEntity;
begin
   result := nil;
   CE := TForest_ParcelDetailConnectionEntity(FindOwnerClass(TForest_ParcelDetailConnectionEntity));
   if CE <> nil then
      result := Tgis_SkupinaParcel(CE.FindOwnerClass(Tgis_SkupinaParcel));
end;

procedure Tgis_PorostniSkupina.SetSkupinaParcel(const Value: Tgis_SkupinaParcel);
var
   SKP: Tgis_SkupinaParcel;
   Por: Tgis_Porost;
begin
   Por := _Porost;
   if Por <> nil then
   begin
      if Por.SkupinaParcel = nil then
      begin
         Por.SkupinaParcel := Value;
         Por.MultiKatastr := false;
         exit;
      end
      else
         if Por.SkupinaParcel = Value then exit;
      Por.MultiKatastr := true;
   end;
   SKP := GetSkupinaParcelDetail;
   if SKP <> nil then
   begin
      if SKP = Value then exit;
      SKP.RemoveSkupina(Self);
   end;
   if Value = nil then exit;
   Value.NextSkupina := Self;
end;

procedure Tgis_PorostniSkupina.Set_Porost(const Value: Tgis_Porost);
var
   P: Tgis_Porost;
begin
   P := _Porost;
   if P = Value then exit;
   if P <> nil then    // vyjmuti ze stavajiciho porostu
   begin
      P.Items.RemoveElement(Self);
   end;
   if Value = nil then exit;
   if Value.Items = nil then
      Value.Items := Value.GetItemsClass.Create;
   Value.Items.AddElement(Self);
end;

function Tgis_PorostniSkupina.GetPureSkupina: string;
var
   S1,S2: string;
begin
   StringAsTwoParams(Skupina,'.',S1,S2);
   result := S1;
end;

function Tgis_PorostniSkupina.GetPropertyStringByName(
  AName: string): string;
begin
   result := inherited GetPropertyStringByName(AName);
   if result <> '' then exit;
   if _Porost <> nil then
      result := _Porost.GetPropertyStringByName(aName);
end;

function Tgis_PorostniSkupina.GetLHC_LCR: integer;
var
   l: Tgis_LHC;
begin
   result := 0;
   l := _LHC;
   if l = nil then exit;
   result := l.LHC_LCR;
end;

function Tgis_PorostniSkupina.GetSameKVAL_P: boolean;
begin
   result := false;
   if NoGeometry then exit;
   result := TAreaElement(Geometry).SameFAI;
end;

function Tgis_PorostniSkupina.GetMAJ_DRUH: integer;
var
   //S: string;
   M: Tgis_Majitel;
begin
   result := 0;
   M := Majitel;
   if M = nil then exit;
   result := M.MAJ_DRUH;
   {
   S := IntToStr(Vlastnik);
   S := S[1];
   result := StrToIntDef(S,0);
   }
end;

function Tgis_PorostniSkupina.GetMAJ_KOD: string;
var
   //S,S1: string;
   M: Tgis_Majitel;
begin
   result := '0';
   M := Majitel;
   if M = nil then exit;
   result := M.MAJ_KOD;
   {
   S1 := IntToStr(Vlastnik);
   if Length(S1) < 2 then
   begin
      result := 0;
      exit;
   end;
   S := Copy(S1,2,Length(S1)-1);
   result := StrToIntDef(S,0);
   }
end;

function Tgis_PorostniSkupina.GetKU_KOD_: integer;
var
   SK: Tgis_SkupinaParcel;
begin
   result := 0;
   Sk := SkupinaParcel;
   if SK = nil then exit;
   result := SK.KU_KOD_;
end;

function Tgis_PorostniSkupina.GetKU_Nazev: string;
var
   SK: Tgis_SkupinaParcel;
begin
   result := '';
   Sk := SkupinaParcel;
   if SK = nil then exit;
   result := SK.KU_Nazev;
end;

function Tgis_PorostniSkupina.GetMajitel: Tgis_Majitel;
var
   SKP: Tgis_SkupinaParcel;
begin
   result := nil;
   SKP := SkupinaParcel;
   if SKP = nil then exit;
   result := SKP.Majitel;
end;

function Tgis_PorostniSkupina.GetMAJ_Adresa: string;
var
   M: Tgis_Majitel;
begin
   result := '';
   M := Majitel;
   if M = nil then exit;
   result := M.Adresa;
end;

function Tgis_PorostniSkupina.GetMAJ_Name: string;
var
   M: Tgis_Majitel;
begin
   result := '';
   M := Majitel;
   if M = nil then exit;
   result := M.FullName;
end;

function Tgis_PorostniSkupina.GetORP: integer;
var
   SK: Tgis_SkupinaParcel;
begin
   result := 0;
   Sk := SkupinaParcel;
   if SK = nil then exit;
   result := SK.ORP;
end;

function Tgis_PorostniSkupina.GetORP_Nazev: string;
var
   SK: Tgis_SkupinaParcel;
begin
   result := '';
   Sk := SkupinaParcel;
   if SK = nil then exit;
   result := SK.ORP_Nazev;
end;

function Tgis_PorostniSkupina.GetKAT: string;
begin
   result := '';
   if _Porost <> nil then
      result := _Porost.KAT;
end;

function Tgis_PorostniSkupina.GetKATS: string;
begin
   result := '';
   if _Porost <> nil then
      result := _Porost.KATS;
end;

procedure Tgis_PorostniSkupina.SetKAT(const Value: string);
begin
   if _Porost <> nil then
      _Porost.KAT := Value;
end;

procedure Tgis_PorostniSkupina.SetKATS(const Value: string);
begin
   if _Porost <> nil then
      _Porost.KATS := Value;
end;

procedure Tgis_PorostniSkupina.SetNextKAT(const Value: string);
begin
   if _Porost <> nil then
      _Porost.NextKAT := Value;
end;

function Tgis_PorostniSkupina.GetKAT_Poznamka: string;
begin
   result := '';
   if _Porost <> nil then
      result := _Porost.KAT_Poznamka;
end;

function Tgis_PorostniSkupina.GetZVL_STATUT: string;
begin
   result := '';
   if _Porost <> nil then
      result := _Porost.ZVL_STATUT;
end;

procedure Tgis_PorostniSkupina.SetKAT_Poznamka(const Value: string);
begin
   if _Porost <> nil then
      _Porost.KAT_Poznamka := Value;
end;

procedure Tgis_PorostniSkupina.SetNextZVL_STATUT(const Value: string);
begin
   if _Porost <> nil then
      _Porost.NextZVL_STATUT := Value;
end;

procedure Tgis_PorostniSkupina.SetZVL_STATUT(const Value: string);
begin
   if _Porost <> nil then
      _Porost.ZVL_STATUT := Value;
end;

function Tgis_PorostniSkupina.GetKraj: string;
var
   SK: Tgis_SkupinaParcel;
begin
   result := '';
   Sk := SkupinaParcel;
   if SK = nil then exit;
   result := SK.Kraj;
end;

function Tgis_PorostniSkupina.GetKraj_Nazev: string;
var
   SK: Tgis_SkupinaParcel;
begin
   result := '';
   Sk := SkupinaParcel;
   if SK = nil then exit;
   result := SK.Kraj_Nazev;
end;

function Tgis_PorostniSkupina.GetLV: integer;
var
   SK: Tgis_SkupinaParcel;
begin
   result := 0;
   Sk := SkupinaParcel;
   if SK = nil then exit;
   result := SK.LV;
end;

function Tgis_PorostniSkupina.GetLVc: integer;
var
   SK: Tgis_SkupinaParcel;
begin
   result := 0;
   Sk := SkupinaParcel;
   if SK = nil then exit;
   result := SK.LVc;
end;

function Tgis_PorostniSkupina.Get_Taxator: Tgis_Taxator;
var
   aPor: Tgis_Porost;
begin
   result := nil;
   aPor := _Porost;
   if (aPor <> nil) and (aPor._Taxator <> nil) then
      result := aPor._Taxator
   else
      result := inherited Get_Taxator;
end;

function Tgis_PorostniSkupina.GetTaxPro: string;
var
   aPor: Tgis_Porost;
begin
   result := '?';
   aPor := _Porost;
   if (aPor <> nil) and (aPor._Taxator <> nil) then
      result := aPor.TaxPro
   else
   if _Taxator <> nil then result := 'PSK';
end;

function Tgis_PorostniSkupina.GetGroup: integer;
begin
   result := 110;
end;

procedure Tgis_PorostniSkupina.SetLVS(const Value: byte);
begin
  FLVS := Value;
end;

{ Tgis_DilecCollection }

function Tgis_DilecCollection.GetName: string;
begin
   result := 'Dilce';
end;


{ Tgis_CisloForestUnit }

function Tgis_CisloForestUnit.CreateItemCislo(const Value: integer): Tgis_CisloForestUnit;
begin
   if Items = nil then
      Items := GetItemsClass.Create;
   result := Tgis_CisloForestUnit(GetItemClass.Create);
   result.Cislo := Value;
   Items.AddElement(result);
end;

function Tgis_CisloForestUnit.GetItemCislo(const Value: integer): Tgis_CisloForestUnit;
var
   i: integer;
   e: TElement;
begin
   result := nil;
   if Items = nil then exit;
   for i := 0 to Items.LastIndex do
   begin
      e := Items.Elements[i];
      if not (e is Tgis_CisloForestUnit) then continue;
      if Tgis_CisloForestUnit(e).Cislo = Value then
      begin
         result := Tgis_CisloForestUnit(e);
         exit;
      end;
   end;
end;

function Tgis_CisloForestUnit.GetName: string;
begin
   result := IntToStr(FCislo);
end;

procedure Tgis_CisloForestUnit.SetCislo(const Value: integer);
begin
   FCislo := Value;
end;

procedure Tgis_CisloForestUnit.SetName(aName: string);
begin
   try
      Cislo := StrToInt(aName);
   except
   end;
end;

{ Tgis_ForestModelCollection }

function Tgis_ForestModelCollection.GetForestUnit(aIndex: integer): Tgis_ForestUnit;
begin
   result := Tgis_ForestUnit(Elements[aIndex]);
end;


procedure Tgis_ForestModelCollection.SetName(aName: string);
begin
end;

{ Tgis_OddeleniCollection }

function Tgis_OddeleniCollection.GetName: string;
begin
   result := 'Oddeleni';
end;

{ Tgis_PorostCollection }

function Tgis_PorostCollection.GetName: string;
begin
   result := 'Porosty';
end;

{ Tgis_PorostniSkupinaCollection }

function Tgis_PorostniSkupinaCollection.GetName: string;
begin
   result := 'Skupiny';
end;

{ Tgis_ItemsForestUnit }

procedure Tgis_ItemsForestUnit.CheckItems;
begin
   if Items = nil then
      Items := GetItemsClass.Create;
end;

function Tgis_ItemsForestUnit.CreateItemName(const Value: string): Tgis_ForestUnit;
begin
   if Items = nil then
      Items := GetItemsClass.Create;
   result := GetItemClass.Create;
   result.Name := Value;
   Items.AddElement(result);
end;

function Tgis_ItemsForestUnit.GetItemClass: Tgis_ForestUnitClass;
begin
   result := nil;
end;

function Tgis_ItemsForestUnit.GetItemName(const Value: string): Tgis_ForestUnit;
begin
   result := nil;
   if Items = nil then exit;
   result := Tgis_ForestUnit(Items.FindElementName(Value));
end;

function Tgis_ItemsForestUnit.GetItems: Tgis_ForestModelCollection;
begin
   if FItems = nil then
      FItems := Tgis_ForestModelCollection(FindElementClass(GetItemsClass));
   result := FItems;
end;

function Tgis_ItemsForestUnit.GetItemsClass: Tgis_ForestModelCollectionClass;
begin
   result := nil;
end;

function Tgis_ItemsForestUnit.GetNoItems: boolean;
begin
   result := (Items = nil) or Items.NoElements;
end;

function Tgis_ItemsForestUnit.GetPlocha: double;
var
   i: integer;
begin
   result := 0;
   if (Items <> nil) and (Items.Count > 0) then
   for i := 0 to Items.LastIndex do
   begin
      result := result + Items.ForestUnit[i].Plocha;
   end;
end;

function Tgis_ItemsForestUnit.GetVymera: double;
var
   i: integer;
begin
   result := 0;
   if (Items <> nil) and (Items.Count > 0) then
   for i := 0 to Items.LastIndex do
   begin
      result := result + Items.ForestUnit[i].Vymera;
   end;
end;

procedure Tgis_ItemsForestUnit.Reset_Taxator(aTaxator: Tgis_Taxator; FUClass: TClass);
var
   i: integer;
begin
  inherited;
  if ClassType <> FUClass then
  if not NoItems then
  begin
     for i := 0 to Items.LastIndex do
     begin
        Items.ForestUnit[i].Reset_Taxator(aTaxator,FUClass);
     end;
  end;
end;

procedure Tgis_ItemsForestUnit.SetItems(const Value: Tgis_ForestModelCollection);
begin
   ExchangeFieldElement(TElement(FItems), Value);
end;

procedure Tgis_ItemsForestUnit.SetUpdateAll(const Value: boolean);
var
   i: integer;
   FU: Tgis_ForestUnit;
   IFU: Tgis_ItemsForestUnit;
begin
   if not Value then exit;
   if not NoGeometry then exit;
   if noItems then exit;
   Rect.Reset;
   for i := 0 to Items.LastIndex do
   begin
      FU := Items.ForestUnit[i];
      if (FU is Tgis_ItemsForestUnit) then
      begin
         IFU := Tgis_ItemsForestUnit(FU);
         IFU.UpdateAll := True;
      end;
      UpdateRect(FU);
   end;
end;

procedure Tgis_ItemsForestUnit.Set_Taxator(const Value: Tgis_Taxator);
var
   i: integer;
begin
  inherited;
  if not NoItems then
  begin
     for i := 0 to Items.LastIndex do
     begin
        Items.ForestUnit[i]._Taxator := nil;
     end;
  end;
end;

{ Tgis_LHC }


function Tgis_LHC.GetItemClass: Tgis_ForestUnitClass;
begin
   result := Tgis_LS;
end;

function Tgis_LHC.GetItemsClass: Tgis_ForestModelCollectionClass;
begin
   result := Tgis_LSCollection;
end;


procedure Tgis_LHC.SetLHC_LCR(const Value: integer);
begin
  FLHC_LCR := Value;
end;

procedure Tgis_LHC.SetLHC_TAX(const Value: string);
begin
  FLHC_TAX := Value;
end;

procedure Tgis_LHC.SetNazev(const Value: string);
begin
   FNazev := Value;
end;

procedure Tgis_LHC.SetROK(const Value: integer);
begin
  FROK := Value;
end;

{ Tgis_ForestModel }

function Tgis_ForestModel.CreateDilecName(const Value: string): Tgis_Dilec;
begin
   result := GetDilecName(Value);
   if result <> nil then exit;
   if ActualOddeleni = nil then exit;
   result := Tgis_Dilec(ActualOddeleni.CreateItemName(Value));
   ActualDilec := result;
end;

function Tgis_ForestModel.CreateLHCCislo(const Value: integer): Tgis_LHC;
begin
   result := GetLHCCislo(Value);
   if result <> nil then exit;
   result := Tgis_LHC.Create;
   result.Cislo := Value;
   AddElement(result);
   ActualLHC := result;
end;

function Tgis_ForestModel.CreateOddeleniCislo(const Value: integer): Tgis_Oddeleni;
begin
   result := GetOddeleniCislo(Value);
   if result <> nil then exit;
   if ActualLS = nil then exit;
   result := Tgis_Oddeleni(ActualLS.CreateItemCislo(Value));
   ActualOddeleni := result;
end;

function Tgis_ForestModel.CreatePorostName(const Value: string): Tgis_Porost;
begin
   result := GetPorostName(Value);
   if result <> nil then exit;
   if ActualDilec = nil then exit;
   result := Tgis_Porost(ActualDilec.CreateItemName(Value));
   ActualPorost := result;
end;

function Tgis_ForestModel.CreateSkupinaName(const Value: string): Tgis_PorostniSkupina;
begin
   result := GetSkupinaName(Value);
   if result <> nil then exit;
   if ActualPorost = nil then exit;
   result := Tgis_PorostniSkupina(ActualPorost.CreateItemName(Value));
   ActualSkupina := result;
end;

function Tgis_ForestModel.GetDilec_Name: string;
begin
   result := '';
   if ActualDilec = nil then exit;
   result := ActualDilec.Name;
end;

function Tgis_ForestModel.GetDilecName(const Value: string): Tgis_Dilec;
begin
   result := nil;
   if (ActualDilec <> nil) and AnsiSameText(Value,ActualDilec.Name) then
   begin
      result := ActualDilec;
      exit;
   end;
   if ActualOddeleni = nil then exit;
   result := Tgis_Dilec(ActualOddeleni.GetItemName(Value));
   ActualDilec := result;
end;

function Tgis_ForestModel.GetLHC_Cislo: integer;
begin
   result := -1;
   if ActualLHC = nil then exit;
   result := ActualLHC.Cislo;
end;

function Tgis_ForestModel.GetLHC_ROK: integer;
var
   i: integer;
   e: TElement;
begin
   result := 0;
   if (ActualLHC <> nil)  then
      result := ActualLHC.ROK;
end;

function Tgis_ForestModel.GetLHCCislo(const Value: integer): Tgis_LHC;
var
   i: integer;
   e: TElement;
begin
   result := nil;
   if (ActualLHC <> nil) and (ActualLHC.Cislo = Value) then
   begin
      result := ActualLHC;
      exit;
   end;
   for i := 0 to LastIndex do
   begin
      e := Elements[i];
      if Tgis_LHC(e).Cislo = Value then
      begin
         result := Tgis_LHC(e);
         break;
      end;
   end;
   ActualLHC := result;
end;

function Tgis_ForestModel.GetOddeleni_Cislo: integer;
begin
   result := -1;
   if ActualOddeleni = nil then exit;
   result := ActualOddeleni.Cislo;
end;

function Tgis_ForestModel.GetOddeleniCislo(const Value: integer): Tgis_Oddeleni;
var
   i: integer;
   aLS: Tgis_LS;
begin
   result := nil;
   if (ActualOddeleni <> nil) and (ActualOddeleni.Cislo = Value) then
   begin
      result := ActualOddeleni;
      exit;
   end;
   if ActualLS = nil then
   begin
      if ActualLHC = nil then exit;
      if ActualLHC.NoItems then exit;
      for i := 0 to ActualLHC.Items.LastIndex do
      begin
         aLS := Tgis_LS(ActualLHC.Items.Elements[i]);
         if aLS.GetItemCislo(Value) <> nil then
         begin
            ActualLS := aLS;
            break;
         end;
      end;
   end;
   if ActualLS = nil then exit;
   result := Tgis_Oddeleni(ActualLS.GetItemCislo(Value));
   ActualOddeleni := result;
end;

function Tgis_ForestModel.GetPorost_Name: string;
begin
   result := '';
   if ActualPorost = nil then exit;
   result := ActualPorost.Name;
end;

function Tgis_ForestModel.GetPorostName(const Value: string): Tgis_Porost;
begin
   result := nil;
   if (ActualPorost <> nil) and AnsiSameText(Value,ActualPorost.Name) then
   begin
      result := ActualPorost;
      exit;
   end;
   if ActualDilec = nil then exit;
   result := Tgis_Porost(ActualDilec.GetItemName(Value));
   ActualPorost := result;
end;

function Tgis_ForestModel.GetSkupina_Name: string;
begin
   result := '';
   if ActualSkupina = nil then exit;
   result := ActualSkupina.Name;
end;

function Tgis_ForestModel.GetSkupinaName(const Value: string): Tgis_PorostniSkupina;
begin
   result := nil;
   if (ActualSkupina <> nil) and AnsiSameText(Value,ActualSkupina.Name) then
   begin
      result := ActualSkupina;
      exit;
   end;
   if ActualPorost = nil then exit;
   result := Tgis_PorostniSkupina(ActualPorost.GetItemName(Value));
   ActualSkupina := result;
end;

procedure Tgis_ForestModel.SetActualDilec(const Value: Tgis_Dilec);
begin
   FActualDilec := Value;
   ActualPorost := nil;
end;

procedure Tgis_ForestModel.SetActualLHC(const Value: Tgis_LHC);
begin
   FActualLHC := Value;
   ActualLS := nil;
end;

procedure Tgis_ForestModel.SetActualOddeleni(const Value: Tgis_Oddeleni);
begin
   FActualOddeleni := Value;
   ActualDilec := nil;
end;

procedure Tgis_ForestModel.SetActualPorost(const Value: Tgis_Porost);
begin
   FActualPorost := Value;
   ActualSkupina := nil;
end;

procedure Tgis_ForestModel.SetActualSkupina(const Value: Tgis_PorostniSkupina);
begin
   FActualSkupina := Value;
end;

procedure Tgis_ForestModel.SetDilec_Name(const Value: string);
begin
   if AutoCreate then
      CreateDilecName(Value)
   else
      GetDilecName(Value);
end;

procedure Tgis_ForestModel.SetLHC_Cislo(const Value: integer);
begin
   if AutoCreate then
      CreateLHCCislo(Value)
   else
      GetLHCCislo(Value);
end;

procedure Tgis_ForestModel.SetOddeleni_Cislo(const Value: integer);
begin
   if AutoCreate then
      CreateOddeleniCislo(Value)
   else
      GetOddeleniCislo(Value);
end;

procedure Tgis_ForestModel.SetPorost_Name(const Value: string);
begin
   if AutoCreate then
      CreatePorostName(Value)
   else
      GetPorostName(Value);
end;

procedure Tgis_ForestModel.SetSkupina_Name(const Value: string);
begin
   if AutoCreate then
      CreateSkupinaName(Value)
   else
      GetSkupinaName(Value);
end;

function Tgis_ForestModel.CreateLSCislo(const Value: integer): Tgis_LS;
begin
   result := GetLSCislo(Value);
   if result <> nil then exit;
   if ActualLHC = nil then exit;
   result := Tgis_LS(ActualLHC.CreateItemCislo(Value));
   ActualLS := result;
end;

function Tgis_ForestModel.GetLS_Cislo: integer;
begin
   result := -1;
   if ActualLS = nil then exit;
   result := ActualLS.Cislo;
end;

function Tgis_ForestModel.GetLSCislo(const Value: integer): Tgis_LS;
begin
   result := nil;
   if (ActualLS <> nil) and (ActualLS.Cislo = Value) then
   begin
      result := ActualLS;
      exit;
   end;
   if ActualLHC = nil then exit;
   result := Tgis_LS(ActualLHC.GetItemCislo(Value));
   ActualLS := result;
end;

procedure Tgis_ForestModel.SetActualLS(const Value: Tgis_LS);
begin
   FActualLS := Value;
   ActualOddeleni := nil;
end;

procedure Tgis_ForestModel.SetLS_Cislo(const Value: integer);
begin
   if AutoCreate then
      CreateLSCislo(Value)
   else
      GetLSCislo(Value);
end;

function Tgis_ForestModel.CreatePorost(aLHC, aLS, aOddeleni: integer;
         aDilec, aPorost: string): Tgis_Porost;
begin
   CreateLHCCislo(aLHC);
   CreateLSCislo(aLS);
   CreateOddeleniCislo(aOddeleni);
   CreateDilecName(aDilec);
   result := CreatePorostName(aPorost);
end;

function Tgis_ForestModel.CreateSkupina(aLHC, aLS, aOddeleni: integer;
         aDilec, aPorost, aSkupina: string): Tgis_PorostniSkupina;
begin
   CreatePorost(aLHC,aLS,aOddeleni,aDilec,aPorost);
   result := CreateSkupinaName(aSkupina);
end;

procedure Tgis_ForestModel.SetUpdateAll(const Value: boolean);
var
   i: integer;
   e: TElement;
begin
   if NoElements then exit;
   for i := 0 to LastIndex do
   begin
      e := Elements[i];
      Tgis_LHC(e).UpdateAll := true;
   end;
   ResetAndUpdate;
end;

procedure Tgis_ForestModel.SetSort(const Value: boolean);
var
   iLHC,iLS,iODD,iDIL,iPOR: integer;
   T,TS: TSortElements;
   aLHC: Tgis_LHC;
   aLS: Tgis_LS;
   aOddeleni: Tgis_Oddeleni;
   aDilec : Tgis_Dilec;
   aPorost : Tgis_Porost;
begin
   T := TSortElementsByInteger.Create;
   T.PropertyName := 'Name';
   T.Input := Self;
   T.DoIt;
   TS := TSortElementsByString.Create;
   TS.PropertyName := 'Name';
   for iLHC := 0 to LastIndex do
   begin
      aLHC := Tgis_LHC(Elements[iLHC]);
      if aLHC.NoItems  then continue;
      T.Input := aLHC.Items;
      T.DoIt;
      for iLS := 0 to aLHC.Items.LastIndex do
      begin
         aLS := Tgis_LS(aLHC.Items.Elements[iLS]);
         if aLS.NoItems  then continue;
         T.Input := aLS.Items;
         T.DoIt;
         for iODD := 0 to aLS.Items.LastIndex do
         begin
            aOddeleni :=  Tgis_Oddeleni(aLS.Items.Elements[iOdd]);
            if aOddeleni.NoItems then continue;
            TS.Input := aOddeleni.Items;
            TS.DoIt;
            for iDIL := 0 to aOddeleni.Items.LastIndex do
            begin
               aDilec := Tgis_Dilec(aOddeleni.Items.Elements[iDIL]);
               if aDilec.NoItems then continue;
               TS.Input := aDilec.Items;
               TS.DoIt;
               for iPor := 0 to aDilec.Items.LastIndex do
               begin
                  aPorost := Tgis_Porost(aDilec.Items.Elements[iPor]);
                  if aPorost.NoItems then continue;
                  T.Input := aPorost.Items;
                  T.PropertyName := 'VEK_ST';
                  T.DoIt;
                  T.PropertyName := 'Name';
               end;
            end;
         end;
      end;
   end;
   T.Free;
   TS.Free;
end;

procedure Tgis_ForestModel.SetAutoCreate(const Value: boolean);
begin
  FAutoCreate := Value;
end;

function Tgis_ForestModel.GetOddCount: integer;
var
   i: integer;
   aLS: Tgis_LS;
begin
   result := 0;
   if ActualLHC = nil then
      ActualLHC := Tgis_LHC(FindElementClass(TGis_LHC));
   if ActualLHC = nil then exit;
   if ActualLHC.NoItems then exit;
   for i := 0 to ActualLHC.Items.LastIndex do
   begin
      aLS := Tgis_LS(ActualLHC.Items.Elements[i]);
      if not aLS.NoItems then
         result := result + aLS.Items.Count;
   end;
end;

function Tgis_ForestModel.FindOdds: TEntity;
var
   i: integer;
   aLS: Tgis_LS;
begin
   result := TOwnerEntity.Create;
   result.Name := 'ODDS';
   if ActualLHC = nil then
      ActualLHC := Tgis_LHC(FindElementClass(TGis_LHC));
   if ActualLHC = nil then exit;
   if ActualLHC.NoItems then exit;
   for i := 0 to ActualLHC.Items.LastIndex do
   begin
      aLS := Tgis_LS(ActualLHC.Items.Elements[i]);
      if not aLS.NoItems then
         result.AddAllFrom := aLS.Items;
   end;
end;

{ Tgis_PorostniSkupinaAlone }

function Tgis_PorostniSkupinaAlone.GetDilec: char;
begin
   result := FDilec;
end;

function Tgis_PorostniSkupinaAlone.GetKATPAR_KOD: byte;
begin
   result := FKATPAR_KOD;
end;

function Tgis_PorostniSkupinaAlone.GetKU_KOD: integer;
begin
   result := FKU_KOD;
end;


function Tgis_PorostniSkupinaAlone.GetLHC: integer;
begin
   result := FLHC;
end;

function Tgis_PorostniSkupinaAlone.GetLS_Kod: integer;
begin
   result := FLS_KOD;
end;

function Tgis_PorostniSkupinaAlone.GetOddeleni: SmallInt;
begin
   result := FOddeleni;
end;

function Tgis_PorostniSkupinaAlone.GetPorost: char;
begin
   result := FPorost;
end;

function Tgis_PorostniSkupinaAlone.GetSK_PARC: integer;
begin
   result := FSK_PARC;
end;

function Tgis_PorostniSkupinaAlone.GetVLASTNIK: string;
begin
   result := FVlastnik;
end;

procedure Tgis_PorostniSkupinaAlone.SetDilec(const Value: char);
begin
   FDilec := Value;
end;

procedure Tgis_PorostniSkupinaAlone.SetKATPAR_KOD(const Value: byte);
begin
   FKATPAR_KOD := Value;
end;

procedure Tgis_PorostniSkupinaAlone.SetKU_KOD(const Value: integer);
begin
   FKU_KOD := Value;
end;


procedure Tgis_PorostniSkupinaAlone.SetLHC(const Value: integer);
begin
   FLHC := Value;
end;

procedure Tgis_PorostniSkupinaAlone.SetLS_KOD(const Value: integer);
begin
   FLS_KOD := Value;
end;

procedure Tgis_PorostniSkupinaAlone.SetOddeleni(const Value: SmallInt);
begin
   FOddeleni := Value;
end;

procedure Tgis_PorostniSkupinaAlone.SetPorost(const Value: char);
begin
   FPorost := Value;
end;

procedure Tgis_PorostniSkupinaAlone.SetSK_PARC(const Value: integer);
begin
   FSK_PARC := Value;
end;

procedure Tgis_PorostniSkupinaAlone.SetVLASTNIK(const Value: string);
begin
   FVlastnik := Value;
end;

{ Tgis_LS }


function Tgis_LS.GetItemClass: Tgis_ForestUnitClass;
begin
   result := Tgis_Oddeleni;
end;

function Tgis_LS.GetItemsClass: Tgis_ForestModelCollectionClass;
begin
   result := Tgis_OddeleniCollection;
end;

function Tgis_LS.GetLHC: integer;
var
   aLHC: Tgis_LHC;
begin
   result := 0;
   aLHC := _LHC;
   if aLHC <> nil then
      result := aLHC.Cislo;
end;

function Tgis_LS.GetLHC_LCR: integer;
var
   aLHC: Tgis_LHC;
begin
   result := 0;
   aLHC := _LHC;
   if aLHC <> nil then
      result := aLHC.LHC_LCR;
end;

function Tgis_LS.Get_LHC: Tgis_LHC;
var
   c: Tgis_ForestModelCollection;
begin
   result := nil;
   c := Tgis_OddeleniCollection(FindOwnerClass(Tgis_LSCollection));
   if c = nil then exit;
   result := Tgis_LHC(c.FindOwnerClass(Tgis_LHC));
end;

procedure Tgis_LS.SetKR(const Value: integer);
begin
  FKR := Value;
end;

procedure Tgis_LS.SetKR_NAZ(const Value: string);
begin
  FKR_NAZ := Value;
end;

procedure Tgis_LS.SetLS(const Value: integer);
begin
   FLS := Value;
end;

procedure Tgis_LS.SetLS_NAZ(const Value: string);
begin
  FLS_NAZ := Value;
end;

procedure Tgis_LS.SetNazev(const Value: string);
begin
   FNazev := Value;
end;

procedure Tgis_LS.SetPoznamka(const Value: string);
begin
  FPoznamka := Value;
end;

procedure Tgis_LS.SetREV_TYP(const Value: integer);
begin
  FREV_TYP := Value;
end;

procedure Tgis_LS.SetUSEK(const Value: integer);
begin
  FUSEK := Value;
end;

procedure Tgis_LS.SetUSEK_NAZ(const Value: string);
begin
  FUSEK_NAZ := Value;
end;

{ Tgis_LSCollection }

function Tgis_LSCollection.GetName: string;
begin
   result := 'LS';
end;

{ Tgis_TezbaCollection }

function Tgis_TezbaCollection.GetName: string;
begin
   result := 'TEZ';
end;

{ Tgis_UserCollection }

function Tgis_UserCollection.FindUserID: integer;
var
   i: integer;
   S: string;
begin
   i := 1;
   result := i;
   S := IntToStr(i);
   if NoElements then exit;
   while FindElementName(S) <> nil do
   begin
      Inc(i);
      S := IntToStr(i);
   end;
   result := i;
end;

function Tgis_UserCollection.GetName: string;
begin
   result := 'USER';
end;

{ Tgis_Taxator }

procedure Tgis_Taxator.AddElement(aElement: TElement);
var
   FU: Tgis_ForestUnit;
   aT: Tgis_Taxator;
begin
   if not (aElement is Tgis_ForestUnit) then
      exit;
   if aElement.IsMyOwner(Self) then
      exit;
   inherited;
   FU := Tgis_ForestUnit(aELement);
   aT := FU._Taxator;
   if aT <> Self then
      aT.ResetFU(FU);
end;

procedure Tgis_Taxator.ResetFU(aFU: Tgis_ForestUnit);
var
   TPro: string;
   FUClass: TClass;
   MyFU: Tgis_ForestUnit;
begin
   if aFU = nil then exit;
   FUClass := aFU.ClassType;
   TPro := aFU.TaxPro;
   MyFU := nil;
   if Tpro = 'ODD' then
      MyFU := aFU._Oddeleni
   else
   if Tpro = 'DIL' then
      MyFU := aFU._Dilec
   else
   if Tpro = 'POR' then
      MyFU := aFU._Porost;
   if MyFU = nil then exit;
   if MyFU.ClassType = FUClass then exit;
   if not (MyFU.IsMyOwner(Self)) then exit;
   RemoveElement(MyFU);
   MyFU.Reset_Taxator(Self,FUClass);
end;

INITIALIZATION
   RegisterOrbisClass(Tgis_UserCollection);
   RegisterOrbisClass(Tgis_TezbaCollection);
   RegisterOrbisClass(Tgis_ForestUnit);
   RegisterOrbisClass(Tgis_Etaz);
   RegisterOrbisClass(Tgis_PorostniSkupina);
   RegisterOrbisClass(Tgis_PorostniSkupinaCollection);
   RegisterOrbisClass(Tgis_Porost);
   RegisterOrbisClass(Tgis_PorostCollection);
   RegisterOrbisClass(Tgis_Dilec);
   RegisterOrbisClass(Tgis_DilecCollection);
   RegisterOrbisClass(Tgis_Oddeleni);
   RegisterOrbisClass(Tgis_OddeleniCollection);
   RegisterOrbisClass(Tgis_LS);
   RegisterOrbisClass(Tgis_LSCollection);
   RegisterOrbisClass(Tgis_LHC);
   RegisterOrbisClass(Tgis_ForestModel);
   RegisterOrbisClass(Tgis_Taxator);
   RegisterOrbisClass(Tgis_Taxator_Collection);
END.
