unit t_LHCUtils;

interface
uses
   e_LHCModel,
   Elements,
   G_Elems,
   Tools;

type
{
  Input  - Entita - linern seznam ploch typu Tgis_PorostniSkupina
  Output - Entita vybranch ploch
  Min    - Minimln hodnota sla PSK
  Max    - Maximln
     Vybere skupiny s slem (prvn numerick hodnota v nzvu) lecm
     v danm intervalu
}
TSelectPorostniSkupinaNumber = class(TTool)
  private
    FMin: integer;
    FMax: integer;
    procedure SetMax(const Value: integer);
    procedure SetMin(const Value: integer);
protected
   function NumberToPropertyName(aNumber: longInt): string; override;
public
   procedure DoIt; override;
published
   property Min: integer read FMin write SetMin;
   property Max: integer read FMax write SetMax;
end;

TSelectForestBorderLines = class(TTool)
  private
    FFrameClassName: string;
    FRefFrameClassName: string;
    FFrameClass: TElementClass;
    FRefFrameClass: TElementClass;
    procedure SetFrameClass(const Value: TElementClass);
    procedure SetFrameClassName(const Value: string);
    procedure SetRefFrameClass(const Value: TElementClass);
    procedure SetRefFrameClassName(const Value: string);
public
   procedure DoIt; override;
   property  FrameClass: TElementClass read FFrameClass write SetFrameClass;
   property  RefFrameClass: TElementClass read FRefFrameClass write SetRefFrameClass;
published
   property  FrameClassName: string read FFrameClassName write SetFrameClassName;
   property  RefFrameClassName: string read FRefFrameClassName write SetRefFrameClassName;
end;


TCreateLHCGeometry = class(TTool)
private
    FFreeEmptyUnit: boolean;
    FNoLines: boolean;
    FSKPColor: integer;
    FUnitClassName: string;
    FComment: string;
    FUnitClass: TElementClass;
    FLIN_DET: TEntity;
    FLIN_Unit: TEntity;
    FARE_Unit: TEntity;
    FForestModel: Tgis_ForestModel;
    FWithoutFinishPrompt: boolean;
    FDontRemoveOldGeometry: boolean;
    FFrameClassName: string;
    FFrameClass: TElementClass;
    FKURespect: boolean;
    procedure SetARE_Unit(const Value: TEntity);
    procedure SetComment(const Value: string);
    procedure SetForestModel(const Value: Tgis_ForestModel);
    procedure SetFreeEmptyUnit(const Value: boolean);
    procedure SetLIN_DET(const Value: TEntity);
    procedure SetLIN_Unit(const Value: TEntity);
    procedure SetNoLines(const Value: boolean);
    procedure SetSKPColor(const Value: integer);
    procedure SetUnitClass(const Value: TElementClass);
    procedure SetUnitClassName(const Value: string);
    procedure SetWithoutFinishPrompt(const Value: boolean);
    procedure SetDontRemoveOldGeometry(const Value: boolean);
    procedure SetFrameClassName(const Value: string);
    procedure SetFrameClass(const Value: TElementClass);
    procedure SetKURespect(const Value: boolean);
public
   procedure DoIt; override;
   procedure ResetAll;
   property  NoLines: boolean read FNoLines write SetNoLines;
   property  UnitClass: TElementClass read FUnitClass write SetUnitClass;
   property  FrameClass: TElementClass read FFrameClass write SetFrameClass;
published
   property  UnitClassName: string read FUnitClassName write SetUnitClassName;
   property  FrameClassName: string read FFrameClassName write SetFrameClassName;
   property  LIN_DET: TEntity read FLIN_DET write SetLIN_DET stored false;
   property  ARE_Unit: TEntity read FARE_Unit write SetARE_Unit stored false;
   property  LIN_Unit: TEntity read FLIN_Unit write SetLIN_Unit stored false;
   property  ForestModel: Tgis_ForestModel read FForestModel write SetForestModel stored false;
   property  Comment: string read FComment write SetComment;
   property  UnitColor: integer read FSKPColor write SetSKPColor;
   property  FreeEmptyUnit: boolean read FFreeEmptyUnit write SetFreeEmptyUnit;
   property  WithoutFinishPrompt: boolean read FWithoutFinishPrompt write SetWithoutFinishPrompt;
   property  DontRemoveOldGeometry: boolean read FDontRemoveOldGeometry write SetDontRemoveOldGeometry;
   property  KURespect: boolean read FKURespect write SetKURespect;
end;

TCheckForestConnection = class(TTool)
  private
    FDetLayer: TEntity;
    FForestModel: Tgis_ForestModel;
    FFixPorost: boolean;
    procedure SetDetLayer(const Value: TEntity);
    procedure SetForestModel(const Value: Tgis_ForestModel);
    procedure SetFixPorost(const Value: boolean);
public
   procedure DoIt; override;
published
   property DetLayer: TEntity read FDetLayer write SetDetLayer;
   property ForestModel: Tgis_ForestModel read FForestModel write SetForestModel;
   property FixPorost: boolean read FFixPorost write SetFixPorost;
end;

TJoinForestModel = class(TTool)
  private
    FForestModel1: Tgis_ForestModel;
    FCHeckLHCNumber: boolean;
    function GetForestModel: Tgis_ForestModel;
    procedure SetForestModel(const Value: Tgis_ForestModel);
    procedure SetForestModel1(const Value: Tgis_ForestModel);
    procedure SetCHeckLHCNumber(const Value: boolean);
protected
   function NumberToPropertyName(aNumber: longInt): string; override;
public
   procedure DoIt; override;
published
   property ForestModel: Tgis_ForestModel read GetForestModel write SetForestModel;
   property ForestModel1: Tgis_ForestModel read FForestModel1 write SetForestModel1;
   property CHeckLHCNumber: boolean read FCHeckLHCNumber write SetCHeckLHCNumber;
end;

TDuplicateForestModelForSelect = class(TJoinForestModel)
  private
    FZAKAreas: TEntity;
    FSelect: TEntity;
    FDETAreas: TEntity;
    FSelectIsZAK: boolean;
    procedure SetDETAreas(const Value: TEntity);
    procedure SetSelect(const Value: TEntity);
    procedure SetZAKAreas(const Value: TEntity);
    procedure SetSelectIsZAK(const Value: boolean);
protected
   function NumberToPropertyName(aNumber: longInt): string; override;
public
   procedure DoIt; override;
published
   property Select: TEntity read FSelect write SetSelect;
   property DETAreas: TEntity read FDETAreas write SetDETAreas;
   property ZAKAreas: TEntity read FZAKAreas write SetZAKAreas;
   property SelectIsZAK: boolean read FSelectIsZAK write SetSelectIsZAK;
end;

TSolveForestModelMishMash = class(TTool)
  private
    FFreeEmptyItems: boolean;
    FEmptyUnits: integer;
    FZAKAreas: TEntity;
    FDETAreas: TEntity;
    FSilentMode: boolean;
    function GetForestModel: Tgis_ForestModel;
    procedure SetForestModel(const Value: Tgis_ForestModel);
    procedure SetFreeEmptyItems(const Value: boolean);
    procedure SetEmptyUnits(const Value: integer);
    procedure SetDETAreas(const Value: TEntity);
    procedure SetZAKAreas(const Value: TEntity);
    procedure SetSilentMode(const Value: boolean);
public
   procedure DoIt; override;
   procedure SolveForestUnit(Value: Tgis_ForestUnit);
   property  EmptyUnits: integer read FEmptyUnits write SetEmptyUnits;
published
   property ForestModel: Tgis_ForestModel read GetForestModel write SetForestModel;
   property FreeEmptyItems: boolean read FFreeEmptyItems write SetFreeEmptyItems;
   property ZAKAreas: TEntity read FZAKAreas write SetZAKAreas;
   property DETAreas: TEntity read FDETAreas write SetDETAreas;
   property SilentMode: boolean read FSilentMode write SetSilentMode;
end;

TVyrovnani_ZAK_DET = class(TTool)
  private
    FOE: TRectEntity;
    FaDigit: integer;
    FFillVymeraByPlocha: boolean;
    FComment: string;
    FByGeometry: boolean;
    procedure SetaDigit(const Value: integer);
    procedure SetFillVymeraByPlocha(const Value: boolean);
    function GetOE: TRectEntity;
    procedure SetComment(const Value: string);
    procedure SetByGeometry(const Value: boolean);
public
   procedure DoIt; override;
   procedure ProcessPorost(aPorost: Tgis_Porost);
   procedure ProcessPorostByGeometry(aPorost: Tgis_Porost);
   property  OE: TRectEntity read GetOE;
published
   property  Comment: string read FComment write SetComment;
   property  aDigit: integer read FaDigit write SetaDigit;
   property  FillVymeraByPlocha: boolean read FFillVymeraByPlocha write SetFillVymeraByPlocha;
   property  ByGeometry: boolean read FByGeometry write SetByGeometry;
end;

implementation
uses
   Dialogs,
   Forms,
   SysUtils,
   e_Class,
   e_Geometry,
   e_GeometryElement,
   e_TopolElements,
   u_Math,
   m_Orbis,
   t_AreMak,
   t_ConfirmDialog,
   t_CopyLinesFromTo,
   t_ForestModelSelector,
   t_LHCTestTools,
   u_String;

{ TSelectPorostniSkupinaNumber }

procedure TSelectPorostniSkupinaNumber.DoIt;
var
   i,ii: integer;
   PSKNumber: integer;
   E1,E2: TEntity;
   E: TElement;
   PSK: Tgis_PorostniSkupina;
   S,SV: string;
begin
   if Input = nil then exit;
   if not (Input is TEntity) then exit;
   E1 := TEntity(Input);
   if Output = nil then  // chyb kontrola zda output je entita - snad jo
   begin
      Output := TAreaCollection.Create;
   end;
   if not (Output is TEntity) then exit; // u nechyb
   E2 := TEntity(Output);
   for i := 0 to E1.LastIndex do
   begin
      E := E1.Elements[i];
      if not (E is Tgis_PorostniSkupina) then continue;
      PSK := Tgis_PorostniSkupina(E);
      if PSK.NoGeometry then Continue;
      SV := '';
      S := PSK.Skupina;
      ii := 1;
      if S <> '' then
      begin
         while not (ii > Length(S)) and (S[ii]=' ') do Inc(ii);
         while not (ii > Length(S)) and (S[ii] in ['0'..'9']) do
         begin
            SV := SV + S[ii];
            Inc(ii);
         end;
         PSKNumber := StrToIntDef(SV,0);
         if (PSKNumber >= Min) and (PSKNumber <= Max) then
         begin
            E2.AddElement(PSK);
         end;
      end;
   end;
   if (E2 is TRectEntity) then
      TRectEntity(E2).ResetAndUpdate;
end;

function TSelectPorostniSkupinaNumber.NumberToPropertyName(aNumber: Integer): string;
begin
  case aNumber of
     0: result := 'Min';
     1: result := 'Max';
  end;
end;

procedure TSelectPorostniSkupinaNumber.SetMax(const Value: integer);
begin
  FMax := Value;
end;

procedure TSelectPorostniSkupinaNumber.SetMin(const Value: integer);
begin
  FMin := Value;
end;

{ TCreateLHCGeometry }

procedure TCreateLHCGeometry.DoIt;
var
   i,j: integer;
   T: TForestBorderTestTool;
   e: TElement;
   TL: TTopolLineElement;
   L: TPolyLineElement;
   ACL: TAreasFromClearLines;
   En : TEntity;
   FMS: TForestModelSelector;
   Are: TPolyLineAreaElement;
   PSK: Tgis_ForestUnit;
   OE: TRectEntity;
   TCL: TCopyLinesFromTo;
   S: string;
begin
   Error := 0;
   if LIN_DET = nil then Error := 1;
   if ARE_Unit = nil then Error := 2;
   if ForestModel = nil then Error := 4;
   if LIN_Unit = nil then
      NoLines := True;
   if Error > 0 then
   begin
      case Error of
         1: S := 'LIN_DET';
         2: S := 'ARE_Unit';
         //3: S := 'LIN_SKP';
         4: S := 'Model LHC';
      end;
      ShowMessage(Comment+#13+#10+'Povinn parametr '+S+' chyb');
      exit;
   end;
   if not (ForestModel is Tgis_ForestModel) or ForestModel.NoElements then
   begin
      ShowMessage(Comment+#13+#10+'Model LHC nen modelem LHC nebo neobsahuje dn objekty!');
      exit;
   end;
   if not ARE_Unit.NoElements and not DontRemoveOldGeometry then
   begin
      if not Confirm('Jednotky PRL u jakousi geometrii maj, ta bude zruena a nahrazena novou. Pokraovat?') then
         exit;
   end;
   if UnitClassName <> '' then
   begin
      UnitClass := GetOrbisClass(UnitClassName);
   end;
   if UnitClass = nil then
   begin
      if not Confirm('Chyb (nebo je patn)tda Jednotky PRL['+UnitClassName+'] a bude tud nahrazena Porostem (Tgis_Porost). Pokraovat?') then
         exit;
   end;
   if FrameClassName <> '' then
   begin
      FrameClass := GetOrbisClass(FrameClassName);
   end
   else
      FrameClass := TGis_PorostniSkupina;
   StartProgress(Comment,0,0);
   ResetAll;
   if LIN_DET = nil then exit;
   ProgressText := 'Penos lini z DET...';
   MaxProgress := LIN_DET.LastIndex;
   Progress := 0;
   Application.ProcessMessages;
   T := TForestBorderTestTool.Create;
   T.FrameClass := UnitClass;
   T.RefFrameClass := FrameClass;
   T.KURespect := KURespect;
   OE := TRectEntity.Create;
   for i := 0 to LIN_DET.LastIndex do
   begin
      if (i mod 50) = 0  then
      begin
         Progress := i;
         Application.ProcessMessages;
      end;
      e := LIN_DET.Elements[i];
      if not(e is TTopolLineElement) then continue;
      TL := TTopolLineElement(e);
      L := TPolyLineElement(TL.Geometry);
      T.Input := L;
      T.DoIt;
      If not T.Result then
         OE.AddElement(TL);
   end;
   OE.ResetAndUpdate;
   Progress := 0;
   MaxProgress := 0;
   ProgressText := 'kopie lini...';
   Application.ProcessMessages;
   if not NoLines then
   begin
      TCL := TCopyLinesFromTo.Create;
      TCL.Input := OE;
      TCL.Output := LIN_Unit;
      TCL.DoIt;
      TCL.Free;
   end;
   //OE.Free;
   {
   if (LIN_SKP is TRectEntity) then
      TRectEntity(LIN_SKP).Update;
   }
   ProgressText := 'Tvorba ploch Jednotek PRL ...';
   Application.ProcessMessages;
   ACL := TAreasFromClearLines.Create;
   ACL.Input := OE;
   ACL.DoIt;
   ForestModel.UpdateAll := true;
   FMS := TForestModelSelector.Create;
   FMS.FrameClass := Tgis_ForestUnitClass(FrameClass); // normalne porostniskupina
   FMS.ReferenceFrameClass := Tgis_ForestUnitClass(UnitClass);
   FMS.FindInput;
   En := TEntity(ACL.Output);
   for i := 0 to En.LastIndex do
   begin
      e := En.Elements[i];
      if not (e is TPolyLineAreaElement) then continue;
      Are := TPolyLineAreaElement(e);
      FMS.X := Are.X;
      FMS.Y := Are.Y;
      FMS.DoIt;
//      PSK := Tgis_PorostniSkupina(FMS.ActualFrame);
      PSK := FMS.ActualFrame;
      if PSK <> nil then
      begin
         if FMS.ActualReferenceFrame <> nil then
            FMS.ActualReferenceFrame.AddGeometry(Are);
      end;
   end;
   ForestModel.UpdateAll := true;
   En.Free;
   ACL.Free;
   FMS.Free;
   OE.Free;
   if not WithoutFinishPrompt then
      ShowMessage('Operace se podaila (patrn)...');
   PostMsg(tm_ModelChanged,nil,nil,false);
end;

procedure TCreateLHCGeometry.ResetAll;
var
   L: Tgis_LHC;
   i,j: integer;
   E: TElement;
   S: string;

   procedure ResetTargetGeometry(aUnit: Tgis_ItemsForestUnit;aClass: TElementClass);
   var
      k: integer;
      TargetLevel: boolean;
      FU: Tgis_ForestUnit;
   begin
      if aUnit = nil then exit;
      if aUnit.ClassType = aClass then
      begin
         if not DontRemoveOldGeometry then
            aUnit.Geometry := nil;
         aUnit.Barva := UnitColor;
         aUnit.Znacka := 1;
         if not DontRemoveOldGeometry then
            ARE_Unit.AddElement(aUnit)
         else
         begin
            if ARE_Unit.IndexOf(aUnit) < 0 then
               ARE_Unit.AddElement(aUnit);
         end;
         exit;
      end;
      if aUnit.NoItems then exit;
      TargetLevel := aUnit.ItemClass = aClass; // Konen rove, n se nepoleze
      for k := 0 to aUnit.Items.LastIndex do
      begin
         FU := aUnit.Items.ForestUnit[k];
         if TargetLevel then
         begin
            if not DontRemoveOldGeometry then
               FU.Geometry := nil;
            FU.Barva := UnitColor;
            FU.Znacka := 1;
            if not DontRemoveOldGeometry then
               ARE_Unit.AddElement(FU)
            else
            begin
               if ARE_Unit.IndexOf(FU) < 0 then
                  ARE_Unit.AddElement(FU);
            end;
         end
         else   // jet nejsme v clov rovni, budeme se noit
         begin
            if not (FU is Tgis_ItemsForestUnit) then continue; // to je asi blbost
            ResetTargetGeometry(Tgis_ItemsForestUnit(FU),aClass);
         end;
      end;
   end;
begin
   Progress := 0;
   S := 'Ruen stvajcch Jednotek PRL';
   ProgressText := S+' Plochy ...';
   Application.ProcessMessages;
   if not DontRemoveOldGeometry then
      ARE_Unit.FreeAll;
   ProgressText := S+' Linie ...';
   Application.ProcessMessages;
   if not DontRemoveOldGeometry then
   if LIN_Unit <> nil then
      LIN_Unit.FreeAll;;
   //MaxProgress := ParcelModel.LastIndex;
   S := 'Inicializace jednotek PRL: ';
   ProgressText := S;
   Progress := 0;
   Application.ProcessMessages;
   for i := 0 to ForestModel.LastIndex do
   begin
      E := ForestModel.Elements[i];
      if not (e is Tgis_LHC) then continue;
      L := Tgis_LHC(E);
      ResetTargetGeometry(L,UnitClass);
   end;
   if (ARE_Unit is TRectEntity) then
      TRectEntity(ARE_Unit).Update;
end;

procedure TCreateLHCGeometry.SetARE_Unit(const Value: TEntity);
begin
  FARE_Unit := Value;
end;

procedure TCreateLHCGeometry.SetComment(const Value: string);
begin
  FComment := Value;
end;

procedure TCreateLHCGeometry.SetDontRemoveOldGeometry(
  const Value: boolean);
begin
  FDontRemoveOldGeometry := Value;
end;

procedure TCreateLHCGeometry.SetForestModel(const Value: Tgis_ForestModel);
begin
  FForestModel := Value;
end;

procedure TCreateLHCGeometry.SetFrameClass(const Value: TElementClass);
begin
  FFrameClass := Value;
end;

procedure TCreateLHCGeometry.SetFrameClassName(const Value: string);
begin
  FFrameClassName := Value;
end;

procedure TCreateLHCGeometry.SetFreeEmptyUnit(const Value: boolean);
begin
  FFreeEmptyUnit := Value;
end;

procedure TCreateLHCGeometry.SetKURespect(const Value: boolean);
begin
  FKURespect := Value;
end;

procedure TCreateLHCGeometry.SetLIN_DET(const Value: TEntity);
begin
  FLIN_DET := Value;
end;

procedure TCreateLHCGeometry.SetLIN_Unit(const Value: TEntity);
begin
  FLIN_Unit := Value;
end;

procedure TCreateLHCGeometry.SetNoLines(const Value: boolean);
begin
  FNoLines := Value;
end;

procedure TCreateLHCGeometry.SetSKPColor(const Value: integer);
begin
  FSKPColor := Value;
end;

procedure TCreateLHCGeometry.SetUnitClass(const Value: TElementClass);
begin
  FUnitClass := Value;
end;

procedure TCreateLHCGeometry.SetUnitClassName(const Value: string);
begin
  FUnitClassName := Value;
end;

procedure TCreateLHCGeometry.SetWithoutFinishPrompt(const Value: boolean);
begin
  FWithoutFinishPrompt := Value;
end;

{ TSelectForestBorderLines }

procedure TSelectForestBorderLines.DoIt;
var
   E,Ei: TEntity;
   i: integer;
   el: TElement;
   TL: TTopolLineElement;
   G: TGeometry;
   T: TForestBorderTestTool;
begin
   if FrameClass = nil then
   begin
      if FrameClassName <> '' then
      begin
         FrameClass := GetOrbisClass(FrameClassName);
         if FrameClass = nil then
            ShowMessage ('ForestBorderTestTool: UnknownClass - '+FrameClassName);
      end;
      if FrameClass = nil then
         FrameClass := Tgis_Porost;
   end;
   if RefFrameClass = nil then
   begin
      if RefFrameClassName <> '' then
      begin
         RefFrameClass := GetOrbisClass(RefFrameClassName);
         if RefFrameClass = nil then
            ShowMessage ('ForestBorderTestTool: UnknownClass - '+RefFrameClassName);
      end;
      if RefFrameClass = nil then
         RefFrameClass := Tgis_PorostniSkupina;
   end;
   if Output = nil then
      Output := TPolyLineCollection.Create;
   E := TEntity(Output);
   if Input = nil then exit;
   if not (Input is TEntity) then exit;
   Ei := TEntity(Input);
   T := TForestBorderTestTool.Create;
   T.FrameClass := FrameClass;
   T.RefFrameClass := RefFrameClass;
   For i := 0 to Ei.LastIndex do
   begin
      el := Ei.Elements[i];
      if not (el is TTopolLineElement) then continue;
      TL := TTopolLineElement(el);
      T.Input := TL.Geometry;
      T.DoIt;
      if not T.Result then
         E.AddElement(TL);
   end;
   if (E is TRectEntity) then
      TRectEntity(E).ResetAndUpdate;
   T.Free;
end;

procedure TSelectForestBorderLines.SetFrameClass(const Value: TElementClass);
begin
  FFrameClass := Value;
end;

procedure TSelectForestBorderLines.SetFrameClassName(const Value: string);
begin
  FFrameClassName := Value;
end;

procedure TSelectForestBorderLines.SetRefFrameClass(const Value: TElementClass);
begin
  FRefFrameClass := Value;
end;

procedure TSelectForestBorderLines.SetRefFrameClassName(const Value: string);
begin
  FRefFrameClassName := Value;
end;

{ TCheckForestConnection }

procedure TCheckForestConnection.DoIt;
var
   FMS: TForestModelSelector;
   i,j: integer;
   e: TElement;
   Sk: Tgis_PorostniSkupina;
   ErrL,ErrM: integer;
   ERE: TOwnerEntity;
   ErLost: TOwnerEntity;
   ErMulti:TOwnerEntity;
   aX,aY: Double;
   BasePOR,POR: Tgis_Porost;
   MA: TMultiAreaElement;
   FixPorostValue: Char;

   function TestPOR: Tgis_Porost;
   begin
      result := nil;
      FMS.X := aX;
      FMS.Y := aY;
      FMS.DoIt;
      result := Tgis_Porost(FMS.ActualFrame);
   end;

   procedure FillPorost;
   var
      aPor: Tgis_Porost;
   begin
      if POR = nil then exit;
      if FixPorost then
      begin
         ForestModel.ActualDilec := POR._Dilec;
         aPor := ForestModel.CreatePorostName(FixPorostValue);
         SK._Porost := aPor;
      end
      else
      begin
         SK._Porost := POR;
      end;
   end;

   procedure AddToLost;
   begin
      ErLost.AddElement(Sk);
      Inc(ErrL);
      ProgressText := IntToStr(ErrL)+' Porost chyb a '+IntToStr(ErrM)+' Skupin ve vce porostech';
      Application.ProcessMessages;
   end;

   procedure AddToMulti;
   begin
      ErMulti.AddElement(Sk);
      Inc(ErrM);
      ProgressText := IntToStr(ErrL)+' Porost chyb a '+IntToStr(ErrM)+' Skupin ve vce porostech';
      Application.ProcessMessages;
   end;
begin
   if DetLayer = nil then
   begin
      ShowMessage('Nen nastavena vrstva ploch DETailu');
      exit;
   end;
   FMS := TForestModelSelector.Create;
   FMS.FrameClassName := 'Tgis_Porost';
   if ForestModel = nil then
   begin
      FMS.FindInput;
      ForestModel := FMS.ForestModel;
   end
   else
      FMS.ForestModel := ForestModel;
   if ForestModel = nil then
   begin
      ShowMessage('Nelze najt Model LHC');
      FMS.Free;
      exit;
   end;
   ForestModel.UpdateAll := true;
   StartProgress('Kontrola propojen Lesa(DET) a Lesa(ZAK)',0,DetLayer.LastIndex);
   ErrL := 0;
   ErrM := 0;
   ERE := TOwnerEntity.Create;
   ERE.Name := 'Chyby propojen DETailu na ZAK';
   AddElement(ERE);
   Output := ERE;
   ErLost := TOwnerEntity.Create;
   ERE.AddElement(ErLost);
   ErLost.Name := 'Pod st skupiny chyb porost';
   ErMulti := TOwnerEntity.Create;
   ERE.AddElement(ErMulti);
   ErMulti.Name := 'Skupina ve vce porostech';
   for i := 0 to DetLayer.LastIndex do
   begin
      progress := i;
      Application.ProcessMessages;
      e := DetLayer.Elements[i];
      if not(e is Tgis_PorostniSkupina) then continue;
      Sk := Tgis_PorostniSkupina(e);
      BasePOR := nil;
      POR := nil;
      FixPorostValue := Sk.Porost;
      if Sk.NoGeometry then
      begin
         aX := 0;
         aY := 0;
      end
      else
      begin
         if Sk.Geometry is TMultiAreaElement then
         begin
            MA := TMultiAreaElement(Sk.Geometry);
            BasePOR := nil;
            for j := 0 to MA.LastIndex do
            begin
               aX := MA.AreaElement[j].X;
               aY := MA.AreaElement[j].Y;
               POR := TestPOR;
               if POR = nil then
               begin
                  AddToLost;
                  continue;
               end;
               if BasePOR = nil then
               begin
                  BasePOR := POR;
                  //SK._Porost := POR;
                  FillPorost;
               end
               else
               begin
                  if BasePOR <> POR then
                  begin
                     AddToMulti;
                     Break;
                  end;
               end;
            end;
         end
         else
         begin
            aX := Sk.Geometry.X;
            aY := Sk.Geometry.Y;
            POR := TestPOR;
            //SK._Porost := POR;
            FillPorost;
            if POR = nil then
               AddToLost;
         end;
      end;
   end;
   FMS.Free;
   ShowMessage('Hotovo, '+IntToStr(DetLayer.Count)+ ' ploch aktualizovno, '+IntToStr(ErrL)+' porost nenalezeno, '+IntToStr(ErrM)+'por.sk ve vce porostech');
end;

procedure TCheckForestConnection.SetDetLayer(const Value: TEntity);
begin
  FDetLayer := Value;
end;


procedure TCheckForestConnection.SetFixPorost(const Value: boolean);
begin
  FFixPorost := Value;
end;

procedure TCheckForestConnection.SetForestModel(
  const Value: Tgis_ForestModel);
begin
  FForestModel := Value;
end;

{ TJoinForestModel }

procedure TJoinForestModel.DoIt;
var
   i,j,k,ii: integer;
   L1: Tgis_LHC;
   O1: Tgis_Oddeleni;
   D1: Tgis_Dilec;
   LS1: Tgis_LS;
begin
   if (ForestModel = nil) or (ForestModel1 = nil) then
   begin
      ShowMessage('Chyb jeden z model LHC pro spojen!');
      exit;
   end;
   for i := 0 to ForestModel1.LastIndex do
   begin
      L1 := Tgis_LHC(ForestModel1.Elements[i]);
      // tady by asi mla bt kontrola sla LHC
      if CHeckLHCNumber or Forestmodel.NoElements then
      begin
         ForestModel.CreateLHCCislo(L1.Cislo);
      end
      else
      begin
         ForestModel.ActualLHC := Tgis_LHC(ForestModel.Elements[0]);
      end;
      if L1.NoItems then continue;
      for j := 0 to L1.Items.LastIndex do
      begin
         LS1 := Tgis_LS(L1.Items.ForestUnit[j]);
         ForestModel.CreateLSCislo(LS1.Cislo);
         if LS1.NoItems then continue;
         for ii := 0 to LS1.Items.LastIndex do
         begin
            O1 := TGis_Oddeleni(LS1.Items.ForestUnit[ii]);
            ForestModel.CreateOddeleniCislo(O1.Cislo);
            if O1.NoItems then continue;
            for k := 0 to O1.Items.LastIndex do
            begin
               D1 := Tgis_Dilec(O1.Items.FOrestUnit[k]);
   { TODO -oPPK -cDodlat : kontrola a uloen duplicit }
               if ForestModel.ActualOddeleni.Items = nil then
                  ForestModel.ActualOddeleni.Items := Tgis_DilecCollection.Create;
               ForestModel.ActualOddeleni.Items.AddElement(D1);
            end;
         end;
      end;
   end;
   ForestModel.UpdateAll := true;
   ForestModel.Sort := true;
end;

function TJoinForestModel.GetForestModel: Tgis_ForestModel;
begin
   result := Tgis_ForestModel(Input);
end;

function TJoinForestModel.NumberToPropertyName(aNumber: Integer): string;
begin
  case aNumber of
     0: result := 'ForestModel1';
     1: result := 'ForestModel';
  end;
end;

procedure TJoinForestModel.SetCHeckLHCNumber(const Value: boolean);
begin
  FCHeckLHCNumber := Value;
end;

procedure TJoinForestModel.SetForestModel(const Value: Tgis_ForestModel);
begin
   Input := Value;
end;

procedure TJoinForestModel.SetForestModel1(const Value: Tgis_ForestModel);
begin
  FForestModel1 := Value;
end;

{ TSolveForestModelMishMash }

procedure TSolveForestModelMishMash.DoIt;
var
   i: integer;
begin
   Error := 0;
   EmptyUnits := 0;
   if (ForestModel = nil) or not (Input is Tgis_ForestModel) then
   begin
      ShowMessage('Model LHC chyb nebo nen modelem LHC!');
      exit;
   end;
   for i := 0 to ForestModel.LastIndex do
   begin
      SolveForestUnit(Tgis_ForestUnit(ForestModel.Elements[i]));
   end;
   //if (Error > 0) or (EmptyUnits > 0) then
   if not SilentMode then
      ShowMessage('V modelu LHC bylo odstranno '+IntToStr(Error)+' duplicit a myma a '+IntToStr(EmptyUnits)+' przdnch jednotek');
end;

function TSolveForestModelMishMash.GetForestModel: Tgis_ForestModel;
begin
   result := Tgis_ForestModel(Input);
end;

procedure TSolveForestModelMishMash.SetDETAreas(const Value: TEntity);
begin
  FDETAreas := Value;
end;

procedure TSolveForestModelMishMash.SetEmptyUnits(const Value: integer);
begin
  FEmptyUnits := Value;
end;

procedure TSolveForestModelMishMash.SetForestModel(const Value: Tgis_ForestModel);
begin
   Input := Value;
end;

procedure TSolveForestModelMishMash.SetFreeEmptyItems(
  const Value: boolean);
begin
  FFreeEmptyItems := Value;
end;

procedure TSolveForestModelMishMash.SetSilentMode(const Value: boolean);
begin
  FSilentMode := Value;
end;

procedure TSolveForestModelMishMash.SetZAKAreas(const Value: TEntity);
begin
  FZAKAreas := Value;
end;

procedure TSolveForestModelMishMash.SolveForestUnit(Value: Tgis_ForestUnit);
var
   i: integer;
   IValue,IFU,IFFU: Tgis_ItemsForestUnit;
   CValue: Tgis_CisloForestUnit;
   FU,FFU: Tgis_ForestUnit;
   _Cislo: integer;
   _Name: string;
begin
   IValue := nil;
   CValue := nil;
   if not (Value is Tgis_ItemsForestUnit) then exit;
   IValue := Tgis_ItemsForestUnit(Value);
   if IValue.NoItems then exit;
   if Value is Tgis_CisloForestUnit then
      CValue := Tgis_CisloForestUnit(Value);
   for i := IValue.Items.LastIndex downto 0 do
   begin
      FFU := nil;
      FU := IValue.Items.ForestUnit[i];
      if FU is Tgis_Dilec then
      begin
         FU.Name := RemoveAllSpaces(FU.Name);
         FU.Name := UpperCase(FU.Name[1]);
         if not (FU.Name[1] in ['A'..'Z']) then
            FU.Name := 'A';
      end
      else if FU is Tgis_Porost then
      begin
         FU.Name := RemoveAllSpaces(FU.Name);
         FU.Name := LowerCase(FU.Name[1]);
         if not (FU.Name[1] in ['a'..'z']) then
            FU.Name := 'a';
      end;
      if FU is Tgis_CisloForestUnit then
      begin
         if CValue <> nil then
            FFU := CValue.GetItemCislo(TGis_CisloForestUnit(FU).Cislo);
      end
      else
      begin
         if IValue <> nil then
            FFU := IValue.GetItemName(FU.Name);
      end;
      if FFU <> FU then  // je tam vckrt
      begin
         if FU is TGis_ItemsForestUnit then
         begin
            IFU := Tgis_ItemsForestUnit(FU);
            IFFU := Tgis_ItemsForestUnit(FFU);
            if not IFU.NoItems then
            begin
               IFFU.CheckItems;
               IFFU.Items.AddAllFrom := IFU.Items;
            end;
         end;
         if not FU.NoGeometry then
            FFU.AddGeometry(FU.Geometry);
         FU.Free;
         Error := Error + 1;
         Continue;
      end;
      SolveForestUnit(FU);
      if FreeEmptyItems then
      begin
         IFU := nil;
         if FU is TGis_ItemsForestUnit then
            IFU := Tgis_ItemsForestUnit(FU);
         if IFU <> nil then
         begin
            if IFU.NoItems and FU.NoGeometry then
            begin
               EmptyUnits := EmptyUnits + 1;
               FU.Free;
               FU := nil;
            end;
         end
         else
         if (FU is Tgis_PorostniSkupina) then
         begin
            if FU.NoGeometry then
            begin
               EmptyUnits := EmptyUnits + 1;
               FU.Free;
               FU := nil;
            end;
         end;
      end;
      if FU <> nil then // kontrola zarazeni do grafiky
      begin
         if (FU is Tgis_Porost) and (ZAKAreas <> nil) then
         begin
            if FU.NoGeometry then continue;
            if FU.IsMyOwner(ZAKAreas) then continue;
            ZAKAreas.AddElement(FU);
         end
         else
         if (FU is Tgis_PorostniSkupina) and (DETAreas <> nil)then
         begin
            if FU.NoGeometry then continue;
            if FU.IsMyOwner(DETAreas)then continue;
            DETAreas.AddElement(FU);
         end;
      end;
   end;
end;

{ TVyrovnani_ZAK_DET }

procedure TVyrovnani_ZAK_DET.DoIt;
var
   FM: Tgis_ForestModel;
   ilhc,ils,iodd,iDil,iPor: integer;
   LHC: Tgis_LHC;
   LS: Tgis_LS;
   Odd: Tgis_Oddeleni;
   Dil: Tgis_Dilec;
   Por: Tgis_Porost;
   e: TElement;
begin
 { TODO -oPPK -cOpravit : Pro prochzen LHC udlat LHC Recursor (ProcessPorost ap.) }
   if Input = nil then exit;
   if not (Input is Tgis_ForestModel) then exit;
   FM := Tgis_ForestModel(Input);
   StartProgress(Comment,0,0);
   for iLhc := 0 to FM.LastIndex do
   begin
      e := FM.Elements[iLhc];
      if not (e is Tgis_LHC) then continue;
      LHC := Tgis_LHC(e);
      if LHC.NoItems then continue;
      for iLS := 0 to LHC.Items.LastIndex do
      begin
         e := LHC.Items.ForestUnit[iLS];
         if not (e is Tgis_LS) then continue;
         LS := Tgis_LS(e);
         if LS.NoItems then continue;
         MaxProgress := LS.Items.LastIndex;
         for iOdd := 0 to LS.Items.LastIndex do
         begin
            Progress := iOdd;
            e := LS.Items.Elements[iOdd];
            if not (e is Tgis_Oddeleni) then continue;
            Odd := Tgis_Oddeleni(e);
            if Odd.NoItems then continue;
            for iDil := 0 to Odd.Items.LastIndex do
            begin
               e := Odd.Items.ForestUnit[iDil];
               if not (e is Tgis_Dilec) then continue;
               DIl := Tgis_Dilec(e);
               if Dil.NoItems then continue;
               for iPor := 0 to Dil.Items.LastIndex do
               begin
                  e := DIl.Items.ForestUnit[iPor];
                  if not (e is Tgis_Porost)then exit;
                  Por := Tgis_Porost(e);
                  ProgressText := 'vyrovnn:'+LHC.Name + '-'+ Odd.Name + '-'+Dil.Name+'-'+Por.Name;
                  Application.ProcessMessages;
                  if ByGeometry then
                     ProcessPorostByGeometry(Por)
                  else
                     ProcessPorost(Por);
               end;
            end;
         end;
      end;
   end;
end;


function TVyrovnani_ZAK_DET.GetOE: TRectEntity;
begin
   if FOE = nil then
   begin
      FOE := TRectEntity.Create;
      FOE.Name := 'OE';
      AddElement(FOE);
      Output := FOE;
   end;
   result := FOE;
end;

procedure TVyrovnani_ZAK_DET.ProcessPorost(aPorost: Tgis_Porost);
var
   i,j,Ai: integer;
   Por: Tgis_Porost;
   PSK: Tgis_PorostniSkupina;
   G: TGeometry;
   A,AA,DArea: TAreaElement;
   refAre: double;
   MA: TMultiAreaElement;
   SumAre,dSumAre,SumAre1: double;
   Ares: Array[0..5]of TAreaElement;
   AreD: Array[0..5]of double;
   PorVym,PorPlo,PorVym_M: double;
   Kvalp: integer;

   procedure SetToAres(Are: TAreaElement);
   var
      ii,ij: integer;
   begin
      for ii := 0 to 5 do
      begin
         if Ares[ii] = nil then
         begin
            Ares[ii] := Are;
            exit;
         end
         else
         begin
            if Are.Area > Ares[ii].Area then
            begin
               for ij := 5 downto ii + 1  do
                  Ares[ij] := Ares[ij - 1];
               Ares[ii] := Are;
            end;
         end;
      end;
   end;
begin
   if aDigit = 0 then
      aDigit := -2;
   //if aSKPDigit = 0 then
   //   aSKPDigit := -4;

   //if aSKP.Vymera > 0 then
   //   aSKP.Vymera := Simple_RoundTo(aSKP.Vymera,aSKPDigit);
   // nalezen porostnch skupin
   // 1) v porostech
   // PredNastaveni vymer a secteni
   SumAre := 0;
   if aPorost.NoItems then
   begin
      OE.AddAndUpdateRect(aPorost);
      exit;
   end;
   PorVym := aPorost.Vymera;
   PorPlo := aPorost.Plocha;
   Kvalp := 1;
   if not aPorost.NoGeometry then
   begin
      G := aPorost.Geometry;
      A := TAreaElement(G);
      if A is TMultiAreaElement then
      begin
         MA := TMultiAreaElement(A);
         A := MA.AreaElement[0];
      end;
      Kvalp := A.FixedAreaIndex;
      if not (KvalP = 1) or (KvalP = 4) then
         KvalP := 1;
   end;
   PorVym_M := PorVym*10000;
   if Abs(PorPlo-PorVym_M)>(0.005*PorVym_M+2.5*SQRT(PorVym_M))then
   begin
      OE.AddAndUpdateRect(aPorost);
   end;
   for i := 0 to aPorost.Items.LastIndex do
   begin
      PSK := Tgis_PorostniSkupina(aPorost.Items.ForestUnit[i]);
      G := PSK.Geometry;
      if G = nil then continue;
      A := TAreaElement(G);
      if A is TMultiAreaElement then
      begin
         MA := TMultiAreaElement(A);
         for j := 0 to MA.LastIndex do
         begin
            A := MA.AreaElement[j];
            if FillVymeraByPlocha then
               A.FixedArea := Simple_RoundTo(A.Area/10000,aDigit)
            else
               A.FixedArea := 0;
            AA := TAreaElement(aPorost.GetSimpleGeometryByXY(A.X,A.Y,0));
            if AA <> nil then
               A.FixedAreaIndex := AA.FixedAreaIndex
            else
               A.FixedAreaIndex := KvalP;
            SumAre := SumAre + A.Area;
         end;
      end
      else
      begin
         if FillVymeraByPlocha then
            A.FixedArea := Simple_RoundTo(A.Area/10000,aDigit)
         else
            A.FixedArea := 0;
         AA := TAreaElement(aPorost.GetSimpleGeometryByXY(A.X,A.Y,0));
         if AA <> nil then
            A.FixedAreaIndex := AA.FixedAreaIndex
         else
            A.FixedAreaIndex := KvalP;
         SumAre := SumAre + A.Area;
      end;
   end;
   //if SumAre = 0 then exit;
   //if aSKP.KVAL_P <> 1 then exit; // nebude se vyrovnvat
   if PorVym = 0  then exit; // nebude se vyrovnvat
   for Ai := 0 to 5 do Ares[Ai] := nil;
   SumAre1 := 0;
   refAre := 0;
   DArea := nil;
   for i := 0 to aPorost.Items.LastIndex do
   begin
      PSK := Tgis_PorostniSkupina(aPorost.Items.ForestUnit[i]);
      G := PSK.Geometry;
      if G = nil then continue;
      A := TAreaElement(G);
      if A is TMultiAreaElement then
      begin
         MA := TMultiAreaElement(A);
         for j := 0 to MA.LastIndex do
         begin
            A := MA.AreaElement[j];
            A.FixedArea := Simple_RoundTo(Porvym / SumAre * A.Area,aDigit);
            SetToAres(A);
            SumAre1 := SumAre1 + A.FixedArea;
         end;
      end
      else
      begin
         A.FixedArea := Simple_RoundTo(Porvym / SumAre * A.Area,aDigit);
         SetToAres(A);
         SumAre1 := SumAre1 + A.FixedArea;
      end;
   end;
   // pokud souty nesed, srovnme to.
   SumAre := Simple_RoundTo(Porvym-SumAre1,aDigit);
   DSumAre := SumAre;
   SumAre1 := 0;
   for i := 0 to 5 do
      if Ares[i] <> nil then SumAre1 := SumAre1 + Ares[i].Area;
   if (Abs(SumAre)> 0) then
   begin
      for i := 0 to 5 do
      begin
         if Ares[i] = nil then
            AreD[i] := 0
         else
         begin
            AreD[i] := Simple_RoundTo(SumAre / SumAre1 * Ares[i].Area,aDigit);
            Ares[i].FixedArea := Ares[i].FixedArea + AreD[i];
            DSumAre := DSumAre - AreD[i];
         end;
      end;
      if DSumAre <> 0 then
         if Ares[0]<> nil then
            Ares[0].FixedArea := Ares[0].FixedArea + DSumAre;
   end;

end;

procedure TVyrovnani_ZAK_DET.ProcessPorostByGeometry(aPorost: Tgis_Porost);
var
   i,j,Ai: integer;
   Por: Tgis_Porost;
   PSK: Tgis_PorostniSkupina;
   G: TGeometry;
   A,AA,DArea: TAreaElement;
   EEA: TEntity;
   refAre: double;
   MA,PMA: TMultiAreaElement;

   SumAre,dSumAre,SumAre1: double;
   Ares: Array[0..5]of TAreaElement;
   AreD: Array[0..5]of double;
   PorVym,PorPlo: double;
   Kvalp: integer;

   procedure SetToAres(Are: TAreaElement);
   var
      ii,ij: integer;
   begin
      for ii := 0 to 5 do
      begin
         if Ares[ii] = nil then
         begin
            Ares[ii] := Are;
            exit;
         end
         else
         begin
            if Are.Area > Ares[ii].Area then
            begin
               for ij := 5 downto ii + 1  do
                  Ares[ij] := Ares[ij - 1];
               Ares[ii] := Are;
            end;
         end;
      end;
   end;

   procedure processPorostArea(anAre: TAreaElement);
   var
      i,j,Ai: integer;
      PorVym_M: double;
   begin
      SumAre := 0;
      EEA.FreeAll;
      PorVym := anAre.FixedArea;
      KvalP := anAre.FixedAreaIndex;
      PorPlo := anAre.Area;
      PorVym_M := PorVym*10000;
      if Abs(PorPlo-PorVym_M)>(0.005*PorVym_M+2.5*SQRT(PorVym_M))then
      begin
         OE.AddAndUpdateRect(anAre);
      end;
      for i := 0 to aPorost.Items.LastIndex do
      begin
         PSK := Tgis_PorostniSkupina(aPorost.Items.ForestUnit[i]);
         G := PSK.Geometry;
         if G = nil then continue;
         A := TAreaElement(G);
         if A is TMultiAreaElement then
         begin
            MA := TMultiAreaElement(A);
            for j := 0 to MA.LastIndex do
            begin
               A := MA.AreaElement[j];
               if anAre.IsHitByXY(A.X,A.Y,0) then
                  EEA.AddElement(A);
            end;
         end
         else
         if anAre.IsHitByXY(A.X,A.Y,0) then
            EEA.AddElement(A);
      end;

      for i := 0 to EEA.LastIndex do
      begin
         A := TAreaElement(EEA.Elements[i]);
         A.FixedArea := 0;
         A.FixedAreaIndex := KvalP;
         SumAre := SumAre + A.Area;
      end;
      //if SumAre = 0 then exit;
      //if aSKP.KVAL_P <> 1 then exit; // nebude se vyrovnvat
      if PorVym = 0  then exit; // nebude se vyrovnvat
      for Ai := 0 to 5 do Ares[Ai] := nil;
      SumAre1 := 0;
      refAre := 0;
      DArea := nil;
      for i := 0 to EEA.LastIndex do
      begin
         A := TAreaElement(EEA.Elements[i]);
         A.FixedArea := Simple_RoundTo(Porvym / SumAre * A.Area,aDigit);
         SetToAres(A);
         SumAre1 := SumAre1 + A.FixedArea;
      end;
      // pokud souty nesed, srovnme to.
      SumAre := Simple_RoundTo(Porvym-SumAre1,aDigit);
      DSumAre := SumAre;
      SumAre1 := 0;
      for i := 0 to 5 do
         if Ares[i] <> nil then SumAre1 := SumAre1 + Ares[i].Area;
      if (Abs(SumAre)> 0) then
      begin
         for i := 0 to 5 do
         begin
            if Ares[i] = nil then
               AreD[i] := 0
            else
            begin
               AreD[i] := Simple_RoundTo(SumAre / SumAre1 * Ares[i].Area,aDigit);
               Ares[i].FixedArea := Ares[i].FixedArea + AreD[i];
               DSumAre := DSumAre - AreD[i];
            end;
         end;
         if DSumAre <> 0 then
            if Ares[0]<> nil then
               Ares[0].FixedArea := Ares[0].FixedArea + DSumAre;
      end;
   end;
begin
   if aDigit = 0 then
      aDigit := -2;
   //if aSKPDigit = 0 then
   //   aSKPDigit := -4;

   //if aSKP.Vymera > 0 then
   //   aSKP.Vymera := Simple_RoundTo(aSKP.Vymera,aSKPDigit);
   // nalezen porostnch skupin
   // 1) v porostech
   // PredNastaveni vymer a secteni
   EEA := TOwnerEntity.Create;
   AddElement(EEA);
   if aPorost.NoItems then
   begin
      OE.AddAndUpdateRect(aPorost);
      EEA.Free;
      exit;
   end;
   Kvalp := 1;
   if not aPorost.NoGeometry then
   begin
      G := aPorost.Geometry;
      A := TAreaElement(G);
      if A is TMultiAreaElement then
      begin
         PMA := TMultiAreaElement(A);
         for i := 0 to PMA.LastIndex do
         begin
            A := PMA.AreaElement[i];
            processPorostArea(A);
         end;
      end
      else
         processPorostArea(A);
   end;
   EEA.Free;

end;

procedure TVyrovnani_ZAK_DET.SetaDigit(const Value: integer);
begin
  FaDigit := Value;
end;


procedure TVyrovnani_ZAK_DET.SetByGeometry(const Value: boolean);
begin
  FByGeometry := Value;
end;

procedure TVyrovnani_ZAK_DET.SetComment(const Value: string);
begin
  FComment := Value;
end;

procedure TVyrovnani_ZAK_DET.SetFillVymeraByPlocha(const Value: boolean);
begin
   FFillVymeraByPlocha := Value;
end;

{ TDuplicateForestModelForSelect }

procedure TDuplicateForestModelForSelect.DoIt;
var
   FM: Tgis_ForestModel;
   FU: Tgis_ForestUnit;
   LHC: Tgis_LHC;
   LS: Tgis_LS;
   Odd: Tgis_Oddeleni;
   Dil: Tgis_Dilec;
   Por: Tgis_Porost;
   PSK,NPSK: Tgis_PorostniSkupina;
   NewLHC: boolean;
   i,j: integer;
   e: TElement;

   procedure CopyPSK;
   begin
      if FM.GetSkupinaName(PSK.Name) <> nil then
      begin
         // to by mela byt chyba
      end;
      FM.CreateSkupinaName(PSK.Name);
      NPSK := FM.ActualSkupina;
      NPSK.VEK_ST := PSK.VEK_ST;
      NPSK.Barva := PSK.Barva;
      NPSK.Znacka := PSK.Znacka;
      NPSK.NextGeometry := PSK.Geometry;
      NPSK.LVS := PSK.LVS;
      NPSK.Properties := PSK.Properties;
      if DETAreas <> nil then
      begin
         DETAreas.AddElement(NPSK);
      end;
   end;
begin
   Error := 0;
   FM := ForestModel;
   if FM = nil then
   begin
      ShowMessage('Chyb Forest Model!');
      Error := 1;
      exit;
   end;
   if (Select = nil) or Select.NoElements then
      exit;
   for i := 0 to Select.LastIndex do
   begin
      e := Select.Elements[i];
{ TODO -oPPK -cDodlat : Duplikace ForestModelu - SelectIsZAK }
      //if SelectIsZAK then dodelat
      if not (e is Tgis_ForestUnit) then
      begin
         ShowMessage('ForestModel neznm typ vbru!');
         Error := 2;
         exit;
      end;
      FU := Tgis_ForestUnit(e);
      if SelectIsZAK then
      begin
         if not (FU is Tgis_Porost) then
         begin
            ShowMessage('FM por Chybn typ vbru!');
            Error := 3;
            exit;
         end
         else
            Por := Tgis_Porost(FU);
      end
      else
      begin
         if not (FU is Tgis_PorostniSkupina) then
         begin
            ShowMessage('FM psk Chybn typ vbru!');
            Error := 3;
            exit;
         end
         else
         begin
            PSK := Tgis_PorostniSkupina(FU);
            Por := PSK._Porost;
         end;
      end;
      LHC := FU._LHC;
      LS  := FU._LS;
      Odd := FU._Oddeleni;
      Dil := FU._Dilec;
      if Por = nil then continue; // Nema porost srab
      NewLHC := false;
      if FM.GetLHCCislo(LHC.Cislo)= nil then
      begin
         FM.CreateLHCCislo(LHC.Cislo);
         NewLHC := true;
      end;
      if (FM.ActualLHC <> nil) and (FM.ActualLHC = LHC) then
      begin
         ShowMessage('Pokoute se duplikovat model LHC do sebe sama!');
         Error := 4;
         exit;
      end;
      if NewLHC and (LHC <> nil) then
      begin
         FM.ActualLHC.Nazev := LHC.Nazev;
         FM.ActualLHC.LHC_LCR := LHC.LHC_LCR;
      end;
      FM.CreateLSCislo(LS.Cislo);
      FM.CreateOddeleniCislo(Odd.Cislo);
      FM.CreateDilecName(Dil.Name);
      if FM.GetPorostName(Por.Name)= nil then // novy porost
      begin
         FM.CreatePorostName(Por.Name);
         FM.ActualPorost.KVAL_P := Por.KVAL_P;
         FM.ActualPorost.Barva := Por.Barva;
         FM.ActualPorost.Znacka := Por.Znacka;
         FM.ActualPorost.KATS := Por.KATS;
         FM.ActualPorost.ZVL_STATUT := Por.ZVL_STATUT;
         FM.ActualPorost.KAT_Poznamka := Por.KAT_Poznamka;
         FM.ActualPorost.Properties := Por.Properties;
         if not Por.NoGeometry then
         begin
            FM.ActualPorost.Geometry := Por.Geometry;
            if ZAKAreas <> nil then
               ZAKAreas.AddElement(FM.ActualPorost);
         end;
      end;
      if SelectIsZAK then
      begin
         if not Por.NoItems then
         for j := 0 to Por.Items.LastIndex do
         begin
            PSK := Tgis_Porostniskupina(Por.items.Elements[j]);
            CopyPSK;
         end;
      end
      else
      begin
         CopyPSK;
      end;
   end;
   FM.UpdateAll := true;
   if (DETAreas <> nil) and (DETAreas is TRectEntity) then
      TRectEntity(DETAreas).Update;
   if (ZAKAreas <> nil) and (ZAKAreas is TRectEntity) then
      TRectEntity(ZAKAreas).Update;
end;

function TDuplicateForestModelForSelect.NumberToPropertyName(aNumber: Integer): string;
begin
  case aNumber of
     0: result := 'Select';
     1: result := 'DETAreas';
     2: result := 'ZAKAreas';
     3: result := 'SelectIsZAK';
  end;
end;

procedure TDuplicateForestModelForSelect.SetDETAreas(const Value: TEntity);
begin
  FDETAreas := Value;
end;

procedure TDuplicateForestModelForSelect.SetSelect(const Value: TEntity);
begin
  FSelect := Value;
end;

procedure TDuplicateForestModelForSelect.SetSelectIsZAK(const Value: boolean);
begin
  FSelectIsZAK := Value;
end;

procedure TDuplicateForestModelForSelect.SetZAKAreas(const Value: TEntity);
begin
  FZAKAreas := Value;
end;

INITIALIZATION
   RegisterOrbisClass(TSelectPorostniSkupinaNumber);
   RegisterOrbisClass(TCreateLHCGeometry);
   RegisterOrbisClass(TSelectForestBorderLines);
   RegisterOrbisClass(TCheckForestConnection);
   RegisterOrbisClass(TJoinForestModel);
   RegisterOrbisClass(TDuplicateForestModelForSelect);
   RegisterOrbisClass(TSolveForestModelMishmash);
   RegisterOrbisClass(TVyrovnani_ZAK_DET);
END.
