unit t_RevPDialog;

interface
uses
   Elements,
   f_REVPDialog,
   T_OrbisDialog;

type

TRevPDialog=class(TOrbisDialog)
  private
    FColor: integer;
    FComment: string;
    FOldObjectText: string;
    FNewObjectText: string;
    FLayer: TEntity;
    FLayerName: String;
    function GetForm: TRevPDialog_Form;
    procedure SetColor(const Value: integer);
    procedure SetComment(const Value: string);
    procedure SetNewObjectText(const Value: string);
    procedure SetOldObjectText(const Value: string);
    function GetNewObject: boolean;
    procedure SetLayer(const Value: TEntity);
    procedure SetLayerName(const Value: String);
public
   procedure DoIt; override;
   procedure CreateForm; override;
   property  Form: TRevPDialog_Form read GetForm;
   property  NewObject: boolean read GetNewObject;
   property  Layer: TEntity read FLayer write SetLayer;
published
   property  Comment: string read FComment write SetComment;
   property  NewObjectText: string read FNewObjectText write SetNewObjectText;
   property  OldObjectText: string read FOldObjectText write SetOldObjectText;
   property  Color: integer read FColor write SetColor;
   property  LayerName: String read FLayerName write SetLayerName;
end;

implementation
uses
   Forms,
   SysUtils,
   t_CMDLn,
   e_Geometry,
   e_LHCModel,
   e_Class;

{ TRevPDialog }

procedure TRevPDialog.CreateForm;
begin
   aForm := TREVPDialog_Form.Create(Application);
   aForm.Tool := Self;
   aForm.IsPossibleToClose := false;
end;

procedure TRevPDialog.DoIt;
var
   REV,REVN: Tgis_LS;
   aCislo, aLS, aTyp, aKR: integer;
   aNazev,aLS_NAZ, aKR_NAZ: string;
   i: integer;
   MA: TMultiAreaElement;

   Procedure FindLayer;
   var
      OIL: TCommandLine;
      E: TElement;
   begin
      OIL := TCommandLine.Create;
      OIL.Line := LayerName;
      OIL.DoIt;
      {$IFDEF WIN64}
      E := TElement(StrToInt64Def(OIL.ResultLine, 0));
      {$ELSE}
      E := TElement(StrToIntDef(OIL.ResultLine, 0));
      {$ENDIF}
      OIL.Free;
      if not (E is TEntity) then exit;
      Layer := TEntity(E);
   end;

   function FindREV(aCislo: integer): Tgis_LS;
   var
      i: integer;
      e: TElement;
      R: Tgis_LS;
   begin
      result := nil;
      if Layer = nil then exit;
      for i := 0 to Layer.LastIndex do
      begin
         e := Layer.Elements[i];
         if not (e is Tgis_LS) then continue;
         R := Tgis_LS(e);
         if R.Cislo = aCislo then
         begin
            result := R;
            exit;
         end;
      end;
   end;
begin
   REV := nil;
   Form.b_Delete.Enabled := not NewObject;
   if NewObject then
   begin
      Form.Caption := NewObjectText + Comment;
   end
   else
   begin
      Form.Caption := OldObjectText + Comment;
      REV := Tgis_LS(Output);
      Form.ed_Nazev.Text  := REV.Nazev;
      Form.ed_Cislo.Value := REV.Cislo;
      Form.ed_Typ.Value   := REV.REV_TYP;
      Form.ed_LS.Value    := REV.LS;
      Form.ed_LS_NAZ.Text := REV.LS_NAZ;
      Form.ed_KR.Value    := REV.KR;
      Form.ed_KR_NAZ.Text := REV.KR_NAZ;
   end;
   inherited;
   if result = 1 then
   begin
      FindLayer;
      aCislo  := Form.ed_Cislo.Value;
      aNazev  := Form.ed_Nazev.Text;
      aTyp    := Form.ed_Typ.Value;
      aLS     := Form.ed_LS.Value;
      aLS_NAZ := Form.ed_LS_NAZ.Text;
      aKR     := Form.ed_KR.Value;
      aKR_NAZ := Form.ed_KR_NAZ.Text;
      if NewObject then
      begin
         REV := FindREV(aCislo);
         if REV = nil then
         begin
            REV := Tgis_LS.Create;
            REV.Cislo   := aCislo;
            REV.Nazev   := aNazev;
            REV.REV_TYP := aTyp;
            REV.LS      := aLS;
            REV.LS_NAZ  := aLS_NAZ;
            REV.KR      := aKR;
            REV.KR_NAZ  := aKR_NAZ;
            REV.Barva := Color;
            REV.Znacka := 1;
         end;
         REV.AddGeometry(TAreaElement(Input));
      end
      else
      begin
         if aCislo <> REV.Cislo then
         begin
            REVN := FindREV(aCislo);
            if REVN <> nil then
            begin
               if REV.Geometry is TMultiAreaElement then
               begin
                  MA := TMultiAreaElement(REV.Geometry);
                  for i := 0 to MA.LastIndex do
                  begin
                     REVN.AddGeometry(MA.AreaElement[i]);
                  end;
               end
               else
                  REVN.AddGeometry(REV.Geometry);
               REV.Geometry := nil;
               Layer.FreeElement(REV);
               REV := REVN;
            end
            else
            begin
               REV.Cislo := aCislo;
            end;
            REV.Nazev := aNazev;
         end;
         REV.Nazev := aNazev;
         REV.REV_TYP := aTyp;
         REV.LS      := aLS;
         REV.LS_NAZ  := aLS_NAZ;
         REV.KR      := aKR;
         REV.KR_NAZ  := aKR_NAZ;
      end;
      Output := REV;
   end;
end;

function TRevPDialog.GetForm: TRevPDialog_Form;
begin
   result := TREVPDialog_Form(aForm);
end;

function TRevPDialog.GetNewObject: boolean;
begin
   result := Output = nil;
end;

procedure TRevPDialog.SetColor(const Value: integer);
begin
  FColor := Value;
end;

procedure TRevPDialog.SetComment(const Value: string);
begin
  FComment := Value;
end;

procedure TRevPDialog.SetLayer(const Value: TEntity);
begin
  FLayer := Value;
end;

procedure TRevPDialog.SetLayerName(const Value: String);
begin
  FLayerName := Value;
end;

procedure TRevPDialog.SetNewObjectText(const Value: string);
begin
  FNewObjectText := Value;
end;

procedure TRevPDialog.SetOldObjectText(const Value: string);
begin
  FOldObjectText := Value;
end;

INITIALIZATION
   RegisterOrbisClass(TRevPDialog);
END.
