unit t_Sluckovani;

interface
uses
   Elements,
   e_QTree,
   e_TopolElements,
   e_PSymb,
   g_Elems,
   e_Type,
   m_InterA,
   Tools,
   t_Snap,
   t_QTreeTools,
   t_GraphicEditors;

type

TSluckovaniStatus = (ssFind,ssAngle,ssPosition,ssFirst,ssSecond,ssOld);

TSluckovani = class(TGraphicEditor)
private
    FSymbols: TTopolSymbols;
    FRepairOnly: boolean;
    FOldY: double;
    FOldX: double;
    FOldAngle: double;
    FOldSymbol: integer;
    FDruhyList: string;
    FError_Tool: boolean;
    FError_AreaBorder: boolean;
    FError_Druh: boolean;
    FErrorToolMessage: string;
    FErrorAreaBorderMessage: string;
    FErrorDruhMessage: string;
    FCheckAreaClassName: string;
    FCheckAreaClass: TElementClass;
    FCheckTool: TBooleanTool;
    procedure SetSymbols(const Value: TTopolSymbols);
    function GetIsSlucka: boolean;
    procedure SetRepairOnly(const Value: boolean);
    procedure SetOldAngle(const Value: double);
    procedure SetOldSymbol(const Value: integer);
    procedure SetOldX(const Value: double);
    procedure SetOldY(const Value: double);
    function GetDruhy: TIntegerListElement;
    procedure SetDruhyList(const Value: string);
    procedure SetError_AreaBorder(const Value: boolean);
    procedure SetError_Druh(const Value: boolean);
    procedure SetError_Tool(const Value: boolean);
    procedure SetErrorAreaBorderMessage(const Value: string);
    procedure SetErrorDruhMessage(const Value: string);
    procedure SetErrorToolMessage(const Value: string);
    procedure SetCheckAreaClassName(const Value: string);
    procedure SetCheckAreaClass(const Value: TElementClass);
    procedure SetCheckTool(const Value: TBooleanTool);
protected
    FDetLevel: string;
    FSLULevel: string;
    FDetLines: TEntity;
    FSluLines: TEntity;
    FPixelTolerance: integer;
    FSLUQT: TRectEntityQTree;
    FDetQT: TRectEntityQTree;
    FY: double;
    FX: double;
    FSluckovaniStatus: TSluckovaniStatus;
    FDetFound: boolean;
    FSlucka: integer;
    FDruhy: TIntegerListElement;
    procedure SetDetLevel(const Value: string);
    procedure SetDetLines(const Value: TEntity);
    procedure SetSLULevel(const Value: string);
    procedure SetSluLines(const Value: TEntity);
    function GetLine: TTopolLineElement;
    procedure SetLine(const Value: TTopolLineElement);
    procedure SetPixelTolerance(const Value: integer);
    procedure SetDetQT(const Value: TRectEntityQTree);
    procedure SetSLUQT(const Value: TRectEntityQTree);
    procedure SetX(const Value: double);
    procedure SetY(const Value: double);
    procedure SetSluckovaniStatus(const Value: TSluckovaniStatus);
    procedure SetDetFound(const Value: boolean);
    procedure SetSlucka(const Value: integer);
    property  Symbols: TTopolSymbols read FSymbols write SetSymbols;
    property  CheckAreaClass: TElementClass read FCheckAreaClass write SetCheckAreaClass;
public
   constructor Create; override;
   procedure   XORDraw; override;
   procedure   On_tm_MouseUp(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
   procedure   On_tm_MouseMove(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
   procedure   On_tm_Resize(var Handled: boolean); override;
   procedure   ChangeCursor; virtual;
   procedure   ChangeAngle; virtual;
   procedure   ChangePosition; virtual;
   procedure   RegisterCursors; override;
   procedure   On_Activate; override;
   procedure   PrepareQTrees;
   procedure   StoreLineToSlu;
   procedure   StoreLineToDet;
   procedure   SetPointOnLine;
   function    CheckDruh: boolean;
   function    CheckAreaBorder: boolean;
   function    CheckForTool: boolean;

   property    DetLines: TEntity read FDetLines write SetDetLines;
   property    SluLines: TEntity read FSluLines write SetSluLines;
   property    DetQT: TRectEntityQTree read FDetQT write SetDetQT;
   property    SLUQT: TRectEntityQTree read FSLUQT write SetSLUQT;
   property    Line: TTopolLineElement read GetLine write SetLine;
   property    X: double read FX write SetX;
   property    Y: double read FY write SetY;
   property    SluckovaniStatus: TSluckovaniStatus read FSluckovaniStatus write SetSluckovaniStatus;
   property    DetFound: boolean read FDetFound write SetDetFound;
   property    Error_Druh: boolean read FError_Druh write SetError_Druh;
   property    Error_AreaBorder: boolean read FError_AreaBorder write SetError_AreaBorder;
   property    Error_Tool: boolean read FError_Tool write SetError_Tool;
   property    IsSlucka: boolean read GetIsSlucka;
   property    OldX: double read FOldX write SetOldX;
   property    OldY: double read FOldY write SetOldY;
   property    OldAngle: double read FOldAngle write SetOldAngle;
   property    OldSymbol: integer read FOldSymbol write SetOldSymbol;
   property    Druhy: TIntegerListElement read GetDruhy;
published
   property DetLevel: string read FDetLevel write SetDetLevel;
   property SLULevel: string read FSLULevel write SetSLULevel;
   property PixelTolerance: integer read FPixelTolerance write SetPixelTolerance;
   property Slucka: integer read FSlucka write SetSlucka;
   property RepairOnly: boolean read FRepairOnly write SetRepairOnly;
   property DruhyList: string read FDruhyList write SetDruhyList;
   property ErrorDruhMessage: string read FErrorDruhMessage write SetErrorDruhMessage;
   property ErrorAreaBorderMessage: string read FErrorAreaBorderMessage write SetErrorAreaBorderMessage;
   property ErrorToolMessage: string read FErrorToolMessage write SetErrorToolMessage;
   property CheckAreaClassName: string read FCheckAreaClassName write SetCheckAreaClassName;
   property CheckTool: TBooleanTool read FCheckTool write SetCheckTool;
end;

TPulslucky = class(TGraphicEditor)
private
    FSymbols: TTopolSymbols;
    FOldY: double;
    FOldX: double;
    FOldAngle: double;
    FOldSymbol: integer;
    FSnapTool: TPointLineSnapTool;
    FSnapped: boolean;
    FGroup: integer;
    FS1: TTopolPointElement;
    FS2: TTopolPointElement;
    FMinDistance: double;
    FGroupColor: integer;
    procedure SetSymbols(const Value: TTopolSymbols);
    procedure SetOldAngle(const Value: double);
    procedure SetOldSymbol(const Value: integer);
    procedure SetOldX(const Value: double);
    procedure SetOldY(const Value: double);
    procedure SetSnapTool(const Value: TPointLineSnapTool);
    procedure SetSnapped(const Value: boolean);
    procedure SetGroup(const Value: integer);
    procedure SetS1(const Value: TTopolPointElement);
    procedure SetS2(const Value: TTopolPointElement);
    procedure SetMinDistance(const Value: double);
    procedure SetGroupColor(const Value: integer);
protected
    FP1,FP2: TDoubleCoord;
    FDetLevel: string;
    FSLULevel: string;
    FDetLines: TEntity;
    FSluPoints: TEntity;
    FPixelTolerance: integer;
    FSLUQT: TRectEntityQTree;
    FDetQT: TRectEntityQTree;
    FY: double;
    FX: double;
    FSluckovaniStatus: TSluckovaniStatus;
    FSluFound: boolean;
    FSlucka: integer;
    procedure SetDetLevel(const Value: string);
    procedure SetDetLines(const Value: TEntity);
    procedure SetSLULevel(const Value: string);
    procedure SetSluPoints(const Value: TEntity);
    procedure SetPixelTolerance(const Value: integer);
    procedure SetDetQT(const Value: TRectEntityQTree);
    procedure SetSLUQT(const Value: TRectEntityQTree);
    procedure SetX(const Value: double);
    procedure SetY(const Value: double);
    procedure SetSluckovaniStatus(const Value: TSluckovaniStatus);
    procedure SetSluFound(const Value: boolean);
    procedure SetSlucka(const Value: integer);
    property  Symbols: TTopolSymbols read FSymbols write SetSymbols;
public
   constructor Create; override;
   procedure   XORDraw; override;
   procedure   On_tm_MouseUp(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
   procedure   On_tm_MouseMove(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
   //procedure   On_tm_Resize(var Handled: boolean); override;
   procedure   ChangeCursor; virtual;
   procedure   ChangePosition; virtual;
   procedure   RegisterCursors; override;
   procedure   On_Activate; override;
   procedure   PrepareQTrees;
   procedure   StoreSlu;
   procedure   DeleteSlu;
   procedure   FindSecondPoint;
   property    DetLines: TEntity read FDetLines write SetDetLines;
   property    SluPoints: TEntity read FSluPoints write SetSluPoints;
   property    DetQT: TRectEntityQTree read FDetQT write SetDetQT;
   property    SLUQT: TRectEntityQTree read FSLUQT write SetSLUQT;
   property    X: double read FX write SetX;
   property    Y: double read FY write SetY;
   property    SluckovaniStatus: TSluckovaniStatus read FSluckovaniStatus write SetSluckovaniStatus;
   property    SluFound: boolean read FSluFound write SetSluFound;
   property    Snapped: boolean read FSnapped write SetSnapped;
   property    OldX: double read FOldX write SetOldX;
   property    OldY: double read FOldY write SetOldY;
   property    OldAngle: double read FOldAngle write SetOldAngle;
   property    OldSymbol: integer read FOldSymbol write SetOldSymbol;
   property    SnapTool: TPointLineSnapTool read FSnapTool write SetSnapTool;
   property    S1: TTopolPointElement read FS1 write SetS1;
   property    S2: TTopolPointElement read FS2 write SetS2;
published
   property DetLevel: string read FDetLevel write SetDetLevel;
   property SLULevel: string read FSLULevel write SetSLULevel;
   property PixelTolerance: integer read FPixelTolerance write SetPixelTolerance;
   property Slucka: integer read FSlucka write SetSlucka;
   property GroupColor: integer read FGroupColor write SetGroupColor;
   property Group: integer read FGroup write SetGroup;
   property MinDistance: double read FMinDistance write SetMinDistance;
end;



implementation
uses
   Dialogs,
   Controls,
   Forms,
   Graphics,
   SysUtils,
   MathLib,
   e_Class,
   e_Geometry,
   i_Top_B,
   m_Orbis,
   t_CmdLn,
   t_ConfirmDialog,
   t_SluckovaniDialog,
   u_TopolT,
   u_Symbol;

{ TSluckovani }

procedure TSluckovani.ChangeAngle;
var
   L: TPolyLineElement;
   v: double;
   A,B: TDoubleCoord;
   anAngle: double;
begin
   if Line = nil then exit;
   L := TPolyLineElement(Line.Geometry);
   if L = nil then exit;
   A.X := L.X;
   A.Y := L.Y;
   B.X := X;
   B.Y := Y;
   XYToPolar(A,B, v,anAngle);
   L.Angle := anAngle;
   XORDraw;
end;


procedure TSluckovani.ChangeCursor;
var
   d: double;
begin
   if QTFinder.Busy then exit;
   Error_Druh := false;
   Error_AreaBorder := false;
   Error_Tool := false;
   DetFound := false;
   d := Device.Projection.GetModelLength(PixelTolerance);
   CursorIndex := 0;
   QTFinder.Input := SluQT;
   QTFinder.X := X;
   QTFInder.Y := Y;
   QTFinder.Tolerance := d;
   QTFinder.DoIt;
   if QTFinder.Output <> nil then
   begin
      if RepairOnly then
         CursorIndex := 5
      else
         CursorIndex := 2;
   end
   else
   begin
      if RepairOnly then exit;
      QTFinder.Input := DetQT;
      QTFinder.DoIt;
      if QTFinder.Output <> nil then
      begin
         DetFound := true;
         if CheckDruh and CheckAreaBorder and CheckForTool then
         begin
            CursorIndex := 1;
         end
         else
            CursorIndex := 6;
      end;
   end;
end;

procedure TSluckovani.ChangePosition;
begin
   SetPointOnLine;
   XORDraw;
end;

function TSluckovani.CheckAreaBorder: boolean;
var
   L: TTopolLineElement;
begin
   result := true;
   if CheckAreaClass = nil then exit;
   if (QTFinder.Output <> nil) and (QTFinder.Output is TTopolLineElement) then
   begin
      L := TTopolLineElement(QTFinder.Output);
      if L.NoGeometry then exit;
      result := L.Geometry.FindClassInOwners(CheckAreaClass) = nil;
      if not result then Error_AreaBorder := true;
   end;
end;

function TSluckovani.CheckDruh: boolean;
var
   L: TTopolLineElement;
begin
   result := true;
   if (QTFinder.Output <> nil) and (QTFinder.Output is TTopolLineElement) then
   begin
      L := TTopolLineElement(QTFinder.Output);
      result :=  Druhy.ValueExists(L.Druh);
      if not result then Error_Druh := true;
   end;
end;

function TSluckovani.CheckForTool: boolean;
var
   L: TTopolLineElement;
begin
   result := true;
   if CheckTool = nil then exit;
   if (QTFinder.Output <> nil) and (QTFinder.Output is TTopolLineElement) then
   begin
      L := TTopolLineElement(QTFinder.Output);
      if L.NoGeometry then exit;
      CheckTool.Input := L.Geometry;
      CheckTool.DoIt;
      result := CheckTool.result;
      if not result then Error_Tool := true;
   end;
end;

constructor TSluckovani.Create;
begin
   inherited;
   Color := clSilver;
   //Interpret := TInterpretTopBodu.Create;
end;

function TSluckovani.GetDruhy: TIntegerListElement;
begin
   if FDruhy = nil then
   begin
      FDruhy := TIntegerListElement.Create;
      FDruhy.Name := 'Druhy';
      AddElement(FDruhy);
   end;
   result := FDruhy;
end;

function TSluckovani.GetIsSlucka: boolean;
begin
   result := true;
   if Dialog = nil then exit;
   if not (Dialog is TSluckovaniDialog) then exit;
   result := TSluckovaniDialog(Dialog).Slucka;
end;

function TSluckovani.GetLine: TTopolLineElement;
begin
   result := TTopolLineElement(Input);
end;


procedure TSluckovani.On_Activate;
begin
   inherited;
   EditorStatus := gesPrepare;
   DetLevel := DetLevel;
   SluLevel := SluLevel;
   PrepareQTrees;
   QTFinder.Busy := false;
   SluckovaniStatus := ssFind;
   if fileExists(PathOfLibraries+'\Fonty.txt') then
   begin
     Symbols := TTopoLSymbols.CreateAll(PathOfLibraries+'\Fonty.txt');
     if Symbols <> nil then
        Symbols.Device := Device;
   end;
   EditorStatus := gesEdit;
end;

procedure TSluckovani.On_tm_MouseMove(var aMessage: TOrbisMouseEvent;var Handled: boolean);
begin
   if ZeroActivity  and not HighActivity then exit;
   if EditorStatus = gesPrepare then exit;
   if SluckovaniStatus <> ssFind then
      XORDraw;
   X := Device.Projection.GetModelX(aMessage.x);
   Y := Device.Projection.GetModelY(aMessage.y);
   Case SluckovaniStatus of
      ssFind: ChangeCursor;
      ssAngle: ChangeAngle;
      ssPosition: ChangePosition;
   end;
end;

procedure TSluckovani.On_tm_MouseUp(var aMessage: TOrbisMouseEvent; var Handled: boolean);
var
   i: integer;
begin
   if EditorStatus = gesPrepare then exit;
   if not (aMessage.Button = mbLeft) then exit;
   if SluckovaniStatus <> ssFind then
   begin
      XORDraw;
      SluckovaniStatus := ssFind;
      XORDraw;
   end
   else
   begin
      if ZeroActivity then exit;
      HighActivity := true;
      if QTFinder.Output = nil then exit;
      Line := TTopolLineElement(QTFinder.output);
      if not DetFound then
      begin
         if not RepairOnly then
         begin
            StoreLineToDet;
            HighActivity := false;
            exit;
         end;
      end;
      if RepairOnly then
      begin
         OldX := Line.Geometry.X;
         OldY := Line.Geometry.Y;
         OldAngle := Line.Geometry.Angle;
         OldSymbol := Line.Symbol;
         if Line.Symbol > 0 then
         begin
            Line.Symbol := 0;
            PostMsgToOwners(tm_ReviewByElement, Line, false);
            Application.ProcessMessages;
            Line.Symbol := Slucka
         end;
      end
      else
      begin
         if Error_Druh then
         begin
            ShowMessage(ErrorDruhMessage);
            HighActivity := false;
            exit;
         end
         else
         if Error_AreaBorder then
         begin
            ShowMessage(ErrorAreaBorderMessage);
            HighActivity := false;
            exit;
         end
         else
         if Error_Tool then
         begin
            ShowMessage(ErrorToolMessage);
            HighActivity := false;
            exit;
         end;
         SetPointOnLine;
         if IsSlucka then
            Line.Symbol := Slucka
         else
            Line.Symbol := 0;
      end;
      XORDraw;
   end;
      //PostMsgToOwners(tm_ReviewByElement, Line, false);
      //Application.ProcessMessages;
   // DIALOG
   if Dialog = nil then
   begin
      HighActivity := false;
      exit;
   end;
   Dialog.Input := Line;
   if RepairOnly then
   begin
      if Dialog is TSluckovaniDialog then
      begin
         TSluckovaniDialog(Dialog).Slucka := Line.Symbol > 0;
      end;
   end;
   Dialog.Left := Mouse.CursorPos.x;
   Dialog.Top := Mouse.CursorPos.y;
   if (Dialog.Left + Dialog.Width) > Screen.WorkAreaWidth then
      Dialog.Left := Screen.WorkAreaWidth - Dialog.Width;
   if (Dialog.Top + Dialog.Height) > Screen.WorkAreaHeight then
      Dialog.Top := Screen.WorkAreaHeight - Dialog.Height;
   Dialog.DoIt;
   XORDraw;
   Case Dialog.Result of
      1: begin
            if RepairOnly then
            begin
               PostMsgToOwners(tm_ReviewByElement, Line, false);
               if (Line.Geometry.X <> OldX) or  (Line.Geometry.Y <> OldY) or
                  (Line.Geometry.Angle <> OldAngle) or (Line.Symbol <> OldSymbol) then
               begin
                  PostMsg(tm_ModelChanged,nil,nil,false);
               end;
               Application.ProcessMessages;
            end
            else
               StoreLineToSlu;
            HighActivity := false;
         end;
      2: begin
            if RepairOnly then
            begin
               if (Line.Geometry.X <> OldX) or  (Line.Geometry.Y <> OldY) or
                  (Line.Geometry.Angle <> OldAngle) or (Line.Symbol <> OldSymbol) then
               begin
                  PostMsg(tm_ModelChanged,nil,nil,false);
               end;
               Line.Geometry.X := OldX;
               Line.Geometry.Y := OldY;
               Line.Geometry.Angle := OldAngle;
               Line.Symbol := OldSymbol;
               PostMsgToOwners(tm_ReviewByElement, Line, false);
               Application.ProcessMessages;
            end
            else
            begin
              Line.Symbol := 0;
              exit;
            end;
            HighActivity := false;
         end;
      3: begin
            SluckovaniStatus := ssAngle;
            XORDraw;
         end;
      4: begin
            SluckovaniStatus := ssPosition;
            XORDraw;
         end;
   end;
end;

procedure TSluckovani.On_tm_Resize(var Handled: boolean);
begin
   if EditorStatus = gesPrepare then exit;
   Handled := true;
   if Line = nil then exit;
   if Dialog = nil then exit;
   XORDraw;
   if IsSlucka then
      Line.Symbol := Slucka
   else
      Line.Symbol := 0;
   XORDraw;
end;

procedure TSluckovani.PrepareQTrees;
var
   TQ: TAddEntityToQTree;
begin
   if not RepairOnly and (DetLines = nil) then exit;
   if SluLines = nil then exit;
   StartProgress('Pprava vrstev...',0,0);
   if not RepairOnly then
   begin
      TQ := TAddEntityToQTree.Create;
      TQ.Input := DetLines;
      TQ.DoIt;
      DetQT :=TRectEntityQTree(TQ.Output);
      DetQT.Name := 'DET QT';
      TQ.Free;
   end;
   TQ := TAddEntityToQTree.Create;
   TQ.Input := SluLines;
   TQ.DoIt;
   SluQT :=TRectEntityQTree(TQ.Output);
   SluQT.Name := 'SLU QT';
   TQ.Free;
   EndProgress;
end;

procedure TSluckovani.RegisterCursors;
begin
   NewCursorName('NOSELECT');
   NewCursorName('DET_SLU');
   NewCursorName('SLU_DET');
   NewCursorName('SLUMOVE');
   NewCursorName('SLUANGLE');
   NewCursorName('SLUREPAIR');
   NewCursorName('SLUVAR');
end;

procedure TSluckovani.SetCheckAreaClass(const Value: TElementClass);
begin
  FCheckAreaClass := Value;
end;

procedure TSluckovani.SetCheckAreaClassName(const Value: string);
begin
   FCheckAreaClassName := Value;
   CheckAreaClass := GetOrbisClass(Value);
end;

procedure TSluckovani.SetCheckTool(const Value: TBooleanTool);
begin
   ExChangeFieldElement(TElement(FCheckTool), Value);
end;

procedure TSluckovani.SetDetFound(const Value: boolean);
begin
  FDetFound := Value;
end;

procedure TSluckovani.SetDetLevel(const Value: string);
begin
   FDetLevel := Value;
   DetLines := FindLevel(Value);
end;

procedure TSluckovani.SetDetLines(const Value: TEntity);
begin
   FDetLines := Value;
end;

procedure TSluckovani.SetDetQT(const Value: TRectEntityQTree);
begin
   ExchangeFieldElement(TElement(FDetQT), Value);
end;

procedure TSluckovani.SetDruhyList(const Value: string);
begin
  FDruhyList := Value;
  Druhy.ValuesString := Value;
end;

procedure TSluckovani.SetErrorAreaBorderMessage(const Value: string);
begin
  FErrorAreaBorderMessage := Value;
end;

procedure TSluckovani.SetErrorDruhMessage(const Value: string);
begin
  FErrorDruhMessage := Value;
end;

procedure TSluckovani.SetErrorToolMessage(const Value: string);
begin
  FErrorToolMessage := Value;
end;

procedure TSluckovani.SetError_AreaBorder(const Value: boolean);
begin
  FError_AreaBorder := Value;
end;

procedure TSluckovani.SetError_Druh(const Value: boolean);
begin
  FError_Druh := Value;
end;

procedure TSluckovani.SetError_Tool(const Value: boolean);
begin
  FError_Tool := Value;
end;

procedure TSluckovani.SetLine(const Value: TTopolLineElement);
begin
   Input := Value;
end;

procedure TSluckovani.SetOldAngle(const Value: double);
begin
  FOldAngle := Value;
end;

procedure TSluckovani.SetOldSymbol(const Value: integer);
begin
  FOldSymbol := Value;
end;

procedure TSluckovani.SetOldX(const Value: double);
begin
  FOldX := Value;
end;

procedure TSluckovani.SetOldY(const Value: double);
begin
  FOldY := Value;
end;

procedure TSluckovani.SetPixelTolerance(const Value: integer);
begin
   FPixelTolerance := Value;
end;

procedure TSluckovani.SetPointOnLine;
var
   i: integer;
   L: TPolyLineElement;
   PataUvnitr: boolean;
   PataX, PataY, Vzdalenost,v: double;
   A,B: TDoubleCoord;
   anAngle: double;
begin
   if Line = nil then exit;
   L := TPolyLineElement(Line.Geometry);
   if L = nil then exit;
   v := 1E30;
   for I:= 0 to L.LastIndex-1 do
   begin
      DejVztahyBod_Usecka( X, Y, L.Xi[i], L.Yi[i], L.Xi[i+1], L.Yi[i+1],
                           PataUvnitr, PataX, PataY, Vzdalenost);

      if PataUvnitr and (Vzdalenost < v) then
      begin
         v := Vzdalenost;
         L.AbsRefX := PataX;
         L.AbsRefY := PataY;
         A.X := X;
         A.Y := Y;
         B.X := PataX;
         B.Y := PataY;
         XYToPolar(A,B, Vzdalenost,anAngle);
         L.Angle := anAngle;
      end;
   end;
end;

procedure TSluckovani.SetRepairOnly(const Value: boolean);
begin
   FRepairOnly := Value;
end;

procedure TSluckovani.SetSlucka(const Value: integer);
begin
   FSlucka := Value;
end;

procedure TSluckovani.SetSluckovaniStatus(const Value: TSluckovaniStatus);
begin
   FSluckovaniStatus := Value;
   Case Value of
      ssFind: CursorIndex := 0;
      ssAngle: CursorIndex := 4;
      ssPosition: CursorIndex := 3;
   end;
end;

procedure TSluckovani.SetSLULevel(const Value: string);
begin
   FSLULevel := Value;
   SluLines := FindLevel(Value);
end;

procedure TSluckovani.SetSluLines(const Value: TEntity);
begin
   FSluLines := Value;
end;

procedure TSluckovani.SetSLUQT(const Value: TRectEntityQTree);
begin
   ExchangeFieldElement(TElement(FSLUQT), Value);
end;

procedure TSluckovani.SetSymbols(const Value: TTopolSymbols);
begin
   ExchangeFieldElement(TElement(FSymbols), Value);
end;

procedure TSluckovani.SetX(const Value: double);
begin
  FX := Value;
end;

procedure TSluckovani.SetY(const Value: double);
begin
  FY := Value;
end;

procedure TSluckovani.StoreLineToDet;
begin
   Line.Symbol := 0;
   RemoveElementOwners(Line);
   DetLines.AddElement(Line);
 { TODO -oPPK -cOpravit : Pospojovat linie v okol }
   DetQT.NextRectEntity := Line;
   PostMsgToOwners(tm_ReviewByElement, Line, false);
   Application.ProcessMessages;
   Line := nil;
   PostMsg(tm_ModelChanged,nil,nil,false);
end;

procedure TSluckovani.StoreLineToSlu;
begin
   RemoveElementOwners(Line);
   SluLines.AddElement(Line);
   SluQT.NextRectEntity := Line;
   Line.DrziSe := NedrziSe;
   PostMsgToOwners(tm_ReviewByElement, Line, false);
   Application.ProcessMessages;
   Line := nil;
   PostMsg(tm_ModelChanged,nil,nil,false);
end;

procedure TSluckovani.XORDraw;
var
   i: integer;
   L: TPolyLineElement;
begin
   if Line = nil then
      Exit;
   if Line.NoGeometry then exit;
   L := TPolyLineElement(Line.Geometry);
   if L.CoordsCount < 2 then exit;
   Device.Pen.Color:= Color;
   Device.Pen.Style:= psSolid;
   Device.Pen.Mode:= pmXOR;
   Device.Pen.Width:= 1;
   Device.Brush.Style:= bsClear;
   if SluckovaniStatus = ssAngle then
   begin
      Device.Pen.Style := psDot;
      Device.MoveTo(L.X, L.Y);
      Device.LineTo(X, Y);
      Device.Pen.Style:= psSolid;
   end;
   //Device.Polyline(PolyElement)
   Device.MoveTo(L.Xi[0], L.Yi[0]);
   for i := 1 to L.LastIndex do
       Device.LineTo(L.Xi[i], L.Yi[i]);
   if (Line.Symbol > 0) and (Symbols <> nil) then
   begin
      Symbols.PaintSymbol(L.X, L.Y, L.Angle+Pi/2, Color, Line.Symbol);
   end;
end;

{ TPulslucky }

constructor TPulslucky.Create;
begin
   inherited;
   Color := clSilver;
   SnapTool := TPointLineSnapTool.Create;
   SnapTool.SnapToLine := true;
   SnapTool.SnapToLinePoint := false;
   SnapTool.SnapToLineBody := true;
   SnapTool.SnapDistance := 10;
end;

procedure TPulslucky.ChangeCursor;
var
   d: double;
begin
   if QTFinder.Busy then exit;
   SluFound := false;
   Snapped := false;
   d := Device.Projection.GetModelLength(PixelTolerance);
   CursorIndex := 0;
   QTFinder.Input := SluQT;
   QTFinder.X := X;
   QTFInder.Y := Y;
   QTFinder.Tolerance := d;
   QTFinder.DoIt;
   if QTFinder.Output <> nil then
   begin
      if TTopolPointElement(QTFinder.Output).Presnost = 10 then
         CursorIndex := 1
      else
      if TTopolPointElement(QTFinder.Output).Presnost = 11 then
         CursorIndex := 2
      else
         CursorIndex := 4;
      SluFound := true;
   end
   else
   begin
      SnapTool.X := X;
      SnapTool.Y := Y;
      SnapTool.DoIt;
      if SnapTool.Snapped then
      begin
         CursorIndex := 3;
         Snapped := true;
      end;
   end;
end;

procedure TPulslucky.ChangePosition;
begin
   //XORDraw;
   Snapped := false;
   SnapTool.X := X;
   SnapTool.Y := Y;
   SnapTool.DoIt;
   if SnapTool.Snapped then
   begin
      Snapped := true;
      case SluckovaniStatus of
         ssFirst:
         begin
            FP1.X := SnapTool.SnapX;
            FP1.Y := SnapTool.SnapY;
         end;
         ssSecond:
         begin
            FP2.X := SnapTool.SnapX;
            FP2.Y := SnapTool.SnapY;
            CursorIndex := 2;
         end;
      end;
   end
   else
   begin
      case SluckovaniStatus of
         ssFirst:
         begin
            CursorIndex := 2;
            SluckovaniStatus := ssSecond;
            FP2.X := X;
            FP2.Y := Y;
         end;
         ssSecond:
         begin
            FP2.X := X;
            FP2.Y := Y;
            CursorIndex := 0;
         end;
      end;
   end;
   XORDraw;
end;


procedure TPulslucky.On_Activate;
begin
   inherited;
   EditorStatus := gesPrepare;
   DetLevel := DetLevel;
   SluLevel := SluLevel;
   PrepareQTrees;
   QTFinder.Busy := false;
   SluckovaniStatus := ssFind;
   if MinDistance < 0.001 then
      MinDistance := 2;
   if fileExists(PathOfLibraries+'\Fonty.txt') then
   begin
     Symbols := TTopoLSymbols.CreateAll(PathOfLibraries+'\Fonty.txt');
     if Symbols <> nil then
        Symbols.Device := Device;
   end;
   EditorStatus := gesEdit;
end;

procedure TPulslucky.On_tm_MouseMove(var aMessage: TOrbisMouseEvent;var Handled: boolean);
begin
   if ZeroActivity and not HighActivity then exit;
   if EditorStatus = gesPrepare then exit;
   if SluckovaniStatus <> ssFind then
      XORDraw;
   X := Device.Projection.GetModelX(aMessage.x);
   Y := Device.Projection.GetModelY(aMessage.y);
   Case SluckovaniStatus of
      ssFind: ChangeCursor;
      ssFirst,ssSecond: ChangePosition;
      //ssAngle: ChangeAngle;
      //ssPosition: ChangePosition;
   end;
end;

procedure TPulslucky.On_tm_MouseUp(var aMessage: TOrbisMouseEvent; var Handled: boolean);
begin
   if ZeroActivity and not HighActivity then exit;
   if EditorStatus = gesPrepare then exit;
   if aMessage.Button <> mbLeft then
   begin
      if SluckovaniStatus <> ssFind then
      begin
         XORDraw;
         SluckovaniStatus := ssFind;
      end;
      HighActivity := false;
      exit;
   end;
   case SluckovaniStatus of
   ssFind:
      begin
         if Snapped then
         begin
            HighActivity := true;
            CursorIndex := 1;
            SluckovaniStatus := ssFirst;
            FP1.X := SnapTool.SnapX;
            FP1.Y := SnapTool.SnapY;
            XORDraw;
         end
         else
         if SluFound then
         begin
            S1 := TTopolPointElement(QTFinder.Output);
            FindSecondPoint;
            SluckovaniStatus := ssOld;
            DeleteSlu;
            HighActivity := false;
         end;
      end;
   ssFirst:
      begin
         XORDraw;
         CursorIndex := 2;
         SluckovaniStatus := ssSecond;
         FP2.X := X;
         FP2.Y := Y;
         XORDraw;
      end;
   ssSecond:
      begin
         if not Snapped then exit;
         XORDraw;
         CursorIndex := 0;
         SluckovaniStatus := ssFind;
         StoreSlu;
         HighActivity := false;
         //Store nekam
      end;
   end;

end;

procedure TPulslucky.PrepareQTrees;
var
   TQ: TAddEntityToQTree;
begin
   //if not RepairOnly and (DetLines = nil) then exit;
   if SluPoints = nil then exit;
   StartProgress('Pprava vrstev...',0,0);
   // Det Lines
   TQ := TAddEntityToQTree.Create;
   TQ.Input := DetLines;
   TQ.DoIt;
   DetQT :=TRectEntityQTree(TQ.Output);
   DetQT.Name := 'DET QT';
   TQ.Free;
   SnapTool.FrameQT := DetQT;
   // SLU Points
   TQ := TAddEntityToQTree.Create;
   TQ.Input := SluPoints;
   TQ.DoIt;
   SluQT :=TRectEntityQTree(TQ.Output);
   SluQT.Name := 'SLU QT';
   TQ.Free;
   EndProgress;
end;

procedure TPulslucky.RegisterCursors;
begin
   NewCursorName('NOSELECT');
   NewCursorName('PULSLU1');
   NewCursorName('PULSLU2');
   NewCursorName('PULSLUNEW');
   NewCursorName('PULSLUOLD');
end;

procedure TPulslucky.SetSluFound(const Value: boolean);
begin
   FSluFound := Value;
end;

procedure TPulslucky.SetDetLevel(const Value: string);
begin
   FDetLevel := Value;
   DetLines := FindLevel(Value);
end;

procedure TPulslucky.SetDetLines(const Value: TEntity);
begin
   FDetLines := Value;
end;

procedure TPulslucky.SetDetQT(const Value: TRectEntityQTree);
begin
   ExchangeFieldElement(TElement(FDetQT), Value);
end;

procedure TPulslucky.SetOldAngle(const Value: double);
begin
  FOldAngle := Value;
end;

procedure TPulslucky.SetOldSymbol(const Value: integer);
begin
  FOldSymbol := Value;
end;

procedure TPulslucky.SetOldX(const Value: double);
begin
  FOldX := Value;
end;

procedure TPulslucky.SetOldY(const Value: double);
begin
  FOldY := Value;
end;

procedure TPulslucky.SetPixelTolerance(const Value: integer);
begin
   FPixelTolerance := Value;
end;



procedure TPulslucky.SetSlucka(const Value: integer);
begin
   FSlucka := Value;
end;

procedure TPulslucky.SetSluckovaniStatus(const Value: TSluckovaniStatus);
begin
   FSluckovaniStatus := Value;
   Case Value of
      ssFind: CursorIndex := 0;
      ssAngle: CursorIndex := 1;
      ssPosition: CursorIndex := 2;
   end;
end;

procedure TPulslucky.SetSLULevel(const Value: string);
begin
   FSLULevel := Value;
   SluPoints := FindLevel(Value);
end;

procedure TPulslucky.SetSluPoints(const Value: TEntity);
begin
   FSluPoints := Value;
end;

procedure TPulslucky.SetSLUQT(const Value: TRectEntityQTree);
begin
   ExchangeFieldElement(TElement(FSLUQT), Value);
end;

procedure TPulslucky.SetSnapTool(const Value: TPointLineSnapTool);
begin
   ExchangeFieldElement(TElement(FSnapTool), Value);
end;

procedure TPulslucky.SetSymbols(const Value: TTopolSymbols);
begin
   ExchangeFieldElement(TElement(FSymbols), Value);
end;

procedure TPulslucky.SetX(const Value: double);
begin
   FX := Value;
end;

procedure TPulslucky.SetY(const Value: double);
begin
   FY := Value;
end;


procedure TPulslucky.XORDraw;
var
   i: integer;
   L: TPolyLineElement;
   anAngle: double;
   Vzdalenost: double;
   P: TDoubleCoord;
begin
   if SluckovaniStatus = ssFirst then
   begin
      Device.Pen.Color:= Color;
      //Device.Pen.Style:= psSolid;
      Device.Pen.Mode:= pmXOR;
      Device.Pen.Width:= 1;
      Device.Brush.Style:= bsClear;

      Device.Pen.Style := psDot;
      Device.MoveTo(FP1.X, FP1.Y);
      Device.LineTo(X, Y);

      Device.Pen.Style:= psSolid;
      P.X := X;
      P.Y := Y;
      XYToPolar(P,FP1, Vzdalenost,anAngle);
      Symbols.PaintSymbol(FP1.X, FP1.Y, anAngle-Pi/2, Color, Slucka);
   end
   else
   if SluckovaniStatus = ssSecond then
   begin
      Device.Pen.Color:= Color;
      //Device.Pen.Style:= psSolid;
      Device.Pen.Mode:= pmXOR;
      Device.Pen.Width:= 1;
      Device.Brush.Style:= bsClear;
      Device.Pen.Style := psDot;
      Device.MoveTo(FP1.X, FP1.Y);
      Device.LineTo(FP2.X, FP2.Y);
      Device.Pen.Style:= psSolid;
      XYToPolar(FP1,FP2, Vzdalenost,anAngle);
      Symbols.PaintSymbol(FP1.X, FP1.Y, anAngle+Pi/2, Color, Slucka);
      Symbols.PaintSymbol(FP2.X, FP2.Y, anAngle-Pi/2, Color, Slucka);
   end
   else
   if SluckovaniStatus = ssOld then
   begin
      Device.Pen.Color:= Color;
      //Device.Pen.Style:= psSolid;
      Device.Pen.Mode:= pmXOR;
      Device.Pen.Width:= 1;
      Device.Brush.Style:= bsClear;
      Device.Pen.Style := psDot;
      if (S1 <> nil) and (S2 <> nil) then
      begin
         Device.MoveTo(S1.X, S1.Y);
         Device.LineTo(S2.X, S2.Y);
      end;
      Device.Pen.Style:= psSolid;
      if S1 <> nil then
         Symbols.PaintSymbol(S1.X, S1.Y, S1.Geometry.Angle, Color, Slucka);
      if S2 <> nil then
         Symbols.PaintSymbol(S2.X, S2.Y, S2.Geometry.Angle, Color, Slucka);
   end
end;

procedure TPulslucky.SetSnapped(const Value: boolean);
begin
  FSnapped := Value;
end;



procedure TPulslucky.DeleteSlu;
var
   S: string;
begin
   XORDraw;
   if S2 = nil then S := 'Byl nalezen pouze jeden bod '
               else S := 'Byly nalezeny oba body';
   S := S +' - Opravdu je chcete zruit?';
   if Confirm(S) then
   begin
      XORDraw;
      Device.Pen.Mode := pmCopy;
      if S1 <> nil then
      begin
         Symbols.PaintSymbol(S1.X, S1.Y, S1.Geometry.Angle, $FFFFFF, Slucka);
         S1.Free;
         S1 := nil;
      end;
      if S2 <> nil then
      begin
         Symbols.PaintSymbol(S2.X, S2.Y, S2.Geometry.Angle, $FFFFFF, Slucka);
         S2.Free;
         S2 := nil;
      end;
      PostMsg(tm_ModelChanged,nil,nil,false);
   end
   else
   begin
      XORDraw;
   end;
   SluckovaniStatus := ssFind;
   CursorIndex := 0;
end;


procedure TPulslucky.StoreSlu;
var
   Vzdalenost,anAngle: double;
   TP: TTopolPointElement;
   P: TPointElement;
begin
   XYToPolar(FP1,FP2, Vzdalenost,anAngle);
   if Vzdalenost < MinDistance then
   begin
      ShowMessage('Pli mal vzdlenost mezi body, operace stornovna cenzurou');
      exit;
   end;
   if GroupColor = 0 then
      GroupColor := 13;  // pokud neni nastavena barva, tak cerna
   // Bod . 1
   P := TPointElement.Create;
   P.Angle := anAngle+PI/2;
   P.X := FP1.X;
   P.Y := FP1.Y;
   TP := TTopolPointElement.Create;
   TP.Geometry := P;
   TP.Barva := GroupColor;
   TP.Znacka := Slucka;
   TP.Druh := Group;
   TP.Presnost := 10;
   SluPoints.AddElement(TP);
   SLUQT.Add(TP);
   PostMsgToOwners(tm_Review,TP,false);
   // Bod . 2
   P := TPointElement.Create;
   P.Angle := anAngle-PI/2;
   P.X := FP2.X;
   P.Y := FP2.Y;
   TP := TTopolPointElement.Create;
   TP.Geometry := P;
   TP.Barva := GroupColor;
   TP.Znacka := Slucka;
   TP.Druh := Group;
   TP.Presnost := 11;
   SluPoints.AddElement(TP);
   SLUQT.Add(TP);
   PostMsgToOwners(tm_Review,TP,false);
   Application.ProcessMessages;
   PostMsg(tm_ModelChanged,nil,nil,false);
   {
   Symbols.PaintSymbol(FP1.X, FP1.Y, anAngle+Pi/2, Color, Slucka);
   Symbols.PaintSymbol(FP2.X, FP2.Y, anAngle-Pi/2, Color, Slucka);
   }
end;

procedure TPulslucky.SetGroup(const Value: integer);
begin
  FGroup := Value;
end;

procedure TPulslucky.SetS1(const Value: TTopolPointElement);
begin
  FS1 := Value;
end;

procedure TPulslucky.SetS2(const Value: TTopolPointElement);
begin
  FS2 := Value;
end;

procedure TPulslucky.FindSecondPoint;
var
   i1,i2,ix: integer;
begin
   S2 := nil;
   i1 := SluPoints.IndexOf(S1);
   i2 := i1;
   if S1.Presnost = 10 then
   begin
      inc(i2);
      ix := 11;
   end
   else
   if S1.Presnost = 11 then
   begin
      dec(i2);
      ix := 10;
   end
   else
      exit;
   if (i2 < 0) or (i2 > SluPoints.LastIndex) then exit;
   S2 := TTopolPointElement(Slupoints.Elements[i2]);
   if S2.Presnost <> ix then
      S2 := nil;
end;

procedure TPulslucky.SetMinDistance(const Value: double);
begin
  FMinDistance := Value;
end;


procedure TPulslucky.SetGroupColor(const Value: integer);
begin
  FGroupColor := Value;
end;

INITIALIZATION
   RegisterOrbisClass(TSluckovani);
   RegisterOrbisClass(TPulSlucky);
END.
