unit t_StoreToLHC;

interface
uses
   Elements,
   e_Space,
   t_CmdLn,
   t_FrameSelect,
   t_LHCSelect,
   Tools;

type

TStoreToLHCTool = class(TTechnology)
private
   FFrameStorePath: string;
   FGlobalStorePath: string;
   FMultiFrameSelector: TMultiSelectSkupina;
   FOIL: TCommandLine;
    FFrameStoreClass: TElementClass;
    FFrameSelector: TFrameSelector;
   procedure SetMultiFrameSelector(const Value: TMultiSelectSkupina);
   procedure SetFrameStorePath(const Value: string);
   procedure SetGlobalStorePath(const Value: string);
    procedure SetOIL(const Value: TCommandLine);
    procedure SetFrameStoreClass(const Value: TElementClass);
    procedure SetFrameStoreClassName(const Value: string);
    function GetFrameStoreClassName: string;
    procedure SetFrameSelector(const Value: TFrameSelector);
public
   constructor Create; override;
   function FindGlobalStoreEntity: TEntity;
   function FindFrameStoreEntity(aFrame: TElement): TEntity;
   property OIL: TCommandLine read FOIL write SetOIL;
   property FrameStoreClass: TElementClass read FFrameStoreClass write SetFrameStoreClass;
   procedure AddElementToEntity(anElement: TElement; E: TEntity); virtual;
published
   property FrameSelector: TFrameSelector read FFrameSelector write SetFrameSelector;
   property MultiFrameSelector: TMultiSelectSkupina read FMultiFrameSelector write SetMultiFrameSelector;
   property GlobalStorePath: string read FGlobalStorePath write SetGlobalStorePath;
   property FrameStorePath: string read FFrameStorePath write SetFrameStorePath;
   property FrameStoreClassName: string read GetFrameStoreClassName write SetFrameStoreClassName;
end;

TStoreLineToLHC = class(TStoreToLHCTool)
private
   function GetFrameLineSelector: TMultiSelectSkupinaByLine;
public
   constructor Create; override;
   procedure DoIt; override;
   property  FrameLineSelector: TMultiSelectSkupinaByLine read GetFrameLineSelector;
end;

TStoreAreaToLHC = class(TStoreToLHCTool)
public
   procedure DoIt; override;
end;

TStorePointToLHC = class(TStoreToLHCTool)
public
   procedure DoIt; override;
end;


implementation
uses
   SysUtils,
   e_Class,
   G_Elems;

{ TStoreToLHCTool }

constructor TStoreToLHCTool.Create;
begin
   inherited;
   ExchangeFieldTool(TTool(FOIL), TCommandLine.Create);
   FrameStoreClass := TRectEntity;
end;

function TStoreToLHCTool.FindFrameStoreEntity(aFrame: TElement): TEntity;
var
   E: TElement;
begin
   result := nil;
   if not (aFrame is TEntity) then exit;
   E := TEntity(aFrame).FindOrCreateElementName(FrameStoreClass, FrameStorePath);
   if not (E is TEntity) then exit;
   result := TEntity(E);
end;

function TStoreToLHCTool.FindGlobalStoreEntity: TEntity;
var
   E: TElement;
begin
   result := nil;
   OIL.DoIt;
   {$IFDEF WIN64}
   E := TElement(StrToInt64Def(OIL.ResultLine, 0));
   {$ELSE}
   E := TElement(StrToIntDef(OIL.ResultLine, 0));
   {$ENDIF}
   if not (E is TEntity) then exit;
   result := TEntity(E);
end;

function TStoreToLHCTool.GetFrameStoreClassName: string;
begin
   if FrameStoreClass = nil then
      FrameStoreClass := TRectEntity;
   result := FrameStoreClass.ClassName;
end;

procedure TStoreToLHCTool.SetMultiFrameSelector(
  const Value: TMultiSelectSkupina);
begin
   ExchangeFieldTool(TTool(FMultiFrameSelector), Value);
end;

procedure TStoreToLHCTool.SetFrameStoreClass(const Value: TElementClass);
begin
   FFrameStoreClass := Value;
end;

procedure TStoreToLHCTool.SetFrameStoreClassName(const Value: string);
var
   EC: TElementClass;
begin
   EC := GetOrbisClass(Value);
   if EC = nil then
      EC := TRectEntity;
   FrameStoreClass := EC;
end;

procedure TStoreToLHCTool.SetFrameStorePath(const Value: string);
begin
   FFrameStorePath := Value;
end;

procedure TStoreToLHCTool.SetGlobalStorePath(const Value: string);
begin
   FGlobalStorePath := Value;
   OIL.Line := Value;
end;


procedure TStoreToLHCTool.SetOIL(const Value: TCommandLine);
begin
  FOIL := Value;
end;

procedure TStoreToLHCTool.SetFrameSelector(const Value: TFrameSelector);
begin
   ExchangeFieldTool(TTool(FFrameSelector), Value);
end;

procedure TStoreToLHCTool.AddElementToEntity(anElement: TElement;
  E: TEntity);
begin
   if anElement = nil then exit;
   if E <> nil then
   begin
      E.AddElement(anElement);
      if E is TRectEntity then
         TRectEntity(E).Update;
   end;
end;

{ TStoreLineToLHC }


constructor TStoreLineToLHC.Create;
begin
   inherited;
   MultiFrameSelector := TMultiSelectSkupinaByLine.Create;
end;

procedure TStoreLineToLHC.DoIt;
var
   Line: TPolyLineElement;
   E: TEntity;
   i: integer;
begin
   Line := TPolyLineElement(Input);
   if Line = nil then exit;
   AddElementToEntity(Line, FindGlobalStoreEntity);
   FrameLineSelector.Line := Line;
   FrameLineSelector.Reset;
   FrameLineSelector.DoIt;
   E := TEntity(FrameLineSelector.Output);
   if E = nil then exit;
   if E.NoElements then exit;
   for i := 0 to E.LastIndex do
   begin
      AddElementToEntity(Line, FindFrameStoreEntity(E.Elements[i]));
   end;
end;

function TStoreLineToLHC.GetFrameLineSelector: TMultiSelectSkupinaByLine;
begin
   result := TMultiSelectSkupinaByLine(MultiFrameSelector);
end;

{ TStoreAreaToLHC }

procedure TStoreAreaToLHC.DoIt;
var
   Area: TAreaElement;
   E: TEntity;
   i: integer;
begin
   Area := TAreaElement(Input);
   if Area = nil then exit;
   AddElementToEntity(Area, FindGlobalStoreEntity);
   if FrameSelector = nil then exit;
   AddElementToEntity(Area, FindFrameStoreEntity(FrameSelector.Output));
end;

{ TStorePointToLHC }

procedure TStorePointToLHC.DoIt;
var
   Point: TPointElement;
   E: TEntity;
   i: integer;
begin
   Point := TPointElement(Input);
   if Point = nil then exit;
   AddElementToEntity(Point, FindGlobalStoreEntity);
   if FrameSelector = nil then exit;
   AddElementToEntity(Point, FindFrameStoreEntity(FrameSelector.Output));
end;

INITIALIZATION
   RegisterOrbisClass(TStoreAreaToLHC);
   RegisterOrbisClass(TStoreLineToLHC);
   RegisterOrbisClass(TStorePointToLHC);
END.
