unit Ed_ums;

INTERFACE
uses
   Controls,
// KERNEL
   Elements,
// GLIB
   g_Elems,
// GUI
   m_Intera,
// MAPPER   
   t_Editor;

type


TNewUserMapSpace = class(TDrawRectTool)
private
   FWidth, FHeight, FScale: double;
protected
   function  GetCursor: TCursor; override;
   function  GetName: string; override;
public
   procedure On_Rect(aRect: TDoubleRect); override;
   procedure On_tm_MouseDown(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
   procedure DoIt; override;
published
   property Width: double read FWidth write FWidth;
   property Height: double read FHeight write FHeight;
   property Scale: double read FScale write FScale;
end;


TUserMapSpaceEditor = class(TRectEntityEditor)
protected
   function  GetCursor: TCursor; override;
   function  GetName: string; override;
   function  GetClassFilter: TRectEntityClass; override;
   function  GetNoElementCursor: TCursor; override;
   procedure SetSelectedElement(aValue: TRectEntity); override;
public
   procedure SetupSelect; override;
end;




IMPLEMENTATION

uses
   Classes,
   Graphics,
   SysUtils,
// KERNEL
   m_Orbis,
// DEVICE
   e_Device,
// MAPPER
   e_Guide,
   e_MSpace,
// MAIN
   m_Proj,
   u_Cursor;


{ TMapSpaceEditor }

function  TNewUserMapSpace.GetName: string;
begin
   result := 'Vkldat uivatelsk mapov prostor';
end;


function  TNewUserMapSpace.GetCursor: TCursor;
begin
   result := cr_NewUMS;
end;



procedure TNewUserMapSpace.On_Rect(aRect: TDoubleRect);
var
   MS: TUserMapSpace;
begin
   if Input = nil then exit;
   if not(Input is TEntity) then exit;
   MS := TUserMapSpace.Create;
   if MS = nil then exit;
   if Scale = 0 then Scale:= 10000;
   MS.Name:= 'Mapa 1:' + IntToStr(trunc(Scale));
   MS.Minx := aRect.Minx;
   MS.Miny := aRect.Miny;
   MS.Maxx := aRect.Maxx;
   MS.Maxy := aRect.Maxy;
   if Scale = 0 then Scale:= 10000;
   MS.Scale:= Scale;
{
   MS.Pen.Color := GetActualColor;
   MS.Brush.Style := bsClear;
}
   TGuide(Input).AddUserMapSpace(MS);
   PostMsgToOwners(tm_ReView, nil, DontKill);
end;


procedure TNewUserMapSpace.On_tm_MouseDown(var aMessage: TOrbisMouseEvent; var Handled: boolean);
var
  HW, HH: integer; { halfwidht, halfheight}
begin
   if aMessage.Button = mbLeft then
   begin
      Device.Pen.Mode  := pmXOR;
      Device.Pen.Color := clLime;
      Device.Pen.Style := psSolid;
      Device.Brush.Style := bsClear;
      HW:= Device.Projection.GetPageLengthHoriz1(Width/2);
      HH:= Device.Projection.GetPageLengthVert1(Height/2);
      ShiftRect := Rect(aMessage.x-hw,aMessage.y-hh,aMessage.x+hw,aMessage.y+hh);
      TCanvasDevice(Device).Canvas.Pen.Width := 1;
      with ShiftRect do TCanvasDevice(Device).Canvas.Rectangle(Left, Top, Right, Bottom);
   end
   else
   if aMessage.Button = mbRight then
   begin
      {
      Editor:= TObjectPropertyEditor.Create;
      Editor.Input:= self;
      Editor.Left:= aMessage.x+20;
      Editor.Top:= aMessage.y+20;
      Editor.DoIt
      }
   end;
   Handled := true;
end;



procedure TNewUserMapSpace.DoIt;
begin
   inherited DoIt;
end;


{ TUserMapSpaceEditor }

function  TUserMapSpaceEditor.GetName: string;
begin
   result := 'Editor uivatelskho mapovho prostoru';
end;


function  TUserMapSpaceEditor.GetCursor: TCursor;
begin
   result := cr_EditUMS;
end;


function  TUserMapSpaceEditor.GetClassFilter;
begin
   result := TUserMapSpace;
end;


function  TUserMapSpaceEditor.GetNoElementCursor: TCursor;
begin
   result := cr_NoUMS;
end;



procedure TUserMapSpaceEditor.SetupSelect;
begin
end;


procedure TUserMapSpaceEditor.SetSelectedElement(aValue: TRectEntity);
begin
   inherited SetSelectedElement(aValue);
   PostMsgToOwners(tm_SetPropertyEditors, nil, DontKill);
end;


END.
