unit Mapper.Components;
// -----------------------------------------------------------------------------------------
//   Zakladni objekty pro Desktop Mapping
// -----------------------------------------------------------------------------------------
//    Topol Pro 2019 Petr Kunz kunzp@tpro.cz
//
//   Map_Component ... zakladni stavebni prvek mapovani ....
//                     obdelnikovy prvek s moznosti zarovnani
//                     souradnice jsou nadale relativni  ne jako u puvodniho Desktopu
//
//   Layout        ... neviditelny prvek s moznosti vkladani dalsich vizualnich komponent
//                     proste layout
//
//   souradny system je jako ve zbytku Orbisu kartezska soustava, pocatek vlevo dole
//   aby to bylo komplikovanejsi, nebo teda ne? asi jo
// -----------------------------------------------------------------------------------------
interface

uses
vcl.Controls,
G_Elems,
Elements;

type

TMap_Component = class(TRectEntity)
  private
    FAlign: TAlign;
    FAnchors: TAnchors;
    procedure SetAlign(const Value: TAlign);
    procedure SetAnchors(const Value: TAnchors);
    function GetAnchorBottom: boolean;
    function GetAnchorLeft: boolean;
    function GetAnchorRight: boolean;
    function GetAnchorTop: boolean;
    procedure SetAnchorBottom(const Value: boolean);
    procedure SetAnchorLeft(const Value: boolean);
    procedure SetAnchorRight(const Value: boolean);
    procedure SetAnchorTop(const Value: boolean);
    function GetBottom: double;
    function GetLeft: double;
    function GetRight: double;
    function GetTop: double;
    procedure SetBottom(const Value: double);
    procedure SetLeft(const Value: double);
    procedure SetRight(const Value: double);
    procedure SetTop(const Value: double);
published
   property Align: TAlign read FAlign write SetAlign;
   property Anchors: TAnchors read FAnchors write SetAnchors;
   property AnchorLeft: boolean read GetAnchorLeft write SetAnchorLeft stored false;
   property AnchorTop: boolean read GetAnchorTop write SetAnchorTop stored false;
   property AnchorRight: boolean read GetAnchorRight write SetAnchorRight stored false;
   property AnchorBottom: boolean read GetAnchorBottom write SetAnchorBottom stored false;
   property Left: double read GetLeft write SetLeft stored false;
   property Top: double read GetTop write SetTop stored false;
   property Right: double read GetRight write SetRight stored false;
   property Bottom : double read GetBottom write SetBottom stored false;
end;

TMap_Layout = class(TMap_Component)


end;

implementation
uses
e_Class;

{ TMap_Component }

function TMap_Component.GetAnchorBottom: boolean;
begin
   result:= akBottom in Anchors;
end;

function TMap_Component.GetAnchorLeft: boolean;
begin
   result:= akLeft in Anchors;
end;

function TMap_Component.GetAnchorRight: boolean;
begin
   result:= akRight in Anchors;
end;

function TMap_Component.GetAnchorTop: boolean;
begin
   result:= akTop in Anchors;
end;

function TMap_Component.GetBottom: double;
begin
   result := MinY;
end;

function TMap_Component.GetLeft: double;
begin
   result := MinY;
end;

function TMap_Component.GetRight: double;
begin
   result := MaxX;
end;

function TMap_Component.GetTop: double;
begin
   result := MaxY;
end;

procedure TMap_Component.SetAlign(const Value: TAlign);
begin
  FAlign := Value;
end;

procedure TMap_Component.SetAnchorBottom(const Value: boolean);
begin
   if Value then
      Anchors := Anchors + [akBottom]
   else
      Anchors := Anchors - [akBottom];
end;

procedure TMap_Component.SetAnchorLeft(const Value: boolean);
begin
   if Value then
      Anchors := Anchors + [akLeft]
   else
      Anchors := Anchors - [akLeft];
end;

procedure TMap_Component.SetAnchorRight(const Value: boolean);
begin
   if Value then
      Anchors := Anchors + [akRight]
   else
      Anchors := Anchors - [akRight];
end;

procedure TMap_Component.SetAnchors(const Value: TAnchors);
begin
  FAnchors := Value;
end;

procedure TMap_Component.SetAnchorTop(const Value: boolean);
begin
   if Value then
      Anchors := Anchors + [akTop]
   else
      Anchors := Anchors - [akTop];
end;

procedure TMap_Component.SetBottom(const Value: double);
begin
   MinY := Value;
end;

procedure TMap_Component.SetLeft(const Value: double);
begin
   MinX := Value;
end;

procedure TMap_Component.SetRight(const Value: double);
begin
   MaxX := Value;
end;

procedure TMap_Component.SetTop(const Value: double);
begin
   MaxY := Value;
end;

initialization
   RegisterOrbisClass(TMap_Component);
   RegisterOrbisClass(TMap_Layout);
end.
