unit F_prndlg;

interface

uses WinTypes, WinProcs, Classes, Graphics, Forms, Controls, Buttons,
  StdCtrls, ExtCtrls,
  Tools;

type
  TPrnDlg = class(TForm)
    OKBtn: TBitBtn;
    CancelBtn: TBitBtn;
    HelpBtn: TBitBtn;
    ed_Scale: TEdit;
    Label1: TLabel;
    Label2: TLabel;
    ed_Overlap: TEdit;
    Panel1: TPanel;
    Image: TImage;
    b_FitToPage: TSpeedButton;
    b_CutMark: TSpeedButton;
    b_CenterPage: TSpeedButton;
    Timer: TTimer;
    lbTime: TLabel;
    lbStart: TLabel;
    Label3: TLabel;
    b_RasterLines: TSpeedButton;
    g_Pages: TGroupBox;
    ed_Pages: TEdit;
    r_AllPages: TRadioButton;
    r_Pages: TRadioButton;
    procedure b_ViewClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure ed_ScaleExit(Sender: TObject);
    procedure ed_OverlapExit(Sender: TObject);
    procedure b_FitToPageClick(Sender: TObject);
    procedure b_CenterPageClick(Sender: TObject);
    procedure OKBtnClick(Sender: TObject);
    procedure TimerTimer(Sender: TObject);
    procedure CancelBtnClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure b_RasterLinesClick(Sender: TObject);
    procedure HelpBtnClick(Sender: TObject);
    procedure r_AllPagesClick(Sender: TObject);
    procedure r_PagesClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    FSilentMode: boolean;
    procedure SetSilentMode(const Value: boolean);
    { Private declarations }
  public
    Tool: TTool;
    TheEnd: boolean;
    Start: TDateTime;
    property SilentMode: boolean read FSilentMode write SetSilentMode;
  end;


implementation

{$R *.DFM}

uses
   SysUtils, Printers_f, Dialogs,
   Elements,
   HelpCtx,
   m_Orbis,
   m_Proj;

procedure TPrnDlg.b_ViewClick(Sender: TObject);
begin
   Tool.PostMsg(tm_Review, Tool, nil, DontKill);
end;

procedure TPrnDlg.FormShow(Sender: TObject);
begin
   Tool.PostMsg(tm_ReView, Tool, nil, DontKill);
end;

procedure TPrnDlg.ed_ScaleExit(Sender: TObject);
begin
   try
      StrToFloat(ed_Scale.Text);
   except
      ed_Scale.SetFocus;
      exit;
   end;
   Tool.PostMsg(tm_PrintScaleChanged, Tool, nil, DontKill);
end;

procedure TPrnDlg.ed_OverlapExit(Sender: TObject);
begin
   try
      StrToInt(ed_OverLap.Text);
   except
      ed_OverLap.SetFocus;
      exit;
   end;
   Tool.PostMsg(tm_PrintOverlapChanged, Tool, nil, DontKill);
end;

procedure TPrnDlg.b_FitToPageClick(Sender: TObject);
begin
   Tool.PostMsg(tm_FitToPage, Tool, nil, DontKill);
end;

procedure TPrnDlg.b_CenterPageClick(Sender: TObject);
begin
   Tool.PostMsg(tm_CenterPage, Tool, nil, DontKill);
end;

procedure TPrnDlg.OKBtnClick(Sender: TObject);
begin
  //Caption:= '... tiskne se';
  OKBtn.Enabled:= false;
  b_FitToPage.Enabled:= false;
  b_CenterPage.Enabled:= false;
  b_CutMark.Enabled:= false;
  b_RasterLines.Enabled:= false;
  ed_Scale.Enabled:= false;
  ed_Overlap.Enabled:= false;
end;

procedure TPrnDlg.TimerTimer(Sender: TObject);
begin
  if Printer.Printing then
  begin
    lbTime.Caption:= TimeToStr(Now-Start);
  end
  else
  begin
    Timer.Enabled:= false;
    Close
  end
end;

procedure TPrnDlg.CancelBtnClick(Sender: TObject);
begin
  Timer.Enabled:= false;
  if TheEnd then
     Close
end;

procedure TPrnDlg.FormClose(Sender: TObject; var Action: TCloseAction);
var
  Finish: TDateTime;
begin
  if TheEnd then
  begin
    if not Printer.Printing then
    begin
       if not SilentMode then
       begin
          Finish:= Now;
          ShowMessage(DateTimeToStr(Finish) + #13 + #13 +
                     'Ukonen tisk na "' + Printer.Printers[Printer.PrinterIndex] + '"' +#13+#13+
                     'Trvn: ' + TimeToStr(Finish-Start));
       end;
    end;
    Tool.Free;
    Action:= caFree
  end
end;

procedure TPrnDlg.b_RasterLinesClick(Sender: TObject);
begin
   Tool.PostMsg(tm_RasterLines, Tool, nil, DontKill);
end;

procedure TPrnDlg.HelpBtnClick(Sender: TObject);
begin
   HelpJump('ctx_PrintDialog');
end;

procedure TPrnDlg.r_AllPagesClick(Sender: TObject);
begin
   if r_AllPages.Checked then
   begin
      r_Pages.Checked := false;
      ed_Pages.Enabled := false;
   end;
end;

procedure TPrnDlg.r_PagesClick(Sender: TObject);
begin
   if r_Pages.Checked then
   begin
      r_AllPages.Checked := false;
      ed_Pages.Enabled := true;
   end;
end;

procedure TPrnDlg.FormCreate(Sender: TObject);
begin
   Image.Picture.Bitmap.Height:= Image.Height;
   Image.Picture.Bitmap.Width:= Image.Width;
end;

procedure TPrnDlg.SetSilentMode(const Value: boolean);
var
  aB: boolean;
begin
   FSilentMode := Value;
   aB := not Value;
   OKBtn.Enabled:= aB;
   b_FitToPage.Enabled:= aB;
   b_CenterPage.Enabled:= aB;
   b_CutMark.Enabled:= aB;
   b_RasterLines.Enabled:= aB;
   ed_Scale.Enabled:= aB;
   ed_Overlap.Enabled:= aB;
end;

end.

