unit T_prndlg;

interface
uses
// KERNEL
   Elements,
// DEVICE
   e_Device,
// GUI
   t_InterA,
// MAPPER
   e_Desktp,
   e_PrnPg,
   f_PrnDlg;


type

TPrintDialogTool = class(TInteractiveTool)
  private
    FMultiPage: boolean;
    FSilentMode: boolean;
    FFileName: string;
    procedure SetMultiPage(const Value: boolean);
    procedure SetSilentMode(const Value: boolean);
    procedure SetFileName(const Value: string);
 protected
   FForm: TPrnDlg;
   FPrintDevice: TPrintDevice;
   FPrintPages: TPrintPages;
   FDeskTop: TDeskTop;
   procedure   SetInput(aInput: TElement); override;
   procedure   CreateForm; virtual;
   procedure   CreateDevice; virtual;
   procedure   LinkDeviceToForm; virtual;
   procedure   CreateInterpret; virtual;
   procedure   LinkInterpretToDevice; virtual;
   function    GetVisible: boolean; virtual;
   procedure   SetVisible(aVisible: boolean);  virtual;
   function    GetPrintPages: TPrintPages;
   function    GetScale: double;
   procedure   SetScale(aValue: double);
{   function    GetDesktop: TDesktop; }

 public
   constructor Create; override;
   procedure   DoIt; override;
   procedure   Show;
   procedure   on_tm_QuitTask; override;
   procedure   On_tm_Review(var aMessage: TOrbisMessage; var Handled: boolean); override;
   procedure   on_tm_FitToPage(var Handled: boolean); virtual;
   procedure   on_tm_PrintScaleChanged(var Handled: boolean); virtual;
   procedure   on_tm_PrintOverlapChanged(var Handled: boolean); virtual;
   procedure   on_tm_CenterPage(var Handled: boolean); virtual;
   procedure   on_tm_RasterLines(var Handled: boolean); virtual;
   procedure   HandleMessage(var aMessage: TOrbisMessage; var Handled: boolean); override;
   procedure   PutStartTime(aTime: TDateTime);
   procedure   PutPageNo(aStr: string);

   property    PrintDevice: TPrintDevice read FPrintDevice write FPrintDevice;
   property    Form: TPrnDlg read FForm;
   property    Visible: boolean read GetVisible write SetVisible;
   property    PrintPages: TPrintPages read GetPrintPages write FPrintPages;
   property    Scale: double read GetScale write SetScale;
   property    Desktop: TDesktop read FDesktop write FDeskTop;
   property    StartTime: TDateTime write PutStartTime;
   property    PageNo: string    write PutPageNo;
   property    MultiPage: boolean read FMultiPage write SetMultiPage;
   property    SilentMode: boolean read FSilentMode write SetSilentMode;
   property    FileName: string read FFileName write SetFileName;
end;



implementation

uses
   Controls,
   Forms,
   Graphics,
   IniFiles,
   SysUtils,
// ELEMENTS
   m_Orbis,
   Worlds,
// GLIB
   g_Elems,
// DEVICE
   d_STDImg,
// INTERPRET
   e_iScrpt,
   i_Uni,
// TOPOL
   t_TisCre,
// APP
   t_MainTl,
// MAIN
   m_Proj;


{ TPrintDialogTool }


constructor TPrintDialogTool.Create;
begin
   inherited Create;
   CreateForm;
   CreateDevice;
   LinkDeviceToForm;
   CreateInterpret;
   LinkInterpretToDevice;
end;



procedure   TPrintDialogTool.DoIt;
var
   Ini: TIniFile;
   S: string;
begin
  if Input = nil then exit;
  S := '';
   if FileName <> '' then
      S := ExtractFileName(FileName);
   Ini := TIniFile.Create(TOldMainTool(World.MainTool).IniFileName);
   TPrnDlg(Self.Form).b_RasterLines.Down := Ini.ReadBool('PRINT', 'RasterizeLinesByOrbis', false);
   PrintDevice.RasterizeLinesByOrbis := Form.b_RasterLines.Down;
   Ini.Free;
   TPrnDlg(Self.Form).b_CenterPage.Down:= true;
   PrintPages.Centering := Form.b_CenterPage.Down;
   TRectEntity(Input).ResetAndUpdate;
   Form.g_Pages.Visible := MultiPage;
   Form.SilentMode := SilentMode;
   Form.Caption := 'Pprava tisku: '+S;

   if not SilentMode and (Form.ShowModal <> mrOK) then
   begin
      Output:= nil;
      Form.Free;
      FForm:= nil;
      Input.Free; {v inputu je nyni neco, co jsem si vytvoril sam}
      Input:= nil;
      //PrintPages.Free;  // PrintPages se uvolnuji s Inputem asi ... :-)
      PrintPages := nil;
      Output := nil;
   end
   else
   begin
     Form.Caption := 'Tisk: '+S;
     Form.Show;
     Form.TheEnd:= true;
     Form.Timer.Enabled:= true
     { ostatn uvolnn a pi zaven formu }
   end;

end;

procedure   TPrintDialogTool.SetInput(aInput: TElement);
var
  DTP: TDeskTop;
  RE:  TDrawRectangle;

begin
   if not (aInput is TDeskTop) then exit;
   DeskTop:= TDeskTop(aInput);
   DTP:= TDeskTop.Create;
   RE:= TDrawRectangle.Create;
   RE.Rect:= DeskTop.Rect.CreateDuplicate;
   RE.BrushStyle:= bsClear;
   RE.PenStyle:= psSolid;
   RE.BrushColor:= $FFFFFF;
   RE.PenColor:= clRed;
   RE.PenWidth:= 0.002;
   aInput:= DTP;
   inherited SetInput(aInput);
   try
      PrintPages.Desktop := DeskTop;
      TRectEntity(Input).AddAndUpdateRect(PrintPages);
      DTP.AddAndUpdateRect(RE);
   except
   end;
end;


procedure   TPrintDialogTool.CreateForm;
begin
   Application.CreateForm(TPrnDlg, FForm);
   Form.Tool := Self;
end;

procedure   TPrintDialogTool.CreateDevice;
begin
   Device := TImageCanvasDevice.Create;
end;

procedure   TPrintDialogTool.LinkDeviceToForm;
begin
   with TImageCanvasDevice(Device) do
   begin
      Bitmap := Form.Image.Picture.Bitmap;
   end;
end;

procedure   TPrintDialogTool.CreateInterpret;
var
   aTool: TUniversalTopolScriptCreator;
   aScript: TInterpretScript;
begin
   aTool := TUniversalTopolScriptCreator.Create;
   aTool.DoIt;
   aScript:= TInterpretScript(aTool.Output);
{   aScript.Smaz; }
   Interpret := TUniversalInterpret.Create;
   TUniversalInterpret(Interpret).Script := aScript;
   aTool.Free;
end;



procedure   TPrintDialogTool.LinkInterpretToDevice;
begin
   Interpret.Device := Device;
end;

function    TPrintDialogTool.GetVisible: boolean;
begin
   try
      result := Form.Visible;
   except
      result := false;
   end;
end;

procedure   TPrintDialogTool.SetVisible(aVisible: boolean);
begin
   try
      Form.Visible := aVisible;
      Device.SetupPage(nil);
   except
   end;
end;

function    TPrintDialogTool.GetPrintPages: TPrintPages;
begin
   result:= nil;
   if PrintDevice = nil then exit;
   if FPrintPages = nil then
   begin
      FPrintPages := TPrintPages.Create;
      FPrintPages.PrintDevice:= PrintDevice;
      FPrintPages.MultiPage := MultiPage;
      Output := FPrintPages;
   end;
   result := FPrintPages;
end;

function    TPrintDialogTool.GetScale: double;
begin
   result := PrintPages.Scale;
end;

procedure   TPrintDialogTool.SetScale(aValue: double);
begin
   PrintPages.Scale := aValue;
end;
{
function    TPrintDialogTool.GetDesktop: TDesktop;
begin
   result := nil;
   if Input is TEntity then result := TDesktop(TEntity(Input).FindElementClass(TDesktop));
end;
}


procedure   TPrintDialogTool.Show;
begin
   Visible := true;
end;



procedure   TPrintDialogTool.On_tm_ReView(var aMessage: TOrbisMessage; var Handled: boolean);
begin
   if not (Input is TRectEntity) then exit;
   Device.SetupPage(nil);
   Device.Page.Color := clGray;
   Device.Projection.ChangeProjectionByRectangle(TRectEntity(Input).Rect, Device.Page);
   inherited On_tm_ReView(aMessage, Handled);
end;

procedure   TPrintDialogTool.on_tm_FitToPage(var Handled: boolean);
var
   Scale1, Scale2: double;
   MapWidth, MapHeight: double;
begin
   if not Form.b_FitToPage.Down then exit;
   MapWidth := Desktop.Width;
   MapHeight:= Desktop.Height;
   Scale1 := Desktop.Width/(PrintDevice.RealWidth - 2*PrintPages.Overlap);
   Scale2 := Desktop.Height/(PrintDevice.RealHeight - 2*PrintPages.Overlap);
   if Scale1 > Scale2 then Scale := Scale1
                      else Scale := Scale2;
   Form.ed_Scale.Text := FloatToStrF(Scale, ffFixed, 10, 2);
   TRectEntity(Input).ResetAndUpdate;
   PostMsg(tm_ReView, Self, nil, DontKill);
end;

procedure   TPrintDialogTool.on_tm_PrintScaleChanged(var Handled: boolean);
begin
   Scale := StrToFloat(Form.ed_Scale.Text);
   Form.b_FitToPage.Down := false;
   TRectEntity(Input).ResetAndUpdate;
   PostMsg(tm_ReView, Self, nil, DontKill);
end;

procedure   TPrintDialogTool.on_tm_PrintOverlapChanged(var Handled: boolean);
var
   Tmp: double;
begin
   Tmp := StrToInt(Form.ed_Overlap.Text);
   PrintPages.Overlap := tmp/1000;
   Form.b_FitToPage.Down := false;
   TRectEntity(Input).ResetAndUpdate;
   PostMsg(tm_ReView, Self, nil, DontKill);
end;


procedure   TPrintDialogTool.on_tm_CenterPage(var Handled: boolean);
var
   Tmp: double;
begin
   PrintPages.Centering := Form.b_CenterPage.Down;
   TRectEntity(Input).ResetAndUpdate;
   PostMsg(tm_ReView, Self, nil, DontKill);
end;

procedure   TPrintDialogTool.on_tm_RasterLines(var Handled: boolean);
var
   Ini: TIniFile;
begin
   Handled := true;
   Ini := TIniFile.Create(TOldMainTool(World.MainTool).IniFileName);
   Ini.WriteBool('PRINT', 'RasterizeLinesByOrbis', Form.b_RasterLines.Down);
   PrintDevice.RasterizeLinesByOrbis := Form.b_RasterLines.Down;
   Ini.Free;
end;


procedure   TPrintDialogTool.on_tm_QuitTask;
begin
  Form.Timer.Enabled:= false;
  if Form <> nil then
     Form.Free;
  if Input <> nil then
     Input.Free
end;



procedure TPrintDialogTool.HandleMessage(var aMessage: TOrbisMessage; var Handled: boolean);
begin
   Handled := false;
   inherited HandleMessage(aMessage, Handled);
   if Handled then exit;
   case aMessage.Message of
      tm_QuitTask            : on_tm_QuitTask;
      tm_FitToPage           : on_tm_FitToPage(Handled);
      tm_PrintScaleChanged   : on_tm_PrintScaleChanged(Handled);
      tm_PrintOverlapChanged : on_tm_PrintOverlapChanged(Handled);
      tm_CenterPage          : on_tm_CenterPage(Handled);
      tm_RasterLines         : on_tm_RasterLines(Handled);
   end;
end;


procedure   TPrintDialogTool.PutStartTime(aTime: TDateTime);
begin
  Form.Start:= aTime;
  Form.lbStart.Caption:= TimeToStr(aTime);
end;


procedure   TPrintDialogTool.PutPageNo(aStr: string);
begin
  Form.Caption:= aStr;
end;



procedure TPrintDialogTool.SetMultiPage(const Value: boolean);
begin
   FMultiPage := Value;
end;

procedure TPrintDialogTool.SetSilentMode(const Value: boolean);
begin
  FSilentMode := Value;
end;

procedure TPrintDialogTool.SetFileName(const Value: string);
begin
  FFileName := Value;
end;

END.
