unit I_Chk;

interface
uses
// KERNEL
  Elements,
  Tools,
// INTERPRET
  i_Basic;

type

TCheckInterpret = class(TInterpret)
private
   FTestCount: longInt;
   FBeforeRow, FAfterRow, FHeader: string;
protected
   procedure WriteInfo(Element: TElement); virtual; abstract;
public
   property TestCount: longInt read FTestCount write FTestCount;
published
   property BeforeRow: string read FBeforeRow write FBeforeRow;
   property Header: string read FHeader write FHeader;
   property AfterRow: string read FAfterRow write FAfterRow;
end;



TCheckPropsInterpret = class(TCheckInterpret)
private
  FWriteProperty: string;
  FWriteCondition: string;
  FInsideProcessCondition: string;
published
  property WriteProperty: string read FWriteProperty write FWriteProperty;
  property WriteCondition: string read FWriteCondition write FWriteCondition;
  property InsideProcessCondition: string read FInsideProcessCondition
                                          write FInsideProcessCondition;
end;



TTestInterpret = class(TCheckInterpret)
protected
   procedure WriteInfo(Element: TElement); override;
   procedure Process(Element: TElement); override;
   procedure CloseWriting; virtual;
public
   procedure Close; override;
end;



implementation
uses
  Forms,
  Dialogs,
  SysUtils,
  d_Txt,
  E_Class,
  CreaCond,
  T_Oper,
  T_Chk;

                          { TTestInterpret }


procedure  TTestInterpret.WriteInfo(Element: TElement);
begin
  Device.WriteLnText(Element.Name);
end;


procedure  TTestInterpret.Process(Element: TElement);
var
  I: longInt;
  T: TTool;
  aTool: TCheckTool;
  AreErrors: boolean;
  SendErrors: boolean;
  S: string;
begin
   AreErrors := false;
   SendErrors := false;
   if Tools.Count = 0 then exit;

   for I:= 0 to Tools.Count-1 do
   begin
      try
      T := Tool[I];
      if not (T is TCheckTool) then continue;
      aTool := TCheckTool(T);
      aTool.Reset;
      aTool.Input:= Element;
      aTool.DoIt;
      if aTool.Error > 0 then
      begin
         aTool.ErrorCount := aTool.ErrorCount + 1;
         AreErrors:= true;
         if not aTool.DontSendErrors then SendErrors := true;
      end;
      except

      end;
   end;

   if AreErrors and SendErrors then
   begin
      if Header <> '' then
         Device.WriteText(Header);

      if BeforeRow <> '' then
         Device.WriteText(BeforeRow);


      WriteInfo(Element);

      for I:= 0 to Tools.Count-1 do
      begin
         T := Tool[I];
         if not (T is TCheckTool) then continue;
         aTool := TCheckTool(T);
         if aTool.Error > 0 then
            {
            Device.WriteLnText('('+ IntToStr(aTool.Error) + ') ' + aTool.ErrorMsg)
            }
            if not aTool.DontSendErrors then
               Device.WriteLnText(aTool.ErrorTxt + ' ' +  aTool.ErrorMsg)
      end;

      if AfterRow <> '' then
         Device.WriteLnText(AfterRow);
      Application.ProcessMessages;
   end;
end;



procedure TTestInterpret.Close;
begin
   CloseWriting;
   inherited Close;
end;


procedure TTestInterpret.CloseWriting;
var
   S: string;
   aTool: TCheckTool;
   i: longInt;
begin
   S := '   ';
   Device.WritelnText(S);
   Device.TextColor := 0;
   Device.WritelnText('******************************************');
   if TestCount > 0 then
      Device.TextColor := $00A000;
   Device.WritelnText(IntToStr(TestCount) + ' objekt bylo testovno nstrojem '+ Name);
   Device.TextColor := 0;
   Device.WritelnText(' Z toho chybnch:');
    for I:= 0 to Tools.Count-1 do
    begin
       aTool := TCheckTool(Tool[I]);
       if not (aTool is TCheckTool) then continue;
       if aTool.ErrorCount > 0 then
          Device.TextColor := 255;
       Device.WriteLnText(S + IntToStr(aTool.ErrorCount) + ' .... ' +  aTool.Name + ' ' + aTool.ResultMsg);
       Device.TextColor := 0;
    end;
   Device.WritelnText('******************************************');
end;


initialization
   RegisterOrbisClass(TCheckInterpret);
   RegisterOrbisClass(TTestInterpret);
end.
