unit t_ChkFile;

interface
uses
   e_Files,
   e_Type,
   Elements,
   t_Chk,
   t_Recurs;

type

TCheckAnyFile = Class(TCheckTool)
private
    FErrorText: string;
    FDir: TDirectoryElement;
    procedure SetDir(const Value: TDirectoryElement);
    procedure SetErrorText(const Value: string);
    procedure SetNextErrorText(const Value: string);
protected
   function  GetError: longInt; override;
   function  GetErrorMsg: string; override;
public
   function FindAllFiles(const aName: string): TEntity;
   function FindPathFile(const aName: string): boolean;
   property ErrorText: string read FErrorText write SetErrorText;
   property NextErrorText: string write SetNextErrorText;
published
   property Dir: TDirectoryElement read FDir write SetDir;
end;


TCheckMDLFile = class(TCheckAnyFile)
private
    FMDL: TStringListElement;
   procedure SetMDL(const Value: TStringListElement);
protected
   function  ElementCondition: boolean; override;
   function  ReadMDL(const aValue: string): boolean;
   procedure CHeckMDLSections;
public
   property MDL: TStringListElement read FMDL write SetMDL;
end;

TCheckNDIFile = class(TCheckAnyFile)
private
    FNDI: TOwnerEntity;
    procedure SetNDI(const Value: TOwnerEntity);
protected
   function  ElementCondition: boolean; override;
   function  ReadNDI(const aValue: string): boolean;
   procedure CHeckNDISections;
public
   property NDI: TOwnerEntity read FNDI write SetNDI;
   function FindTableItemFiles(const ItemName: string): boolean;
end;

TFindAllFileElements = class(TRecursor)
  private
    FFileName: string;
    procedure SetFileName(const Value: string);
protected
   function  QGoInside(Element: TElement): boolean; override;
   function  QProcessBefore(Element: TElement): boolean; override;
   procedure Process(Element: TElement); override;
   procedure PrepareOutPut; override;
public
   property FileName: string read FFileName write SetFileName;
end;

implementation
uses
   Classes,
   DB,
   DBTables,
   Forms,
   SysUtils,
   e_Class,
   u_String;

{ TCheckNDIFile }

procedure TCheckNDIFile.CHeckNDISections;
var
   i: integer;
   SL: TStringListElement;
   SFile,SItem,STab: string;
begin
   //Error := 1;
   for i := 0 to NDI.LastIndex do
   begin
      SL := TStringListElement(NDI.Elements[i]);
      STab := SL.GetStringByName('Tab');
      SFile := SL.GetStringByName('File');
      SItem := SL.GetStringByName('Item_for_file');
      //if (SFile <> '') or (SItem <> '') then
      //   NextErrorText := ' TAB: '+STab;
      if (SFile <> '') then
      begin
         if not FindPathFile(SFile) then
         begin
            NextErrorText := '    File: '+SFile;
            Error := 1;
         end;
      end;
      if (SItem <> '') then
      begin
         if not FindTableItemFiles(SItem) then
         begin
            Error := 1;
         end;
      end;
   end;
end;

function TCheckNDIFile.ElementCondition: boolean;
var
   F: TFileElement;
begin
   result := true;
   Error := 0;
   ErrorText := '';
   if not (Input is TFileElement) then exit;
   F := TFileElement(Input);
   if CompareText(F.FileExt,'NDI') <> 0 then exit;
   result := false;
   if not ReadNDI(F.FileName) then exit;
   CheckNDISections;
   //ErrorText := F.FileName;
   //ErrorText := Dir.FileName;
   //NextErrorText := F.FileName;
   result := Error = 0;
end;

function TCheckNDIFile.FindTableItemFiles(const ItemName: string): boolean;
var
   S,SF: string;
   E: TEntity;
   i: integer;
   T: TTable;
   F: TField;
   ErrCount: integer;
begin
   result := true;
   S := Input.Name;
   S := ChangeFileExt(S,'.DBF');
   E := FindAllFiles(S);
   for i := 0 to E.LastIndex do
   begin
      T := TTable.Create(Application);
      S := '    Tabulka: '+TFileElement(E.Elements[i]).FileName+ ' poloka: '+ItemName;
      T.TableName := TFileElement(E.Elements[i]).FileName;
      T.Active := true;
      if not T.Active then
      begin
         T.Free;
         Error := 1;
         result := false;
         NextErrorText := S;
         NextErrorText := '       ...Ble neuren chyba DB tabulky';
         exit;
      end;
      T.First;
      ErrorText := '';
      //Error := 0;
      F := T.FindField(ItemName);
      if F = nil then
      begin
         T.Free;
         Error := 1;
         result := false;
         NextErrorText := S;
         NextErrorText := '       poloka  neexistuje!';
         exit;
      end;
      if T.RecordCount > 10 then
      begin
         StartProgress(Name,0,T.RecordCount - 1);
         ProgressText := S;
         Application.ProcessMessages;
      end;
      ErrCount := 0;
      While not T.Eof do
      begin
         F := T.FindField(ItemName);
         SF := F.AsString;
         if BezVnejsichMezer(SF) <> '' then
         if not FindPathFile(SF) then
         begin
            if ErrCount = 0 then
               NextErrorText :=  S ;
            NextErrorText := '       '+ SF;
            Inc(ErrCount);
            Error := 1;
            result := false;
         end;
         if T.RecNo mod 5 = 0 then
         begin
            Progress := T.RecNo;
            ProgressText := S+' '+IntToStr(T.RecNo)+'/'+IntToStr(ErrCount);
            Application.ProcessMessages;
         end;
         T.Next;
      end;
      EndProgress;
      T.Free;
   end;
end;

function TCheckNDIFile.ReadNDI(const aValue: string): boolean;
var
   S: string;
   L: TStringList;
   SL: TStringListElement;
   i: integer;
begin
   result := true;
   L := TstringList.Create;
   try
      L.LoadFromFile(aValue);
   except
      L.Free;
      Error := 1;
      ErrorText := '...chyba ten souboru';
      result := false;
      exit;
   end;
   NDI := TOwnerEntity.Create;
   NDI.Name := aValue;
   SL := nil;
   for i := 0 to L.Count - 1 do
   begin
      S := L.Strings[i];
      if S = '' then continue;
      if S[1]='[' then
      begin
         SL := TStringListElement.Create;
         SL.Name := S;
         NDI.AddElement(SL);
      end
      else
      begin
         if SL <> nil then
            SL.AddNameIsValue(S);
      end;
   end;
   L.Free;
end;

procedure TCheckNDIFile.SetNDI(const Value: TOwnerEntity);
begin
   ExchangeFieldElement(TElement(FNDI), Value);
end;

{ TCheckAnyFile }

procedure TCheckAnyFile.SetDir(const Value: TDirectoryElement);
begin
  FDir := Value;
end;

function TCheckAnyFile.FindAllFiles(const aName: string): TEntity;
var
   FAF: TFindAllFileElements;
begin
   FAF := TFindAllFileElements.Create;
   FAF.Input := Dir;
   FAF.FileName := aName;
   FAF.DoIt;
   result := TEntity(FAF.Output);
   FAF.Free;
end;

function TCheckAnyFile.FindPathFile(const aName: string): boolean;
var
   S: string;
   E: TEntity;
   El: TElement;
   i: integer;
   F: TFileElement;
begin
   result := false;
   S := ExtractFileName(aName);
   E := FindAllFiles(S);
   for i := 0 to E.LastIndex do
   begin
      El := E.Elements[i];
      if not (El is TFileElement) then continue;
      F := TFileElement(El);
      //NextErrorText := '     '+F.FileName;
      if AnsiPos(AnsiUpperCase(aName),AnsiUpperCase(F.FileName)) > 0 then
      begin
         result := true;
         E.Free;
         exit;//one file found ... OK
      end;
   end;
   E.Free;
end;

function TCheckAnyFile.GetError: longInt;
begin
   result := error;
end;

function TCheckAnyFile.GetErrorMsg: string;
begin
   result := ErrorText;
end;


procedure TCheckAnyFile.SetErrorText(const Value: string);
begin
  FErrorText := Value;
end;

procedure TCheckAnyFile.SetNextErrorText(const Value: string);
begin
   FErrorText := FErrortext + #13+#10+Value;
end;



{ TFindAllFileElements }

procedure TFindAllFileElements.PrepareOutPut;
begin
   Output := TEntity.Create;
end;

procedure TFindAllFileElements.Process(Element: TElement);
begin
   if AnsiCompareText(Element.Name,FileName) = 0 then
      TEntity(Output).AddElement(Element);
end;

function TFindAllFileElements.QGoInside(Element: TElement): boolean;
begin
   result := Element is TDirectoryElement;
end;

function TFindAllFileElements.QProcessBefore(Element: TElement): boolean;
begin
   result := Element.ClassType = TFileElement;
end;

procedure TFindAllFileElements.SetFileName(const Value: string);
begin
  FFileName := Value;
end;


{ TCheckMDLFile }

function TCheckMDLFile.ElementCondition: boolean;
var
   F: TFIleElement;
begin
   result := true;
   Error := 0;
   ErrorText := '';
   if not (Input is TFileElement) then exit;
   F := TFileElement(Input);
   if CompareText(F.FileExt,'MDL') <> 0 then exit;
   result := ReadMDL(F.FileName);
end;

procedure TCheckMDLFile.CHeckMDLSections;
begin

end;

function TCheckMDLFile.ReadMDL(const aValue: string): boolean;
var
   S,S1: string;
   L: TStringList;
   i,j: integer;
   E: TEntity;
   F: TFileElement;
begin
   result := true;
   L := TstringList.Create;
   try
      L.LoadFromFile(aValue);
   except
      L.Free;
      Error := 1;
      ErrorText := '...chyba ten souboru';
      result := false;
      exit;
   end;
   //MDL := TStringListElement.Create;
   //MDL.Name := aValue;
   for i := 0 to L.Count - 1 do
   begin
      if CompareText(L.Names[i],'SEC_DBF') <> 0 then continue;
      S := L.Strings[i];
      StringAsTwoParams(S,'=',S1,S);
      //MDL.AddNameIsValue(S);
      if (S <> '') then
      begin
         if not FindPathFile(S) then
         begin
            NextErrorText := '    File: '+S;
            S1 := ExtractFileName(S);
            E := FindAllFiles(S1);
            if not E.NoElements then
            begin
               NextErrorText := '       podobn soubory: ';
               for j := 0 to E.LastIndex do
               begin
                  F := TFileElement(E.Elements[j]);
                  NextErrorText := '        '+F.FileName;
               end;
            end;
            E.Free;
            Error := 1;
            result := false;
         end;
      end;
   end;
   L.Free;
end;

procedure TCheckMDLFile.SetMDL(const Value: TStringListElement);
begin
  FMDL := Value;
end;

INITIALIZATION
   RegisterOrbisClass(TCheckNDIFile);
   RegisterOrbisClass(TCheckMDLFile);
END.
