unit T_prpflt;

interface

uses
   e_Type,
   Elements,
   Tools;

type

TIntegerPropertyFilter = class(TBooleanTool)
private
   FPropertyName: string;
   FValues: TIntegerListElement;
   FExclude: boolean;
   FLastValue: longInt;
   function GetValues: TIntegerListElement;
   function GetValuesString: string;
   procedure SetValues(aValue: TIntegerListElement);
   procedure SetValuesString(aValue: string);
   procedure SetNextValue(aValue: longInt);
public
   procedure DoIt; override;
   property Values: TIntegerListElement read GetValues write SetValues;
   property LastValue: longInt read FlastValue;
published
   property Exclude: boolean read FExclude write FExclude;
   property PropertyName: string read FPropertyName write FPropertyName;
   property ValuesString: string read GetValuesString write SetValuesString;
   property NextValue: longInt write SetNextValue;
end;

implementation

uses
   SysUtils,
   e_Class,
   u_String;

{ TIntegerPropertyFilter }


function TIntegerPropertyFilter.GetValues: TIntegerListElement;
begin
   if FValues = nil then
   begin
      FValues := TIntegerListElement.Create;
      AddElement(FValues);
   end;
   result := FValues;
end;

function TIntegerPropertyFilter.GetValuesString: string;
var
   i: longInt;
begin
   result := '';
   if Values.NoElements then exit;
   for i := 0 to Values.LastIndex do
   begin
      if i > 0 then result := result + ',';
      result := result + IntToStr(Values.Integers[i]);
   end;
end;

procedure TIntegerPropertyFilter.SetValues(aValue: TIntegerListElement);
begin
   if FValues <> nil then
   begin
      FreeElement(FValues);
      FValues := nil;
   end;
   FValues := aValue;
   AddElement(FValues);
end;

procedure TIntegerPropertyFilter.SetValuesString(aValue: string);
var
   S: string;
   B: boolean;
begin
   if aValue = '' then exit;
   repeat
      B := StringAsTwoParams(aValue, ',', S, aValue);
      try
         NextValue := StrToInt(S);
      except
      end;
   until not B;
end;

procedure TIntegerPropertyFilter.DoIt;
var
   S: string;
   i: longInt;
begin
   Result := Exclude;
   if Input = nil then exit;
   if PropertyName = '' then exit;
   FLastValue := Input.GetPropertyLongintByName(PropertyName);
   for i := 0 to Values.LastIndex do
   begin
      if FLastValue = Values.Integers[i] then
      begin
         Result := not Exclude;
         break;
      end;
   end;
end;

procedure TIntegerPropertyFilter.SetNextValue(aValue: longInt);
begin
   Values.AddInteger(aValue);
end;


INITIALIZATION
   RegisterOrbisClass(TIntegerPropertyFilter);
END.
