unit T_FiArLi;

interface
uses
// KERNEL
   Elements,
   Tools,
// SPACE
   E_Reg;

type

TFindAreaThroughLines = class(TTool)
private
  FError: integer;
  FReg: TRegister;
  FQuality: integer;
  function GetReg: TRegister;
  procedure SetQuality(aVal: integer);
    function GetClearFinding: boolean;
public
  procedure DoIt; override;
  property Reg: TRegister read GetReg;
published
  property Error: integer read FError write FError;
  property Quality: integer read FQuality;
  property ClearFinding: boolean read GetClearFinding;
// ClearFinding - nalezena jedna a prv jedna plocha z tho bloku jako vechny linie
end;

implementation
uses
  SysUtils,
// KERNEL
  Worlds,  
// SPACE
  e_Space,
// TOPOL
  e_TplPrp;

            { TFindAreaThroughLines }

procedure TFindAreaThroughLines.DoIt;
var
  anAre: TPolyLineAreaElement;

  AreaNo: longInt;
  NeibArea, BestNeibArea: TAreaElement;
  aLineProps: TTopolLineProps;
  aLine: TPolyLineElement;
  I: longInt;
  FirstBlockReg, aBlockReg, BestBlockReg: TBlockRegister;

begin
  if Reg = nil then
  begin
    Error:= 1;
    exit;
  end;

  if not (input is TPolyLineAreaElement) then
  begin
    Error:= 2;
    exit;
  end;

  anAre:= TPolyLineAreaElement(input);

{ hledn ploch pes linie }
  BestBlockReg:= nil;
  FirstBlockReg:= nil;
  BestNeibArea:= nil;
  FQuality:= 0;

  for I:= 0 to anAre.Count-1 do
  { cyklus pes vechny linie, nalezen nejvt plochy od kter je mon pevzt
    vlastnosti }
  begin
  try
  { naten sprvn linie }
    aLine:= TPolyLineElement(anAre.Elements[I]);
    if not (aLine is TPolyLineElement) then continue;
    if not (aLine.Properties is TTopolLineProps) then continue;

  { zjitn sla plochy ze sprvn strany a nalezen Registru objekt }
    aLineProps:= TTopolLineProps(aLine.Properties);
    AreaNo:= aLineProps.GetRightAreaNoForBegin(anAre.LinesOrigines[I]);
    aBlockReg:= Reg.GetBlockRegister(aLineProps.BlockName);

    if aBlockReg = nil then
    begin
      SetQuality(1); { neregistrovan linie }
      continue;
    end;

    if FirstBlockReg = nil then
    begin
      FirstBlockReg:= aBlockReg;
    end
    else
    begin
      if (aBlockReg <> FirstBlockReg) then
      SetQuality(2) { obklopujc linie jsou z rznch blok }
    end;


  { nalezen ve stejnem registru vlastn plochy, mus mt nastaveny vlastnosti }
    NeibArea:= TAreaElement(aBlockReg.GetObject('P', AreaNo));
    if NeibArea = nil then
    begin
      SetQuality(4); { linie ukazuje na neregistrovanou plochu }
      continue;
    end;

    if NeibArea.Properties = nil then
    begin
      SetQuality(8); { linie ukazuje na plochu bez vlastnost }
      continue
    end;

  { nalezen nejvt - nejreprezentativnj plochy }
    if BestNeibArea <> nil then
    begin
      if NeibArea <> BestNeibArea then
      begin
        SetQuality(16); { linie ukazuj na rzn plochy }
        if NeibArea.WholeArea > BestNeibArea.WholeArea then
        begin
          BestNeibArea:= NeibArea;
          BestBlockReg:= aBlockReg
        end
      end
    end
    else
    begin
      BestNeibArea:= NeibArea;
      BestBlockReg:= aBlockReg
    end;
  except
      on EX: Exception  do
      begin
        Error:= 3;
      end

  end
  end;

  Output:= BestNeibArea;
  if BestNeibArea = nil then
     SetQuality(32)
end;


procedure TFindAreaThroughLines.SetQuality(aVal: integer);
begin
  FQuality:= FQuality or aVal;
end;


function TFindAreaThroughLines.GetReg: TRegister;
begin
  result:= FReg;
  if result <> nil then exit;
  result:= TRegister(World.ObjectModel.FindElementClass(TRegister));
end;

function TFindAreaThroughLines.GetClearFinding: boolean;
begin
  result:= (Quality = 0) and (Error = 0)
end;

end.
