unit T_exlntl;
{ Soustava nstroj pro export lini do topolsk struktury }

interface
uses
// KERNEL
  Elements,
  Tools,
// SPACE
  //E_Space,
// SPACETOOL
  T_ToTool,
// TOPOL
  e_TopoLElements,
  T_AreLin,
  T_DBFExp;

type


TPrepareExportLinesToTopol = class(TTool)
{ Pprava entity lini pro export
  input - TEntity exportovanch lini - sekvenn
  output = input
  DoIt:
    nejdve pro vechny linie nastav sla lini. Topologick vazby nastav jen
    vzhledem k tm linim kter jsou t soust entity.
}
private
  FTopolNo: longInt;
    FNoTopology: boolean;
  procedure DoLine(aLine: TTopoLLineElement);
    procedure SetNoTopology(const Value: boolean);
public
  procedure DoIt; override;
  property  NoTopology: boolean read FNoTopology write SetNoTopology;
end;


TExportLinesToTopol = class(TExportTopolElements)
  private
    FNoTopology: boolean;
    procedure SetNoTopology(const Value: boolean);
{ input - TEntity exportovanch lini - sekvenn
  output = input
  DoIt
    provede vlastn export do soubor TN.LIN tak jak jsou nastaveny. Na plochy
    nebere zetel.
}
public
  procedure DoIt; override;
  property  NoTopology: boolean read FNoTopology write SetNoTopology;
end;

(*
TSelectLinesFromGroups = class(TProchazecLiniiGauge)
private
  FGroupID: longInt;
  function GetOwnerGroup(aArea: TTopoLAreaElement): TEntity;
  function DifferentGroups(aLine: TTopoLLineElement): boolean;
public
  procedure Process(Element: TElement); override;
  procedure DoIt; override;
published
  property GroupId: longInt read FGroupId write FGroupID;
end;

*)

implementation
uses
  Forms,
// KERNEL
  e_Class,
// GLIB
  e_WCS,
// DIALOG
  //f_Gauge,
// SPACE
  e_Geometry,
// SPACETOOL
  e_Node,
// TOPOL
  //E_TplPrp,
  T_TopExp,
  U_TopolT;


{TPrepareExportLinesToTopol}

procedure TPrepareExportLinesToTopol.DoLine(aLine: TTopoLLineElement);
var
  aNode: TNode;
  aBranch, LeftBranch, RightBranch: TNodeBranch;
  aDrziSe: TDrziSe;
  Lin: TPolyLineElement;
  //aProps: TTopolLineProps;
begin
  //if not (aLine.Properties is TTopolLineProps) then exit;
  if not (TElement(aLine) is TTopolLineElement) then exit;
  //aProps:= TTopolLineProps(aLine.Properties);
  aDrziSe:= aLine.DrziSe;

  Lin := TPolyLineElement(aLine.Geometry);
  { hledn soused linie, nejdve sousednch vazeb }
  GiveNodeBranchForLine(Lin, true, aNode, aBranch);
  { koho se drm rukama }
  if aNode <> nil then
  begin
    LeftBranch:= aNode.GetNextBranchLeft(aBranch);
    aDrziSe.Pravou.Rukou.ZaRuku:= LeftBranch.LineBegin;
    //aDrziSe.Pravou.Rukou.Koho:= TTopoLLineElement(LeftBranch.Line.FindOwnerClass(TTopoLLineElement)).ID;
    aDrziSe.Pravou.Rukou.Koho:= TTopoLLineElement(LeftBranch.Line.FindClassWithOwnerInOwners(TTopoLLineElement,TEntity(Input))).ID;

    RightBranch:= aNode.GetNextBranchRight(aBranch);
    //aDrziSe.Levou.Rukou.Koho:= TTopoLLineElement(RightBranch.Line.FindOwnerClass(TTopoLLineElement)).ID;
    aDrziSe.Levou.Rukou.Koho:= TTopoLLineElement(RightBranch.Line.FindClassWithOwnerInOwners(TTopoLLineElement,TEntity(Input))).ID;
    aDrziSe.Levou.Rukou.ZaRuku:= RightBranch.LineBegin;
  end;

  GiveNodeBranchForLine(Lin, false, aNode, aBranch);
  { koho se drm nohama }
  if aNode <> nil then
  begin
    LeftBranch:= aNode.GetNextBranchLeft(aBranch);
    //aDrziSe.Levou.Nohou.Koho:= TTopoLLineElement(LeftBranch.Line.FindOwnerClass(TTopoLLineElement)).ID;
    aDrziSe.Levou.Nohou.Koho:= TTopoLLineElement(LeftBranch.Line.FindClassWithOwnerInOwners(TTopoLLineElement,TEntity(Input))).ID;
    aDrziSe.Levou.Nohou.ZaRuku:= LeftBranch.LineBegin;

    RightBranch:= aNode.GetNextBranchRight(aBranch);
    //aDrziSe.Pravou.Nohou.Koho:= TTopoLLineElement(RightBranch.Line.FindOwnerClass(TTopoLLineElement)).ID;
    aDrziSe.Pravou.Nohou.Koho:= TTopoLLineElement(RightBranch.Line.FindClassWithOwnerInOwners(TTopoLLineElement,TEntity(Input))).ID;
    aDrziSe.Pravou.Nohou.ZaRuku:= RightBranch.LineBegin
  end;
  aLine.DrziSe:= aDrziSe
end;



procedure TPrepareExportLinesToTopol.DoIt;
{ problematika slovn:
  nelze se spolhat na CISLO v TTopolLineProps, nebo nkolik linii me mt
  stejn property - vznikly rozdlenm star linie,
  spolehnme se proto na ID jako na loit doasnho indexu }
var
  I: longInt;
  aLine: TTopoLLineElement;
begin
  if not (Input is TEntity) then exit;
  { oslovn vech lini }
  FTopolNo:= 0;
  for I:= 0 to TEntity(Input).Count-1 do
  begin
    aLine:= TTopoLLineElement(TEntity(Input).Elements[I]);
    if not (TElement(aLine) is TTopoLLineElement) then continue;
    inc(FTopolNo);
    aLine.ID:= FTopolNo;
  end;

  { zpracovn vazeb vech lini }

  for I:= 0 to TEntity(Input).Count-1 do
  begin
    aLine:= TTopoLLineElement(TEntity(Input).Elements[I]);
    if not (TElement(aLine) is TTopoLLineElement) then continue;
    if NoTopology then
       aLine.DrziSe := NedrziSe
    else
       DoLine(aLine);
  end;

  Output:= Input
end;



procedure TPrepareExportLinesToTopol.SetNoTopology(const Value: boolean);
begin
  FNoTopology := Value;
end;

{TExportLinesToTopol}

procedure TExportLinesToTopol.DoIt;
var
  PELT: TPrepareExportLinesToTopol;
  aLine: TTopoLLineElement;
  I: longInt;
  TLW: TTopolLinWriter;
  //F_GaugeForm: TF_GaugeForm;
begin
  if input = nil then exit;
  if not (Input is TEntity) then exit;
  if BlockName = '' then exit;

  { pprava lini - oslovn a nastaven topologie }
  PELT:= TPrepareExportLinesToTopol.Create;
  PELT.Input:= Input;
  PELT.NoTopology := NoTopology;
  PELT.DoIt;
  PELT.Free;

  TLW:= TTopolLinWriter.CreateAll(FBlockName, TJTSK.Create, true);

  if TLW = nil then exit;
  TLW.NoTopology := NoTopology;
  if JedenDruh > 0 then
     TLW.Druh:= JedenDruh;
  {
  Application.CreateForm(TF_GaugeForm, F_GaugeForm);
  StartProgress('Export lini do ' + BlockName, 0, TEntity(Input).Count);
  F_GaugeForm.MaxValue:= TEntity(Input).Count;
  F_GaugeForm.Caption:= 'Export lini do ' + BlockName;
  F_GaugeForm.Show;
  }
  StartProgress('Export lini do ' + BlockName, 0, TEntity(Input).Count);

  for I:= 0 to TEntity(Input).Count-1 do
  begin
    {
    if F_gaugeForm.DoCloseQuery and
       F_gaugeForm.RealyClose('Peruit export lini ?') then
       break;
    }
    Progress:= I;
    Application.ProcessMessages;

    aLine:= TTopoLLineElement(TEntity(Input).Elements[I]);
    if not (TElement(aLine) is TTopoLLineElement) then continue;
    TLW.WriteLin2(aLine);
    ExportObject(aLine);
  end;
  //F_GaugeForm.Free;
  ProgressText := 'Konec exportu ...';
  Application.ProcessMessages;
  TLW.Free;
end;

(*
              { TSelectLinesFromGroups }


function TSelectLinesFromGroups.GetOwnerGroup(aArea: TTopoLAreaElement): TEntity;
var
  aOwn: TEntity;
  I: LongInt;
begin
  result:= nil;

  if aArea.IsHole then
  begin
    aOwn:= TEntity(aArea.findOwnerClass(THolesEntity));
    if aOwn = nil then exit;
    aArea:= THolesEntity(aOwn).OwnerArea;
    if aArea = nil then exit;
  end;

  for I:= 0 to aArea.OwnersCount-1 do
  begin
    aOwn:= aArea.Owners[I];
    if aOwn.ID = FGroupID then
    begin
      result:= aOwn;
      break
    end;
  end;
end;


function TSelectLinesFromGroups.DifferentGroups(aLine: TPolyLineElement): boolean;
var
  Area1, Area2: TAreaElement;
  ArOw1, ArOw2: TEntity;
  I: longInt;
begin
  result:= true;
  Area1:= nil;
  Area2:= nil;
  for I:= 0 to aLine.OwnersCount-1 do
  begin
    Area2:= TAreaElement(aLine.Owners[I]);
    if not (Area2 is TAreaElement) then continue;

    if Area1=nil then
    begin
       Area1:= Area2;
       Area2:= nil
    end;

    if Area2<> nil then break;
  end;

  if (Area1=nil) or (Area2=nil) then exit;

  ArOw1:= GetOwnerGroup(Area1);
  ArOw2:= GetOwnerGroup(Area2);

  result:= ArOw1 <> ArOw2;
end;


procedure TSelectLinesFromGroups.Process(Element: TElement);
var
  OutLines: TPolyLineString;
  aLine: TPolyLineElement;
begin
  if not (output is TPolyLineString) then exit;
  OutLines:= TPolyLineString(Output);
  aLine:= TPolyLineElement(Element);

  if DifferentGroups(aLine) then
  begin
    if not (output is TPolyLineString) then exit;
    OutLines:= TPolyLineString(Output);
    OutLines.AddAndUpdateRect(aLine);
  end
end;


procedure TSelectLinesFromGroups.DoIt;
begin
  if not (output is TPolyLineString) then exit;
  inherited DoIt;
end;

*)


procedure TExportLinesToTopol.SetNoTopology(const Value: boolean);
begin
  FNoTopology := Value;
end;

initialization
  RegisterOrbisClass(TPrepareExportLinesToTopol);
  RegisterOrbisClass(TExportLinesToTopol);
  //RegisterOrbisClass(TSelectLinesFromGroups);
end.
