unit TabletUnit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  WinTablet, WintabConsts, Buttons, StdCtrls, ExtCtrls, ComCtrls;

type
  TMainForm = class(TForm)
    StatusBar1: TStatusBar;
    PressureBar: TProgressBar;
    DataList: TListView;
    Panel1: TPanel;
    chkEnable: TCheckBox;
    Tablet: TWIN_Tablet;
    cmbOutput: TComboBox;
    pnlCursor: TPanel;
    lblOutput: TLabel;
    lblCursor: TLabel;
    Button1: TButton;
    procedure TabletPacket(Sender: TObject; PacketNumber, ContextHandle: HCtx; pPacket: Pointer);
    procedure TabletCursorChanged(Sender: TObject; PacketNumber, ContextHandle: HCtx; pPacket: Pointer);
    procedure TabletProximity(Sender: TObject; ContextHandle: HCtx; Entering: Boolean);
    procedure chkEnableClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure cmbOutputChange(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure Button1Click(Sender: TObject);
  public
    procedure AddListRow(A, B: string);
  end;

  { Application-specific packet structure. Setting the context
    PktData variable will define what data to retrieve. }
  TMyPacket = record
    Cursor: Integer;
    Buttons: Integer;
    Coords: TPoint;
    Pressure: Integer;
    Orientation: TOrientation;
  end;
  PMyPacket = ^TMyPacket;

const
  lcPktData = PK_CURSOR or PK_BUTTONS or PK_X or PK_Y or PK_NORMAL_PRESSURE or PK_ORIENTATION;
  lcOptions = {CXO_SYSTEM or }CXO_MESSAGES or CXO_CSRMESSAGES;
  lcBtnMask = SBN_LCLICK or SBN_RCLICK or SBN_LDBLCLICK or SBN_RDBLCLICK;

var
  MainForm: TMainForm;

implementation

{$R *.DFM}

procedure TMainForm.AddListRow(A, B: string);
var
  Item: TListItem;
begin
  Item := DataList.Items.Add;
  Item.Caption := A;
  Item.SubItems.Add(B);
end;

procedure TMainForm.chkEnableClick(Sender: TObject);
begin
  Tablet.Enabled := TCheckBox(Sender).Checked;
end;

procedure TMainForm.TabletPacket(Sender: TObject; PacketNumber, ContextHandle: HCtx;
  pPacket: Pointer);
begin
  with TMyPacket(pPacket^), StatusBar1 do
  begin
    PressureBar.Position := Pressure;
    Panels[0].Text := IntToStr(Cursor);
    Panels[1].Text := IntToStr(Coords.X);
    Panels[2].Text := IntToStr(Coords.Y);
    Panels[3].Text := IntToStr(Pressure);
    Panels[4].Text := IntToStr(Buttons);
  end;
end;

procedure TMainForm.TabletCursorChanged(Sender: TObject; PacketNumber,
  ContextHandle: HCtx; pPacket: Pointer);
var
  Csr: Integer;
  CsrType: Integer;
  CsrPhysID: Integer;
begin
  Csr := TMyPacket(pPacket^).Cursor;
  WTInfo(WTI_CURSORS + Csr, CSR_TYPE, @CsrType);
  WTInfo(WTI_CURSORS + Csr, CSR_PHYSID, @CsrPhysID);
  lblCursor.Caption := 'Cursor: ' + GetInfoAsString(WTI_CURSORS, CSR_NAME) + #13#10 + ' (type = $' +
  IntToHex(CsrType and $0F06, 4)
  + ', physid = $' + IntToHex(CsrPhysID, 4) + ')';
  case Csr of
    1: pnlCursor.Color := $996633;
    2: pnlCursor.Color := $663399;
    3: pnlCursor.Color := $339966;
  end;
end;

procedure TMainForm.TabletProximity(Sender: TObject; ContextHandle: HCtx;
  Entering: Boolean);
begin
  if Entering then
    StatusBar1.Panels[5].Text := 'Pen down'
  else
  begin
    StatusBar1.Panels[5].Text := 'Pen up';
    pnlCursor.Color := clBlack;
  end;
end;

procedure TMainForm.cmbOutputChange(Sender: TObject);
var
  TabletX, TabletY: TAxis;
  AContext: TLogContext;
begin
  if not Tablet.Present then Exit;
  AContext := Tablet.Context;
  WTInfo(WTI_DEVICES, DVC_X, @TabletX);
  WTInfo(WTI_DEVICES, DVC_Y, @TabletY);
  with AContext do
    case TComboBox(Sender).ItemIndex of
      0: { Output coordinates using digitizer resolution }
      begin
      	OutExtX := TabletX.Max;
      	OutExtY := TabletY.Max;
      end;
      1: { Output coordinates using screen resolution }
      begin
      	OutExtX := GetSystemMetrics(SM_CXSCREEN);
      	OutExtY := -GetSystemMetrics(SM_CYSCREEN);
      end;
    end;
  Tablet.Context := AContext;
end;

procedure TMainForm.FormCreate(Sender: TObject);
var
  AContext: TLogContext;
  ResX, ResY: TAxis;
begin
  with Tablet, AContext do
  begin
    if not Present then
    begin
      ShowMessage('No digitizer present on system.');
      Exit;
    end;
    { Fill context with defaults }
    DefaultContext(AContext);

    { Set option flags to initialize messaging }
  	Options := Options or lcOptions;
    { Bitfields in PktData indicates what data to receive in packets }
  	PktData := lcPktData;
    { Mask buttons (optional) }
    BtnDnMask := lcBtnMask;
    BtnUpMask := BtnDnMask;
    MsgBase := WT_DEFBASE;
    { This will allocate memory for the packets }
    PacketSize := SizeOf(TMyPacket);
    { Open the context and get handle }
    Enabled := True;
    Open(AContext);
  end;

  cmbOutput.ItemIndex := 0;
  with Tablet.Pressure do
  begin
    PressureBar.Min := Min;
    PressureBar.Max := Max;
  end;

  WTInfo(WTI_DEVICES, DVC_X, @ResX);
  WTInfo(WTI_DEVICES, DVC_Y, @ResY);

  AddListRow('Device Name', Tablet.DeviceName);
  AddListRow('Wintab ID', Tablet.WintabID);
  AddListRow('Packet rate', IntToStr(Tablet.PacketRate) + ' Hz');
  AddListRow('Tablet resolution', IntToStr(ResX.Max)  + ' x ' + IntToStr(ResY.Max));
  AddListRow('Number of devices supported',
    IntToStr(GetInfoAsInteger(WTI_INTERFACE, IFC_NDEVICES)));
  AddListRow('Number of cursors supported',
    IntToStr(GetInfoAsInteger(WTI_INTERFACE, IFC_NCURSORS)));
  AddListRow('Number of contexts supported',
    IntToStr(GetInfoAsInteger(WTI_INTERFACE, IFC_NCONTEXTS)));
end;

procedure TMainForm.FormDestroy(Sender: TObject);
begin
  Tablet.Close;
end;

procedure TMainForm.Button1Click(Sender: TObject);
begin
  WTConfig(Tablet.ContextHandle, Handle);
end;

end.
